﻿using System;
using System.Collections.Generic;
using System.Linq;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Infrastructure.Controls;
using System.Windows;
using System.ComponentModel;
using System.Windows.Input;

namespace ClinSchd.Infrastructure
{
    /// <summary>
    /// 
    /// </summary>
    public abstract class AsyncValidatableModel: DependencyObject, INotifyPropertyChanged, IDataErrorInfo
    {	
        /// <summary>
        /// 
        /// </summary>
        public AsyncValidatableModel ()
        {
            IsAsync = true;
        }

        #region Async

        public AsyncWrapper AsyncWrapper
        {
            get
            {
                try
                {
                    return (AsyncWrapper)GetValue(AsyncWrapperProperty);
                }
                catch
                {
                    return null;
                }
            }
            set
            {
                try
                {
                    SetValue(AsyncWrapperProperty, value);
                }
                catch
                {
                }
            }
        }

        public static readonly DependencyProperty AsyncWrapperProperty =
            DependencyProperty.Register ("AsyncWrapper", typeof (AsyncWrapper), typeof (AsyncValidatableModel));

        WorkCompletedMethod _externalCompletedMethod;
        private bool _useDefultBusyIndicator = true;
        private bool _closeBeforeWorkCompleted;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="s"></param>
        /// <param name="args"></param>
        public delegate void WorkMethod (object s, DoWorkEventArgs args);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="s"></param>
        /// <param name="args"></param>
        public delegate void WorkCompletedMethod (object s, RunWorkerCompletedEventArgs args);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="workMethod"></param>
        /// <param name="workCompletedMethod"></param>
        /// <param name="useDefultBusyIndicator"></param>
        /// <param name="message"></param>
        /// <param name="closeBeforeWorkCompleted"></param>
        /// <param name="subMessage"></param>
        /// <param name="displayDelay"></param>
        public void DoWorkAsync(WorkMethod workMethod, WorkCompletedMethod workCompletedMethod, bool useDefultBusyIndicator = true, bool closeBeforeWorkCompleted = false, string message = "", string subMessage = "", TimeSpan? displayDelay = null)
        {
            _useDefultBusyIndicator = useDefultBusyIndicator;
            _closeBeforeWorkCompleted = closeBeforeWorkCompleted;
            _externalCompletedMethod = workCompletedMethod;

            if (IsAsync)
            {
                if (useDefultBusyIndicator)
                    MakeBusy (message, subMessage, displayDelay);

                BackgroundWorker worker = new BackgroundWorker();
                worker.DoWork += new DoWorkEventHandler(workMethod);
                worker.RunWorkerCompleted += OnWorkerCompleted;
                worker.RunWorkerAsync();
            }
            else
            {
                DoWorkEventArgs args = new DoWorkEventArgs (null);
                workMethod.Invoke(null, args);
                RunWorkerCompletedEventArgs completedArgs = new RunWorkerCompletedEventArgs (args.Result, null, args.Cancel);
                _externalCompletedMethod.Invoke(null, completedArgs);
            }
        }

        private void MakeBusy (string message = "", string subMessage = "", TimeSpan? displayDelay = null)
        {
            if (AsyncWrapper != null)
            {
                AsyncWrapper.Dispatcher.BeginInvoke (new Action (() => {
                    AsyncWrapper.DisableScreen (message, subMessage, displayDelay);
                }));
            }
        }

        protected void OnWorkerCompleted(object s, RunWorkerCompletedEventArgs args)
        {
            if (_useDefultBusyIndicator && _closeBeforeWorkCompleted)
                MakeNotBusy(s, args);

            if (_externalCompletedMethod != null)
                _externalCompletedMethod(s, args);

            if (_useDefultBusyIndicator && !_closeBeforeWorkCompleted)
                MakeNotBusy(s, args);
        }

        private void MakeNotBusy (object s, RunWorkerCompletedEventArgs args)
        {
            if (AsyncWrapper != null)
            {
                AsyncWrapper.StopDelayTimer();
                AsyncWrapper.Dispatcher.BeginInvoke (new Action (() => {
                    AsyncWrapper.EnableScreen ();
                }));                
            }
        }
        #endregion

        #region Validation
        Dictionary<string, string> _errors = new Dictionary<string, string> ();
        public Dictionary<string, string> Errors
        {
            get { return _errors; }
            set
            {
                _errors = value;
                BuildErrorSummary ();
            }
        }

        //string _error_summary_header_template = "Please correct the following errors:\n";
        string _errorSummaryHeaderTemplate = string.Empty;
        public string ErrorSummaryHeaderTemplate
        {
            get { return _errorSummaryHeaderTemplate; }
            set { _errorSummaryHeaderTemplate = value; }
        }

        string _errorSummaryItemTemplate = "{0}: {1}\n";
        public string ErrorSummaryItemTemplate
        {
            get { return _errorSummaryItemTemplate; }
            set { _errorSummaryItemTemplate = value; }
        }


        /// <summary>
        /// Can be used for Save command binding to prohibit Save
        /// </summary>
        public bool HasErrors
        {
            get { return _errors.Count > 0; }
        }


        protected void AddError (string field, string error)
        {
            if (_errors.ContainsKey (field))
                _errors[field] = error;
            else
                _errors.Add (field, error);
            BuildErrorSummary ();
        }

        protected void ClearError (string field)
        {
            if (_errors.ContainsKey (field))
                _errors.Remove (field);
            BuildErrorSummary ();
        }

        public void ClearErrors ()
        {
            Error = string.Empty;
            Errors.Clear ();
        }

        /// <summary>
        /// Error summary
        /// </summary>
        public string Error { get; set; }
        
        /// <summary>
        /// Should async calls be made or not.
        /// </summary>
        public bool IsAsync { get; set; }

        /// <summary>
        /// IDataErrorInfo method for access to errors by the view.
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public string this[string name]
        {
            get
            {
                if (_errors.ContainsKey (name))
                    return _errors[name];
                else
                    return null;
            }
        }

        void BuildErrorSummary ()
        {
            if (_errors.Any()) {
                Error = _errorSummaryHeaderTemplate;

                foreach (KeyValuePair<string, string> err in _errors)
                    Error += string.Format (_errorSummaryItemTemplate, err.Key, err.Value);
                Error = Error.TrimEnd ("\n".ToCharArray ());
            } else
                Error = string.Empty;

            NotifyPropertyChanged ("Error");	// cause view to refresh error summary
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;
        public virtual void NotifyPropertyChanged (string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged (this, new PropertyChangedEventArgs (propertyName));
        }
        #endregion

        #endregion

        protected ICommand GetCommand( string name, Action<object> execute, Predicate<object> canExecute )
        {
            ICommand command;

            if ( _commands.ContainsKey( name ) )
            {
                command = _commands[name];
            }
            else
            {
                command = new CommandRelay( execute, canExecute );
                _commands.Add( name, command );
            }

            return command;
        }

        private readonly Dictionary<string, ICommand> _commands = new Dictionary<string, ICommand>();
    }
}
