﻿using System;
using System.Windows;
using System.Windows.Input;
using System.Windows.Threading;


namespace ClinSchd.Infrastructure.Controls
{
    /// <summary>
    /// Interaction logic for BusyIndicatorPanel.xaml
    /// </summary>
    public partial class BusyIndicatorPanel
    {
        private DispatcherTimer _timer;

        public static readonly DependencyProperty IsCloseButtonVisibleProperty =
            DependencyProperty.Register("IsCloseButtonVisible", typeof(bool), typeof(BusyIndicatorPanel), new UIPropertyMetadata(false));

        public static readonly DependencyProperty IsBusyProperty =
            DependencyProperty.Register("IsBusy", typeof(bool), typeof(BusyIndicatorPanel), new UIPropertyMetadata(false));

        public static readonly DependencyProperty IsLoadingProperty =
            DependencyProperty.Register("IsLoading", typeof(bool), typeof(BusyIndicatorPanel), new UIPropertyMetadata(false));
        public static readonly DependencyProperty IsMessageVisibleProperty =
            DependencyProperty.Register("IsMessageVisible", typeof(bool), typeof(BusyIndicatorPanel), new UIPropertyMetadata(true));

        public static readonly DependencyProperty IsSubMessageVisibleProperty =
            DependencyProperty.Register("IsSubMessageVisible", typeof(bool), typeof(BusyIndicatorPanel), new UIPropertyMetadata(true));

        public static readonly DependencyProperty MessageProperty =
            DependencyProperty.Register("Message", typeof(string), typeof(BusyIndicatorPanel), new UIPropertyMetadata("Loading..."));

        public static readonly DependencyProperty SubMessageProperty =
            DependencyProperty.Register("SubMessage", typeof(string), typeof(BusyIndicatorPanel), new UIPropertyMetadata(string.Empty));

        public static readonly DependencyProperty ClosePanelCommandProperty =
            DependencyProperty.Register("ClosePanelCommand", typeof(ICommand), typeof(BusyIndicatorPanel));

        public static readonly DependencyProperty DisplayAfterProperty =
            DependencyProperty.Register("DiaplsyAfter", typeof(TimeSpan?), typeof(BusyIndicatorPanel));

        /// <summary>
        /// Initializes a new instance of the <see cref="BusyIndicatorPanel"/> class.
        /// </summary>
        public BusyIndicatorPanel()
        {
            InitializeComponent();
            _timer = new DispatcherTimer();
            _timer.Tick += _timer_Tick;
            CanCloseWindow = false;
            DisplayAfter = null;
        }

        private void _timer_Tick(object sender, EventArgs e)
        {
            _timer.Stop();
            IsBusy = true;
        }

        public TimeSpan? DisplayAfter
        {
            get { return (TimeSpan?)GetValue(DisplayAfterProperty); }
            set
            {
                SetValue(DisplayAfterProperty, value);
                _timer.Interval = value.HasValue ? value.Value : TimeSpan.FromTicks(0);
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the control is visible.
        /// </summary>
        /// <value>
        /// <c>true</c> if the control is visible, <c>false</c>.
        /// </value>
        private bool IsBusy
        {
            get { return (bool)GetValue(IsBusyProperty); }
            set { SetValue(IsBusyProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the close button is visible.
        /// </summary>
        /// <value>
        /// <c>true</c> if the close button is visible, <c>false</c>.
        /// </value>
        public bool IsCloseButtonVisible
        {
            get { return (bool)GetValue(IsCloseButtonVisibleProperty); }
            set { SetValue(IsCloseButtonVisibleProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is loading.
        /// </summary>
        /// <value>
        /// <c>true</c> if this instance is loading; otherwise, <c>false</c>.
        /// </value>
        public bool IsLoading
        {
            get { return (bool)GetValue(IsLoadingProperty); }
            set
            {
                SetValue(IsLoadingProperty, value);

                if (value)
                {
                    if (DisplayAfter.HasValue)
                        _timer.Start();
                    else
                        IsBusy = true;
                }
                else
                {
                    _timer.Stop();
                    IsBusy = false;
                }
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the Message is visible.
        /// </summary>
        /// <value>
        /// <c>true</c> if the Message is visible, <c>false</c>.
        /// </value>
        public bool IsMessageVisible
        {
            get { return (bool)GetValue(IsMessageVisibleProperty); }
            set { SetValue(IsMessageVisibleProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the Submessage is visible.
        /// </summary>
        /// <value>
        /// <c>true</c> if the Submessage is visible, <c>false</c>.
        /// </value>
        public bool IsSubMessageVisible
        {
            get { return (bool)GetValue(IsSubMessageVisibleProperty); }
            set { SetValue(IsSubMessageVisibleProperty, value); }
        }

        /// <summary>
        /// Gets or sets the message.
        /// </summary>
        /// <value>The message.</value>
        public string Message
        {
            get { return (string)GetValue(MessageProperty); }
            set { SetValue(MessageProperty, value); }
        }

        /// <summary>
        /// Gets or sets the sub message.
        /// </summary>
        /// <value>The sub message.</value>
        public string SubMessage
        {
            get { return (string)GetValue(SubMessageProperty); }
            set { SetValue(SubMessageProperty, value); }
        }

        /// <summary>
        /// Gets or sets the close panel command.
        /// </summary>
        /// <value>The close panel command.</value>
        public ICommand ClosePanelCommand
        {
            get { return (ICommand)GetValue(ClosePanelCommandProperty); }
            set { SetValue(ClosePanelCommandProperty, value); }
        }

        /// <summary>
        /// Called when [close click].
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Windows.RoutedEventArgs"/> instance containing the event data.</param>
        private void OnCloseClick(object sender, RoutedEventArgs e)
        {
            if (ClosePanelCommand != null)
            {
                ClosePanelCommand.Execute(null);
            }
        }

        public bool CanCloseWindow { get; set; }

        /// <summary>
        /// Closes the parent window when this control is so hosted
        ///   Must set CanCloseWindow = true first
        /// </summary>
        public void CloseThisWindow()
        {
            if (CanCloseWindow)
            {
                var window = Window.GetWindow(this);
                if (window != null)
                    window.Close();
                System.Threading.Thread.Sleep(200);
            }
        }

        public System.Windows.Media.Brush BackgroundColor { get { return BackgrounPanel.Background; } } 
    }
}