﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace ClinSchd.Infrastructure.Controls
{
    /// <summary>
    /// Interaction logic for FlaggedTextBlock.xaml
    /// </summary>
    public partial class FlaggedTextBlock : TextBlock
    {
        #region constructor

        public FlaggedTextBlock()
        {
            this.InvalidateMeasure();
        }

        #endregion constructor

        #region IsFlagged Dependency Property

        public Boolean IsFlagged
        {
            get { return (Boolean)GetValue(IsFlaggedProperty); }
            set { SetValue(IsFlaggedProperty, value); }
        }

        public static FrameworkPropertyMetadata IsFlaggedPropertyMetadata = new FrameworkPropertyMetadata(
            false,
            FrameworkPropertyMetadataOptions.AffectsRender | FrameworkPropertyMetadataOptions.AffectsArrange,
            new  PropertyChangedCallback(IsFlaggedProperty_PropertyChanged)
        );

        private static void IsFlaggedProperty_PropertyChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
        {
             var textBlock = (FlaggedTextBlock)obj;
             if ((bool)e.OldValue) textBlock.DrawingVisual = null;
            if ((bool)e.NewValue) textBlock.DrawingVisual = textBlock.CreateDrawingVisualFlag();
        }

        // Using a DependencyProperty as the backing store for IsFlagged.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty IsFlaggedProperty = DependencyProperty.Register(
            "IsFlagged",
            typeof(Boolean),
            typeof(FlaggedTextBlock),
            IsFlaggedPropertyMetadata
        );

        #endregion IsFlagged Dependency Property

        #region FlagBrush Dependency Property

        public Brush FlagBrush
        {
            get { return (Brush)GetValue(FlagBrushProperty); }
            set { SetValue(FlagBrushProperty, value); }
        }

        public static FrameworkPropertyMetadata FlagBrushPropertyMetadata = new FrameworkPropertyMetadata(
            System.Windows.Media.Brushes.Red,
            FrameworkPropertyMetadataOptions.AffectsRender | FrameworkPropertyMetadataOptions.AffectsArrange,
            new PropertyChangedCallback(FlagBrushProperty_PropertyChanged)
        );

        private static void FlagBrushProperty_PropertyChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
        {
            var textBlock = (FlaggedTextBlock)obj;
            if (textBlock.IsFlagged) textBlock.DrawingVisual = textBlock.CreateDrawingVisualFlag();
        }

        // Using a DependencyProperty as the backing store for FlagBrush.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty FlagBrushProperty = DependencyProperty.Register(
            "FlagBrush",
            typeof(Brush),
            typeof(FlaggedTextBlock),
            FlagBrushPropertyMetadata
        );

        #endregion FlagBrush Dependency Property


        public DrawingVisual DrawingVisual
        {
            get { return _drawingVisual; }
            set
            {
                if (_drawingVisual != null)
                    RemoveVisualChild(_drawingVisual);
                _drawingVisual = value;

                if (_drawingVisual != null)
                    if(IsFlagged) AddVisualChild(_drawingVisual);
            }
        }

        private DrawingVisual _drawingVisual;

        protected override int VisualChildrenCount
        {
            get { return base.VisualChildrenCount + (IsFlagged ? 1 : 0); }
        }

        protected override Visual GetVisualChild(int index)
        {
            return index==base.VisualChildrenCount ? DrawingVisual : base.GetVisualChild(index);
        }

        // Create a DrawingVisual that contains a triangle.
        private DrawingVisual CreateDrawingVisualFlag()
        {
            if (_lastSize.IsEmpty)
                return null;

            var triangle = new DrawingVisual();
            using (DrawingContext dc = triangle.RenderOpen())
            {
                double legSize = (_lastSize.Height / 3) * 2;
                var start = new Point(_lastSize.Width, 0);

                var segments = new[]
                { 
                    new LineSegment(new Point(_lastSize.Width, legSize), true), 
                    new LineSegment(new Point(_lastSize.Width - legSize, 0), true)
                };

                var figure = new PathFigure(start, segments, true);
                var geo = new PathGeometry(new[] { figure });
                dc.DrawGeometry(FlagBrush, null, geo);
            }

            return triangle;
        }

        private Size _lastSize = Size.Empty;

        protected override void OnRenderSizeChanged(SizeChangedInfo sizeInfo)
        {
            base.OnRenderSizeChanged(sizeInfo);
            _lastSize = sizeInfo.NewSize;

            DrawingVisual = null;

            if (IsFlagged)
                DrawingVisual = CreateDrawingVisualFlag();
        }
    }
}