﻿using System;
using System.Runtime.InteropServices;
using System.Windows;
using System.Windows.Input;
using System.Windows.Interop;
using ClinSchd.Infrastructure.Interfaces;

namespace ClinSchd.Infrastructure.Models
{
    public class BaseDialogView : Window, IDialogView
    {
        #region IDialogView members

        public object Context
        {
            set
            {
                DataContext = value;
                var closeCommandProvider = value as ICloseCommandProvider;
                if ( closeCommandProvider != null )
                {
                    CloseCommandProvider = closeCommandProvider;
                }
                var saveCommandProvider = value as ISaveCommandProvider;
                if ( saveCommandProvider != null )
                {
                    SaveCommandProvider = saveCommandProvider;
                }
            }
        }

        public ICloseCommandProvider CloseCommandProvider { get; set; }

        public ISaveCommandProvider SaveCommandProvider { get; set; }

        private bool CloseOnSave { get; set; }

        public new void Close()
        {
            Closing -= BaseDialogView_Closing;
            if ( !_isClosing )
            {
                base.Close();
            }
        }

        #endregion IDialogView members

        #region constructor

        protected BaseDialogView()
        {
            CommandBindings.Add( new CommandBinding( ApplicationCommands.Close, CloseCommandBinding_Executed, CloseCommandBinding_CanExecute ) );
            CommandBindings.Add( new CommandBinding( ApplicationCommands.Save, SaveCommandBinding_Executed, SaveCommandBinding_CanExecute ) );

            Closing += BaseDialogView_Closing;
        }

        #endregion constructor

        #region private fields

        private bool _isClosing;

        #endregion

        #region private methods

        private void BaseDialogView_Closing( object sender, System.ComponentModel.CancelEventArgs e )
        {
            if ( CloseCommandProvider != null && CloseCommandProvider.CloseCommand != null )
            {
                if ( CloseCommandProvider.CloseCommand.CanExecute( null ) )
                {
                    _isClosing = true;
                    CloseCommandProvider.CloseCommand.Execute( null );
                }
                else
                {
                    e.Cancel = true;
                }
            }
        }

        private void SaveCommandBinding_CanExecute( object sender, CanExecuteRoutedEventArgs e )
        {
            e.CanExecute = SaveCommandProvider == null || SaveCommandProvider.SaveCommand.CanExecute( null );
        }

        private void SaveCommandBinding_Executed( object sender, ExecutedRoutedEventArgs e )
        {
            if ( SaveCommandProvider != null )
            {

                SaveCommandProvider.SaveCommand.Execute( null );
                if ( CloseOnSave )
                {
                    Close();
                }
            }
        }

        private void CloseCommandBinding_CanExecute( object sender, CanExecuteRoutedEventArgs e )
        {
            e.CanExecute = DialogCanClose();
        }

        private void CloseCommandBinding_Executed( object sender, ExecutedRoutedEventArgs e )
        {
            Close();
        }

        private bool DialogCanClose()
        {
            return CloseCommandProvider == null ||
                   CloseCommandProvider.CloseCommand.CanExecute( null );
        }

        #endregion private methods

        private const int GWL_STYLE = -16;
        private const int WS_SYSMENU = 0x80000;

        [DllImport( "user32.dll", SetLastError = true )]
        private static extern int GetWindowLong( IntPtr hWnd, int nIndex );

        [DllImport( "user32.dll" )]
        private static extern int SetWindowLong( IntPtr hWnd, int nIndex, int dwNewLong );

        protected void RemoveXCloseButton()
        {
            // Remove 'x' close button
            var hwnd = new WindowInteropHelper( this ).Handle;
            SetWindowLong( hwnd, GWL_STYLE, GetWindowLong( hwnd, GWL_STYLE ) & ~WS_SYSMENU );
        }
    }
}
