﻿using System;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using ClinSchd.Infrastructure.Interfaces;

namespace ClinSchd.Infrastructure.Models
{
    public class ContactInfo : MonitoredValuesBaseClass, INotifyPropertyChanged
    {
        #region private fields

        private readonly IDataAccessService _dataAccessService;

        private State _state = new State();
        private Country _country = new Country();
        private bool _isActive;
        private DateTime? _minEndDate = null;

        #endregion private fields

        #region constructor

        public ContactInfo(IDataAccessService dataAccessService, string patientIEN)
        {
            _dataAccessService = dataAccessService;
            PatientIEN = patientIEN;
            Start = null;
            End = null;
            IsActive = true;
        }

        #endregion constructor

        #region public members

        public bool IsActive
        {
            get { return _isActive; }
            set
            {
                _isActive = value; 
                OnPropertyChanged( "IsActive" );
            }
        }

        public string PatientIEN { get; set; }

        public string Street1
        {
            get { return GetMonitoredValue("Street1"); }
            set
            {
                SetMonitoredValue("Street1", value);
                OnPropertyChanged( "Street1" );
            }
        }

        public string Street2
        {
            get { return GetMonitoredValue("Street2"); }
            set
            {
                SetMonitoredValue("Street2", value);
                OnPropertyChanged( "Street2" );
            }
        }

        public string Street3
        {
            get { return GetMonitoredValue("Street3"); }
            set
            {
                SetMonitoredValue("Street3", value);
                OnPropertyChanged( "Street3" );
            }
        }

        public string City
        {
            get { return GetMonitoredValue("City"); }
            set
            {
                SetMonitoredValue("City", value);
                OnPropertyChanged( "City" );
            }
        }

        public State State
        {
            get { return _state ?? ( _state = _dataAccessService.GetPatientState( PatientIEN ) ); }
            set
            {
                _state = value;

                if (_state == null)
                    SetMonitoredValue("State.Ien", String.Empty);
                else
                    SetMonitoredValue("State.Ien", _state.Ien);

                OnPropertyChanged( "State" );
            }
        }

        public string CityState
        {
            get
            {
                string cityState = GetMonitoredValue("CityState");

                if ( string.IsNullOrEmpty( cityState ) )
                {
                    cityState = _dataAccessService.GetPatientCityState( PatientIEN );
                    SetMonitoredValue( "CityState", cityState );
                }

                return cityState;
            }
            set
            {
                SetMonitoredValue("CityState", value);
                OnPropertyChanged( "CityState" );
            }
        }

        //public string ZipCode
        //{
        //    get { return GetMonitoredValue("ZipCode"); }
        //    set
        //    {
        //        SetMonitoredValue("ZipCode", value);
        //        OnPropertyChanged( "ZipCode" );
        //    }
        //}

        public string ZipPlus4
        {
            get { return GetMonitoredValue("ZipPlus4"); }
            set
            {
                SetMonitoredValue("ZipPlus4", value);
                OnPropertyChanged( "ZipPlus4" );
            }
        }

        public string County
        {
            get { return GetMonitoredValue("County"); }
            set
            {
                SetMonitoredValue("County", value);
                OnPropertyChanged( "County" );
            }
        }

        public Country Country
        {
            get
            {
                if ( _country == null )
                    _country = new Country();
                return _country;
            }
            set
            {
                _country = value;

                if (_country == null)
                    SetMonitoredValue("Country.Ien", String.Empty);
                else
                    SetMonitoredValue("Country.Ien", _country.Ien);

                OnPropertyChanged( "Country" );
            }
        }

        [Required(ErrorMessage = @"Not a valid phone number")]
        [RegularExpression(@"^(\+\d{1,2}\s)?\(?\d{3}\)?[\s.-]\d{3}[\s.-]\d{4}$")]
        public string Phone
        {
            get { return GetMonitoredValue("Phone"); }
            set
            {
                SetMonitoredValue("Phone", value);
                OnPropertyChanged( "Phone" );
            }
        }

        public string PhoneFormatted { get { return Keys.Methods.StandardPhoneFormat(Phone); } }

        [Required(ErrorMessage = @"Not a valid phone number")]
        [RegularExpression(@"^(\+\d{1,2}\s)?\(?\d{3}\)?[\s.-]\d{3}[\s.-]\d{4}$")]
        public string PhoneCell
        {
            get { return GetMonitoredValue("PhoneCell"); }
            set
            {
                SetMonitoredValue("PhoneCell", value);
                OnPropertyChanged( "PhoneCell" );
            }
        }

        public string PhoneCellFormatted { get { return Keys.Methods.StandardPhoneFormat(PhoneCell); } }

        [Required(ErrorMessage = @"Not a valid phone number")]
        [RegularExpression(@"^(\+\d{1,2}\s)?\(?\d{3}\)?[\s.-]\d{3}[\s.-]\d{4}$")]
        public string PhoneResidence
        {
            get { return GetMonitoredValue("PhoneResidence"); }
            set
            {
                SetMonitoredValue("PhoneResidence", value);
                OnPropertyChanged( "PhoneResidence" );
            }
        }

        public string PhoneResidenceFormatted { get { return Keys.Methods.StandardPhoneFormat(PhoneResidence); } }

        [Required(ErrorMessage = @"Not a valid phone number")]
        [RegularExpression(@"^(\+\d{1,2}\s)?\(?\d{3}\)?[\s.-]\d{3}[\s.-]\d{4}$")]
        public string PhoneWork
        {
            get { return GetMonitoredValue("PhoneWork"); }
            set
            {
                SetMonitoredValue( "PhoneWork", value );
                OnPropertyChanged( "PhoneWork" );
            }
        }

        public string PhoneWorkFormatted { get { return Keys.Methods.StandardPhoneFormat(PhoneWork); } }

        [Required(ErrorMessage = @"Not a valid phone number")]
        [RegularExpression(@"^(\+\d{1,2}\s)?\(?\d{3}\)?[\s.-]\d{3}[\s.-]\d{4}$")]
        public string PhoneMessage
        {
            get { return GetMonitoredValue("PhoneMessage"); }
            set
            {
                SetMonitoredValue("PhoneMessage", value);
                OnPropertyChanged( "PhoneMessage" );
            }
        }

        public string PhoneMessageFormatted { get { return Keys.Methods.StandardPhoneFormat(PhoneMessage); } }

        [Required(ErrorMessage = @"Not a valid phone number")]
        [RegularExpression(@"^(\+\d{1,2}\s)?\(?\d{3}\)?[\s.-]\d{3}[\s.-]\d{4}$")]
        public string PhonePager
        {
            get { return GetMonitoredValue("PhonePager"); }
            set
            {
                SetMonitoredValue("PhonePager", value);
                OnPropertyChanged( "PhonePager" );
            }
        }

        public string PhonePagerFormatted { get { return Keys.Methods.StandardPhoneFormat(PhonePager); } }

        public string EmailAddress
        {
            get { return GetMonitoredValue("EmailAddress"); }
            set
            {
                SetMonitoredValue("EmailAddress", value);
                OnPropertyChanged( "EmailAddress" );
            }
        }

        public DateTime? Start
        {
            get
            {
                string getValue = GetMonitoredValue("Start");

                if (string.IsNullOrEmpty(getValue))
                    return null;

                return Convert.ToDateTime(getValue);
            }
            set
            {
                string setValue = string.Empty;

                if (value.HasValue)
                {
                    setValue = value.Value.ToString();
                    SetMonitoredValue("Start", setValue);
                    MinEndDate = Start;

                    if (End.HasValue)
                        if (value.Value.CompareTo(End.Value) > 0)
                            End = value;
                }

                OnPropertyChanged("Start");
            }
        }

        public string StartRPCValue
        {
            get
            {
                string getValue = GetRPCValue("Start");

                if (string.IsNullOrEmpty(getValue) || getValue == "@")
                    return getValue;

                return Start.Value.ToShortDateString();
            }
        }

        public DateTime? End
        {
            get
            {
                string getValue = GetMonitoredValue("End");

                if (string.IsNullOrEmpty(getValue))
                    return null;

                return Convert.ToDateTime(getValue);
            }
            set
            {
                string setValue = string.Empty;

                if (value.HasValue)
                    setValue = value.Value.ToString();

                SetMonitoredValue("End", setValue);
                OnPropertyChanged("End");
            }
        }

        public string EndRPCValue
        {
            get
            {
                string getValue = GetRPCValue("End");

                if (string.IsNullOrEmpty(getValue) || getValue == "@")
                    return getValue;

                return End.Value.ToShortDateString();
            }
        }

        public DateTime? MinEndDate
        {
            get {  return _minEndDate; }
            set
            {
                _minEndDate = value;
                OnPropertyChanged("MinEndDate");
            }
        }

        #endregion public members

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged( string propertyName )
        {
            PropertyChanged?.Invoke( this, new PropertyChangedEventArgs( propertyName ) );
        }
    }
}
