﻿using System;
using System.Windows.Media;

using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ScheduleView;

namespace ClinSchd.Infrastructure.Models
{
	/// <summary>
	/// This class wraps a Telerik Appointment
	/// </summary>
	public class CustomAppointment : Appointment
	{
        #region private fields

        private SchdAppointment _schdAppointment;

        private bool _isEnableUpdateAppointment = false;
        private bool _isEditAccessBlockEnabled = true;
        private bool _isDeleteAccessBlockEnabled = true;
        private string _toolTipText = String.Empty;

        #endregion private fields

        #region constructor

        public CustomAppointment() : base()
        {
            // Just to insure base valus have no default value
            base.Body = String.Empty;
            base.Category = null;
            base.Subject = String.Empty;
            _schdAppointment = new SchdAppointment();
        }

        /// <summary>
        /// CustomAppointment
        /// </summary>
        public CustomAppointment(SchdAppointment schdAppointment) : this()
        {
            SchdAppointment = schdAppointment;
		}

        #endregion constructor

        #region Appointment overrides

        public override IAppointment Copy()
        {
            var newAppointment = new CustomAppointment();
            newAppointment.CopyFrom(this);
            return newAppointment;
        }

        public override void CopyFrom(IAppointment other)
        {
            if (other.GetType() == typeof(CustomAppointment))
            {
                CustomAppointment source = (CustomAppointment)other;
                this.SchdAppointment = this.SchdAppointment;
                this.Importance = source.Importance;
                this.IsAllDayEvent = source.IsAllDayEvent;
                this.RecurrenceRule = source.RecurrenceRule;

                foreach (Resource res in source.Resources)
                    this.Resources.Add(new Resource(res.ResourceName, res.ResourceType));

                this.TimeMarker = source.TimeMarker;
                this.TimeZone = source.TimeZone;
                this.IsEditAccessBlockEnabled = source.IsEditAccessBlockEnabled;
                this.IsDeleteAccessBlockEnabled = source.IsDeleteAccessBlockEnabled;
            }
            else
                base.CopyFrom(other);
        }

        //
        // Summary:
        //     Gets or sets the System.String value representing the body of the Telerik.Windows.Controls.ScheduleView.IAppointment
        //     object.
        public override string Body
        {
            get
            {
                if (String.IsNullOrEmpty(base.Body))
                    if (_schdAppointment != null)
                        return _schdAppointment.NOTE;

                return base.Body;
            }
            set { base.Body = value; }
        }

        //
        // Summary:
        //     Gets or sets the System.String value representing the category of the Telerik.Windows.Controls.ScheduleView.IAppointment
        //     object.
        public override ICategory Category
        {
            get
            {
                if (base.Category == null)
                    return Keys.ColorCodes.AppointmentCategories[_schdAppointment.CategoryName];

                return base.Category;
            }
            set { base.Category = value; }
        } 

        //
        // Summary:
        //     Gets or sets the System.DateTime value determining the end date and time of the
        //     Telerik.Windows.Controls.ScheduleView.IAppointment.
        public override DateTime End
        {
            get
            {
                DateTime retValue = new DateTime();

                if (!string.IsNullOrEmpty(_schdAppointment?.END_TIME))
                    retValue = Convert.ToDateTime(_schdAppointment.END_TIME);

                if (IsTimeLineView)
                    return retValue.Date.AddDays(1).AddMinutes(-1);

                return retValue;
            }
            set
            {
                if (_schdAppointment != null)
                    _schdAppointment.END_TIME = value.ToString();

                OnPropertyChanged("End");
            }
        }

        //
        // Summary:
        //     Gets or sets the importance.
        /* public override Importance Importance { get; set; } */

        //
        // Summary:
        //     Gets or sets the System.Boolean value indicating whether the current Telerik.Windows.Controls.ScheduleView.IAppointment
        //     is an All-day one.
        /* public override bool IsAllDayEvent { get; set; } */

        //
        // Summary:
        //     Gets or sets the Telerik.Windows.Controls.ScheduleView.IRecurrenceRule value
        //     determining the recurrence rule of the Telerik.Windows.Controls.ScheduleView.IAppointment.
        /* public override IRecurrenceRule RecurrenceRule { get; set; } */

        //
        // Summary:
        //     Gets the collection containing the resources, associated with the appointment.
        /* public override ResourceCollection Resources { get; } */

        //
        // Summary:
        //     Gets or sets the System.DateTime value determining the start date and time of
        //     the Telerik.Windows.Controls.ScheduleView.IAppointment.
        public override DateTime Start
        {
            get
            {
                DateTime retValue = new DateTime();

                if (!string.IsNullOrEmpty(_schdAppointment?.START_TIME))
                    retValue = Convert.ToDateTime(_schdAppointment.START_TIME);

                if (IsTimeLineView)
                    return retValue.Date;

                return retValue;
            }
            set
            {
                if (_schdAppointment != null)
                    _schdAppointment.START_TIME = value.ToString();

                OnPropertyChanged("Start");
            }
        }

        //
        // Summary:
        //     Gets or sets the System.String value representing the subject of the Telerik.Windows.Controls.ScheduleView.IAppointment
        //     object.
        public override string Subject
        {
            get
            {
                if (String.IsNullOrEmpty(base.Subject))
                    if (_schdAppointment != null)
                        return _schdAppointment.PATIENTNAME;

                return base.Subject;
            }
            set
            { base.Subject = value; }
        }

        //
        // Summary:
        //     Gets or sets the System.String value representing the body of the Telerik.Windows.Controls.ScheduleView.IAppointment
        //     object.
        /* public override ITimeMarker TimeMarker { get; set; } */

        //
        // Summary:
        //     Gets or sets the time zone.
        /* public override TimeZoneInfo TimeZone { get; set; } */

        #endregion Appointment overrides

        #region public members

        public Brush BackgroundColor
        {
            get { return ((Category)this.Category).CategoryBrush; }
        }

        public SchdAppointment SchdAppointment
        {
            get { return _schdAppointment; }
            set
            {
                _schdAppointment = value;
                OnPropertyChanged("");
                UniqueId = _schdAppointment.APPOINTMENTID;
            }
        }

        public string Initials
        {
            get
            {
                if (_schdAppointment != null)
                    if (!String.IsNullOrEmpty(_schdAppointment.PATIENTNAME))
                    {
                        string[] names = _schdAppointment.PATIENTNAME.Split(',');

                        if (names.Length > 1)
                            return names[0].Substring(0, 1) + names[1].Substring(0, 1);
                        else
                            return _schdAppointment.PATIENTNAME.Substring(0, 1);
                    }

                return Subject;
            }
        }

        public bool IsEnableUpdateAppointment
        {
            get { return _isEnableUpdateAppointment && !Keys.LoggedInUserSDECViewOnlyKey; }
            set
            {
                _isEnableUpdateAppointment = value;
                OnPropertyChanged("");
            }
        }

        public bool IsNoShowAppointmentEnabled
		{
			get
			{
                if (_schdAppointment == null)
                    return false;

                return IsEnableUpdateAppointment && _schdAppointment.CanUndoMarkAsNoShow;
			}
		}

		public bool IsCheckedInAppointmentEnabled
		{
			get
			{
                if (_schdAppointment == null)
                    return false;

                return IsEnableUpdateAppointment && _schdAppointment.CanUndoCheckIn;
			}
		}

		public bool IsCheckedOutAppointmentEnabled
		{
			get
			{
                if (_schdAppointment == null)
                    return false;

                return IsEnableUpdateAppointment && _schdAppointment.CanUndoCheckOut;
            }
		}

		public bool IsCancelledAppointmentEnabled
		{
			get
			{
                if (_schdAppointment == null)
                    return false;

                return IsEnableUpdateAppointment && _schdAppointment.CanUndoCancel;
            }
		}

		public bool IsNotNoShowAppointmentEnabled
		{
			get
			{
                if (_schdAppointment == null)
                    return false;

                return IsEnableUpdateAppointment && _schdAppointment.CanMarkAsNoShow;
            }
		}

		public bool IsNotCheckedInAppointmentEnabled
		{
			get
			{
                if (_schdAppointment == null)
                    return false;

                return IsEnableUpdateAppointment && _schdAppointment.CanCheckIn;
            }
		}

		public bool IsNotCheckedOutAppointmentEnabled
		{
			get
			{
                if (_schdAppointment == null)
                    return false;

                return IsEnableUpdateAppointment && _schdAppointment.CanCheckOut;
            }
		}

		public bool IsNotCancelledAppointmentEnabled
		{
			get
			{
                if (_schdAppointment == null)
                    return false;

                return _schdAppointment.CanCancel && !Keys.LoggedInUserSDECViewOnlyKey;             }
		}

		public bool IsEditAppointmentEnabled
		{
			get
			{
                if (_schdAppointment == null)
                    return false;

                return IsEnableUpdateAppointment && _schdAppointment.CanEdit;
			}
		}

		public bool IsEditAccessBlockEnabled
		{
			get
			{
				return this._isEditAccessBlockEnabled;
			}
			set
			{
				if (_isEditAccessBlockEnabled != value) {
					this._isEditAccessBlockEnabled = value;
					this.OnPropertyChanged ("IsEditAccessBlockEnabled");
				}
			}
		}

		public bool IsDeleteAccessBlockEnabled
		{
			get
			{
				return this._isDeleteAccessBlockEnabled;
			}
			set
			{
				if (_isDeleteAccessBlockEnabled != value) {
					this._isDeleteAccessBlockEnabled = value;
					this.OnPropertyChanged ("IsDeleteAccessBlockEnabled");
				}
			}
		}

        public AppointmentStatuses Status
        {
            get
            {
                if (_schdAppointment == null)
                    return AppointmentStatuses.Unknown;

                return _schdAppointment.AppointmentStatus;
            }
        }

        public string StatusText
        {
            get
            {
                if (_schdAppointment == null)
                    return String.Empty;

                return _schdAppointment.AppointmentStatusText;
            }
        }

        public string ToolTipText
        {
            get { return _toolTipText; }
            set
            {
                _toolTipText = value;
                OnPropertyChanged("ToolTipText");
            }
        }

        public bool ToolTipEnabled {  get { return !String.IsNullOrEmpty(ToolTipText); } }

        public bool IsTimeLineView { get; set; } = false;

	    public DateTime StartActual
	    {
	        get
	        {
                if (_schdAppointment != null)
                    if (!String.IsNullOrEmpty(_schdAppointment.START_TIME))
                        return Convert.ToDateTime(_schdAppointment.START_TIME);

                return new DateTime();
            }
        }

        public DateTime EndActual
        {
            get
            {
                if (_schdAppointment != null)
                    if (!String.IsNullOrEmpty(_schdAppointment.END_TIME))
                        return Convert.ToDateTime(_schdAppointment.END_TIME);

                return new DateTime();
            }
            set
            {
                if (_schdAppointment != null)
                    _schdAppointment.END_TIME = value.ToString();

                OnPropertyChanged("End");
            }
        }

	    public string PatientDOB
        {
	        get
	        {
                if (_schdAppointment?.Patient != null)
    	            return _schdAppointment.Patient.DOBFormatted;
                
                return  string.Empty;
            }
        }

	    public string PatientSSN
	    {
	        get
	        {
                if (_schdAppointment?.Patient != null)
                    return _schdAppointment.Patient.SSNFormatted;

                return string.Empty;
            }
        }

	    public string MRTC
	    {
	        get
	        {
                string mrtc = string.Empty;

                if (!string.IsNullOrEmpty(_schdAppointment?.MRTCParentRequestIEN))
                {
                    mrtc = "\nMRTC";

                    if (_schdAppointment?.AppointmentRequest != null)
                        mrtc += ": " + _schdAppointment.AppointmentRequest.NumberOfAppointmentsRequired.ToString() +
                                " of " + _schdAppointment.AppointmentRequest.MrtcAppointmentCount.ToString();
                }

                return mrtc;
            }
        }

	    public bool IsMRTC
	    {
            get { return !string.IsNullOrEmpty(_schdAppointment?.MRTCParentRequestIEN); }
	    }

	    public string FormattedSubject
	    {
	        get
	        {
                if (String.IsNullOrEmpty(base.Subject))
                {
                    if (_schdAppointment != null)
                    {
                        return _schdAppointment.PATIENTNAME +
                               "\nSSN: " + ((_schdAppointment.Patient == null) ? String.Empty : _schdAppointment.Patient.SSNFormatted) +
                               "\tDOB: " + ((_schdAppointment.Patient == null) ? String.Empty : _schdAppointment.Patient.DOBFormatted);
                    }
                }

                return base.Subject;
            }
        }

        public void RaisePropertyChanged()
        {
            OnPropertyChanged("");
        }

        #endregion public members
    }
}
