using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using ClinSchd.Infrastructure.Converters;
using ClinSchd.Infrastructure.Interfaces;
using System.ComponentModel.DataAnnotations;
using System.Text.RegularExpressions;

namespace ClinSchd.Infrastructure.Models
{
    public class Patient : AsyncValidatableModel
    {
        public string LastSub { get; set; }

        public bool IsDead { get; set; }

        public SpecialNeedsPreferences SpecialNeedsPreferences { get; set; }

        /*
         * Bad Address Indicator is a Set of codes:  1 = Undeliverable, 2 = Homeless, 3 = Other, 4 = Address Not Found*/

        public AppointmentRequest Appointment { get; set; }

        private PatientSensitiveRecord _sensitiveRecord;
        public PatientSensitiveRecord SensitiveRecord
        {
            get
            {
                if (_sensitiveRecord == null)
                    _sensitiveRecord = new PatientSensitiveRecord();

                return _sensitiveRecord;
            }
            set
            {
                _sensitiveRecord = value;
                IsSensitiveRecord = _sensitiveRecord.IsSensitiveRecord;
                NotifyPropertyChanged( "SensitiveRecord" );
            }
        }

        public bool IsSensitiveRecord { get; set; }

        public bool IsBadAddress
        {
            get { return _isBadAddress; }
            set
            {
                _isBadAddress = value;
                NotifyPropertyChanged( "IsBadAddress" );
            }
        }

        private string _badAddressType;

        public string BadAddressType
        {
            get
            {
                if ( Keys.Methods.IsNumeric( _badAddressType ) && Convert.ToInt32( _badAddressType ) > 0 && Convert.ToInt32( _badAddressType ) <= BadAddressTypes.Count )
                    return BadAddressTypes[Convert.ToInt32( _badAddressType ) - 1];
                return _badAddressType;
            }
            set
            {
                _badAddressType = value;
                IsBadAddress = !string.IsNullOrEmpty(_badAddressType);
                NotifyPropertyChanged("BadAddressType");
            }
        }

        public List<string> BadAddressTypes { get; } = new List<string>(new[] { "Undeliverable", "Homeless", "Other", "Address Not Found" });

        private readonly IDataAccessService _dataAccessService;

        private string _ien = string.Empty;

        public string IEN //Patient IEN from Patient File
        {
            get { return _ien; }
            set
            {
                _ien = value;
                PrimaryContactInfo.PatientIEN = value;
                TemporaryContactInfo.PatientIEN = value;
                PrimaryNextOfKin.PatientIEN = value;
                SecondaryNextOfKin.PatientIEN = value;
            }
        }

        public string DFN { get; set; } //WaitList IEN for Patient on WaitList....

        public string Name
        {
            get { return _name; }
            set
            {
                _name = value; 
                NotifyPropertyChanged( "Name" );
            }
        }

        public string Suffix
        {
            get { return _suffix; }
            set
            {
                _suffix = value;
                NotifyPropertyChanged( "Suffix" );
            }
        }

        public string MedicareNo { get; set; }

        public string HRN { get; set; }

        public string MaritalStatus
        {
            get { return _maritalStatus; }
            set
            {
                _maritalStatus = value;
                NotifyPropertyChanged( "MaritalStatue" );
            }
        }

        public string ReligiousPreference { get; set; }

        public string RegistrationComments { get; set; }

        public static bool MaskSensitiveInfo(string patientIEN, bool isSensitiveReord)
        {
            return (isSensitiveReord && (patientIEN != Keys.PatientIen_SensitiveRecordCheck));
        }

        private string _ssn;

        public string SSNRaw { get { return _ssn; } }

        public string SSN
        {
            get { return SSNFormatted; }
            set
            {
                _ssn = value;
                NotifyPropertyChanged( "SSN" );
                NotifyPropertyChanged("SSNRaw");
                NotifyPropertyChanged("SSNFormatted");
            }
        }

        public string SSNFormatted { get { return FormattedSSN(_ssn, IEN, IsSensitiveRecord); } }

        public static string FormattedSSN(string ssn, string patientIEN, bool isSensitiveRecord)
        {
            if (string.IsNullOrEmpty(ssn))
                return string.Empty;

            if (MaskSensitiveInfo(patientIEN, isSensitiveRecord))
                return "*SENSITIVE*";

            // The VA asked this formatting option not be used.
            //  Keeping it in the event they change their minds
            //if (isSensitiveRecord)
            //    return "***-**-" + ssn.Substring(Math.Max(0, ssn.Length - 4));

            if (ssn.Length == 9)
                return Regex.Replace(ssn, @"(\d{3})(\d{2})(\d{4})", "$1-$2-$3");

            return ssn;
        }

        public string SSN4
        {
            get
            {
                if (!string.IsNullOrEmpty(_ssn) && _ssn.Length > 8)
                    return _ssn.Substring(5, 4);

                return string.Empty;
            }
        }

        public string Gender
        {
            get { return _gender; }
            set
            {
                _gender = value;
                NotifyPropertyChanged( "Gender" );
            }
        }

        public string InstitutionIen { get; set; }

        private string _institution;
        public string Institution
        {
            get
            {
                if(string.IsNullOrEmpty(_institution))
                {
                    var patients = _dataAccessService.GetPatients(Name, Keys.MaxPatientsToReturn);
                    foreach (var pat in patients.Where(pat => pat.IEN == IEN))
                    {
                        _institution = pat.Institution;
                        break;
                    }
                }
                return _institution;
            }
            set
            {
                _institution = value;
                NotifyPropertyChanged( "Institution" );
            }
        }

        public string OriginatingUser { get; set; }

        public IList<Provider> OriginatingUsers { get; set; }

        public string OriginatingUserName { get; set; }

        public Provider Provider
        {
            get { return _provider; }
            set
            {
                _provider = value;
                NotifyPropertyChanged( "Provider" );
            }
        }

        public IList<Provider> Providers { get; set; }

        public string EligibilityIen { get; set; }

        public string EligibilityName { get; set; }

        public string SCVisit
        {
            get { return _scVisit; }
            set
            {
                _scVisit = value; 
                NotifyPropertyChanged( "SCVisit" );
            }
        }

        public bool SCVisitBool
        {
            get
            {
                if ( string.IsNullOrEmpty( SCVisit ) )
                    return false;
                return SCVisit == "YES";
            }
        }

        public bool ServiceConnectionBool
        {
            get { return ServiceConnection.ToUpper() != "NO"; }
        }

        private string _serviceConnection;

        public string ServiceConnection
        {
            get { return string.IsNullOrEmpty( _serviceConnection ) ? "NO" : _serviceConnection; }
            set
            {
                _serviceConnection = value;
                NotifyPropertyChanged( "ServiceConnection" );
            }
        } //YES/NO

        public string ServiceConnectionPercentage
        {
            get { return _serviceConnectionPercentage; }
            set
            {
                _serviceConnectionPercentage = value;
                if ( !string.IsNullOrEmpty( _serviceConnectionPercentage ) )
                    ServiceConnectionPercent = Convert.ToInt32( _serviceConnectionPercentage );
                NotifyPropertyChanged( "ServiceConnectionPercentage" );
            }
        }

        public int ServiceConnectionPercent { get; set; }

        public bool ServiceRelated
        {
            get { return _serviceRelated; }
            set {
                _serviceRelated = value;
                NotifyPropertyChanged("ServiceRelated");
            }
        }

        public string TypeIen { get; set; }

        public string TypeName
        {
            get { return _typeName; }
            set
            {
                _typeName = value;
                NotifyPropertyChanged( "TypeName" );
            }
        }

        public string PriorityGroup { get; set; }

        public PatientGAF GAF { get; set; }

        public string GenderFullString
        {
            get
            {
                if (Gender == null)
                    return "";

                if (Gender.ToUpper() == "F")
                    return "Female";

                if (Gender.ToUpper() == "M")
                    return "Male";
                return "";
            }
        }

        public DateTime? DOBDate { get; private set; }

        public string DOBFormatted { get { return FormattedDOB(_dob, IEN, IsSensitiveRecord); } }

        public string DOBRaw {  get { return _dob; } }

        private string _dob;
        [Required(ErrorMessage = @"Please provide date of birth")]
        public string DOB
        {
            get { return DOBFormatted; }
            set
            {
                _dob = value;
                DOBDate = DOBToDateTime(_dob);
                NotifyPropertyChanged( "DOB" );
                NotifyPropertyChanged( "DOBDate");
                NotifyPropertyChanged("Age");
                NotifyPropertyChanged( "DOBFormatted" );
            }
        }

        public string Age {  get { return CalculateAge(_dob); } }

        public static string CalculateAge(string dob)
        {
            DateTime? dobDate = DOBToDateTime(dob);

            if (dobDate.HasValue)
            {
                DateTime now = DateTime.Today;
                DateTime bday = dobDate.Value.Date;
                var age = now.Year - bday.Year;

                if (bday > now.AddYears(-age))
                    age--;

                return age.ToString();
            }
            else
                return string.Empty;
        }

        public static DateTime? DOBToDateTime(string dob)
        {
            DateTime? DOBDate = null;

            if (!string.IsNullOrEmpty(dob))
            {
                var provider = CultureInfo.InvariantCulture;
                DateTime rslt;

                if (DateTime.TryParse(dob, provider, DateTimeStyles.None, out rslt))
                    DOBDate = rslt.Date;
                else
                {
                    var numPieces = dob.Split(' ');

                    if (numPieces.Length == 3)
                        DOBDate = Convert.ToDateTime(ShortDateTimeConverter.ConvertMonthStringDateToShortDateTime(dob));

                    if (numPieces.Length == 2)
                        DOBDate = Convert.ToDateTime(ShortDateTimeConverter.ConvertMonthStringDateToShortDateTime2(dob));
                }
            }

            return DOBDate;
        }

        public static string FormattedDOB(string dob, string patientIEN, bool isSensitiveRecord)
        {
            if (string.IsNullOrEmpty(dob))
                return string.Empty;

            if (MaskSensitiveInfo(patientIEN, isSensitiveRecord))
                return "*SENSITIVE*";

            // PII, 01/27/2017 - VA indicated full DOB view if Sensitive and User accepts, 
            // so this is commented out, but left in case it need to be reinstated
            //if (isSensitiveRecord)
            //{
            //    if (DOBDate.HasValue)
            //        return DOBDate.Value.ToString("**/**/yyyy");
            //    else
            //        return "**/**/" + _dob.Substring(Math.Max(0, _ssn.Length - 4));
            //}

            DateTime? dobDate = DOBToDateTime(dob);

            if (dobDate.HasValue)
                return dobDate.Value.ToString("M/d/yyyy");
            else
                return dob;
        }

        public Patient (IDataAccessService dataAccessService)
        {
            _dataAccessService      = dataAccessService;
            PrimaryContactInfo      = new ContactInfo(dataAccessService, IEN);
            TemporaryContactInfo    = new ContactInfo(dataAccessService, IEN);
            PrimaryNextOfKin        = new PatientNextOfKin(dataAccessService, IEN);
            SecondaryNextOfKin      = new PatientNextOfKin(dataAccessService, IEN);
            SpecialNeedsPreferences = new SpecialNeedsPreferences();
            GAF                     = new PatientGAF();
        }

        public void LoadPatientInfoByIEN()
        {
            var ptInfo = _dataAccessService.GetPatientInfo( IEN );
            if ( ptInfo == null )
                return;

            DOB               = ptInfo.DOBRaw;
            HRN               = ptInfo.HRN;
            IEN               = ptInfo.IEN;
            Name              = ptInfo.Name;
            SSN               = ptInfo.SSNRaw;
            IsSensitiveRecord = ptInfo.IsSensitiveRecord;
        }

        public void GetPatients( string searchString, WorkCompletedMethod workCompletedMethod )
        {
            DoWorkAsync( ( s, args ) => {
                args.Result = _dataAccessService.GetPatients( searchString == string.Empty ? null : searchString, Keys.MaxPatientsToReturn );
            }, workCompletedMethod );
        }

        public void SetEthnicity(string ien, string name)
        {
            var eth = _dataAccessService.GetEthnicities().FirstOrDefault(e => e.Ien == ien || e.Name == name);

            if (eth == null)
                eth = new Ethnicity
                {
                    Ien = ien,
                    Name = name
                };

            _ethnicity = eth;
        }

        private Ethnicity _ethnicity;
        public Ethnicity Ethnicity
        {
            get
            {
                if (_ethnicity != null)
                    return _ethnicity;

                var e = _dataAccessService.GetEthnicity(IEN);
                if (e == null)
                    return new Ethnicity();
                return e;
            }
            set
            {
                _ethnicity = value; 
                NotifyPropertyChanged( "Ethnicity" );
            }
        }

        [Required(ErrorMessage = @"Please provide ethnicity")]
        public List<Ethnicity> Ethnicities
        {
            get { return _dataAccessService.GetEthnicities(); }
        }

        private string _ward;

        public string Ward
        {
            get
            {
                if (_ward == null)
                    _ward = _dataAccessService.GetPatientWard(IEN);
                return _ward;
            }
            set
            {
                _ward = value; 
                NotifyPropertyChanged( "Ward" );
            }
        }

        private IList<Country> _countries;
        public IList<Country> Countries
        {
            get{return _countries ?? _dataAccessService.GetCountries();}
            set { _countries = value; }
        }

        private List<string> _preferences;

        public List<string> Preferences
        {
            get { return _preferences = _preferences ?? new List<string>(); }
            set { _preferences = value; }
        }

        private IList<Race> _raceList;

        public IList<Race> RaceList
        {
            get
            {
                if ( _raceList != null )
                    return _raceList;

                _raceList = new List<Race>();

                foreach ( var r in _dataAccessService.GetRaces() )
                    _raceList.Add(
                        new Race
                        {
                            Ien          = r.Ien,
                            Name         = r.Name,
                            Abbreviation = r.Abbreviation
                        } );

                return _raceList;
            }
        }

        public void SetRaces(string ienList, string nameList)
        {
            // Resets IsSelected, LastIsSelected, IsDeafult to false
            RaceList.ToList().ForEach(r => r.Reset());

            if (string.IsNullOrEmpty(ienList) || string.IsNullOrEmpty(nameList))
                return;

            var iens = ienList.Split('|');
            var defaultSet = false;

            foreach(var ien in iens)
            {
                var race = RaceList.FirstOrDefault(r => r.Ien == ien);

                if (race != null)
                {
                    if (!defaultSet)
                    {
                        race.IsDefault = true;
                        defaultSet = true;
                    }

                    race.IsSelected = true;
                    race.LastIsSelected = true;
                }
            }
        }

        public Race Race
        {
            get { return RaceList.FirstOrDefault( r => r.IsDefault ); }
        }

        public ContactInfo PrimaryContactInfo { get; set; }

        public ContactInfo TemporaryContactInfo { get; set; }

        public PatientNextOfKin PrimaryNextOfKin { get; set; }

        public PatientNextOfKin SecondaryNextOfKin { get; set; }

        public IList<PatientFlag> PatientFlags { get; } = new List<PatientFlag>();

        private string _enrollmentSubGroup = string.Empty;
        private string _serviceConnectionPercentage;
        private string _scVisit;
        private bool _serviceRelated;
        private bool _isBadAddress;
        private string _name;
        private string _suffix;
        private string _maritalStatus;
        private Provider _provider;
        private string _typeName;
        private string _gender;

        public string EnrollmentSubGroup
        {
            get { return _enrollmentSubGroup; }
            set
            {
                if (!string.IsNullOrEmpty(value) && !"abcdeg".Contains(value))
                    Keys.AlertUser(null, value + " is not a valid Enrollment Subgroup.", "Enrollment Subgroup");
                else
                    _enrollmentSubGroup = value;
            }
        }

        public bool CatastrophicDisabilityFlag { get; set; } = false;

        public bool Category8Flag { get; set; } = false;

        public bool FugitiveFelonFlag { get; set; } = false;

        public bool LoadRMGrid { get; set; } = false;

        public override string ToString()
        {
            return Name;
        }

        public bool PTLookRSRead { get; set; } = false;

        public bool GetRegARead { get; set; } = false;
    }
}
