﻿using System;
using System.Collections.Generic;
using System.Linq;
using ClinSchd.Infrastructure.Interfaces;

namespace ClinSchd.Infrastructure.Models
{
    public class SchdResource : AsyncValidatableModel
    {
        public bool IsProhibitedClinic { get; set; }
        public bool IsProvidersSchedule { get; set; }
        private readonly IDataAccessService _dataAccessService;
        private RPMSClinic _clinic;
        private string _timescale = string.Empty;
        private bool _clinicNotFound;
        public string RESOURCEID { get; set; }

        public string GroupResourceName { get; set; }

        public string RESOURCE_NAME
        {
            get { return _resourceName; }
            set
            {
                _resourceName = value;
                NotifyPropertyChanged( "RESOURCE_NAME" );
            }
        }

        public string INACTIVE { get; set; }
        public string TIMESCALE
        {
            get
            {
                if (string.IsNullOrEmpty(_timescale))
                {
                    _timescale = "30";
                    int displayIncrements = 2;

                    if (Clinic != null)
                        displayIncrements = Clinic.DISPLAY_INCREMENTS;

                    switch (displayIncrements)
                    {
                        case 1:
                            _timescale = "60";
                            break;
                        case 3:
                            _timescale = "20";
                            break;
                        case 4:
                            _timescale = "15";
                            break;
                        case 6:
                            _timescale = "10";
                            break;
                        default:
                            _timescale = "30";
                            break;
                    }
                }

                return _timescale;
            }
            set { _timescale = value; }
        }
        public string HOSPITAL_LOCATION_ID { get; set; }
        public string LETTER_TEXT { get; set; }
        public string NO_SHOW_LETTER { get; set; }
        public string CLINIC_CANCELLATION_LETTER { get; set; }
        public string VIEW { get; set; }
        public string OVERBOOK { get; set; }
        public string MODIFY_SCHEDULE { get; set; }
        public string MODIFY_APPOINTMENTS { get; set; }
        public string ResourceType
        {
            get { return ResourceTypeCode + "|" + ResourceTypeIEN + "|" + ResourceTypeName; }
            set
            {
                ResourceTypeCode = string.Empty;
                ResourceTypeIEN = string.Empty;
                ResourceTypeName = string.Empty;

                if (!string.IsNullOrEmpty(value))
                {
                    string[] resValues = value.Split('|');

                    if (resValues.Length >= 1)
                        ResourceTypeCode = resValues[0];

                    if (resValues.Length >= 2)
                        ResourceTypeIEN = resValues[1];

                    if (resValues.Length >= 3)
                        ResourceTypeName = resValues[2];
                }
            }
        }
        public string ResourceTypeCode { get; set; }
        public string ResourceTypeIEN { get; set; }
        public string ResourceTypeName { get; set; }
        public string Date { get; set; }
        public string UserIEN { get; set; }
        public string UserName { get; set; }
        public string InactiveDate { get; set; }
        public string InactiveUserIEN { get; set; }
        public string InactiveUserName { get; set; }
        public string ReactivatedDate { get; set; }
        public string ReactivatedUserIEN { get; set; }
        public string ReactivatedUserName { get; set; }
        public string ClinicName { get; set; }
        public string LastSub { get; set; }
        public string ParentProviderResourceID { get; set; }
        public string ParentProviderResourceName { get; set; }
        public string ScheduleTabCaption
        {
            get
            {
                if (string.IsNullOrEmpty(ParentProviderResourceName))
                    return RESOURCE_NAME;

                return ParentProviderResourceName + "-" + RESOURCE_NAME;
            }
        }
        public string ScheduleID
        {
            get
            {
                if (string.IsNullOrEmpty(ParentProviderResourceID))
                    return RESOURCEID;

                return ParentProviderResourceID + "^" + RESOURCEID;
            }
        }
        public Guid LoadingID { get; set; }
        public string DefaultProvider { get; set; }
        public string StopCodeNumber { get; set; }
        public IList<CSlot> AppointmentSlots { get; private set; }
        public IList<SchdAppointment> Appointments { get; private set; }
        public IList<SchdAvailability> AccessBlocks { get; private set; }

        public SchdResource ()
        {
            AppointmentSlots = new List<CSlot>();
            Appointments = new List<SchdAppointment>();
            AccessBlocks = new List<SchdAvailability>();
        }

        public SchdResource (IDataAccessService dataAccessService) : this()
        {
            _dataAccessService = dataAccessService;
        }

        public RPMSClinic Clinic
        {
            get
            {
                if (_clinic == null && !string.IsNullOrEmpty(HOSPITAL_LOCATION_ID) && !_clinicNotFound)
                {
                    _clinic = _dataAccessService.GetClinicByID(HOSPITAL_LOCATION_ID);
                    _clinicNotFound = _clinic == null;
                }

                if (_clinic != null)
                    _clinic.ParentResource = this;

                return _clinic;
            }
            set
            {
                _clinic = value;
            }
        }

        private bool? _variableLengthAppointments;
        public bool VariableLengthAppointments
        {
            get
            {
                if (!_variableLengthAppointments.HasValue)
                {
                    if (Clinic != null)
                        _variableLengthAppointments = Clinic.VariableAppointmentLength;
                    else
                    {
                        if (string.IsNullOrEmpty(HOSPITAL_LOCATION_ID))
                            _variableLengthAppointments = _dataAccessService.GetVariableAppointmentsFlag(HOSPITAL_LOCATION_ID);
                    }
                }
                return _variableLengthAppointments.HasValue && _variableLengthAppointments.Value;
            }
        }

        private IList<Provider> _providers;
        public IList<Provider> Providers
        {
            get
            {
                if (_providers == null && !string.IsNullOrEmpty(HOSPITAL_LOCATION_ID))
                {
                    _providers = _dataAccessService.GetProvidersByClinic (HOSPITAL_LOCATION_ID);
                }
                return _providers;
            }
        }

        public void GetSchedule(DateTime startTime, DateTime endTime, bool adjustTimes = true)
        {
            bool oldIsAsync = IsAsync;
            IsAsync = false;

            try
            {
                Appointments = GetAppointments(startTime, endTime);

                foreach (SchdAppointment appt in Appointments)
                    appt.ResourceScheduleID = ScheduleID;

                GetAvailability(startTime, endTime);
            }
            finally
            {
                IsAsync = oldIsAsync;
            }
        }

        public void GetVisibleAppointments (DateTime startTime, DateTime endTime, WorkCompletedMethod workCompletedMethod)
        {
            DoWorkAsync( (s, args) => {
                args.Result = _dataAccessService.GetAppointments (this, startTime.ToShortDateString (), endTime.ToShortDateString () + "@23:59");
            }, workCompletedMethod);
        }

        public IList<SchdAppointment> GetAppointments(DateTime startTime, DateTime endTime)
        {
            return _dataAccessService.GetAppointments(this, startTime.ToShortDateString(), endTime.ToShortDateString() + "@23:59");
        }

        public IList<SchdAvailability> GetAvailability(DateTime startTime, DateTime endTime)
        {
            var results = _dataAccessService.GetAvailabilities(this, startTime.ToShortDateString(), endTime.ToShortDateString());
            AppointmentSlots = Keys.AppointmentSlots.Slots;

            foreach (SchdAvailability block in results)
                block.ResourceScheduleID = ScheduleID;

            AccessBlocks = results;
            return results;
        } 

        public List<CustomAppointment> CustomAppointments { get; } = new List<CustomAppointment>();

        public List<CustomSlot> CustomSlots { get; } = new List<CustomSlot>();

        //public void GetVisibleAvailabilities (DateTime startTime, DateTime endTime, WorkCompletedMethod workCompletedMethod)
        //{
        //    /*if(Keys.SelectedAppointmentRequest != null)
        //        if (!string.IsNullOrEmpty(Keys.SelectedAppointmentRequest.ClinName))
        //            if(Keys.SelectedAppointmentRequest.ClinName.ToUpper() != RESOURCE_NAME.ToUpper())
        //                return;*/

        //    DoWorkAsync ((s, args) => {
        //        args.Result = _dataAccessService.GetAvailabilities (this, startTime.ToShortDateString (), endTime.ToShortDateString ());
        //    }, workCompletedMethod);
        //}

        private string _resourceName;
    }
}
