﻿using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Globalization;
using System.Linq;
using System.Text;

namespace ClinSchd.Infrastructure.Models
{
    public static class StringExtensions
    {
        private const string CRLF = "\r\n";
        // returns the string with a max length of maxLen and \r\n removed.
        public static string MaxStr( this string text, int maxLen )
        {
            var nstr = text.Replace( "\r\n", " " );
            var len = Math.Min( nstr.Length, maxLen );
            return nstr.Substring( 0, len );
        }

        public static string RemoveInvalidChars( this string text, string invalidChars )
        {
            foreach ( var ch in text.Where( ch => invalidChars.Contains( ch.ToString() ) ) )
                text = text.Replace( ch.ToString(), "" );
            return text;
        }

        public static List<string> ToList( this string str )
        {
            string[] delim = {CRLF};
            var parts = str.Split( delim, StringSplitOptions.None );

            var list = new List<string>();
            list.AddRange( parts );
            return list;
        }

        public static string ToStr( this List<string> list )
        {
            var str = "";
            foreach ( var item in list )
                str += item + CRLF;
            return str;
        }

        public static int ToIntDef( this string data, int defaultValue )
        {
            int num;
            return int.TryParse( data, out num ) ? num : defaultValue;
        }

        //public static string Piece( this String data, int pos)
        //{
        //    return MFunctions.Piece( data, "^", pos );
        //}

        public static string Piece( this string data, string delim, int start )
        {
            return MFunctions.Piece( data, delim, start );
        }

        public static string Piece( this string data, string delim, int start, int end )
        {
            return MFunctions.Piece( data, delim, start, end );
        }

        public static int PieceCount( this string data, string delim )
        {
            return MFunctions.PieceCount( data, delim );
        }

        public static string ToTitleCase( this string s )
        {
            return System.Threading.Thread.CurrentThread.CurrentCulture.TextInfo.ToTitleCase( s.ToLower() );
        }

        public static bool IsNullOrWhiteSpace( this string s )
        {
            var ans = string.IsNullOrEmpty( s );
            if ( !ans )
            {
                ans = s.Trim().Length == 0;
            }
            return ans;
        }

        public static bool IsSimpleNumber( this string s )
        {
            var str = s.Trim();
            foreach ( var c in str )
            {
                var uc = CharUnicodeInfo.GetUnicodeCategory( c );
                if ( uc != UnicodeCategory.DecimalDigitNumber && c != '.' )
                    return false;
            }
            return true;
        }

        public static string Format( this string s, params object[] args )
        {
            return string.Format( s, args );
        }

        public static string AddBraces( this string s )
        {
            return "{ " + s + " }";
        }

        public static string AddQuotes( this string s )
        {
            var lcl = new StringBuilder( s );
            if ( lcl.ToString().Contains( "\"" ) )
                lcl.Replace( "\"", "\\\"" );
            return "\"" + lcl + "\"";
        }

        public static string Repeat( this string s, int reps )
        {
            var ans = new StringBuilder( reps*s.Length );
            if ( reps > 0 )
            {
                for ( var ndx = 0; ndx < reps; ndx++ )
                    ans.Append( s );
            }
            return ans.ToString();
        }

        public static string TextBefore( this string s, string stop )
        {
            var ans = string.Empty;
            if ( string.IsNullOrEmpty( stop ) )
                return ans;

            var stopLoc = s.IndexOf( stop, StringComparison.Ordinal );
            ans = stopLoc > 0 ? s.Substring( 0, stopLoc ) : s; //"XXX" looking for "yY" is implied as "XXXyY" to return full string
            return ans;
        }

        public static string TextAfter( this string s, string stop )
        {
            var ans = string.Empty;
            if ( string.IsNullOrEmpty( stop ) )
                return ans;

            var stopLoc = s.IndexOf( stop, StringComparison.Ordinal );
            if ( stopLoc > 0 )
                ans = s.Substring( stopLoc + stop.Length );
            return ans;
        }
    }

    ///
    /// Work around class for Tuple to remain in Net 3.5
    /// 
    public class Tuple<T, TU> : IComparable, IEquatable<Tuple<T, TU>>
    {
        public T Item1 { get; private set; }
        public TU Item2 { get; set; }

        public static Tuple<T, TU> Create( T t, TU u )
        {
            var x = new Tuple<T, TU>
            {
                Item1 = t,
                Item2 = u
            };
            return x;
        }

        [SuppressMessage( "Microsoft.Naming", "CA2204:Literals should be spelled correctly", MessageId = "CompareTo" )]
        public int CompareTo( object obj )
        {
            var objType = obj.GetType();
            if ( objType == GetType() )
                return CompareTo( obj as Tuple<T, TU> );

            throw new ApplicationException( "Tuple<T,U>: CompareTo(Object).  Object is not a Tuple<T,U>." );
        }

        public int CompareTo( Tuple<T, TU> to )
        {
            var ans = string.CompareOrdinal(Item1.ToString(), to.Item1.ToString());
            if ( ans == 0 )
                ans = string.CompareOrdinal(Item2.ToString(), to.Item2.ToString());
            return ans;
        }

        public bool Equals( Tuple<T, TU> x )
        {
            return CompareTo( x ) == 0;
        }
    }

    public class MFunctions
    {
        //	Returns the index of the item whose specified Piece contains the specified Value.
        /// <summary>
        /// Indexes the of piece.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <param name="delimiter">The delimiter.</param>
        /// <param name="data">The data.</param>
        /// <param name="pieceNum">The piece num.</param>
        /// <param name="useCase">if set to <c>true</c> [use case].</param>
        /// <returns></returns>
        public static int IndexOfPiece( string value, string delimiter, string[] data, int pieceNum, bool useCase ) // defaults int PieceNum = 1, bool UseCase = true
        {
            var result = -1;
            if ( data == null )
                return result;

            for ( var i = 0; i < data.Length; i++ )
            {
                if ( ( useCase && string.Compare( Piece( data[i], delimiter, pieceNum ), value, StringComparison.Ordinal ) == 0 )
                     | ( !useCase && string.Compare( Piece( data[i], delimiter, pieceNum ), value, StringComparison.Ordinal ) == 0 ) )
                {
                    result = i;
                    break;
                }
            }
            return result;
        }

        //public static string Piece( string source, string delimiter )
        //{
        //    return Piece( source, delimiter, 1, 1 );
        //}

        //public static string Piece( string source, string delimiter, int start )
        //{
        //    return Piece( source, delimiter, start, start );
        //}
        //	This emulates the M $PIECE function.
        /// <summary>
        /// Pieces the specified source.
        /// </summary>
        /// <param name="source">The source.</param>
        /// <param name="delimiter">The delimiter.</param>
        /// <param name="start">The start.</param>
        /// <param name="end">The end.</param>
        /// <returns></returns>
        public static string Piece( string source, string delimiter, int start = 1, int end = -1 )
        {
            if ( string.IsNullOrEmpty( source ) )
                return "";
            if ( string.IsNullOrEmpty( delimiter ) )
                return "";

            if ( end == -1 )
                end = start;
            if ( end < start || start < 1 || delimiter.Length == 0 || source.Length == 0 )
                return "";

            var ln = delimiter.Length;
            int pc;
            var pc1 = 0;
            var pc2 = ln;
            var pos = -ln;

            for ( pc = 1; pc < start; pc++ )
            {
                pos = source.IndexOf( delimiter, pos + ln, StringComparison.Ordinal );
                if ( pos < 0 )
                    return "";
                pc1 = pos + ln;
            }

            for ( ; pc <= end; pc++ )
            {
                pos = source.IndexOf( delimiter, pos + ln, StringComparison.Ordinal );
                if ( pos < 0 )
                {
                    pc2 = source.Length;
                    break;
                }
                pc2 = pos;
            }
            return source.Substring( pc1, pc2 - pc1 );
        }

        //	Emulates the M $LENGTH function when used with a second parameter.
        /// <summary>
        /// Pieces the count.
        /// </summary>
        /// <param name="source">The source.</param>
        /// <param name="delimiter">The delimiter.</param>
        /// <returns></returns>
        public static int PieceCount( string source, string delimiter )
        {
            if ( source == null )
                return 0;
            if ( delimiter == null )
                return 0;
            if ( delimiter.Length == 0 )
                return 0;
            if ( source.Length == 0 )
                return 0;

            var delimiters = delimiter.ToCharArray();

            var split = source.Split( delimiters, 10000 );
            return split.Length;
        }
    }

    public class Triple<T, TU, TV> : IComparable, IEquatable<Triple<T, TU, TV>>
    {
        public T Item1 { get; private set; }
        public TU Item2 { get; set; }
        public TV Item3 { get; set; }

        public static Triple<T, TU, TV> Create( T t, TU u, TV v )
        {
            var x = new Triple<T, TU, TV>
            {
                Item1 = t,
                Item2 = u,
                Item3 = v
            };
            return x;
        }

        [SuppressMessage( "Microsoft.Naming", "CA2204:Literals should be spelled correctly", MessageId = "CompareTo" )]
        public int CompareTo( object obj )
        {
            var objType = obj.GetType();
            if ( objType == GetType() )
                return CompareTo( obj as Triple<T, TU, TV> );

            throw new ApplicationException( "Tuple<T,U, V>: CompareTo(Object).  Object is not a Triple<T,U,V>." );
        }

        public int CompareTo( Triple<T, TU, TV> to )
        {
            var ans = string.CompareOrdinal( Item1.ToString(), to.Item1.ToString() );
            if ( ans != 0 )
                return ans;

            ans = string.CompareOrdinal( Item2.ToString(), to.Item2.ToString() );
            if ( ans == 0 )
                ans = string.CompareOrdinal( Item3.ToString(), to.Item3.ToString() );
            return ans;
        }

        public bool Equals( Triple<T, TU, TV> x )
        {
            return CompareTo( x ) == 0;
        }
    }

    public class Quad<T, TU, TV, TW> : IComparable, IEquatable<Quad<T, TU, TV, TW>>
    {
        public T Item1 { get; private set; }
        public TU Item2 { get; set; }
        public TV Item3 { get; set; }
        public TW Item4 { get; set; }

        public static Quad<T, TU, TV, TW> Create( T t, TU u, TV v, TW w )
        {
            var x = new Quad<T, TU, TV, TW>
            {
                Item1 = t,
                Item2 = u,
                Item3 = v,
                Item4 = w
            };
            return x;
        }

        public Quad<T, TU, TV, TW> Clone()
        {
            return Create( Item1, Item2, Item3, Item4 );
        }

        [SuppressMessage( "Microsoft.Naming", "CA2204:Literals should be spelled correctly", MessageId = "CompareTo" )]
        public int CompareTo( object obj )
        {
            var objType = obj.GetType();
            if ( objType == GetType() )
                return CompareTo( obj as Quad<T, TU, TV, TW> );

            throw new ApplicationException( "Quad<T,U,V,W>: CompareTo(Object).  Object is not a Quad<T,U,V,W>." );
        }

        public int CompareTo( Quad<T, TU, TV, TW> to )
        {
            var ans = string.CompareOrdinal( Item1.ToString(), to.Item1.ToString() );
            if ( ans != 0 )
                return ans;

            ans = string.CompareOrdinal( Item2.ToString(), to.Item2.ToString() );
            if ( ans != 0 )
                return ans;

            ans = string.CompareOrdinal( Item3.ToString(), to.Item3.ToString() );
            if ( ans == 0 )
                ans = string.CompareOrdinal( Item4.ToString(), to.Item4.ToString() );
            return ans;
        }

        public bool Equals( Quad<T, TU, TV, TW> x )
        {
            return CompareTo( x ) == 0;
        }
    }

    
    // Tracing Support
    //
//    public class TraceSupport : IDisposable
//    {
//        //Singleton class
//        private static TraceSupport _singleton;
//        private readonly TextWriter _traceWriter;
//        // communicate trace setting to world
//#if DEBUG
//        private static bool TraceAvailable = false;
//#else
//        private static bool TraceAvailable = false;

//#endif

//        [SuppressMessage( "Microsoft.Reliability", "CA2000:Dispose objects before losing scope" )]
//        private TraceSupport()
//        {
//            if ( !TraceAvailable ) return;
//            _traceWriter = Console.Out; //TextWriter.Synchronized( new StreamWriter( "C:\\Temp\\Temp.txt", true, Encoding.UTF8 ) );
//            _traceWriter.WriteLine( "\n=============================\nTrace Started {0}", VCInit.VCSession().HostDateTime.ToString( "yyyy-MMM-dd HH:mm" ) );
//            _traceWriter.WriteLine( "=============================\n" );
//        }

//        public static TraceSupport GetTraceSupport()
//        {
//            return _singleton ?? ( _singleton = new TraceSupport() );
//        }

//        ~TraceSupport()
//        {
//            Dispose( false );
//        }

//        public void Dispose()
//        {
//            Dispose( true );
//            GC.SuppressFinalize( this );
//        }

//        protected virtual void Dispose( bool disposing )
//        {
//            if ( disposing )
//            {
//                if ( !TraceAvailable ) return;
//                _traceWriter.Close();
//                _traceWriter.Dispose();
//            }
//        }

//        public void Write( string text )
//        {
//            if ( !TraceAvailable ) return;
//            _traceWriter.Write( text );
//            _traceWriter.Flush();
//            Debug.WriteLine( text );
//            Debug.Flush();
//        }

//        public void Write( string text, params object[] values )
//        {
//            if ( !TraceAvailable ) return;
//            _traceWriter.Write( text, values );
//            _traceWriter.Flush();
//            Debug.Write( string.Format( text, values ) );
//            Debug.Flush();
//        }

//        public void WriteLine( string text )
//        {
//            if ( !TraceAvailable ) return;
//            _traceWriter.WriteLine( text );
//            _traceWriter.Flush();
//            Debug.WriteLine( text );
//            Debug.Flush();
//        }

//        public void WriteLine( string text, params object[] values )
//        {
//            if ( !TraceAvailable ) return;
//            _traceWriter.WriteLine( text, values );
//            _traceWriter.Flush();
//            Debug.WriteLine( string.Format( text, values ) );
//            Debug.Flush();
//        }

//        public void ShowControlStructure( Control ctl, int indent )
//        {
//            if ( !TraceAvailable ) return;
//            var indention = "|   ".Repeat( indent );
//            if ( ctl != null )
//            {
//                //TraceWriter.WriteLine( "{0}Name={7} X={1} Y={2} Type={3} Width={4} Height={5} Text={6}",
//                //    indention, ctl.Location.X, ctl.Location.Y, ctl.GetType().FullName, ctl.Width, ctl.Height, ctl.Text, ctl.Name );
//                //foreach ( Control sub in ctl.Controls )
//                //    ShowControlStructure( sub, indent < int.MaxValue ? indent + 1 : indent );
//            }
//            else
//                _traceWriter.WriteLine( "{0}null Control passed ToolBar ShowControlStructure", indention );
//        }

//        //public void ShowFormStructure( Form frm, int indent )
//        //{
//        //    if ( TraceAvailable )
//        //    {
//        //        String indention = "    ".Repeat( indent );
//        //        TraceWriter.WriteLine( "{0}Form={1} Title={2} Width={3} Height={4} Controls.Count={5}",
//        //            indention, frm.Name, frm.Text, frm.Width, frm.Height, frm.Controls.Count );
//        //        foreach ( Control ctl in frm.Controls )
//        //        {
//        //            ShowControlStructure( ctl, indent < int.MaxValue ? indent + 1 : indent );
//        //        }
//        //    }
//        //}
//    }
}

