using System;
using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Commands;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.CancelAppt.Services;
using ClinSchd.Infrastructure.Factories;

namespace ClinSchd.Modules.CancelAppt.CancelAppt
{
    public class CancelApptPresentationModel : ICancelApptPresentationModel, INotifyPropertyChanged
    {
        private readonly ICancelApptService _cancelApptService;
        public readonly IDataAccessService DataAccessService;
        public readonly IEventAggregator EventAggregator;
        private ValidationMessage _validationMessage;
        private CancelAppointment _cancelAppointment;
        private AutoRebookData _autoRebook;

        public DateTime Today
        {
            get { return DateTime.Today; }
        }

        public CancelApptPresentationModel (
            ICancelApptView view,
            ICancelApptService cancelApptService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            View = view;
            View.Model = this;
            EventAggregator = eventAggregator;
            _cancelApptService = cancelApptService;
            DataAccessService = dataAccessService;
            ValidationMessage = new ValidationMessage ();
            InitialErrorMessage ();

            CancelApptAppointmentCommand = new DelegateCommand<string> (ExecuteCancelApptAppointmentCommand, CanExecuteCancelApptAppointmentCommand);
            PrintLetterCommand = new DelegateCommand<string> (ExecutePrintLetterCommand, CanExecutePrintLetterCommand);
        }

        public void InitialErrorMessage ()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
        }

        private DateTime _originalDesiredDate;
        private IList<NameValue> _reasons;
        private bool _isAutoRebookChecked;

        public void CancelApptAppointment (SchdAppointment newAppointment)
        {
            CancelAppointment cancelAppointment = new CancelAppointment ();
            AutoRebookData autoRebook = new AutoRebookData ();
            if (newAppointment == null || newAppointment.APPOINTMENTID == null) {
                return;
            }
            cancelAppointment.ApptID = newAppointment.APPOINTMENTID;
            cancelAppointment.PatientID = newAppointment.PATIENTID;
            cancelAppointment.CancelledByClinic = true;
            cancelAppointment.PropertyChanged += (s,args) => {
                if (args.PropertyName == "CancelledByClinic")
                {
                    var reasonsType = cancelAppointment.CancelledByClinic ? "CLINIC" : "PATIENT";
                    Reasons = DataAccessService.GetCancellationReasons(reasonsType);
                }
            };
            if (newAppointment.DesiredAppointmentDate != null)
                cancelAppointment.PatientDesiredDate = newAppointment.DesiredAppointmentDate.Value.Date;
            else
                cancelAppointment.PatientDesiredDate = Convert.ToDateTime(newAppointment.START_TIME);

            _originalDesiredDate = cancelAppointment.PatientDesiredDate;
            autoRebook.RebookAppointment = newAppointment;
            autoRebook.IsAccessTypeChecked = true;
            autoRebook.MinimumDays = 7;
            autoRebook.MaximumDays = 30;
            IsAutoRebookChecked = false;

            //reason processing
            Reasons = DataAccessService.GetCancellationReasons ("CLINIC");

            //this.ReasonIEN = Reasons[0].Value;
            cancelAppointment.ReasonIEN = ReasonIEN;

            CancelAppointment = cancelAppointment;

            AutoRebook = autoRebook;
        }

        public void ExecuteCancelApptAppointmentCommand (string command)
        {
            var appointmentWaitTimeInfo = DataAccessService.GetExpandedEntryWaitTime(CancelAppointment.PatientID,
            ExtensionMethods.ConvertToFilemanDateTime(Convert.ToDateTime(SelectedAppointment.START_TIME))).Split('^');

            var patientContactDesiredDate = "";
            if (appointmentWaitTimeInfo.Length > 3)
                patientContactDesiredDate = appointmentWaitTimeInfo[2];

            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
            CancelAppointment.OriginalDesiredDate = _originalDesiredDate;
            string errorMessage;
            if(SelectedAppointment.AppointmentRequest != null)
                errorMessage = DataAccessService.CancelAppointment (CancelAppointment, SelectedAppointment);
            else  
                errorMessage = DataAccessService.CancelAppointment (CancelAppointment);


            if (errorMessage == string.Empty)
            {
                //Reset Patient Contacts Sequence Number
                SelectedAppointment.DesiredAppointmentDate = Convert.ToDateTime(patientContactDesiredDate);
                var patientContactReadRequestParams = new List<string>();
                if (SelectedAppointment.AppointmentRequest != null)
                    patientContactReadRequestParams = PatientContactRequestFactory.CreateReadRequestParams(SelectedAppointment.AppointmentRequest);
                else
                    patientContactReadRequestParams = PatientContactRequestFactory.CreateReadRequestParams(SelectedAppointment);

                if (!string.IsNullOrWhiteSpace(DataAccessService.GetPatientContact(patientContactReadRequestParams)))
                    DataAccessService.ResetPatientContactMainSequence(patientContactReadRequestParams);


                SelectedAppointment.DesiredAppointmentDate = DesiredDate;

                if (SelectedAppointment.AppointmentRequest != null)
                    SelectedAppointment.AppointmentRequest.DesiredApptDate = DesiredDate;

                EventAggregator.GetEvent<RefreshCurrentScheduleEvent>().Publish(String.Empty);
                EventAggregator.GetEvent<UpdateFutureAppointmentsEvent>().Publish(null);
            }
            else
            {
                _validationMessage.IsValid = false;
                _validationMessage.Title = "Cancel Appointment";
                _validationMessage.Message = errorMessage;
                return;
            }

            if (IsAutoRebookChecked)
            {
                if (AutoRebook.IsAccessTypeChecked)
                {
                    AutoRebook.AccessTypeID = int.Parse (AutoRebook.RebookAppointment.ACCESSTYPEID);
                } else {
                    AutoRebook.AccessTypeID = 0;
                }
                errorMessage = DataAccessService.AutoRebook (AutoRebook);
                if (errorMessage != string.Empty) {
                    _validationMessage.IsValid = false;
                    _validationMessage.Title = "AutoRebook Appointment";
                    _validationMessage.Message = errorMessage;
                }
            }
        }

        public bool CanExecuteCancelApptAppointmentCommand (string command)
        {
            return true;
        }

        private void ExecutePrintLetterCommand (string command)
        {
            object[] args = new object[3];
            args[0] = "Patient Cancellation Letter";
            args[1] = CancelAppointment.ApptID;
            args[2] = CancelAppointment.CancelledByClinic ? "C" : "A";
            EventAggregator.GetEvent<DisplayReportEvent> ().Publish (args);
        }

        private bool CanExecutePrintLetterCommand (string command)
        {
            return true;
        }

        public void OnClose()
        {
            View.Close();
        }

        #region ICancelApptPresentationModel Members

        public ICancelApptView View { get; }
        public SchdAppointment SelectedAppointment { get; set; }
        public AppointmentRequest SelectedRequest { get; set; }
        public string ReasonIEN { get; set; }
        public string AccessType { get; set; }

        public bool IsAutoRebookChecked
        {
            get { return _isAutoRebookChecked; }
            set
            {
                _isAutoRebookChecked = value;
                OnPropertyChanged( "IsAutoRebookChecked" );
            }
        }

        public IList<NameValue> Reasons
        {
            get { return _reasons; }
            set
            {
                _reasons = value;
                OnPropertyChanged( "Reasons" );
            }
        }

        public DelegateCommand<string> CancelApptAppointmentCommand { get; private set; }
        public DelegateCommand<string> PrintLetterCommand { get; private set; }

        public AutoRebookData AutoRebook
        {
            get { return _autoRebook; }
            set
            {
                _autoRebook = value;
                OnPropertyChanged( "AutoRebook" );
            }
        }

        public CancelAppointment CancelAppointment
        {
            get { return _cancelAppointment; }
            set
            {
                _cancelAppointment = value;
                OnPropertyChanged( "CancelAppointment" );
            }
        }

        public ValidationMessage ValidationMessage 
        {
            get
            {
                return _validationMessage;
            }
            set
            {
                _validationMessage = value;
            }
        }

        public DateTime DesiredDate
        {
            get
            {
                return CancelAppointment.PatientDesiredDate;
            }
            set
            {
                CancelAppointment.PatientDesiredDate = value;
            }
        }

        public string CloseTheAppointmentRequest()
        {
            string errorText = string.Empty;

            if (SelectedRequest != null)
            {
                if (SelectedRequest.IsEwl)
                    DataAccessService.WLClose(SelectedRequest.RequestManagmentIen, Keys.WaitListDispositions.RemovedScheduledAssigned);
                else
                    DataAccessService.ARClose(SelectedRequest.RequestManagmentIen, Keys.WaitListDispositions.RemovedScheduledAssigned);
            }

            return errorText;
        }

        #endregion

        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
