using System.Collections.Generic;
using System.ComponentModel;

using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Commands;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Modules.ChangeDivision.Services;

namespace ClinSchd.Modules.ChangeDivision.ChangeDivision
{
    public class ChangeDivisionPresentationModel : IChangeDivisionPresentationModel, INotifyPropertyChanged
    {
        private readonly IChangeDivisionService _changeDivisionService;
        private readonly IDataAccessService _dataAccessService;
        private readonly IEventAggregator _eventAggregator;
        private ValidationMessage _validationMessage;
        private IList<Divisions> _divisions;
        private string _divisionIEN;

        public ChangeDivisionPresentationModel (
            IChangeDivisionView view,
            IChangeDivisionService changeDivisionService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            View = view;
            View.Model = this;
            _eventAggregator = eventAggregator;
            _changeDivisionService = changeDivisionService;
            _dataAccessService = dataAccessService;
            ValidationMessage = new ValidationMessage ();
            InitialErrorMessage ();

            ChangeDivisionCommand = new DelegateCommand<string> (ExecuteChangeDivisionCommand, CanExecuteChangeDivisionCommand);
        }

        public void InitialErrorMessage ()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
        }

        public void GetDivisions ()
        {
            DivisionList = _dataAccessService.GetDivisions ();

            foreach (Divisions d in DivisionList) {
                if (d.IsDefault) {
                    DivisionIEN = d.IEN;
                    break;
                }
            }
        }

        public void ExecuteChangeDivisionCommand (string command)
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;

            var returnValue = _dataAccessService.SetDivision (DivisionIEN);

            if (returnValue) {
                _dataAccessService.LoadGlobalRecordsets ();
                Divisions div = _dataAccessService.GetCurrentDivision();
                _dataAccessService.CurrentDivision = div;
                string payload = "^";

                if (div != null)
                    payload = string.Format("{0}^{1}", div.IEN, div.NAME);

                _eventAggregator.GetEvent<DisplayDefaultDivisionEvent> ().Publish (payload);
            } else {
                _validationMessage.IsValid = false;
                _validationMessage.Title = "Change Division";
                _validationMessage.Message = "The current division cannot be changed";
            }
        }

        public bool CanExecuteChangeDivisionCommand (string command)
        {
            return true;
        }


        public void OnClose()
        {
            View.Close();
        }

        #region IChangeDivisionPresentationModel Members

        public DelegateCommand<string> ChangeDivisionCommand { get; private set; }

        public IChangeDivisionView View { get; }

        public string DivisionIEN
        {
            get { return _divisionIEN; }
            set
            {
                _divisionIEN = value;
                OnPropertyChanged( "DivisionIEN" );
            }
        }

        public IList<Divisions> DivisionList
        {
            get { return _divisions; }
            set
            {
                _divisions = value;
                OnPropertyChanged( "DivisionList" );
            }
        }

        public ValidationMessage ValidationMessage
        {
            get { return _validationMessage; }
            set
            {
                _validationMessage = value;
                OnPropertyChanged( "ValidationMessage" );
            }
        }

        #endregion

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
