using System;
using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Commands;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.CheckOut.Services;
using Microsoft.Practices.Unity;

namespace ClinSchd.Modules.CheckOut.CheckOut
{
    public class CheckOutPresentationModel : ICheckOutPresentationModel, INotifyPropertyChanged
    {
        private readonly ICheckOutService _checkOutService;
        private readonly IDataAccessService _dataAccessService;
        public readonly IEventAggregator EventAggregator;
        private ValidationMessage _validationMessage;

        public CheckOutPatient CheckOutData
        {
            get { return _checkOutData; }
            set
            {
                _checkOutData = value;
                OnPropertyChanged( "CheckOutData" );
            }
        }

        private SchdAppointment Appointment { get; set; }

        public CheckOutPresentationModel (
            ICheckOutView view,
            ICheckOutService checkOutService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            View = view;
            View.Model = this;
            EventAggregator = eventAggregator;
            _checkOutService = checkOutService;
            _dataAccessService = dataAccessService;
            ValidationMessage = new ValidationMessage ();
            InitialErrorMessage ();

            CheckOutAppointmentCommand = new DelegateCommand<string> (ExecuteCheckOutAppointmentCommand, CanExecuteCheckOutAppointmentCommand);
        }

        public void InitialErrorMessage ()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
        }

        public void GetCheckOutAppointment (SchdAppointment newAppointment)
        {
            var checkOutData = new CheckOutPatient ();
            if ( newAppointment == null || newAppointment.APPOINTMENTID == null )
                return;

            Appointment = newAppointment;
            checkOutData.ApptID = newAppointment.APPOINTMENTID;
            checkOutData.AppointmentStartTime = newAppointment.START_TIME;
            checkOutData.PatientID = newAppointment.PATIENTID;
            checkOutData.ProviderIEN = newAppointment.VPROVIDER;

            PatientName = newAppointment.PATIENTNAME;

            //Find the default provider for the resource & load into combo box
            /*DataView rv = new DataView (dataAccessService.GetResources());
            rv.Sort = "RESOURCE_NAME ASC";
            int nFind = rv.Find ((string)newAppointment.RESOURCENAME);
            DataRowView drv = rv[nFind];*/

            var schdRes = _dataAccessService.GetSdecResourceByName(newAppointment.RESOURCENAME);

            var sHospLoc = schdRes.HOSPITAL_LOCATION_ID; // drv["HOSPITAL_LOCATION_ID"].ToString ();
            sHospLoc = sHospLoc == string.Empty ? "0" : sHospLoc;
            var nHospLoc = 0;
            try {
                nHospLoc = Convert.ToInt32 (sHospLoc);
            } catch (Exception ex) {
                _validationMessage.IsValid = false;
                _validationMessage.Title = "Check Out Patient";
                _validationMessage.Message = ex.Message;
            }

            //string sProv = "0";

            if (nHospLoc > 0)
            {
                //Get hospital locations
                /*DataView hv = new DataView(this.dataAccessService.GetHospitalLocations());
                //DataView hv = new DataView(Keys.Tables.HospitalResourceTable);
                hv.Sort = "HOSPITAL_LOCATION_ID";
                int hFind = hv.Find (nHospLoc);
                DataRowView dhv = hv[hFind];*/

                //sProv = schdRes.DefaultProvider; // dhv["DEFAULT_PROVIDER"].ToString ();
            } else {
                _validationMessage.IsValid = false;
                _validationMessage.Title = "Check Out Patient";
                _validationMessage.Message = "You need to select a clinic for this resource in Scheduling Management.";
                return;
            }

            //Provider processing
/*
            var clinicProviders = dataAccessService.GetProvidersByClinic(sHospLoc);

            Providers = new List<Provider>();

            foreach (var item in clinicProviders)
            {
                Providers.Add(item);
            }

            if (clinicProviders.Count > 0)
            {
                Providers.Add(new Provider { Name = "", }); // Separator
            }
                
            foreach (var item in this.dataAccessService.GetProviders())
            {
                Providers.Add(item);
            }

            bool isNone = false;
            foreach (Provider nameValue in Providers) {
                if (nameValue.Name == "<None>")
                    isNone = true;
            }

            if (!isNone) {
                Provider newProv = ProviderFactory.Create ();
                newProv.Name = "<None>";
                newProv.IEN = "0";
                Providers.Insert(0, newProv);
            }
            OnPropertyChanged ("Providers");

            if (newAppointment.VPROVIDER != string.Empty) {
                foreach (Provider n in Providers) {
                    if (n.IEN == newAppointment.VPROVIDER)
                        this.ProviderIEN = n.IEN;
                }
            } else {
                this.ProviderIEN = sProv;
            }
            CheckOutData.ProviderIEN = this.ProviderIEN;
            OnPropertyChanged ("ProviderIEN");
//*/
            //default the check in time
            checkOutData.CheckOutDateTime = DateTime.Now;

            CheckOutData = checkOutData;
        }

        public void ExecuteCheckOutAppointmentCommand (string command)
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;

            var errorMessage = _dataAccessService.CheckOutPatient (CheckOut);

            if (errorMessage == string.Empty) {
                EventAggregator.GetEvent<RefreshCurrentScheduleEvent>().Publish(String.Empty);
                EventAggregator.GetEvent<UpdateFutureAppointmentsEvent>().Publish(null);

                if (RebookRequired)
                {
                    var requestParams = new NewAppointmentRequestParams
                    {
                        PatientID = Appointment.PATIENTID,
                        PatientName = Appointment.PATIENTNAME,
                        ProviderIEN = Appointment.VPROVIDER,
                        ResourceIEN = Appointment.ResourceId,
                        ResourceName = Appointment.RESOURCENAME,
                        DesiredDateTime = RebookDate
                    };
                    EventAggregator.GetEvent<OpenNewAppointmentRequest>().Publish(requestParams);
                }
            } else {
                _validationMessage.IsValid = false;
                _validationMessage.Title = "Check Out Patient";
                _validationMessage.Message = errorMessage;
            }
        }

        public bool CanExecuteCheckOutAppointmentCommand (string command)
        {
            return true;
        }

        public void OnClose()
        {
            View.Close();
        }

        [Dependency]
        public Factory<Provider> ProviderFactory { get; set; }

        #region ICheckOutPresentationModel Members

        public ICheckOutView View { get; }

        public string PatientName
        {
            get { return _patientName; }
            set
            {
                _patientName = value;
                OnPropertyChanged( "PatientName" );
            }
        }

        public string ProviderIEN { get; set; }
        public string PCCClinicIEN { get; set; }
        public string PCCFormIEN { get; set; }
        public string SummaryReportIEN { get; set; }
        public bool IsSummaryReportChecked { get; set; }
        public SchdAppointment SelectedAppointment { get; set; }
        public IList<Provider> Providers { get; set; }
        public IList<NameValue> SummaryReport { get; set; }
        public DelegateCommand<string> CheckOutAppointmentCommand { get; private set; }

        private bool _rebookRequired;

        public bool RebookRequired
        {
            get { return _rebookRequired; }
            set
            {
                if ( _rebookRequired != value )
                {
                    _rebookRequired = value;
                    OnPropertyChanged( "RebookRequired" );
                }
            }
        }

        private DateTime _rebookDate = DateTime.Today;
        private CheckOutPatient _checkOutData;
        private string _patientName;

        public DateTime RebookDate
        {
            get { return _rebookDate; }
            set
            {
                if ( _rebookDate != value )
                {
                    _rebookDate = value;
                    OnPropertyChanged( "RebookDate" );
                }
            }
        }

        public CheckOutPatient CheckOut
        {
            get { return CheckOutData; }
            set
            {
                CheckOutData = value;
                OnPropertyChanged( "CheckOut" );
            }
        }

        public ValidationMessage ValidationMessage
        {
            get { return _validationMessage; }
            set
            {
                _validationMessage = value;
                OnPropertyChanged( "ValidationMessage" );
            }
        }

        #endregion

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
