﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Models;

namespace ClinSchd.Modules.DataAccess.Views
{
    /// <summary>
    /// Interaction logic for PatientPrompt.xaml
    /// </summary>
    public partial class PatientPrompt : UserControl
    {
        public Patient CurrentPatient { get; set; }
        public string PatientAlertInfo { get; set; }
        public bool BDialogResult { get; set; }
        public int TotalFlags { get; set; }
        public bool ShowDetails { get; set; } = false;

        public PatientPrompt()
        {
            InitializeComponent();
            FugitiveFelonFlag.Text = "*** WARNING - FFP FLAG ACTIVE ***\n* PLEASE NOTIFY YOUR SUPERVISOR *";
            CatastrophicDisabilityFlag.Text = "*** CATASTROPHIC DISABILITY ***";
            Catagory8gFlag.Text = "*** CATEGORY 8g PATIENT ***" +
                                "\n***      NOT  ENROLLED      ***";
            SetFlagFontSize(12);
        }

        private void OK_Click(object sender, RoutedEventArgs e)
        {
            BDialogResult = true;
            var window = Window.GetWindow(this);
            if (window != null)
                window.Close();
        }

        private void PatientPrompts_Loaded(object sender, RoutedEventArgs e)
        {
            Infrastructure.Keys.SetRenderingMode(this);

            if (String.IsNullOrEmpty(PatientAlertInfo.Trim()))
                LstPatientPrompts.Visibility = Visibility.Collapsed;
            else
            {
                var lines = PatientAlertInfo.Split(new string[] { Environment.NewLine }, StringSplitOptions.None);
                if (TotalFlags > 0)
                    LstPatientPrompts.Items.Add("Total Flags = " + TotalFlags);
                LstPatientPrompts.Items.Add("");
                foreach (var line in lines)
                    LstPatientPrompts.Items.Add(line);
            }

            OKButton.Focus();
            Keyboard.Focus(OKButton);
        }

        private void Cancel_Click(object sender, RoutedEventArgs e)
        {
            BDialogResult = false;
            var window = Window.GetWindow(this);
            if (window != null)
                window.Close();
        }

        public void SetFlagFontSize(double fontSize)
        {
            LstPatientPrompts.FontSize = fontSize;
            FugitiveFelonFlag.FontSize = fontSize + 2;
            CatastrophicDisabilityFlag.FontSize = fontSize + 2;
            Catagory8gFlag.FontSize = fontSize + 2;
        }

        public bool IsButtonBarVisible
        {
            get { return (ButtonBar.Visibility == Visibility.Visible); }
            set
            {
                if (value)
                    ButtonBar.Visibility = Visibility.Visible;
                else
                    ButtonBar.Visibility = Visibility.Collapsed;
            }
        }

        public static PatientPrompt ProcessFlags(Patient patient, double flagFontSize = 12, bool flagNameOnly = false, bool isButtonBarVisible = true, bool showDetails = false)
        {
            if (patient.PatientFlags.Count == 0 && !patient.FugitiveFelonFlag && !patient.CatastrophicDisabilityFlag && !patient.Category8Flag)
                return null;

            var prompt = "";

            if (patient.PatientFlags.Count > 0)
                foreach (PatientFlag flag in patient.PatientFlags)
                    if (flagNameOnly)
                        prompt += flag.FlagName + Environment.NewLine;
                    else
                        prompt += (((flag.FlagDetails.Count > 0) ? String.Join("\n", flag.FlagDetails.ToArray()) : flag.FlagName)) + Environment.NewLine + Environment.NewLine;

            var patientAlerts = new PatientPrompt
            {
                CurrentPatient = patient,
                ShowDetails = showDetails,
                IsButtonBarVisible = isButtonBarVisible,
                PatientAlertInfo = prompt,
                TotalFlags = patient.PatientFlags.Count
            };

            patientAlerts.SetFlagFontSize(flagFontSize);
            patientAlerts.FugitiveFelonFlag.Visibility = (patient.FugitiveFelonFlag ? Visibility.Visible : Visibility.Collapsed);
            patientAlerts.CatastrophicDisabilityFlag.Visibility = (patient.CatastrophicDisabilityFlag ? Visibility.Visible : Visibility.Collapsed);
            patientAlerts.Catagory8gFlag.Visibility = (patient.Category8Flag ? Visibility.Visible : Visibility.Collapsed);
            return patientAlerts;
        }

        public static PatientPrompt ProcessSensitiveInformation(Patient patient, NameValue sensitiveInfo, /*double flagFontSize = 12, bool flagNameOnly = false,*/ bool isButtonBarVisible = true) //, bool showDetails = false)
        {
            if ((sensitiveInfo == null) || string.IsNullOrEmpty(sensitiveInfo.Value))
                return null;

            var prompt = sensitiveInfo.Value;

            var patientAlerts = new PatientPrompt
            {
                CurrentPatient = patient,
                ShowDetails = false,
                IsButtonBarVisible = isButtonBarVisible,
                PatientAlertInfo = prompt,
                TotalFlags = 0
            };

            switch (sensitiveInfo.Name)
            {
                case "-1":
                case "0":
                case "1":
                    break;
                case "2":
                    patientAlerts.LstPatientPrompts.Background = Brushes.Yellow;
                    break;
                case "3":
                case "4":
                    patientAlerts.LstPatientPrompts.Background = Brushes.Red;
                    patientAlerts.LstPatientPrompts.Foreground = Brushes.White;
                    break;
            }

            patientAlerts.LstPatientPrompts.SelectionChanged -= patientAlerts.LstPatientPrompts_SelectionChanged;
            patientAlerts.CancelBt.Visibility = (sensitiveInfo.Name == "2" ? System.Windows.Visibility.Visible : System.Windows.Visibility.Collapsed);
            patientAlerts.FontFamily = new FontFamily("Courier New");
            patientAlerts.SetFlagFontSize(12);
            patientAlerts.FugitiveFelonFlag.Visibility = Visibility.Collapsed;
            patientAlerts.CatastrophicDisabilityFlag.Visibility = Visibility.Collapsed;
            patientAlerts.Catagory8gFlag.Visibility = Visibility.Collapsed;
            return patientAlerts;
        }

        private void LstPatientPrompts_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (!ShowDetails)
                return;

            if (e.AddedItems.Count == 0)
                return;

            string prompt = (string)e.AddedItems[0];
            string[] lines = prompt.Split('\n');
            PatientFlag flag = null;

            switch (lines.Count())
            {
                case 0: return;
                case 1:
                    flag = CurrentPatient.PatientFlags.FirstOrDefault(f => (f.FlagName == lines[0]));
                    break;
                default:
                    flag = CurrentPatient.PatientFlags.FirstOrDefault(f => (String.Join("\n", f.FlagDetails.ToArray()) == prompt));
                    break;
            }

            if (flag != null)
                Keys.AlertUser(this, String.Join("\n", flag.FlagDetails.ToArray()), "Patient Flag Details");
        }

        private void HotKeyHandler(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                Key key = e.Key == Key.System ? e.SystemKey : e.Key;
                switch (key)
                {
                    case Key.O:
                        Keyboard.Focus(OKButton);
                        break;
                    case Key.C:
                        Keyboard.Focus(CancelBt);
                        break;
                }
            }
        }
    }
}
