﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Windows;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using Microsoft.Practices.Composite.Presentation.Commands;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.FindAppt.FindAppt;
using ClinSchd.Modules.FindAppt.Services;

namespace ClinSchd.Modules.FindAppt.CancelMRTC
{
    public class ConfirmCloseMRTCRequestPresentationModel : IConfirmCloseMRTCRequestPresentationModel, INotifyPropertyChanged
    {
        #region fields

        private readonly IEventAggregator _eventAggregator;
        private readonly IDataAccessService _dataAccessService;

        private bool _isRemoveOnlyThisChecked = true;
        private bool _isRemoveAllChecked;
        private bool _isRemoveParentChecked;
        private bool _isRemoveAllAssociationsChecked;
        private bool _isRemoveAppointmentRequestChecked;
        private AppointmentRequest _parentRequest;
        private BookApptPresentationModel _firstCancelAppt;
        private BookApptPresentationModel _selectedBookingRecord;
        private BookingStatusView _bookingStatusView;

        private readonly IConfirmCloseMRTCRequestView _view;

        #endregion fields

        #region constructor

        public ConfirmCloseMRTCRequestPresentationModel(IEventAggregator eventAggregator, IDataAccessService dataAccessService, IConfirmCloseMRTCRequestView view)
        {
            IsCancelStatusVisible = true;
            Title = "MRTC Cancel Status";
            SelectedClause = Keys.WaitListDispositions.RemovedNoLongerNecessary;
            _eventAggregator = eventAggregator;
            _dataAccessService = dataAccessService;
            // Create the Commands
            CancelCommand = new DelegateCommand<string>(ExecuteCancelCommand, CanExecuteCancelCommand);
            CloseRequestsCommand = new DelegateCommand<string>(ExecuteCloseRequestsCommand, CanExecuteCloseRequestsCommand);
            _view = view;
            _view.Model = this;
        }

        #endregion constructor

        #region bindings

        public Visibility AppointmentOptionVisibility
        {
            get
            {
                if (Appointment != null)
                    return Visibility.Visible;
                else
                    return Visibility.Collapsed;
            }
        }

        public bool IsCancelStatusVisible { get; private set; }

        public Visibility OptionsVisibility
        {
            get
            {
                if (Request != null)
                    if (Request.IsMRTCParent)
                        return Visibility.Collapsed;

                return Visibility.Visible;
            }
        }

        public bool IsRemoveAllAssociationsChecked
        {
            get { return _isRemoveAllAssociationsChecked; }
            set
            {
                _isRemoveAllAssociationsChecked = value;
                OnPropertyChanged("IsRemoveAllAssociationsChecked");
            }
        }

        public bool IsRemoveOnlyThisChecked
        {
            get { return _isRemoveOnlyThisChecked; }
            set
            {
                _isRemoveOnlyThisChecked = value;
                OnPropertyChanged("IsRemoveOnlyThisChecked");
//                IsRemoveParentChecked = !_isRemoveOnlyThisChecked;
            }
        }

        public bool IsRemoveAllChecked
        {
            get { return _isRemoveAllChecked; }
            set
            {
                _isRemoveAllChecked = value;
                OnPropertyChanged("IsRemoveAllChecked");
                IsRemoveParentChecked = _isRemoveAllChecked;
            }
        }

        public bool IsRemoveParentChecked
        {
            get { return _isRemoveParentChecked; }
            set
            {
                _isRemoveParentChecked = value;
                OnPropertyChanged("IsRemoveParentChecked");
            }
        }

        public bool IsRemoveAppointmentRequestChecked
        {
            get { return _isRemoveAppointmentRequestChecked; }
            set
            {
                _isRemoveAppointmentRequestChecked = value;
                OnPropertyChanged("IsRemoveAppointmentRequestChecked");
            }
        }

        public AppointmentRequest ParentRequest { get { return _parentRequest; } }

        public AppointmentRequest Request { get; private set; }

        public SchdAppointment Appointment { get; private set; }

        public string RemoveOnlyThisText
        {
            get
            {
                if (Request != null)
                    return "Close the selected appointment request only";
                if (Appointment != null)
                    return "Cancel the selected appointment only";
                return string.Empty;
            }
        }

        public string RemoveAllAssociationsText
        {
            get
            {
                if (Request != null)
                    return "Cancel all appointment requests in the series.";
                else if (Appointment != null)
                    return "Close all associated appointment requests in the series.";
                else
                    return string.Empty;
            }

        }

        public string RemoveAllText
        {
            get
            {
                if (Request != null)
                    return "Close all appointment requests in the series.";
                else if (Appointment != null)
                    return "Cancel all pending appointments in the series.";
                else
                    return string.Empty;
            }

        }

        public BookApptPresentationModel SelectedBookingRecord
        {
            get { return _selectedBookingRecord; }
            set
            {
                _selectedBookingRecord = value;
                OnPropertyChanged("SelectedBookingRecord");
            }
        }

        #endregion bindings

        #region commands

        public DelegateCommand<string> CancelCommand { get; }
        public DelegateCommand<string> CloseRequestsCommand { get; }

        #endregion commands

        #region Command methods

        public bool CanExecuteCancelCommand(string command)
        {
            return true;
        }

        public bool CanExecuteCloseRequestsCommand(string command)
        {
            return true;
        }

        public void ExecuteCancelCommand(string command)
        {
            CloseBookingStatusView();
            DialogResult = false;
            _view.Close();
        }

        public void ExecuteCloseRequestsCommand(string command)
        {
            _view.Close();
            // Load the MRTC Children
            LoadRequestData();

            // Open the Status View
            if (_bookingStatusView == null)
            {
                //Window owner = Window.GetWindow((DependencyObject)View);
                _bookingStatusView = new BookingStatusView
                {
                    Owner = Keys.MainWindow,
                    DataContext = this,
                    WindowStartupLocation = WindowStartupLocation.CenterOwner
                };
            }

            _bookingStatusView.Show();

            if (IsRemoveOnlyThisChecked)
            {
                BookApptPresentationModel bModel = null;

                // is this a Close Request process ?
                if (IsCloseRequest)
                {
                    // Find the current request to close
                    bModel = BookedAppointments.FirstOrDefault(br => br.Request != null && br.Request.ApptRequestIen == Request.ApptRequestIen);

                    if (bModel != null)
                    {
                        bModel.CloseTheRequest = true;
                        // Should never have an appointment
                        bModel.CancelTheAppointment = false;
                    }
                }
                else if (IsCloseAppointment)
                {
                    // Find the current appointment to cancel - only iterate booking records with appointments
                    foreach(var item in BookedAppointments.Where(br => br.Appointment != null).ToList())
                    {
                        if (item.Appointment.APPOINTMENTID == Appointment.APPOINTMENTID)
                        {
                            bModel = item;
                            break;
                        }
                    }

                    if (bModel != null)
                    {
                        bModel.CancelTheAppointment = true;
                        bModel.CloseTheRequest = IsRemoveAppointmentRequestChecked;
                    }
                }
            }
            else if (IsRemoveAllChecked)
            {

                if (IsCloseRequest)
                {
                    foreach(var item in BookedAppointments)
                    {
                        item.CloseTheRequest = true;
                        item.CancelTheAppointment = IsRemoveAllAssociationsChecked;
                    }
                }
                else if (IsCloseAppointment)
                {
                    foreach(var item in BookedAppointments)
                    {
                        item.CloseTheRequest = IsRemoveAllAssociationsChecked;
                        item.CancelTheAppointment = true;
                    }
                }
            }

            // Select out the first appointment to be canceled, if any
            _firstCancelAppt = BookedAppointments.FirstOrDefault(br => br.CanCancelAppointment && br.CancelTheAppointment);

            if (_firstCancelAppt != null)
            {
                // Here we want to call the Cancel Appointment Process, which will trigger canceeling other appointments as well as closing any requests
                _firstCancelAppt.CancelStatus = "Canceling the Appointment";
                _eventAggregator.GetEvent<AppointmentCancelledEvent>().Subscribe(CancelAppointmentEventHandler, ThreadOption.UIThread, false);
                _eventAggregator.GetEvent<LaunchCancelApptDialogEvent>().Publish(new CancelApptDialogEventPayload(_firstCancelAppt.Appointment, null, false));
            }
            else
                CloseRequests();
        }

        #endregion Command methods

        #region properties

        public ObservableCollection<BookApptPresentationModel> BookedAppointments { get; } = new ObservableCollection<BookApptPresentationModel>();

        public bool DialogResult { get; set; }

        public bool IsCloseAppointment { get { return Appointment != null; } }

        public bool IsCloseRequest { get { return Request != null; } }

        public string SelectedClause { get; private set; }

        public string Title { get; set; }

        public IConfirmCloseMRTCRequestView View { get { return _view; } }

        #endregion properties

        #region private methods

        private void CancelAppointmentEventHandler(string payload)
        {
            if (payload == null)
            {
                CloseBookingStatusView();
                _view.Close();
                return;
            }

            var apptID = payload;
            var cancelByClinic = true;
            var cancelReasonIEN = "10";
            var cancelNotes = string.Empty;

            if (payload.Contains('^'))
            {
                var parts = payload.Split('^');
                apptID = parts[0];
                cancelByClinic = parts[1] == "1";
                cancelReasonIEN = parts[2];
                cancelNotes = parts[3];
            }

            // Are we getting back the appointment we sent to be cancelled from ExecuteClosedRequestsCommand
                // If not we are ignoring it because it is not ours
            if (apptID == _firstCancelAppt.Appointment.APPOINTMENTID)
            {
                // Unsubscribe from the AppointmentCancelledEvent
                _eventAggregator.GetEvent<AppointmentCancelledEvent>().Unsubscribe(CancelAppointmentEventHandler);
                _firstCancelAppt.CancelStatus = "Appointment Cancelled";
                // Remove the appointment ID from the parent request record
                _parentRequest.MRTCAppointmentIENs.Remove(_firstCancelAppt.Appointment.APPOINTMENTID);
                // Remove the appointment from the booking record
                _firstCancelAppt.Appointment = null;
                // Update the request, removing the appointment link
//                _dataAccessService.SetMRTCChildRequest(_firstCancelAppt.Request, _firstCancelAppt.Request.ApptRequestIen.ToString());
                // get the newly updated request
                _firstCancelAppt.Request = _dataAccessService.GetApptRequestByRequestIEN(_firstCancelAppt.Request.ApptRequestIen.ToString());

                // iterate through the remaining appointments to be cancelled - iterate only bookings with appointments
                foreach(var bModel in BookedAppointments.Where(br => br.Appointment != null).ToList())
                {
                    // Can we cancel this appointment
                    if (!bModel.CanCancelAppointment || !bModel.CancelTheAppointment)
                        continue;

                    bModel.CancelStatus = "Canceling the Appointment";
                    if ( bModel.Request.DesiredApptDate != null )
                    {
                        var cancelAppt = new CancelAppointment
                        {
                            ApptID = bModel.Appointment.APPOINTMENTID,
                            CancelledByClinic = cancelByClinic,
                            ReasonIEN = cancelReasonIEN,
                            OriginalDesiredDate = bModel.Request.DesiredApptDate.Value,
                            PatientDesiredDate = bModel.Request.DesiredApptDate.Value,
                            Notes = cancelNotes
                        };

                        var errorMessage = _dataAccessService.CancelAppointment(cancelAppt, bModel.Appointment);

                        if (string.IsNullOrEmpty(errorMessage))
                        {
                            // Cancel succeeded, notify everyone
                            _eventAggregator.GetEvent<AppointmentCancelledEvent>().Publish(
                                                string.Format(
                                                    "{0}^{1}^{2}^{3}",
                                                    bModel.Appointment.APPOINTMENTID,
                                                    cancelByClinic ? "1" : "0",
                                                    cancelReasonIEN,
                                                    cancelNotes
                                                )
                                            );

                            bModel.CancelStatus = "Appointment Canceled";
                            // prepare to re-open the request item
                            var name = bModel.Appointment.PATIENTNAME; //?????
                            var patIen = bModel.Appointment.PATIENTID;
                            // re-open the request
                            var waitListIen = _dataAccessService.ReopenWaitListRecordAfterCancelAppointment(
                                bModel.Appointment.APPTREQTYPE_REQTYPE,
                                bModel.Appointment.APPTREQTYPE_REQIEN,
                                bModel.Appointment.APPOINTMENTID,
                                name,
                                patIen,
                                cancelAppt.PatientDesiredDate.ToShortDateString(),
                                false,
                                false
                            );
                            // Update the request, removing the appointment link
                            _dataAccessService.SetMRTCChildRequest(bModel.Request, bModel.Request.ApptRequestIen.ToString());
                            // get the newly opened request
                            bModel.Request = _dataAccessService.GetApptRequestByRequestIEN(bModel.Request.ApptRequestIen.ToString());
                            // remove the cancelled appointment from the booking record
                            bModel.Appointment = null;
                        }
                    }

                    System.Windows.Forms.Application.DoEvents();
                }

                // Clear the initial cancel appointment object
                _firstCancelAppt = null;
                // Call the Close requests method
                CloseRequests();
            }
        }

        private void CloseRequests()
        {
            // a place to save the MRTC link values
            var links = new List<string>();
            var prefDates = new List<string>();
            var requestCount = 0;

            // iterate the booking records, need to iterate all to update the links
            foreach (var bModel in BookedAppointments)
            {
                if (bModel.CanCloseRequest && bModel.CloseTheRequest)
                {
                    // Close the current child request
                    bModel.CancelStatus = "Closing the Request";
                    _dataAccessService.ARClose(bModel.Request.ApptRequestIen.ToString(), SelectedClause);
                    System.Windows.Forms.Application.DoEvents();
                    // Remove the request IEN from the parent request ien list
                    _parentRequest.MRTCAppointmentRequestIENs.Remove(bModel.Request.ApptRequestIen.ToString());
                    // Remove the request from the booking record
                    bModel.Request = null;
                    bModel.CancelStatus = "Request Closed";
                    System.Windows.Forms.Application.DoEvents();
                }

                if (bModel.Request != null)
                {
                    links.Add(
                        string.Format(
                            "{0}~{1}",
                            bModel.Appointment == null ? string.Empty : bModel.Appointment.APPOINTMENTID,
                            bModel.Request == null ? string.Empty : bModel.Request.ApptRequestIen.ToString()
                        )
                    );

                    prefDates.Add(bModel.DesiredDateStr);
                    requestCount++;
                }
            }

            // Update the MRTC Child Links
            _dataAccessService.UpdateMRTCChildLinks(_parentRequest.ApptRequestIen, links);

            // If we are closing the parent request, do so now
            if (IsRemoveParentChecked)
            {
                Keys.MrtcCloseRequestIen = _parentRequest.ApptRequestIen.ToString();
                _dataAccessService.ARClose(_parentRequest.ApptRequestIen.ToString(), SelectedClause);
                System.Windows.Forms.Application.DoEvents();
            }
            else
            {
                int openRequestCount = 0;

                foreach (var bModel in BookedAppointments)
                    if (bModel.RequestIsOpen)
                        openRequestCount++;

                if (openRequestCount > 0)
                {
                    // Make sure the Parent request is reopned
                    var reqType = _parentRequest.AppointmentRequestType;
                    _dataAccessService.ReopenAppointmentRequestByRequestIen(_parentRequest.ApptRequestIen.ToString(), ref reqType);
                }
            }

            // Refresh the wait list
            _eventAggregator.GetEvent<CloseFindAppointmentEvent>().Publish(_parentRequest.ApptRequestIen.ToString());
            // Update the current schedule
            _eventAggregator.GetEvent<RefreshCurrentScheduleEvent>().Publish(string.Empty);
            // Update future appointments
            _eventAggregator.GetEvent<UpdateFutureAppointmentsEvent>().Publish(null);
            CloseBookingStatusView();
            DialogResult = true;
        }

        private void LoadRequestData()
        {
            BookedAppointments.Clear();
            var list = FindApptService.LoadMRTCChildRequests(_dataAccessService, ParentRequest);

            if (list.Count > 0)
                foreach (var bRecord in list)
                    BookedAppointments.Add(bRecord);
        }

        #endregion private methods

        #region IConfirmCloseMRTCRequestPresentationModel members

        public bool Initialize(LaunchMRTCCancelDialogArgs args)
        {
            Request = args.Request;
            Appointment = args.Appointment;

            if (!string.IsNullOrEmpty(args.SelectedClause))
                SelectedClause = args.SelectedClause;

            // Get the Parent Request
            if (Request != null)
            {
                if (Request.IsMRTCParent)
                {
                    _parentRequest = Request;
                    IsRemoveOnlyThisChecked = false;
                    IsRemoveAllChecked = true;
                    IsRemoveAllAssociationsChecked = true;
                    IsRemoveParentChecked = true;
                }
                else
                    _parentRequest = _dataAccessService.GetApptRequestByRequestIEN(Request.MRTCParentRequestIEN);
            }
            else if (Appointment != null && !string.IsNullOrEmpty(Appointment.MRTCParentRequestIEN))
                _parentRequest = _dataAccessService.GetApptRequestByRequestIEN(Appointment.MRTCParentRequestIEN);
            else
            {
                Keys.AlertUser(this, "Canceling/Closing MRTC records requires either a MRTC Appointment Request or MRTC Appointment be selected.", "MRTC Cancel");
                return false;
            }

            OnPropertyChanged("AppointmentOptionVisibility");
            OnPropertyChanged("OptionsVisibility");
            OnPropertyChanged("RemoveOnlyThisText");
            OnPropertyChanged("RemoveAllAssociationsText");
            OnPropertyChanged("RemoveAllText");

            return true;
        }

        public void CloseBookingStatusView()
        {
            if (_bookingStatusView != null)
            {
                _bookingStatusView.Close();
                _bookingStatusView = null;
            }
        }

        #endregion IConfirmCloseMRTCRequestPresentationModel members

        #region INotifyPropertyChanged members

        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion INotifyPropertyChanged members
    }
}
