using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Linq;
using System.Windows;
using System.Windows.Media;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Modules.PatientAppt.Group;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Commands;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.FindAppt.Services;
using Microsoft.Practices.Unity;
using System.Globalization;

namespace ClinSchd.Modules.FindAppt.FindAppt
{
    public class FindApptPresentationModel : AsyncValidatableModel, IFindApptPresentationModel
    {
        #region fields

        private AppointmentRequest _appointmentRequest;
        readonly PatientAppt.Controllers.PatientApptController _patientApptController;
        readonly PatientAppt.Services.PatientApptService _patientApptService;

        public readonly IDataAccessService DataAccessService;
        public readonly IEventAggregator EventAggregator;
        private ValidationMessage _validationMessage;
        private IList<NameValue> _accessGroupList;
        private IList<FindGroupedAccessTypes> _schdGroupedAccessTypes;
        private string _accessGroupIEN = "0";
        private ObservableCollection<FindAppointmentResult> _findAppointmentResult = new ObservableCollection<FindAppointmentResult>();
        public IList<FindAppointmentResult> AllAppointmentResults { get; set; }
        private IList<NameValue> _clinicGroupList;
        private bool _isAMChecked;
        private bool _isPMChecked;
        private bool _isBothChecked = true;
        private bool _isMonChecked;
        private bool _isTueChecked;
        private bool _isWedChecked;
        private bool _isThuChecked;
        private bool _isFriChecked;
        private bool _isSatChecked;
        private bool _isSunChecked;
        private string _prevStartTime = string.Empty;
        private string _prevEndTime = string.Empty;
        private DateTime _minimumSelectableDate;
        private DateTime? _selectedSearchDate;
        private string _holidayIndicator = string.Empty;
        private FindAppointmentResult _selectedFindAppointmentResult;
        private bool _isWorking;
        private IList<Clinic> _ptCSchClinics = new List<Clinic>();
        private Clinic _selectedSearchClinic;
        private Clinic _originalSearchClinic;
        private DateTime _desiredAppointmentDate;
        private IList<DateTime> _desiredAppointmentDates;
        private BookApptPresentationModel _selectedBookingRecord;
        private BookingStatusView _bookingStatusView;
        private bool _changesMade;
        private readonly List<SchdResource> _cachedResources = new List<SchdResource>();

        #endregion fields

        #region constructors

        public FindApptPresentationModel(
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            IsCancelStatusVisible = false;
            Title = "MRTC Booking Status";
            EventAggregator = eventAggregator;
            DataAccessService = dataAccessService;
            // Get the mechanism to call the Book Appointment process
            _patientApptController = new PatientAppt.Controllers.PatientApptController(new UnityContainer(), dataAccessService, eventAggregator);
            _patientApptService = new PatientAppt.Services.PatientApptService();

            FindAppointmentResult.CollectionChanged += FindAppointmentResult_CollectionChanged;

            // Create the Commands
            CloseCommand = new DelegateCommand<string>(ExecuteCloseCommand, CanExecuteCloseCommand);
            FindApptAppointmentCommand = new DelegateCommand<string>(ExecuteFindApptAppointmentCommand, CanExecuteFindApptAppointmentCommand);
            NewAppointmentCommand = new DelegateCommand<string>(ExecuteNewAppointmentCommand, CanExecuteNewAppointmentCommand);
            NextDesiredDateCommand = new DelegateCommand<string>(ExecuteNextDesiredDateCommand, CanExecuteNextDesiredDateCommand);
            SpecialInstructionsCommand = new DelegateCommand<string>(ExecuteSpecialInstructionsCommand, CanExecuteSpecialInstructionsCommand);

            ValidationMessage = new ValidationMessage();
            SelectedDates = new List<string>();
            IsBothChecked = true;
            AppointmentData.Resources = new List<SchdResource>();

            InitialErrorMessage();
            //LoadAccessTypeGroups();
            //LoadGroupedAccessTypes();
            //GetClinicGroups();
        }

        #endregion constructors

        #region INotifyPropertyChanged members

        public void OnPropertyChanged(string propertyName)
        {
            base.NotifyPropertyChanged(propertyName);
        }

        #endregion INotifyPropertyChanged members

        #region IFindApptPresentationModel members

        public AppointmentRequest AppointmentRequest
        {
            get { return _appointmentRequest; }
            set
            {
                _appointmentRequest = value;

                if (DesiredAppointmentDates == null)
                    DesiredAppointmentDates = new List<DateTime>(); 
                else
                    DesiredAppointmentDates.Clear();

                SelectedSearchDate = DateTime.Today;

                if (_appointmentRequest.DesiredApptDate.HasValue && _appointmentRequest.DesiredApptDate.Value.CompareTo(SelectedSearchDate) >= 0)
                    SelectedSearchDate = _appointmentRequest.DesiredApptDate;

                DesiredAppointmentDates.Add(SelectedSearchDate.Value);

                _appointmentRequest.Patient.ServiceConnection = _appointmentRequest.Patient.SCVisit;

                OnPropertyChanged("AppointmentRequest");
                InitializeFindAppt();
            }
        }

        public void OnClose()
        {
            if (_bookingStatusView != null)
            {
                _bookingStatusView.Close();
                _bookingStatusView = null;
            }

            foreach (Window window in Application.Current.Windows)
            {
                var title = window.Title;
                if (window.Title.Contains("Find Appointment for"))
                    window.Close();
            }
        }

        public IList<string> SelectedDates { get; set; }

        public SchdResource SelectedResource
        {
            get
            {
                // No SelectedSearch clinic, nnothing to search with
                if (SelectedSearchClinic == null)
                    return null;

                // Check locally cached resources
                var result = _cachedResources.FirstOrDefault(r => r.HOSPITAL_LOCATION_ID == SelectedSearchClinic.HospitalLocationId);

                // if found then return it
                if (result != null)
                    return result;

                // Still have not found a resource, check the SelectedResourcesList
                if (SelectedResourceList != null)
                    result = SelectedResourceList.FirstOrDefault(r => r.HOSPITAL_LOCATION_ID == SelectedSearchClinic.HospitalLocationId);

                // If still do not have a resource, retrieve it from the database
                if (result == null)
                    result = DataAccessService.GetSdecResourceByName(SelectedSearchClinic.ClinicName);

                // If we found a resource add it to the local cache
                if (result != null)
                    _cachedResources.Add(result);
                    
                // return the results
                return result;
            }
        }

        public IList<SchdResource> SelectedResourceList { get; set; }

        public ValidationMessage ValidationMessage
        {
            get { return _validationMessage; }
            set { _validationMessage = value; }
        }

        public IFindApptView View { get; set; }

        #endregion IFindApptPresentationModel members

        #region Commands

        public DelegateCommand<string> CloseCommand { get; }

        public DelegateCommand<string> FindApptAppointmentCommand { get; }

        public DelegateCommand<string> NewAppointmentCommand { get; }

        public DelegateCommand<string> NextDesiredDateCommand { get; }

        public DelegateCommand<string> SpecialInstructionsCommand { get; private set; }

        #endregion Commands

        #region Command methods

        public bool CanExecuteCloseCommand(string command)
        {
            return !IsWorking;
        }

        public bool CanExecuteFindApptAppointmentCommand(string command)
        {
            if (AppointmentRequest == null)
                return false;

            return ( command == "IGNORE WORKING" || !IsWorking )
                   &&
                   SelectedSearchDate.HasValue
                   &&
                   SelectedSearchClinic != null
                   &&
                   ( IsAMChecked || _isPMChecked || IsBothChecked );
        }

        public bool CanExecuteNewAppointmentCommand(string command)
        {
            return !IsWorking
                   &&
                   SelectedSearchClinic != null
                   &&
                   FindAppointmentResult_SelectedForBooking().Count > 0
                   &&
                   (
                       SelectedSearchClinic.VariableAppointmentLength
                       ||
                       (SelectedSearchClinic.Duration == FindAppointmentResult_SelectedDuration())
                   );
        }

        public bool CanExecuteNextDesiredDateCommand(string command)
        {
            bool desiredDateSet = false;

            if ( DesiredAppointmentDates != null && DesiredAppointmentDates.Count > 1 )
                desiredDateSet = DesiredAppointmentDate.CompareTo( DesiredAppointmentDates[DesiredAppointmentDates.Count - 1] ) < 0;

            return !IsWorking && desiredDateSet;
        }

        public bool CanExecuteSpecialInstructionsCommand( string command )
        {
            return true;
        }

        public void ExecuteCloseCommand(string command)
        {
            if (_changesMade)
            {
                EventAggregator.GetEvent<CloseFindAppointmentEvent>().Publish(AppointmentRequest.ApptRequestIen.ToString());
                EventAggregator.GetEvent<UpdateFutureAppointmentsEvent>().Publish(null);
            }

            OnClose();
        }

        public void ExecuteFindApptAppointmentCommand(string command)
        {
            IsWorking = true;

            try
            {
                FindMultipleReturnToClinicAppointments();
            }
            finally
            {
                IsWorking = false;
            }
        }

        public void ExecuteNewAppointmentCommand(string command)
        {
            IsWorking = true;

            try
            {
                try
                {
                    // Get the list of slots that have been checked, soted by startTime.
                    List<FindAppointmentResult> bookingList = FindAppointmentResult_SelectedForBooking();
                    // Variable length is allowed so 
                    //      startTime is startTime of 1st slot in the list
                    //      endTime is endTime of last slot in the list.
                    // This will cover a single checked slot as well
                    DateTime sTime = Convert.ToDateTime(bookingList[0].StartTime);
                    DateTime eTime = Convert.ToDateTime(bookingList[bookingList.Count - 1].EndTime);

                    if (SelectedResource.Clinic.VariableAppointmentLength)
                    {
                        // Using SELECTED duration, or Appointment MAX duration, which ever is smaller
                        int duration = Math.Min((int)(eTime - sTime).TotalMinutes, Keys.NewAppointmentMaxDuration);

                        if (duration < SelectedResource.Clinic.Duration)
                            eTime = sTime.AddMinutes(SelectedResource.Clinic.Duration);
                        else if (duration > SelectedResource.Clinic.Duration)
                            eTime = sTime.AddMinutes(duration);
                    }

                    string startTime = sTime.ToString();
                    string endTime = eTime.ToString();

                    // Create the Appointment object
                    //      AppointmentData.Resources[0] is set to the current SelectedSearchClinic, to it is matched and can be used here
                    var newSchdAppt = new SchdAppointment
                    {
                        PATIENTNAME = AppointmentRequest.Patient.Name,
                        Patient = AppointmentRequest.Patient,
                        PATIENTID = AppointmentRequest.Patient.IEN,
                        ResourceId = AppointmentData.Resources[0].RESOURCEID,
                        RESOURCENAME = AppointmentData.Resources[0].RESOURCE_NAME,
                        SelectedResource = AppointmentData.Resources[0],
                        START_TIME = startTime,
                        END_TIME = endTime,
                        DesiredAppointmentDate = DesiredAppointmentDate,
                        MRTCParentRequestIEN = AppointmentRequest.ApptRequestIen.ToString(),
                        Command = "New Appointment"
                    };
                    
                    // Call the booking method
                    BookNewAppointment(newSchdAppt);
                }
                catch (Exception ex)
                {
                    TraceLog.LogException(GetType().Name, "ExecuteNewApptAppointmentCommand", ex);
                    View.AlertUser("There was a problem booking an appointment\n" + ex.Message + "\nReview the Trace Log for more details.", "MRTC Booking Error");
                }
            }
            finally
            {
                IsWorking = false;
            }
        }

        public void ExecuteNextDesiredDateCommand(string command)
        {
            IsWorking = true;

            try
            {
                try
                {
                    DesiredAppointmentDate = DesiredAppointmentDates.FirstOrDefault(d => d.CompareTo(DesiredAppointmentDate) > 0);
                }
                catch (Exception ex)
                {
                    TraceLog.LogException(GetType().Name, "ExecuteNextDesiredDateCommand", ex);
                    View.AlertUser("There was a problem retrieving the next desired date\n" + ex.Message + "\nReview the Trace Log for more details.", "MRTC Navigation Error");
                }
            }
            finally
            {
                IsWorking = false;
            }
        }

        public void ExecuteSpecialInstructionsCommand(string command) { View.AlertUser(ClinicSpecialInstructions, "Special Instructions"); }

        #endregion Command methods

        #region Bindings

        public string Title { get; set; }

        public ObservableCollection<BookApptPresentationModel> BookedAppointments { get; } = new ObservableCollection<BookApptPresentationModel>();

        public string BookingStatus
        {
            get
            {
                if (AppointmentRequest == null)
                    return String.Empty;

                return string.Format("Booking {0} of {1}", BookingItemNumber, AppointmentRequest.NumberOfAppointmentsRequired);
            }
        }

        public Visibility BookingStatusVisibility
        {
            get { return BookedAppointments.Count > 0 ? Visibility.Visible : Visibility.Hidden; }
        }

        public bool IsCancelStatusVisible { get; private set; }

        public string ClinicGroupIEN { get; set; }

        public IList<NameValue> ClinicGroupList
        {
            get { return _clinicGroupList; }
            set
            {
                if ( _clinicGroupList != value )
                {
                    _clinicGroupList = value;
                    OnPropertyChanged( "ClinicGroupList" );
                }
            }
        }

        public string ClinicSpecialInstructions
        {
            get
            {
                if ((SelectedSearchClinic == null) || String.IsNullOrEmpty(SelectedSearchClinic.SpecialInstructions))
                    return string.Empty;

                return SelectedSearchClinic.SpecialInstructions.Trim();
            }
        }

        public Brush ClinicSpecialInstructionsBackground
        {
            get { return Keys.ColorCodes.SpecialInstructionsColor; }
        }

        public Visibility ClinicSpecialInstructionsVisibility
        {
            get { return string.IsNullOrEmpty(ClinicSpecialInstructions) ? Visibility.Collapsed : Visibility.Visible; }
        }

        private bool _desiredAppointmentDateChanging;
        public DateTime DesiredAppointmentDate
        {
            get { return _desiredAppointmentDate; }
            set
            {
                _desiredAppointmentDate = value;
                OnPropertyChanged("DesiredAppointmentDate");
                SelectedSearchDate = DesiredAppointmentDate;

                if (!_desiredAppointmentDateChanging && (BookedAppointments.Count > 0))
                {
                    _desiredAppointmentDateChanging = true;

                    try
                    {
                        SelectedBookingRecord = BookedAppointments.FirstOrDefault(br => br.DesiredDate.Value.CompareTo(_desiredAppointmentDate) == 0);
                    }
                    finally
                    {
                        _desiredAppointmentDateChanging = false;
                    }
                }

                if (CanExecuteFindApptAppointmentCommand("IGNORE WORKING"))
                    ExecuteFindApptAppointmentCommand(string.Empty);
            }
        }

        public IList<DateTime> DesiredAppointmentDates
        {
            get { return _desiredAppointmentDates; }
            set
            {
                _desiredAppointmentDates = value;
                OnPropertyChanged("DesiredAppointmentDates");
            }
        }

        public ObservableCollection<FindAppointmentResult> FindAppointmentResult
        {
            get { return _findAppointmentResult; }
            set
            {
                _findAppointmentResult = value;
                OnPropertyChanged( "FindAppointmentResult" );
            }
        }

        public string HolidayIndicator
        {
            get { return _holidayIndicator; }
            set
            {
                _holidayIndicator = value;
                OnPropertyChanged("HolidayIndicator");
                OnPropertyChanged("HolidayIndicatorVisibility");
            }
        }

        public Brush HolidayIndicatorBackground { get { return Keys.ColorCodes.HolidayNotWorkingColor; } }

        public Visibility HolidayIndicatorVisibility { get { return string.IsNullOrEmpty(HolidayIndicator) ? Visibility.Hidden : Visibility.Visible; } }

        public Brush InactiveClinicIndicatorBackground { get { return Keys.ColorCodes.InactiveClinicBackgroundColor; } }

        public Brush InactiveClinicIndicatorForeground { get { return Keys.ColorCodes.InactiveClinicForegroundColor; } }

        public string InactiveClinicIndicatorText
        {
            get
            {                
                string result = string.Empty;

                if (SelectedResource != null)
                    if (!string.IsNullOrEmpty(SelectedResource.InactiveDate))
                    {
                        DateTime inactiveDate = Convert.ToDateTime(SelectedResource.InactiveDate);
                        DateTime? reactivateDate = null;

                        if (!string.IsNullOrEmpty(SelectedResource.ReactivatedDate))
                            reactivateDate = Convert.ToDateTime(SelectedResource.ReactivatedDate);

                        if (DateTime.Today.CompareTo(inactiveDate) < 0)
                            result = "Clinic Deactivates on " + inactiveDate.ToShortDateString();
                        else
                            result = "Clinic Deactivated on " + inactiveDate.ToShortDateString();

                        if (!string.IsNullOrEmpty(result))
                            if (reactivateDate.HasValue)
                                if (inactiveDate.CompareTo(reactivateDate.Value) < 0)
                                    result += "Reactivates on " + reactivateDate.Value.ToShortDateString();
                    }

                return result;
            }
        }

        public Visibility InactiveClinicIndicatorVisibility
        {
            get
            {
                Visibility result = Visibility.Collapsed;

                if ( SelectedResource != null && !string.IsNullOrEmpty( SelectedResource.InactiveDate ) )
                    result = Visibility.Visible;

                return result;
            }
        }

        public bool IsAMChecked
        {
            get { return _isAMChecked; }
            set
            {
                _isAMChecked = value;
                OnPropertyChanged("IsAMChecked");
                EnableCommands();
            }
        }

        public bool IsPMChecked
        {
            get { return _isPMChecked; }
            set
            {
                _isPMChecked = value;
                OnPropertyChanged("IsPMChecked");
                EnableCommands();
            }
        }

        public bool IsBothChecked
        {
            get { return _isBothChecked; }
            set
            {
                _isBothChecked = value;
                OnPropertyChanged("IsBothChecked");
                EnableCommands();
            }
        }

        public bool IsMonChecked
        {
            get { return _isMonChecked; }
            set
            {
                _isMonChecked = value;
                OnPropertyChanged("IsMonChecked");
            }
        }

        public bool IsTueChecked
        {
            get { return _isTueChecked; }
            set
            {
                _isTueChecked = value;
                OnPropertyChanged("IsTueChecked");
            }
        }

        public bool IsWedChecked
        {
            get { return _isWedChecked; }
            set
            {
                _isWedChecked = value;
                OnPropertyChanged("IsWedChecked");
            }
        }

        public bool IsThuChecked
        {
            get { return _isThuChecked; }
            set
            {
                _isThuChecked = value;
                OnPropertyChanged("IsThuChecked");
            }
        }

        public bool IsFriChecked
        {
            get { return _isFriChecked; }
            set
            {
                _isFriChecked = value;
                OnPropertyChanged("IsFriChecked");
            }
        }

        public bool IsSatChecked
        {
            get { return _isSatChecked; }
            set
            {
                _isSatChecked = value;
                OnPropertyChanged("IsSatChecked");
            }
        }

        public bool IsSunChecked
        {
            get { return _isSunChecked; }
            set
            {
                _isSunChecked = value;
                OnPropertyChanged("IsSunChecked");
            }
        }

        public DateTime MinimumSelectableDate
        {
            get { return _minimumSelectableDate; }
            set
            {
                _minimumSelectableDate = value;
                OnPropertyChanged("MinimumSelectableDate");
            }
        }

        public string NoAvailabilityWarningText
        {
            get
            {
                if (SelectedResource == null)
                    return string.Empty;

                string result = "There are no slots available.\n\n{0}\nand click Search to complete the MRTC request.";
                string warning = "Please modify the search criteria";

                if (!string.IsNullOrEmpty(SelectedResource.InactiveDate))
                {
                    DateTime inactiveDate = Convert.ToDateTime(SelectedResource.InactiveDate);
                    string tense = "will be";

                    if (inactiveDate.CompareTo(DateTime.Today) < 0)
                        tense = "was";

                    if (inactiveDate.CompareTo(SelectedSearchDate.Value) < 0)
                        warning = string.Format
                            (
                                "The selected clinic {0} deactivated on {1}." +
                                "\nPlease select another clinic",
                                tense,
                                inactiveDate.ToShortDateString()
                            );
                }

                return string.Format(result, warning);
            }
        }

        public Visibility NoAvailabilityWarningVisibility
        {
            get { return FindAppointmentResult.Count > 0 ? Visibility.Collapsed : Visibility.Visible; }
        }

        public int NumberOfAvailableSlots
        {
            get { return FindAppointmentResult == null ? 0 : FindAppointmentResult.Sum(s => s.AvailabilityCount); }
        }

        public Visibility NumberOfAvailableSlotsVisibility
        {
            get { return FindAppointmentResult == null ? Visibility.Hidden : Visibility.Visible; }
        }

        public IList<Clinic> PtCSchClinics
        {
            get { return _ptCSchClinics; }
            set
            {
                _ptCSchClinics = value;
                OnPropertyChanged("PtCSchClinics");
            }
        }

        public Visibility SearchResultGridVisibility
        {
            get { return FindAppointmentResult.Count > 0 ? Visibility.Visible : Visibility.Collapsed; }
        }

        public BookApptPresentationModel SelectedBookingRecord
        {
            get { return _selectedBookingRecord; }
            set
            {
                _selectedBookingRecord = value;
                OnPropertyChanged("SelectedBookingRecord");
                OnPropertyChanged("BookingItemNumber");
                OnPropertyChanged("BookingStatus");

                if (!_desiredAppointmentDateChanging)
                {
                    _desiredAppointmentDateChanging =  true;

                    try
                    {
                        DesiredAppointmentDate = _selectedBookingRecord.DesiredDate.Value;
                    }
                    finally
                    {
                        _desiredAppointmentDateChanging = false;
                    }
                }
            }
        }

        public FindAppointmentResult SelectedFindAppointmentResult
        {
            get { return _selectedFindAppointmentResult; }
            set
            {
                _selectedFindAppointmentResult = value;
                OnPropertyChanged("SelectedFindAppointmentResult");

                if (_selectedFindAppointmentResult == null)
                    HolidayIndicator = string.Empty;
                else
                    HolidayIndicator = DataAccessService.IsHoliday(Convert.ToDateTime(_selectedFindAppointmentResult.StartTime));
            }
        }

        public Clinic SelectedSearchClinic
        {
            get { return _selectedSearchClinic; }
            set
            {
                if (AppointmentData.Resources.Count == 0)
                    AppointmentData.Resources.Add(new SchdResource(DataAccessService));
                else
                    AppointmentData.Resources[0] = new SchdResource(DataAccessService);

                bool selectedClinicIsProhibited = false;

                if (value != null)
                    selectedClinicIsProhibited = DataAccessService.IsProhibitedClinic(value.ClinicName, value.ClinicIen);

                if (selectedClinicIsProhibited)
                    _selectedSearchClinic = null;
                else
                {
                    _selectedSearchClinic = value;

                    if (_selectedSearchClinic != null)
                    {
                        if (!_selectedSearchClinic.IsCompleteRecord)
                            DataAccessService.RefreshClinic((RPMSClinic)_selectedSearchClinic);

                        AppointmentData.Resources[0].RESOURCEID = _selectedSearchClinic.ParentResource.RESOURCEID;
                        AppointmentData.Resources[0].RESOURCE_NAME = _selectedSearchClinic.ClinicName;
                        AppointmentData.Resources[0].HOSPITAL_LOCATION_ID = _selectedSearchClinic.HospitalLocationId;
                        AppointmentData.Resources[0].TIMESCALE = _selectedSearchClinic.Duration.ToString();
                        AppointmentData.Resources[0].OVERBOOK = _selectedSearchClinic.MaxOverBooksRaw;

                        //AppointmentData.Resources[0].RESOURCEID = DataAccessService.GetResourceIdByClinicIdAndName(_selectedSearchClinic.HospitalLocationId, _selectedSearchClinic.ClinicName, false);
                        //AppointmentData.Resources[0].RESOURCE_NAME = _selectedSearchClinic.ClinicName;
                        //AppointmentData.Resources[0].HOSPITAL_LOCATION_ID = _selectedSearchClinic.HospitalLocationId;
                        //AppointmentData.Resources[0].TIMESCALE = _selectedSearchClinic.Duration.ToString();
                        //AppointmentData.Resources[0].OVERBOOK = _selectedSearchClinic.MaxOverBooksRaw;
                    }
                }

                OnPropertyChanged("SelectedSearchClinic");
                OnPropertyChanged("ClinicSpecialInstructions");
                OnPropertyChanged("ClinicSpecialInstructionsVisibility");
                OnPropertyChanged("VariableLengthIndicator");
                OnPropertyChanged("VariableLengthIndicatorVisibility");
                OnPropertyChanged("InactiveClinicIndicatorText");
                OnPropertyChanged("InactiveClinicIndicatorVisibility");
                ExecuteFindApptAppointmentCommand(null);
                EnableCommands();
            }
        }

        public DateTime? SelectedSearchDate
        {
            get { return _selectedSearchDate; }
            set
            {
                _selectedSearchDate = value;
                OnPropertyChanged("SelectedSearchDate");
                EnableCommands();
            }
        }

        public string VariableLengthIndicator
        {
            get
            {
                if (SelectedSearchClinic == null)
                    return string.Empty;

                return string.Format(
                    "{0}{1} min Appt{2}",
                    SelectedSearchClinic.VariableAppointmentLength ? "VL " : string.Empty,
                    SelectedSearchClinic.Duration,
                    string.IsNullOrEmpty(SelectedSearchClinic.MaxOverBooksRaw) ? string.Empty : " | Max OB: " + SelectedSearchClinic.MaxOverBooksRaw
                );
            }
        }

        public Brush VariableLengthIndicatorBackground { get { return Keys.ColorCodes.VariableLengthApptColor; } }

        #endregion Bindings

        #region public properties

        public string AccessGroupIEN
        {
            get { return _accessGroupIEN; }
            set
            {
                if ( _accessGroupIEN != value )
                {
                    _accessGroupIEN = value;
                    OnPropertyChanged( "AccessGroupIEN" );
                }
            }
        }

        public IList<NameValue> AccessGroupList
        {
            get { return _accessGroupList; }
            set
            {
                if ( _accessGroupList != value )
                {
                    _accessGroupList = value;
                    OnPropertyChanged( "AccessGroupList" );
                }
            }
        }

        public FindAppointmentData AppointmentData = new FindAppointmentData();
        private string _groupedAccessTypeIEN;

        public List<FindAppointmentResult> AppointmentsToBook { get; set; }

        public int BookingItemNumber
        {
            get
            {
                if (SelectedBookingRecord == null)
                    return -1;
                else
                    return SelectedBookingRecord.Index;
            }
        }

        public string GroupedAccessTypeIEN
        {
            get { return _groupedAccessTypeIEN; }
            set
            {
                _groupedAccessTypeIEN = value;
                OnPropertyChanged( "GroupedAccessTypeIEN" );
            }
        }

        public bool IsWorking
        {
            get { return _isWorking; }
            set
            {
                _isWorking = value;
                OnPropertyChanged("IsWorking");
                EnableCommands();
            }
        }

        public IList<FindGroupedAccessTypes> SchdGroupedAccessTypes
        {
            get { return _schdGroupedAccessTypes; }
            set
            {
                if ( _schdGroupedAccessTypes != value )
                {
                    _schdGroupedAccessTypes = value;
                    OnPropertyChanged( "SchdGroupedAccessTypes" );
                }
            }
        }

        #endregion public properties

        #region public methods

        public void BookingStatusView_Show()
        {
            if (BookedAppointments.Count == 0)
                return;

            if (_bookingStatusView == null)
            {
                Window owner = Window.GetWindow((DependencyObject)View);
                _bookingStatusView = new BookingStatusView
                {
                    Owner = owner,
                    DataContext = this
                };

                // Positioning Calculations

                Rect screenRect = Keys.CurrentScreenDimensions();

                // Attempting to center the 2 forms
                // Width of both forms + 10 pixel separation
                double combinedWidth = owner.Width + _bookingStatusView.Width + 20;
                // Calculate owner left position
                double ownerLeft = screenRect.Left + (screenRect.Right - screenRect.Left + 1 - combinedWidth) / 2;

                // if owner left is less than the current screen left make an adjustment
                if (ownerLeft < screenRect.Left)
                    ownerLeft = screenRect.Left + 10;

                // Set the child form to be 20 pixels to the right of the owner
                double childLeft = ownerLeft + owner.Width + 20;
                // Get the new right position of the child form
                double childRight = childLeft + _bookingStatusView.Width;

                // if the child form right position is greater than the current screen right make an adjustment 
                if (childRight > screenRect.Right)
                    childLeft = childLeft - (childRight - screenRect.Right + 10);

                // Get the owner bottom
                double ownerBottom = owner.Top + owner.Height;
                // Set the child view bottom the same as the owner form bottom
                double childTop = ownerBottom - _bookingStatusView.Height;

                // if the new child view top is above the current screen top make an adjustment
                if (childTop < screenRect.Top)
                    childTop = screenRect.Top + 30;

                // Set the form positions
                owner.Left = ownerLeft;
                _bookingStatusView.Top = childTop;
                _bookingStatusView.Left = childLeft;
                _bookingStatusView.Owner = owner;
                _bookingStatusView.Show();
            }
        }

        public void ClinicsStringChanged(string newSearchString)
        {
            if (newSearchString == null)
                return;

            if (newSearchString.Length < Keys.ClinSetSearchCharMin)
                return;

            SelectedSearchClinic = null;
            var ptCSchClinics = DataAccessService.GetClinicsWithMatchingStopCode(AppointmentRequest.ClinicIen, newSearchString);

            if (ptCSchClinics != null)
            {
                View.SetCmbClinicsComboBoxOpen(ptCSchClinics.Count > 1);
                PtCSchClinics = ptCSchClinics.Where(r => r.ClinicName.StartsWith(newSearchString, true, new CultureInfo("en-us"))).ToList();

                if (PtCSchClinics.Count == 1)
                    SelectedSearchClinic = PtCSchClinics[0];
            }
            else
                PtCSchClinics = new List<Clinic>();
        }

        public void FilterAvailableAppointmentsByDesiredDate(string desiredDate)
        {
            FindAppointmentResult.Clear();
            foreach (var appt in AllAppointmentResults)
                if (AppointmentRequest.DesiredApptDate == Convert.ToDateTime(desiredDate))
                    FindAppointmentResult.Add(appt);
            OnPropertyChanged( "FindAppointmentResult" );
        }

        public void OnClose(string arg)
        {
            OnClose();
        }

        #endregion public methods

        #region private methods

        private void BookNewAppointment(SchdAppointment newAppointment)
        {
            // If there are no booking records we need to create one for each desired date in the series
            if (BookedAppointments.Count == 0)
            {
                DateTime desiredDate = newAppointment.DesiredAppointmentDate.HasValue ? newAppointment.DesiredAppointmentDate.Value : Convert.ToDateTime(newAppointment.START_TIME).Date;
                DesiredAppointmentDates[0] = desiredDate;
                DesiredAppointmentDate = DesiredAppointmentDates[0];
                // Store all desired dates for writing to MRTC records
                List<DateTime> mrtcCalcPrefDates = new List<DateTime> {desiredDate};     // Add the first desired date to the list

                // First booking record
                BookApptPresentationModel firstBooking = CreateBookingRecord(desiredDate);
                // Create the new request
                firstBooking.Request = Request_Create(desiredDate, firstBooking.Index);

                if (firstBooking.Request == null)
                {
                    View.AlertUser("Canceling the booking process.", "MRTC Booking Error");
                    return;
                }

                // Add the subsequent desired dates
                for (int i = 1; i < AppointmentRequest.NumberOfAppointmentsRequired; i++)
                {
                    // Calculate the desired date, starting from 1st appointment booking date
                    //DateTime nextDesiredDate = desiredDate.AddDays(AppointmentRequest.AppointmentIntervalInDays * i);
                    DateTime nextDesiredDate = Convert.ToDateTime(newAppointment.START_TIME).Date.AddDays(AppointmentRequest.AppointmentIntervalInDays * i);
                    // Add the new Booking Record
                    BookApptPresentationModel bRecord = CreateBookingRecord(nextDesiredDate);
                    // Create the new request
                    bRecord.Request = Request_Create(nextDesiredDate, bRecord.Index);
                    // Add the new Preferred Date
                    mrtcCalcPrefDates.Add(nextDesiredDate);
                }

                // Update the MRTC Desired Dates for the MRTC Master Request
                DataAccessService.UpdateMRTCChildPreferredDates(AppointmentRequest.ApptRequestIen, mrtcCalcPrefDates);
            }

            // Get the booking record for the current desired date
            BookApptPresentationModel bookingRecord = BookedAppointments.FirstOrDefault(ba => ba.DesiredDate.Value.CompareTo(DesiredAppointmentDate) == 0);
            
            // if a bookingRecord is not found then we have a problem
            if (bookingRecord == null)
            {
                View.AlertUser("Unable to find a booking record for the '" + DesiredAppointmentDate.ToShortDateString() + "' desired date.", "MRTC Booking Error");
                return;
            }

            // Need to update the selected booking record here, but do not want to trigger a search for new appointments
            SelectedBookingRecord = bookingRecord;
            DesiredAppointmentDate = _selectedBookingRecord.DesiredDate.Value;
            SelectedSearchDate = _desiredAppointmentDate;
            BookingStatusView_Show();

            if (bookingRecord.Request == null)
            {
                // If there is no Appointment Request associated with the booking record we need to create a new request
                bookingRecord.Request = Request_Create(DesiredAppointmentDate, bookingRecord.Index);

                // Update the MRTC Master Record
                if (bookingRecord.Request != null)
                    AppointmentRequest.MRTCAppointmentRequestIENs.Add(bookingRecord.Request.ApptRequestIen.ToString());
            }

            // See if the DesiredDate has already been booked
            if (bookingRecord.AppointmentBooked)
            {
                View.AlertUser("An appointment has previously been booked for the CID date " + DesiredAppointmentDate.ToShortDateString() + ".", "Appointment Previously Booked");
                return;
            }

            if (bookingRecord.Request != null)
            {
                // Time to book an appointment for the current desired date
                if (bookingRecord.Request.ClinicIen != SelectedSearchClinic.ClinicIen)
                {
                    bookingRecord.Request.ClinicIen = SelectedSearchClinic.ClinicIen;
                    bookingRecord.Request.ClinName = SelectedSearchClinic.ClinicName;
                    bookingRecord.Request.Clinic = SelectedSearchClinic;
                    bookingRecord.Request.ClinicStopName = SelectedSearchClinic.ClinicName;

                    foreach (var spec in bookingRecord.Request.GetApptServiceSpecialties( bookingRecord.Request.ClinicStopName ) )
                        if (spec.Name.ToUpper() == bookingRecord.Request.ClinicStopName.ToUpper())
                            bookingRecord.Request.ClinicStopIen = spec.Ien;

                    if (SelectedResource != null)
                        bookingRecord.Request.ClinicResourceId = SelectedResource.RESOURCEID;
                }
            }

            newAppointment.AppointmentRequest = bookingRecord.Request;

            // Set up the New Appointment dialog
            bool result = false;
            IGroupView groupView = new GroupView();
            groupView.RemoveRequestCommand = "REMOVE";
            var groupPresentationModel = new GroupPresentationModel(groupView, DataAccessService, _patientApptService, EventAggregator);
            Keys.IsAddAppointmentOrLeftClickOnScheduler = true;
            // Call the new appointment dialog
            _patientApptController.LaunchNewAppointmentDialog(newAppointment, _patientApptService, groupPresentationModel, ref result);

            // New Appointment Created ?
            if (result)
            {
                _changesMade = true;
                bookingRecord.Appointment = newAppointment;
                // Update the MRTC Master Record
                AppointmentRequest.MRTCAppointmentIENs.Add(newAppointment.APPOINTMENTID);

                // Grab desired dates for only unbooked appointments from the BookeAppointments list.
                IList<DateTime> dds = new List<DateTime>();

                // Update the CID drop down list with only unbooked requests
                foreach (BookApptPresentationModel booking in BookedAppointments.Where(ba => !ba.AppointmentBooked).ToList())
                    dds.Add(booking.DesiredDate.Value);

                DesiredAppointmentDates = dds;

                // What to do next . . . next booking or close the MRTC request.
                switch (DesiredAppointmentDates.Count)
                {
                    case 0:
                        //  All appointments have been booked,
                        // Remove \ Close  the MRTC Parent Request.
                        EventAggregator.GetEvent<RemoveRequestEvent>().Publish(AppointmentRequest.ApptRequestIen.ToString());
                        // Close the dialog
                        OnClose();
                        return;
                    case 1:
                        DesiredAppointmentDate = DesiredAppointmentDates[0];
                        break;
                    default:
                        ExecuteNextDesiredDateCommand(string.Empty);
                        break;
                }

                // Reset the minimum selectable date in the calendar
                MinimumSelectableDate = Convert.ToDateTime(newAppointment.START_TIME).Date.AddDays(1);
            }

            OnPropertyChanged("BookedAppointments");
            OnPropertyChanged("BookingStatusVisibiliy");
        }

        private BookApptPresentationModel CreateBookingRecord(DateTime desiredDate)
        {
            var bookingRecord = FindApptService.CreateBookingRecord(BookedAppointments, desiredDate);
            OnPropertyChanged("BookingItemNumber");
            OnPropertyChanged("BookingStatus");
            OnPropertyChanged("BookingStatusVisibility");
            return bookingRecord;
        }

        private void EnableCommands()
        {
            CloseCommand.RaiseCanExecuteChanged();
            FindApptAppointmentCommand.RaiseCanExecuteChanged();
            NewAppointmentCommand.RaiseCanExecuteChanged();
            NextDesiredDateCommand.RaiseCanExecuteChanged();
        }

        private void FindAvailability()
        {
            if (SelectedSearchClinic == null)
                return;

            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
            FindAppointmentData findAppointmentData = new FindAppointmentData();

            if (AppointmentData.Resources != null)
                findAppointmentData.Resources = AppointmentData.Resources;

            if (SelectedDates.Count > 0)
            {
                findAppointmentData.StartDate = SelectedDates[0];
                findAppointmentData.EndDate = SelectedDates[1];
            }
            else if (string.IsNullOrEmpty(_prevStartTime))
            {
                findAppointmentData.StartDate = DateTime.Today.ToShortDateString();
                findAppointmentData.EndDate = DateTime.Today.ToShortDateString();
            }
            else
            {
                findAppointmentData.StartDate = _prevStartTime;
                findAppointmentData.EndDate = _prevEndTime;
            }

            _prevStartTime = findAppointmentData.StartDate;
            _prevEndTime = findAppointmentData.EndDate;

            if (findAppointmentData.Resources == null || findAppointmentData.Resources.Count == 0)
                findAppointmentData.Resources = SelectedResourceList;

            foreach (FindGroupedAccessTypes a in SchdGroupedAccessTypes)
                if (a.IsChecked)
                    findAppointmentData.AccessTypes.Insert(0, a);

            if (IsAMChecked)
                findAppointmentData.AMPM = "AM";
            else if (IsPMChecked)
                findAppointmentData.AMPM = "PM";
            else if (IsBothChecked)
                findAppointmentData.AMPM = "BOTH";

            findAppointmentData.Monday = IsMonChecked;
            findAppointmentData.Tuesday = IsTueChecked;
            findAppointmentData.Wednesday = IsWedChecked;
            findAppointmentData.Thursday = IsThuChecked;
            findAppointmentData.Friday = IsFriChecked;
            findAppointmentData.Saturday = IsSatChecked;
            findAppointmentData.Sunday = IsSunChecked;

            if ((findAppointmentData.Resources != null) && (findAppointmentData.Resources.Count > 0))
            {
                IList<FindAppointmentResult> newResultList = DataAccessService.FindAppointments(findAppointmentData);
                List<string> s = new List<string>();

                if (SelectedDates == null)
                    SelectedDates = s;

                newResultList = newResultList.OrderBy(p => Convert.ToDateTime(p.StartTime)).ToList();

                foreach (var item in newResultList)
                    FindAppointmentResult.Add(item);
            }
            else
            {
                _validationMessage.IsValid = false;
                _validationMessage.Title = "Find Appointment";
                _validationMessage.Message = "Please select a clinic.";
            }
        }

        private void FindMultipleReturnToClinicAppointments()
        {
            try
            {
                DateTime? date1;
                DateTime? date2;
                SelectedDates.Clear();

                foreach (var item in FindAppointmentResult.ToList())
                    FindAppointmentResult.Remove(item);

                //TimeSpan? searchDateDiff = SelectedSearchDate.Value - DateTime.Now.Date;
                int searchDateDiff = (int)(SelectedSearchDate.Value - DateTime.Now.Date).TotalDays;

//                if (Convert.ToInt32(searchDateDiff.Value.TotalDays) == 0)
                if (searchDateDiff < 0)
                {
                    date1 = SelectedSearchDate.Value;
                    date2 = SelectedSearchDate.Value.AddDays(7);
                }
                else if (searchDateDiff == 0)
                {
                    date1 = SelectedSearchDate.Value;
                    date2 = SelectedSearchDate.Value.AddDays(7);
                }
                //else if ((searchDateDiff.Value.TotalDays > 0) && (searchDateDiff.Value.TotalDays < 3))
                else if ((searchDateDiff > 0) && (searchDateDiff < 3))
                {
                    //date1 = SelectedSearchDate.Value.AddDays(-searchDateDiff.Value.TotalDays);
                    date1 = SelectedSearchDate.Value.AddDays(-searchDateDiff);
                    date2 = SelectedSearchDate.Value.AddDays(3);
                }
                else
                {
                    date1 = SelectedSearchDate.Value.AddDays(-3);
                    date2 = SelectedSearchDate.Value.AddDays(3);
                }

                SelectedDates.Add(date1.ToString());
                SelectedDates.Add(date2.ToString());

                FindAvailability();

                OnPropertyChanged("FindAppointmentResult");
                OnPropertyChanged("NumberOfAvailableSlots");
                OnPropertyChanged("NumberOfAvailableSlotsVisibility");
                OnPropertyChanged("SearchResultGridVisibility");
                OnPropertyChanged("NoAvailabilityWarningText");
                OnPropertyChanged("NoAvailabilityWarningVisibility");

                System.Windows.Forms.Application.DoEvents();
                System.Threading.Thread.Sleep(250);
                System.Windows.Forms.Application.DoEvents();

                if (!ValidationMessage.IsValid)
                    View.AlertUser(ValidationMessage.Message, ValidationMessage.Title);
            }
            catch (Exception e)
            {
                TraceLog.LogException(GetType().Name, "FindMultipleReturnToClinicAppointments", e);
                Keys.AlertUserError(this, e, "MRTC Find Availability");
            }
        }

        private void InitializeFindAppt()
        {
            if (_appointmentRequest != null)
                if (_appointmentRequest.MultipleAppointmentsRequired)
                    BookedAppointments.Clear();

            IsAsync = true;
            DoWorkAsync(
                (s, args) =>
                {
                    lock (args)
                    {
                        WorkAsyncResults results = new WorkAsyncResults();
                        string action = "Initializing";
                        List<string> messages = new List<string>();

                        try
                        {
                            //action = "Read Clinics List";
                            //if ((PtCSchClinics == null) || (PtCSchClinics.Count == 0))
                            //  results.List.Add(DataAccessService.GetClinicsWithMatchingStopCode(AppointmentRequest.ClinicIen));
                            //else
                            // Need this as a place holder
                            results.List.Add(new List<Clinic>() { AppointmentRequest.Clinic });

                            action = "Read Access Groups";
                            if (AccessGroupList == null)
                                results.List.Add(DataAccessService.GetAccessGroups());
                            else
                                results.List.Add(null);

                            action = "Read Access Type Group";
                            if (SchdGroupedAccessTypes == null)
                                results.List.Add(DataAccessService.GetFindAccessTypeByGroupID(AccessGroupIEN));
                            else
                                results.List.Add(null);

                            action = "Read Clinic Groups";
                            if (ClinicGroupList == null)
                                results.List.Add(ClinicGroupList = DataAccessService.GetResourceGroupList());
                            else
                                results.List.Add(null);

                            if (_appointmentRequest != null)
                            {
                                if (_appointmentRequest.MultipleAppointmentsRequired)
                                    results.List.Add(FindApptService.LoadMRTCChildRequests(DataAccessService, _appointmentRequest));
                            }

                            results.Messages = messages;
                            results.Success = true;
                        }
                        catch (Exception ex)
                        {
                            results.Error = new Exception(string.Format("While initializing Find Appt there was a problem {0}.", action), ex);
                        }

                        args.Result = results;
                    }
                },
                (s, args) =>
                {
                    lock (args)
                    {
                        if (args.Result != null)
                        {
                            WorkAsyncResults results = (WorkAsyncResults)args.Result;

                            if (results.Error != null)
                            {
                                Keys.AlertUserError(View, results.Error, "Initialize Find Appt Error");
                                return;
                            }

                            if (results.Messages != null)
                                if (results.Messages.Count > 0)
                                    View.AlertUser(string.Join("\n", results.Messages.ToArray()), "Initialize Find Appt");

                            if (results.Success)
                            {
                                if (results.List[0] != null)
                                    PtCSchClinics = (IList<Clinic>)results.List[0];

                                if (results.List[1] != null)
                                {
                                    AccessGroupList = (IList<NameValue>)results.List[1];
                                    AccessGroupList.Insert(0, new NameValue("<Show All Access Types>", "0"));
                                    _accessGroupIEN = AccessGroupList[0].Value;
                                }

                                if (results.List[2] != null)
                                {
                                    SchdGroupedAccessTypes = (IList<FindGroupedAccessTypes>)results.List[2];
                                    GroupedAccessTypeIEN = SchdGroupedAccessTypes[0].ACCESS_TYPE_ID;
                                }

                                if (results.List[3] != null)
                                {
                                    ClinicGroupList = (IList<NameValue>)results.List[3];

                                    if (ClinicGroupList.Count > 0)
                                        ClinicGroupIEN = ClinicGroupList[0].Value;
                                }

                                SelectedSearchClinic = PtCSchClinics.FirstOrDefault(c => c.ClinicIen == _appointmentRequest.ClinicIen);

                                if (_appointmentRequest != null)
                                {
                                    if (_appointmentRequest.MultipleAppointmentsRequired)
                                    {

                                        ObservableCollection<BookApptPresentationModel> list = (ObservableCollection<BookApptPresentationModel>)results.List[4];

                                        if (list.Count > 0)
                                        {
                                            DesiredAppointmentDates.Clear();

                                            foreach (BookApptPresentationModel bRecord in list)
                                            {
                                                if (!bRecord.AppointmentBooked)
                                                    DesiredAppointmentDates.Add(bRecord.DesiredDate.Value.Date);

                                                BookedAppointments.Add(bRecord);
                                            }
                                        }

                                        if (DesiredAppointmentDates.Count > 0)
                                            DesiredAppointmentDate = DesiredAppointmentDates[0];
                                    }
                                    else
                                    {
                                        SelectedDates.Add(_appointmentRequest.DesiredApptDate.Value.ToShortDateString());
                                        DesiredAppointmentDates.Add(_appointmentRequest.DesiredApptDate.Value);
                                        DesiredAppointmentDate = _appointmentRequest.DesiredApptDate.Value;
                                    }
                                }
                            }
                            else
                            {
                                TraceLog.LogException(GetType().Name, "InitializeFindAppt", results.Error);
                                Keys.AlertUserError(View, results.Error, "Initialize Find Appt Error");
                            }
                        }
                    }
                },
                true,
                false,
                "Initializing Find Appt"
            );
        }

        private void InitialErrorMessage()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
        }

        //private void LoadGroupedAccessTypes()
        //{
        //    SchdGroupedAccessTypes = DataAccessService.GetFindAccessTypeByGroupID(AccessGroupIEN);
        //    if (SchdGroupedAccessTypes.Count > 0)
        //    {
        //        GroupedAccessTypeIEN = SchdGroupedAccessTypes[0].ACCESS_TYPE_ID;
        //    }
        //}

        //private void LoadAccessTypeGroups()
        //{
        //    AccessGroupList = DataAccessService.GetAccessGroups();
        //    AccessGroupList.Insert(0, new NameValue("<Show All Access Types>", "0"));
        //    _accessGroupIEN = AccessGroupList[0].Value;
        //}

        private void Request_Update(AppointmentRequest request, string appointmentIEN)
        {
            DataAccessService.SetMRTCChildRequest(request, request.ApptRequestIen.ToString(), appointmentIEN);
        }

        private AppointmentRequest Request_Create(DateTime? desiredDate, int index)
        {
            AppointmentRequest request = new AppointmentRequest(DataAccessService);
            //  1. Request IEN (Optional)
            request.RequestManagmentIen = string.Empty;
            //  2. Patient IEN
            request.Patient = AppointmentRequest.Patient;
            //  3. Originating Date
            request.DateEntered = AppointmentRequest.DateEntered;
            request.OriginatingDate = AppointmentRequest.OriginatingDate;
            //  4. Institution Name
            request.Institution = AppointmentRequest.Institution;
            //  5. Request Type
            request.AppointmentRequestType = AppointmentRequest.AppointmentRequestType;
            //  6. Request Specific Clinc, Name from file 44
            if (request.ClinicIen != SelectedSearchClinic.ClinicIen)
            {
                request.ClinicIen = SelectedSearchClinic.ClinicIen;
                request.ClinName = SelectedSearchClinic.ClinicName;
                request.Clinic = SelectedSearchClinic;
                request.ClinicStopName = SelectedSearchClinic.ClinicName;

                foreach (var spec in request.GetApptServiceSpecialties( request.ClinicStopName ) )
                    if (spec.Name.ToUpper() == request.ClinicStopName.ToUpper())
                        request.ClinicStopIen = spec.Ien;

                if (SelectedResource != null)
                    request.ClinicResourceId = SelectedResource.RESOURCEID;
            }
            //  7. Originating User Name
            request.OriginatingUser = AppointmentRequest.OriginatingUser;
            //  8. Priority
//            request.PriorityText = AppointmentRequest.PriorityText;
            request.Priority = AppointmentRequest.Priority;
            //  9. Requested By
            request.RequestedBy = AppointmentRequest.RequestedBy;
            request.RequestedByText = AppointmentRequest.RequestedByText;
            // 10. Requesting Provider Name, if Provider Request
            request.Provider = AppointmentRequest.Provider;
            request.RequestedByProviderMerge = AppointmentRequest.RequestedByProviderMerge;
            // 11. Desired Date
            request.DesiredApptDate = desiredDate;
            // 12. Comment
            request.PtCSchComment = AppointmentRequest.PtCSchComment;
            request.RequestManagmentComment = AppointmentRequest.RequestManagmentComment;
            // 13. Enrollment Priority
            request.EnrollmentPriority = AppointmentRequest.EnrollmentPriority;
            // 14. MRTC ?
            request.MultipleAppointmentsRequired = true;
            // 15. MRTC Appointment Interval - For child MRTC Records this the "index" of the child request
            request.AppointmentIntervalInDays = index; // SelectedBookingRecord.Index + 1;
            // 16. MRTC MRTC Appointment Count
            request.NumberOfAppointmentsRequired = AppointmentRequest.NumberOfAppointmentsRequired;
            // 17. Patient Contacts
            request.PatientContact = AppointmentRequest.PatientContact;
            // 18. Service Connected Priority (Optional)
            // Set when Patient Set in # 2 above
            // 19. Service Connected % (Optional)
            // Set when Patient Set in # 2 above
            // 20. MRTC Calculated preferred dates, separated by pipe "|" (Optional)
            // 21. Clinic Stop Pointer (Optional)
            request.ClinicStopIen = "";
            // 22. Appointment Type, pointer to Appointment Type file (Optional)

            if ( AppointmentRequest.AppointmentType == null )
                request.AppointmentType = SelectedSearchClinic.DefaultAppointmentTypeName;
            else
                request.AppointmentType = AppointmentRequest.AppointmentType;           
            // 23. Patient Status (Optinal)
            request.EnrolleeStatus = AppointmentRequest.EnrolleeStatus;
            // 24. (MRTC Parent Only) List of Child Appointment IENs
            // 25. (MRTC Child Only) MRTC Parent Request IEN
            request.MRTCParentRequestIEN = AppointmentRequest.ApptRequestIen.ToString();
            //26. 

            //27.

            //28.

            if (DataAccessService.SetMRTCChildRequest(request))
            {
                _changesMade = true;
                return request;
            }
            View.AlertUser("Unable to create an appointment request for CID " + desiredDate.Value.ToShortDateString() + ".", "MRTC Booking Error");
            return null;
        }

        #endregion private methods

        #region FindAppointmentResults methods

        private void FindAppointmentResult_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            switch (e.Action)
            {
                case NotifyCollectionChangedAction.Add:
                    FindAppointmentResult_Register(e.NewItems);
                    break;

                case NotifyCollectionChangedAction.Replace:
                    FindAppointmentResult_Register(e.NewItems);
                    FindAppointmentResult_DeRegister(e.OldItems);
                    break;

                case NotifyCollectionChangedAction.Remove:
                    FindAppointmentResult_DeRegister(e.OldItems);
                    break;
            }
        }

        private void FindAppointmentResult_DeRegister(IList items)
        {
            foreach (FindAppointmentResult item in items)
                item.PropertyChanged -= FindAppointmentResult_PropertyChanged;
        }

        private void FindAppointmentResult_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Book")
            {
                FindAppointmentResult far = (FindAppointmentResult)sender;
                List<FindAppointmentResult> list = FindAppointmentResult_SelectedForBooking();
                list.Remove(far);

                if (far.Book)
                {
                    if (SelectedSearchClinic.VariableAppointmentLength)
                    {
                        // Can check multiple slots, but must all be contigous such that a large time slot can be created from them
                        // If a slot is checked that is not contigous all other selected slots are unchecked
                        if (!FindAppointmentResult_IsContiguousSlot(far, list))
                            // Not contigous, keep the selected slot and remove all others
                            FindAppointmentResult_ThereCanBeOnlyOne(far, list);
                    }
                    else if (SelectedSearchClinic.Duration >= FindAppointmentResult_SlotDuration())
                    {
                        // Can check multiple slots, but must all be contigous such that a large time slot can be created from them
                        // If a slot is checked that is not contigous all other selected slots are unchecked
                        if (FindAppointmentResult_IsContiguousSlot(far, list))
                        {
                            // If Total selected slot duration exceeds the Clinic Appointment Duration then keep the selected slot and remove all others
                            if (FindAppointmentResult_SelectedDuration() > SelectedSearchClinic.Duration)
                                FindAppointmentResult_ThereCanBeOnlyOne(far, list);
                        }
                        else
                            // Not contigous, keep the selected slot and remove all others
                            FindAppointmentResult_ThereCanBeOnlyOne(far, list);
                    }
                    else
                        // Only 1 slot can be checked for booking
                        FindAppointmentResult_ThereCanBeOnlyOne(far, list);
                }

                EnableCommands();
            }
        }

        private bool FindAppointmentResult_IsContiguousSlot(FindAppointmentResult slot, List<FindAppointmentResult> list)
        {
            if (list.Count == 0)
                return true;

            // Check for contiguos checked slots
            return slot.EndTime.CompareTo(list[0].StartTime) == 0 || slot.StartTime.CompareTo(list[list.Count - 1].EndTime) == 0;
        }

        private void FindAppointmentResult_Register(IList items)
        {
            foreach (FindAppointmentResult item in items)
                item.PropertyChanged += FindAppointmentResult_PropertyChanged;
        }

        private int FindAppointmentResult_SelectedDuration()
        {
            List<FindAppointmentResult> selected = FindAppointmentResult_SelectedForBooking();

            if (selected.Count == 0)
                return 0;

            return (int)Convert.ToDateTime(selected[selected.Count - 1].EndTime).Subtract(Convert.ToDateTime(selected[0].StartTime)).TotalMinutes;
        }

        private List<FindAppointmentResult> FindAppointmentResult_SelectedForBooking()
        {
            if (FindAppointmentResult == null)
                return new List<FindAppointmentResult>();
            
            return FindAppointmentResult.Where(f => f.Book).OrderBy(p => Convert.ToDateTime(p.StartTime)).ToList();
        }

        private int FindAppointmentResult_SlotDuration()
        {
            if (FindAppointmentResult == null)
                return 0;

            return (int)Convert.ToDateTime(FindAppointmentResult[0].EndTime).Subtract(Convert.ToDateTime(FindAppointmentResult[0].StartTime)).TotalMinutes;
        }

        private void FindAppointmentResult_ThereCanBeOnlyOne(FindAppointmentResult far, List<FindAppointmentResult> list)
        {
            foreach (var item in list)
                if (!far.Equals(item))
                    item.Book = false;
        }

        #endregion FindAppointmentResults methods methods
    }
}
