using System.Collections.Generic;
using System.ComponentModel;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Management.AccessGroups.Services;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Commands;
using Microsoft.Practices.Composite.Presentation.Events;

namespace ClinSchd.Modules.Management.AccessGroups
{
    public class AccessGroupsPresentationModel : IAccessGroupsPresentationModel, INotifyPropertyChanged
    {
        private readonly IManagementAccessGroupsService _managementAccessGroupsService;
        private readonly IDataAccessService _dataAccessService;
        private readonly IEventAggregator _eventAggregator;

        public event PropertyChangedEventHandler PropertyChanged;

        private IList<NameValue> _accessGroupList;
        private IList<SchdAccessType> _accessTypeList;
        private IList<SchdGroupedAccessTypes> _schdGroupedAccessTypes;
        private string _accessTypeIEN;
        private string _accessGroupIEN;
        private string _groupedAccessTypeIEN;

        public AccessGroupsPresentationModel( IAccessGroupsView view,
            IManagementAccessGroupsService managementAccessGroupsService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator )
        {
            View = view;
            View.Model = this;
            _managementAccessGroupsService = managementAccessGroupsService;
            _dataAccessService = dataAccessService;
            _eventAggregator = eventAggregator;

            LoadAccessTypes();
            LoadAccessTypeGroups();

            AddAccessTypeCommand = new DelegateCommand<string>( ExecuteAddAccessTypeCommand, CanExecuteAddAccessTypeCommand );
            EditAccessTypeCommand = new DelegateCommand<string>( ExecuteEditAccessTypeCommand, CanExecuteEditAccessTypeCommand );
            AddAccessGroupCommand = new DelegateCommand<string>( ExecuteAddAccessGroupCommand, CanExecuteAddAccessGroupCommand );
            EditAccessGroupCommand = new DelegateCommand<string>( ExecuteEditAccessGroupCommand, CanExecuteEditAccessGroupCommand );
        }

        public void LoadAccessTypes()
        {
            AccessTypeList = _dataAccessService.GetAccessTypes();
            if ( AccessTypeList.Count > 0 )
            {
                AccessTypeIEN = AccessTypeList[0].BSDX_ACCESS_TYPE_IEN;
            }
        }

        public void LoadAccessTypeGroups()
        {
            AccessGroupList = _dataAccessService.GetAccessGroups();
            if ( AccessGroupList.Count > 0 )
            {
                AccessGroupIEN = AccessGroupList[0].Value;
            }
        }

        public void LoadGroupedAccessTypes()
        {
            SchdGroupedAccessTypes = _dataAccessService.GetAccessTypeByGroupID( AccessGroupIEN );
            if ( SchdGroupedAccessTypes.Count > 0 )
            {
                GroupedAccessTypeIEN = SchdGroupedAccessTypes[0].ACCESS_TYPE_ID;
            }
        }

        public void AddAccessTypeToGroup()
        {
            if ( AccessGroupIEN == null )
            {
                View.AlertUser( "Please select a access type", "Access Groups" );
                return;
            }
            if ( AccessTypeIEN == null )
            {
                View.AlertUser( "Please select a group", "Access Groups" );
                return;
            }
            SchdGroupedAccessTypes = _dataAccessService.AddAccessTypeToGroupByID( AccessGroupIEN, AccessTypeIEN );
            if ( SchdGroupedAccessTypes.Count > 0 )
            {
                GroupedAccessTypeIEN = SchdGroupedAccessTypes[0].ACCESS_TYPE_ID;
            }
        }

        public void AddAllAccessTypesToGroup()
        {
            if ( AccessGroupIEN == null )
            {
                View.AlertUser( "Please select a group", "Access Groups" );
                return;
            }
            foreach ( SchdAccessType accessType in AccessTypeList )
            {
                SchdGroupedAccessTypes = _dataAccessService.AddAccessTypeToGroupByID( AccessGroupIEN, accessType.BSDX_ACCESS_TYPE_IEN );
            }
        }

        public void RemoveGroupedAccessType()
        {
            if ( AccessGroupIEN == null )
            {
                View.AlertUser( "Please select a group", "Access Groups" );
                return;
            }
            if ( GroupedAccessTypeIEN == null )
            {
                View.AlertUser( "Please select an access type", "Access Groups" );
                return;
            }
            if ( !View.ConfirmUser( "Are you sure you want to remove this access type?", "Access Groups" ) )
                return;

            SchdGroupedAccessTypes = _dataAccessService.RemoveGroupedAccessTypeByID( AccessGroupIEN, GroupedAccessTypeIEN );
            if ( SchdGroupedAccessTypes.Count > 0 )
            {
                GroupedAccessTypeIEN = SchdGroupedAccessTypes[0].ACCESS_TYPE_ID;
            }
        }

        public void RemoveAllGroupedAccessTypes()
        {
            if ( AccessGroupIEN == null )
            {
                View.AlertUser( "Please select a group", "Access Groups" );
                return;
            }
            if ( !View.ConfirmUser( "Are you sure you want to remove all access types?", "Access Groups" ) )
                return;

            foreach ( SchdGroupedAccessTypes accessType in SchdGroupedAccessTypes )
            {
                SchdGroupedAccessTypes = _dataAccessService.RemoveGroupedAccessTypeByID( AccessGroupIEN, accessType.ACCESS_TYPE_ID );
            }
        }

        public void RemoveGroupByID()
        {
            if ( AccessGroupIEN == null )
            {
                View.AlertUser( "Please select a group", "Access Groups" );
                return;
            }
            if ( !View.ConfirmUser( "Are you sure you want to remove this group?", "Access Groups" ) )
                return;

            AccessGroupList = _dataAccessService.RemoveAccessTypeGroup( AccessGroupIEN );
            if ( AccessGroupList.Count > 0 )
            {
                AccessGroupIEN = AccessGroupList[0].Value;
            }
        }

        public void RefreshListBoxes( string s )
        {
            if ( s == "AccessTypeGroups" )
                LoadAccessTypeGroups();
            else
                LoadAccessTypes();
        }

        public void ExecuteAddAccessTypeCommand( string command )
        {
            _eventAggregator.GetEvent<ManagementItemAddedEvent>().Subscribe( RefreshListBoxes, ThreadOption.UIThread, true );
            _eventAggregator.GetEvent<LaunchAddAccessTypeDialogEvent>().Publish( "Add Access Type" );
        }

        public bool CanExecuteAddAccessTypeCommand( string title )
        {
            return true;
        }

        public void ExecuteEditAccessTypeCommand( string command )
        {
            _eventAggregator.GetEvent<ManagementItemAddedEvent>().Subscribe( RefreshListBoxes, ThreadOption.UIThread, true );
            _eventAggregator.GetEvent<LaunchAddAccessTypeDialogEvent>().Publish( AccessTypeIEN );
        }

        public bool CanExecuteEditAccessTypeCommand( string title )
        {
            return true;
        }

        public void ExecuteAddAccessGroupCommand( string command )
        {
            _eventAggregator.GetEvent<ManagementItemAddedEvent>().Subscribe( RefreshListBoxes, ThreadOption.UIThread, true );
            _eventAggregator.GetEvent<LaunchAddAccessGroupDialogEvent>().Publish( "Add Access Group" );
        }

        public bool CanExecuteAddAccessGroupCommand( string title )
        {
            return true;
        }

        public void ExecuteEditAccessGroupCommand( string command )
        {
            if ( AccessGroupIEN == null )
            {
                View.AlertUser( "Please select a group", "Access Groups" );
                return;
            }
            _eventAggregator.GetEvent<ManagementItemAddedEvent>().Subscribe( RefreshListBoxes, ThreadOption.UIThread, true );
            _eventAggregator.GetEvent<LaunchAddAccessGroupDialogEvent>().Publish( AccessGroupIEN );
        }

        public bool CanExecuteEditAccessGroupCommand( string title )
        {
            return true;
        }

        public IAccessGroupsView View { get; }
        public Patient Patient { get; set; }
        public DelegateCommand<string> AddAccessTypeCommand { get; private set; }
        public DelegateCommand<string> EditAccessTypeCommand { get; private set; }
        public DelegateCommand<string> AddAccessGroupCommand { get; private set; }
        public DelegateCommand<string> EditAccessGroupCommand { get; private set; }

        public string AccessTypeIEN
        {
            get { return _accessTypeIEN; }
            set
            {
                _accessTypeIEN = value;
                OnPropertyChanged( "AccessTypeIEN" );
            }
        }

        public string AccessGroupIEN
        {
            get { return _accessGroupIEN; }
            set
            {
                _accessGroupIEN = value;
                OnPropertyChanged( "AccessGroupIEN" );
            }
        }

        public string GroupedAccessTypeIEN
        {
            get { return _groupedAccessTypeIEN; }
            set
            {
                _groupedAccessTypeIEN = value;
                OnPropertyChanged( "GroupedAccessTypeIEN" );
            }
        }

        public IList<SchdGroupedAccessTypes> SchdGroupedAccessTypes
        {
            get { return _schdGroupedAccessTypes; }
            private set
            {
                if ( _schdGroupedAccessTypes != value )
                {
                    _schdGroupedAccessTypes = value;
                    OnPropertyChanged( "SchdGroupedAccessTypes" );
                }
            }
        }

        public IList<SchdAccessType> AccessTypeList
        {
            get { return _accessTypeList; }
            private set
            {
                if ( _accessTypeList != value )
                {
                    _accessTypeList = value;
                    OnPropertyChanged( "AccessTypeList" );
                }
            }
        }

        public IList<NameValue> AccessGroupList
        {
            get { return _accessGroupList; }
            private set
            {
                if ( _accessGroupList != value )
                {
                    _accessGroupList = value;
                    OnPropertyChanged( "AccessGroupList" );
                }
            }
        }

        private void OnPropertyChanged( string propertyName )
        {
            var handler = PropertyChanged;
            if ( handler != null )
                handler( this, new PropertyChangedEventArgs( propertyName ) );
        }
    }
}