using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Commands;
using Microsoft.Practices.Composite.Presentation.Events;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Management.ResourceGroups.Services;
using System.Windows;
using System.Windows.Input;

namespace ClinSchd.Modules.Management.ResourceGroups
{
    public class ResourceGroupsPresentationModel : IResourceGroupsPresentationModel, INotifyPropertyChanged
    {
        private readonly IManagementResourceGroupsService _managementResourceGroupsService;
        private readonly IDataAccessService _dataAccessService;
        private readonly IEventAggregator _eventAggregator;
        private IList<SchdResource> _resourceList;
        private IList<NameValue> _clinicGroupList;
        private IList<SchdGroupedResources> _schdGroupedResources;

        public ResourceGroupsPresentationModel(IResourceGroupsView view,
            IManagementResourceGroupsService managementResourceGroupsService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            View = view;
            View.Model = this;
            _managementResourceGroupsService = managementResourceGroupsService;
            _dataAccessService = dataAccessService;
            _eventAggregator = eventAggregator;

            LoadClinicGroups ();
            AddResourceGroupCommand = new DelegateCommand<string> (ExecuteAddResourceGroupCommand, CanExecuteAddResourceGroupCommand);
            EditResourceGroupCommand = new DelegateCommand<string> (ExecuteEditResourceGroupCommand, CanExecuteEditResourceGroupCommand);

            _eventAggregator.GetEvent<ManagementItemAddedEvent> ().Subscribe (RefreshListBoxes, ThreadOption.UIThread, true);
            _eventAggregator.GetEvent<SchedulingManagementUserGrpHotKeys>().Subscribe(SchedulingManagementUserGrpHotKeys, ThreadOption.UIThread, false);
            _eventAggregator.GetEvent<SchedulingManagementUserGrpTabToggle>().Subscribe(SchedulingManagementUserGrpTabToggle, ThreadOption.UIThread, false);
        }

        private string _lastClinicSearchString = string.Empty;
        private string _resourceIEN;
        private string _clinicGroupIEN;
        private string _groupedClinicIEN;

        public void SchedulingManagementUserGrpHotKeys(Key k)
        {
            View.SchedulingManagementUserGrpHotKeys(k);
        }

        public void SchedulingManagementUserGrpTabToggle(string s)
        {
            View.SchedulingManagementUserGrpTabToggle();
        }
        public void LoadClinics(string searchString)
        {
            string resourceIEN = ResourceIEN;
            _lastClinicSearchString = searchString;
            ResourceList = _dataAccessService.GetResources(searchString, false);

            if (ResourceList.Count > 0)
            {
                string lastSub = ResourceList[ResourceList.Count - 1].LastSub;

                if ((ResourceList.Count == Keys.ClinicResourceSearchMaxRecords) && !string.IsNullOrEmpty(lastSub))
                {
                    View.AlertUser($"The current search criteria returns more than {Keys.ClinicResourceSearchMaxRecords} records.\nPlease refine your search and try again.", "Resource Search Criteria");
                    View.SetResourceSearchFocus();
                }

                if (string.IsNullOrEmpty(resourceIEN))
                    ResourceIEN = ResourceList[0].RESOURCEID;
                else
                    ResourceIEN = resourceIEN;
            }
        }

        public void LoadClinicGroups ()
        {
            string clinicGroupIEN = ClinicGroupIEN;
            ClinicGroupList = _dataAccessService.GetResourceGroupList ();

            if (ClinicGroupList.Count > 0)
            {
                if (string.IsNullOrEmpty(clinicGroupIEN))
                    ClinicGroupIEN = ClinicGroupList[0].Value;
                else
                    ClinicGroupIEN = clinicGroupIEN;
            }
        }

        public void LoadGroupedClinics ()
        {
            SchdGroupedResources = _dataAccessService.GetResourcesByGroupID (ClinicGroupIEN);
            if (SchdGroupedResources.Count > 0)
            {
                GroupedClinicIEN = SchdGroupedResources[0].RESOURCE_GROUP_ITEMID;
            }
        }

        public void AddClinicToGroup ()
        {
            if (ClinicGroupIEN == null)
            {
                View.AlertUser ("Please select a group", "Clinic Groups");
                return;
            }
            if (ResourceIEN == null)
            {
                View.AlertUser ("Please select a resource", "Clinic Groups");
                return;
            }
            SchdGroupedResources = _dataAccessService.AddClinicToGroupByID (ClinicGroupIEN, ResourceIEN);
            if (SchdGroupedResources.Count > 0) {
                GroupedClinicIEN = SchdGroupedResources[0].RESOURCE_GROUP_ITEMID;
            }
            _eventAggregator.GetEvent<ManagementItemAddedEvent> ().Publish ("ResourceGroups");
        }

        public void AddAllClinicsToGroup()
        {
            if ( ClinicGroupIEN == null )
            {
                View.AlertUser( "Please select a group", "Clinic Groups" );
                return;
            }
            foreach ( SchdResource clinics in ResourceList )
            {
                SchdGroupedResources = _dataAccessService.AddClinicToGroupByID( ClinicGroupIEN, clinics.RESOURCEID );
            }
        }

        public void RemoveGroupedClinic ()
        {
            if (ClinicGroupIEN == null) {
                View.AlertUser ("Please select a group", "Clinic Groups");
                return;
            }
            if (GroupedClinicIEN == null) {
                View.AlertUser ("Please select a clinic", "Clinic Groups");
                return;
            }
            if ( !View.ConfirmUser( "Are you sure you want to remove this clinic?", "Clinic Groups" ) )
                return;

            var resourceId = "";
            foreach (var item in _schdGroupedResources)
            {
                if (item.RESOURCE_GROUP_ITEMID == GroupedClinicIEN)
                {
                    resourceId = item.RESOURCE_ID;
                    break;
                }
            }
            if (string.IsNullOrEmpty(resourceId))
            {
                MessageBox.Show("Invalid Resource ID: null");
                return;
            }
            SchdGroupedResources = _dataAccessService.RemoveGroupedClinicByID(ClinicGroupIEN, resourceId);
            if (SchdGroupedResources.Count > 0) {
                GroupedClinicIEN = SchdGroupedResources[0].RESOURCE_GROUP_ITEMID;
            }
            _eventAggregator.GetEvent<ManagementItemAddedEvent> ().Publish ("ResourceGroups");
        }

        public void RemoveAllGroupedClinics ()
        {
            if (ClinicGroupIEN == null) {
                View.AlertUser ("Please select a group", "Clinic Groups");
                return;
            }
            if ( !View.ConfirmUser( "Are you sure you want to remove all clinics?", "Clinic Groups" ) )
                return;

            foreach (SchdGroupedResources clinics in SchdGroupedResources)
            {
                SchdGroupedResources = _dataAccessService.RemoveGroupedClinicByID (ClinicGroupIEN, clinics.RESOURCE_ID);
            }
        }

        public void RemoveGroupByName ()
        {
            if (ClinicGroupIEN == null) {
                View.AlertUser ("Please select a group", "Clinic Groups");
                return;
            }
            if ( !View.ConfirmUser( "Are you sure you want to remove this group?", "Clinic Groups" ) )
                return;

            string groupName = string.Empty;
            foreach (NameValue c in ClinicGroupList)
            {
                if ( ClinicGroupIEN == c.Value )
                    groupName = c.Name;
            }
            ClinicGroupList = _dataAccessService.RemoveResourceGroup (groupName);
            if (ClinicGroupList.Count > 0)
            {
                ClinicGroupIEN = ClinicGroupList[0].Value;
            }
            _eventAggregator.GetEvent<ManagementItemAddedEvent>().Publish("ResourceGroups");
        }

        public void RefreshListBoxes (string s)
        {
            if ( s == "ResourceGroups" )
                LoadClinicGroups();

            if ( s == "ResourceInGroups" && !string.IsNullOrEmpty( _lastClinicSearchString ) )
                LoadClinics( _lastClinicSearchString );
        }

        public void ExecuteAddResourceGroupCommand (string command)
        {
            _eventAggregator.GetEvent<ManagementItemAddedEvent> ().Subscribe (RefreshListBoxes, ThreadOption.UIThread, true);
            _eventAggregator.GetEvent<LaunchAddResourceGroupDialogEvent> ().Publish ("Add Clinic Group");
        }

        public bool CanExecuteAddResourceGroupCommand (string title)
        {
            return true;
        }

        public void ExecuteEditResourceGroupCommand( string command )
        {
            if ( ClinicGroupIEN == null )
            {
                View.AlertUser( "Please select a group", "Clinic Groups" );
                return;
            }
            _eventAggregator.GetEvent<ManagementItemAddedEvent>().Subscribe( RefreshListBoxes, ThreadOption.UIThread, true );
            _eventAggregator.GetEvent<LaunchAddResourceGroupDialogEvent>().Publish( ClinicGroupIEN );
        }

        public bool CanExecuteEditResourceGroupCommand (string title)
        {
            return true;
        }

        public DelegateCommand<string> AddResourceGroupCommand { get; private set; }
        public DelegateCommand<string> EditResourceGroupCommand { get; private set; }
        public IResourceGroupsView View { get; }
        public Patient Patient { get; set; }

        public string ResourceIEN
        {
            get { return _resourceIEN; }
            set
            {
                _resourceIEN = value;
                OnPropertyChanged( "ResourceIEN" );
            }
        }

        public string ClinicGroupIEN
        {
            get { return _clinicGroupIEN; }
            set
            {
                _clinicGroupIEN = value;
                OnPropertyChanged( "ClinicGroupIEN" );
            }
        }

        public string GroupedClinicIEN
        {
            get { return _groupedClinicIEN; }
            set
            {
                _groupedClinicIEN = value;
                OnPropertyChanged( "GroupedClinicIEN" );
            }
        }

        public IList<NameValue> ClinicGroupList
        {
            get { return _clinicGroupList; }
            private set
            {
                if ( _clinicGroupList != value )
                {
                    _clinicGroupList = value;
                    OnPropertyChanged( "ClinicGroupList" );
                }
            }
        }

        public IList<SchdResource> ResourceList
        {
            get { return _resourceList; }
            private set
            {
                if ( _resourceList != value )
                {
                    _resourceList = value;
                    OnPropertyChanged( "ResourceList" );
                }
            }
        }

        public IList<SchdGroupedResources> SchdGroupedResources
        {
            get { return _schdGroupedResources; }
            private set
            {
                if ( _schdGroupedResources != value )
                {
                    _schdGroupedResources = value;
                    OnPropertyChanged( "SchdGroupedResources" );
                }
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if ( handler != null )
                handler( this, new PropertyChangedEventArgs( propertyName ) );
        }
    }
}
