using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using ClinSchd.Infrastructure.Events;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Commands;
using Microsoft.Practices.Composite.Presentation.Events;
using System.Windows.Input;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Management.Resources.Services;

namespace ClinSchd.Modules.Management.Resources
{
    public class ResourcesPresentationModel : IResourcesPresentationModel, INotifyPropertyChanged
    {
        private readonly IManagementResourcesService _managementResourcesService;
        private readonly IDataAccessService _dataAccessService;
        private readonly IEventAggregator _eventAggregator;
        private IList<NameValue> _users;
        private NameValue _selectedUser;
        private IList<ResourceUser> _resourceUserList;
        private IList<ProhibitedClinic> _resourceList;
        private ResourceUser _selectedResourceUser;
        private string _lastClinicSearch = string.Empty;

        public ResourcesPresentationModel(IResourcesView view,
            IManagementResourcesService managementResourcesService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            View                        = view;
            View.Model                  = this;
            _managementResourcesService = managementResourcesService;
            _dataAccessService          = dataAccessService;
            _eventAggregator            = eventAggregator;

            LoadUsers ();
            AddResourceCommand          = new DelegateCommand<string> (ExecuteAddResourceCommand, CanExecuteAddResourceCommand);
            EditResourceCommand         = new DelegateCommand<string> (ExecuteEditResourceCommand, CanExecuteEditResourceCommand);
            DeleteResourceCommand       = new DelegateCommand<string> (ExecuteDeleteResourceCommand, CanExecuteDeleteResourceCommand);
            ReactivateResourceCommand   = new DelegateCommand<string>(ExecuteReactivateResourceCommand, CanExecuteReactivateResourceCommand);
            AddUserCommand              = new DelegateCommand<string> (ExecuteAddUserCommand, CanExecuteAddUserCommand);
            EdiUserCommand              = new DelegateCommand<string> (ExecuteEditUserCommand, CanExecuteEditUserCommand);

            _eventAggregator.GetEvent<RefreshProhibitedClinicsEvent>().Subscribe(RefreshProhibitedClinics, ThreadOption.UIThread, true);
            _eventAggregator.GetEvent<SchedulingManagementUserHotKeys>().Subscribe(SchedulingManagementUserHotKeys, ThreadOption.UIThread, false);
            _eventAggregator.GetEvent<SchedulingManagementUserTabToggle>().Subscribe(SchedulingManagementUserTabToggle, ThreadOption.UIThread, false);
        }

        public void SchedulingManagementUserHotKeys(Key k)
        {
            View.SchedulingManagementUserHotKeys(k);
        }

        public void SchedulingManagementUserTabToggle(string s)
        {
            View.SchedulingManagementUserTabToggle();
        }
        private void RefreshProhibitedClinics(string arg)
        {
            //MessageBox.Show("Inside ResourcePresModel");
        }
        public int ProhibitedClinicsFound { get; set; }

        public void LoadResources (string searchString)
        {
            _lastClinicSearch = searchString;
            ResourceList = _dataAccessService.SearchSdecProhibitedClinics(searchString).Where(pc => pc.ResourceTypeCode == "H").ToList();
            ProhibitedClinicsFound = ResourceList.Count;
            if (ResourceList.Count > 0)
            {
                string lastSub = ResourceList[ResourceList.Count - 1].LastSub;

                if ((ResourceList.Count == Keys.ClinicResourceSearchMaxRecords) && !string.IsNullOrEmpty(lastSub))
                {
                    View.AlertUser($"The current search criteria returns more than {Keys.ClinicResourceSearchMaxRecords} records.\nPlease refine your search and try again.", "Prohibited Clinic Search Criteria");
                    View.SetProhibitedClinicFocus();
                }

                SelectedResource = ResourceList[0];
                ResourceIEN = ResourceList[0].Ien;
            }            
        }

        private ProhibitedClinic _selectedResource;
        private string _resourceUserIEN;
        private string _userIEN;
        private string _resourceIEN;

        public ProhibitedClinic SelectedResource
        {
            get { return _selectedResource; }
            set
            {
                _selectedResource = value;
                OnPropertyChanged( "SelectedResource" );
            }
        }

        public void LoadUsers ()
        {
            Users = _dataAccessService.GetScheduleUsers ();
            if (Users.Count > 0)
            {
                UserIEN = Users[0].Value;
                SelectedUser = Users[0];
            }
        }

        public void LoadResourceUsers (string resourceIen = "")
        {
            if(!string.IsNullOrEmpty(resourceIen))
                ResourceIEN = resourceIen;
            ResourceUserList = _dataAccessService.GetResourceUsersByResourceID (ResourceIEN);

            if (ResourceUserList.Count > 0)
            {
                ResourceUserIEN = ResourceUserList[0].BSDX_RESOURCE_USER_IEN;
                SelectedResourceUser = ResourceUserList[0];
            }			
        }

        public void UserSelectionChanged ()
        {
            foreach (var n in Users)
            {
                if (UserIEN == n.Value)
                {
                    SelectedUser = n;
                    return;
                }
            }
        }

        public void ResourceUserSelectionChanged ()
        {
            foreach (var r in _resourceUserList)
            {
                if (ResourceUserIEN == r.BSDX_RESOURCE_USER_IEN)
                {
                    SelectedResourceUser = r;
                    return;
                }
            }
        }

        public void RemoveResourceUser ()
        {
            if (ResourceUserIEN == null) {
                View.AlertUser ("Please select a user", "Clinics and Users");
                return;
            }
            if (ResourceIEN == null)
            {
                View.AlertUser ("Please select a clinic", "Clinics and Users");
                return;
            }
            if ( !View.ConfirmUser( "Are you sure you want to delete this user?", "Clinics and Users" ) )
                return;

            ResourceUserList = _dataAccessService.RemoveResourceUserByID (ResourceUserIEN, ResourceIEN);
            if ( ResourceUserList.Count > 0 )
                ResourceUserIEN = ResourceUserList[0].USER_ID;
        }

        public void RemoveAllResourceUsers ()
        {
            if (ResourceIEN == null)
            {
                View.AlertUser ("Please select a clinic", "Clinics and Users");
                return;
            }

            if ( !View.ConfirmUser( "Are you sure you want to delete all users?", "Clinics and Users" ) )
                return;

            if(ResourceUserList.Any())
            {
                var hospLocId = _dataAccessService.GetHospitalLocationIdByResourceIen(ResourceIEN);
                var parms = new List<string> {hospLocId, ""};
                var rslt = _dataAccessService.UpdatePrivilegedUsers(parms);
                if (rslt)
                    ResourceUserList = null;
            }

            /*while (ResourceUserList.Any())
                    {
                        foreach (ResourceUser user in this.ResourceUserList)
                            ResourceUserList = this.dataAccessService.RemoveResourceUserByID(user.BSDX_RESOURCE_USER_IEN, this.ResourceIEN);
                    }*/
        }

        public void RefreshListBoxes (string s)
        {
            if ( s == "Resources" )
                LoadResources( _lastClinicSearch );
            else
                LoadResourceUsers();
        }

        public void ExecuteAddResourceCommand (string command)
        {
            _eventAggregator.GetEvent<ManagementItemAddedEvent> ().Subscribe (RefreshListBoxes, ThreadOption.UIThread, true);
            _eventAggregator.GetEvent<LaunchAddResourceDialogEvent> ().Publish ("Add Clinic");
        }

        public bool CanExecuteAddResourceCommand (string title)
        {
            return true;
        }

        public void ExecuteEditResourceCommand (string command)
        {
            if (ResourceIEN == null)
            {
                View.AlertUser ("Please select a clinic", "Clinics and Users");
                return;
            }
            _eventAggregator.GetEvent<ManagementItemAddedEvent> ().Subscribe (RefreshListBoxes, ThreadOption.UIThread, true);
            _eventAggregator.GetEvent<LaunchAddResourceDialogEvent> ().Publish (ResourceIEN);
        }

        public bool CanExecuteEditResourceCommand (string title)
        {
            return true;
        }

        public void ExecuteDeleteResourceCommand (string command)
        {
            if (ResourceIEN == null)
            {
                View.AlertUser ("Please select a clinic", "Clinics and Users");
                return;
            }

            var sHospitalLocationID = string.Empty;
            foreach (var r in ResourceList)
            {
                if (ResourceIEN == r.Ien)
                {
                    sHospitalLocationID = r.HospitalLocationId;
                }
            }
            if ( !View.ConfirmUser( "Are you sure you want to delete this clinic?", "Clinics and Users" ) )
                return;

            var sError = _dataAccessService.DeleteClinic (ResourceIEN);
            if (sError != string.Empty)
            {
                View.AlertUser (sError, "Clinics and Users");
                return;
            }

            RefreshListBoxes ("Resources");
            _eventAggregator.GetEvent<CloseSchedulesEvent> ().Publish (sHospitalLocationID);
            _eventAggregator.GetEvent<ManagementItemAddedEvent> ().Publish (null);
        }

        public bool CanExecuteDeleteResourceCommand(string title)
        {
            return true;
        }

        public void ExecuteReactivateResourceCommand(string command)
        {
            _eventAggregator.GetEvent<ManagementItemAddedEvent>().Subscribe(RefreshListBoxes, ThreadOption.UIThread, true);
            _eventAggregator.GetEvent<LaunchReactivateResourceDialogEvent>().Publish(ResourceIEN);
        }

        public bool CanExecuteReactivateResourceCommand(string title)
        {
            return true;
        }

        public void ExecuteAddUserCommand(string command)
        {
            if (SelectedResource == null)
            {
                View.AlertUser("Please select a clinic", "Clinics and Users");
                return;
            }
            if (SelectedUser == null)
            {
                View.AlertUser("Please select a user", "Clinics and Users");
                return;
            }
            if (_resourceUserList.FirstOrDefault(u => u.USER_ID == SelectedUser.Value) != null)
            {
                View.AlertUser("The selected user is already a member of the Privileged Users list.", "Clinics and Users");
                return;
            }
            _eventAggregator.GetEvent<ManagementItemAddedEvent>().Subscribe(RefreshListBoxes, ThreadOption.UIThread, true);
            _eventAggregator.GetEvent<LaunchAddResourceUserDialogEvent>().Publish("Add Clinic User");

            var r = new ResourceUser
            {
                USER_ID     = SelectedUser.Value,
                USERNAME    = SelectedUser.Name,
                RESOURCE_ID = ResourceIEN
            };
            _eventAggregator.GetEvent<AddEditResourceUserEvent>().Publish(r);
        }

        public bool CanExecuteAddUserCommand (string title)
        {
            return true;
        }

        public void ExecuteEditUserCommand (string command)
        {
            if (ResourceUserIEN == null || SelectedResourceUser == null)
            {
                View.AlertUser ("Please select a user", "Clinics and Users");
                return;
            }
            _eventAggregator.GetEvent<ManagementItemAddedEvent> ().Subscribe (RefreshListBoxes, ThreadOption.UIThread, true);
            _eventAggregator.GetEvent<LaunchAddResourceUserDialogEvent> ().Publish (ResourceUserIEN);
            _eventAggregator.GetEvent<AddEditResourceUserEvent> ().Publish (SelectedResourceUser);
        }

        public bool CanExecuteEditUserCommand (string title)
        {
            return true;
        }

        public DelegateCommand<string> AddResourceCommand { get; private set; }
        public DelegateCommand<string> EditResourceCommand { get; private set; }
        public DelegateCommand<string> DeleteResourceCommand { get; private set; }
        public DelegateCommand<string> ReactivateResourceCommand { get; private set; }
        public DelegateCommand<string> AddUserCommand { get; private set; }
        public DelegateCommand<string> EdiUserCommand { get; private set; }
        public IResourcesView View { get; }
        public Patient Patient { get; set; }

        public string ResourceIEN
        {
            get { return _resourceIEN; }
            set
            {
                _resourceIEN = value;
                OnPropertyChanged( "ResourceIEN" );
            }
        }

        public string ResourceUserIEN
        {
            get { return _resourceUserIEN; }
            set
            {
                _resourceUserIEN = value; 
                OnPropertyChanged( "ResourceUserIEN" );
            }
        }

        public string UserIEN
        {
            get { return _userIEN; }
            set
            {
                _userIEN = value;
                OnPropertyChanged( "UserIEN" );
            }
        }

        public IList<ResourceUser> ResourceUserList
        {
            get { return _resourceUserList; }
            set
            {
                if ( _resourceUserList != value )
                {
                    _resourceUserList = value ?? new List<ResourceUser>();
                }
                OnPropertyChanged( "ResourceUserList" );
            }
        }

        public IList<NameValue> Users
        {
            get { return _users; }
            set
            {
                _users = value;
                OnPropertyChanged( "Users" );
            }
        }

        public NameValue SelectedUser
        {
            get { return _selectedUser; }
            set
            {
                _selectedUser = value;
                OnPropertyChanged( "SelectedUser" );
            }
        }

        public IList<ProhibitedClinic> ResourceList
        {
            get { return _resourceList; }
            set
            {
                _resourceList = value;
                OnPropertyChanged( "ResourceList" );
            }
        }

        public ResourceUser SelectedResourceUser
        {
            get { return _selectedResourceUser; }
            set
            {
                _selectedResourceUser = value;
                OnPropertyChanged( "SelectedResourceUser" );
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if ( handler != null )
                handler( this, new PropertyChangedEventArgs( propertyName ) );
        }
    }
}
