using System.ComponentModel;

using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Commands;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.MarkAsNoShow.Services;
using System;

namespace ClinSchd.Modules.MarkAsNoShow.MarkAsNoShow
{
    public class MarkAsNoShowPresentationModel : IMarkAsNoShowPresentationModel, INotifyPropertyChanged
    {
        private readonly IMarkAsNoShowService _markAsNoShowService;
        public readonly IDataAccessService DataAccessService;
        public readonly IEventAggregator EventAggregator;
        private bool _isAutoRebookChecked;
        private AutoRebookData _autoRebook;

        public MarkAsNoShowPresentationModel (
            IMarkAsNoShowView view,
            IMarkAsNoShowService markAsNoShowService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            View = view;
            View.Model = this;
            EventAggregator = eventAggregator;
            _markAsNoShowService = markAsNoShowService;
            DataAccessService = dataAccessService;
            ValidationMessage = new ValidationMessage ();
            InitialErrorMessage ();
            NoShowAppointmentCommand = new DelegateCommand<string> (ExecuteNoShowAppointmentCommand, CanExecuteNoShowAppointmentCommand);
            PrintLetterCommand = new DelegateCommand<string> (ExecutePrintLetterCommand, CanExecutePrintLetterCommand);
        }

        public void NoShowAppointment (SchdAppointment newAppointment)
        {
            AutoRebookData autoRebook = new AutoRebookData ();
            if ( newAppointment == null || newAppointment.APPOINTMENTID == null )
                return;

            SelectedAppointment = newAppointment;
            autoRebook.RebookAppointment = newAppointment;
            autoRebook.IsAccessTypeChecked = true;
            autoRebook.MinimumDays = 7;
            autoRebook.MaximumDays = 30;
            IsAutoRebookChecked = false;

            AutoRebook = autoRebook;
        }

        public void ExecuteNoShowAppointmentCommand (string command)
        {
            ValidationMessage.IsValid = true;
            ValidationMessage.Title = string.Empty;
            ValidationMessage.Message = string.Empty;

            if (SelectedAppointment == null)
            {
                ValidationMessage.IsValid = false;
                ValidationMessage.Title = "Mark As NoShow Appointment";
                ValidationMessage.Message = "No appointment selected";
                return;
            }
            string errorMessage = DataAccessService.AppointmentNoShow (SelectedAppointment.APPOINTMENTID, true);

            if (errorMessage != string.Empty)
            {
                ValidationMessage.IsValid = false;
                ValidationMessage.Title = "Mark As NoShow Appointment";
                ValidationMessage.Message = errorMessage;
                return;
            }
            EventAggregator.GetEvent<RefreshCurrentScheduleEvent>().Publish(string.Empty);
            EventAggregator.GetEvent<UpdateFutureAppointmentsEvent>().Publish(null);

            if (IsAutoRebookChecked)
            {
                AutoRebook.AccessTypeID = AutoRebook.IsAccessTypeChecked ? int.Parse(SelectedAppointment.ACCESSTYPEID) : 0;
                errorMessage = DataAccessService.AutoRebook (AutoRebook);
                if (errorMessage != string.Empty)
                {
                    ValidationMessage.IsValid = false;
                    ValidationMessage.Title = "AutoRebook Appointment";
                    ValidationMessage.Message = errorMessage;
                }
            }
        }

        private void ExecutePrintLetterCommand (string command)
        {
            var args = new object[3];
            args[0] = "Patient No Show Letter";
            args[1] = SelectedAppointment.APPOINTMENTID;
            args[2] = "N";
            EventAggregator.GetEvent<DisplayReportEvent> ().Publish (args);
        }

        public bool CanExecuteNoShowAppointmentCommand (string command)
        {
            return true;
        }

        private bool CanExecutePrintLetterCommand (string command)
        {
            return true;
        }

        public void InitialErrorMessage ()
        {
            ValidationMessage.IsValid = true;
            ValidationMessage.Title = string.Empty;
            ValidationMessage.Message = string.Empty;
        }

        public void OnClose ()
        {
            View.Close ();
        }

        public DelegateCommand<string> PrintLetterCommand { get; private set; }

        #region IMarkAsNoShowPresentationModel Members

        public DelegateCommand<string> NoShowAppointmentCommand { get; private set; }

        public IMarkAsNoShowView View { get; }

        public SchdAppointment SelectedAppointment { get; set; }

        public bool IsAutoRebookChecked
        {
            get { return _isAutoRebookChecked; }
            set
            {
                _isAutoRebookChecked = value;
                OnPropertyChanged( "IsAutoRebookChecked" );
            }
        }

        public ValidationMessage ValidationMessage { get; set; }

        public AutoRebookData AutoRebook
        {
            get { return _autoRebook; }
            set
            {
                _autoRebook = value;
                OnPropertyChanged( "AutoRebook" );
            }
        }

        public string GetVistAPatientAppointmentStatus(SchdAppointment appointment)
        {
            return DataAccessService.GetVistAPatientAppointmentStatus(appointment.Patient.IEN, Convert.ToDateTime(appointment.START_TIME));
        }

        #endregion

        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
