using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using Microsoft.Practices.Composite.Regions;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.PatientAppt.Group;
using ClinSchd.Modules.PatientAppt.Services;
using Microsoft.Practices.Unity;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Modules.PatientAppt.Controllers;
using Microsoft.Practices.Composite.Presentation.Commands;
using System.Collections.ObjectModel;
using System.Runtime.Remoting.Metadata.W3cXsd2001;
using ClinSchd.Modules.PatientAppt.Group.Factories;

namespace ClinSchd.Modules.PatientAppt.Controllers
{
    public class PatientApptController : IPatientApptController
    {
        private readonly IUnityContainer container;
        private readonly IEventAggregator eventAggregator;
        private readonly IDataAccessService dataAccessService;
        private Patient _patient;
        public PatientApptController (IUnityContainer container, IDataAccessService dataAccessService,
                                IEventAggregator eventAggregator)
        {
            this.container = container;
            this.eventAggregator = eventAggregator;
            this.dataAccessService = dataAccessService;
        }

        public void Run()
        {
            this.eventAggregator.GetEvent<ViewAppointmentEvent> ().Subscribe (LaunchViewAppointmentDialog, ThreadOption.UIThread, true);
            this.eventAggregator.GetEvent<AddNewAppointmentEvent> ().Subscribe (LaunchNewAppointmentDialog, ThreadOption.UIThread, true);
            this.eventAggregator.GetEvent<AddWalkInAppointmentEvent> ().Subscribe (LaunchWalkinAppointmentDialog, ThreadOption.UIThread, true);
            this.eventAggregator.GetEvent<EditAppointmentEvent> ().Subscribe (LaunchEditAppointmentDialog, ThreadOption.UIThread, true);
            this.eventAggregator.GetEvent<PatientSelectedEvent>().Subscribe(PatientSelected, ThreadOption.UIThread, false);
        }
        private void PatientSelected(ClinSchd.Infrastructure.Models.Patient selectedPatient)
        {
            _patient = selectedPatient;
        }
        private void LaunchViewAppointmentDialog (SchdAppointment appointment)
        {
            IPatientApptService PatientApptService = container.Resolve<IPatientApptService> ();
            IGroupPresentationModel Model = container.Resolve<IGroupPresentationModel> ();
            Model.GetPatientInformation (appointment.PATIENTID);
            Model.AppointmentRequest = appointment.AppointmentRequest;
            Model.GetViewAppointment (appointment);
            Model.Controller = this;
            Model.Command = "View Appointment";
            Model.PaneTitle = "View Appointment";
            Model.IsNoteEnabled = false;
            PatientApptService.ShowDialog (Model.View, Model, () => Model.OnClose ());
        }

        private void LaunchNewAppointmentDialog (SchdAppointment appointment)
        {
            IPatientApptService patientApptService = container.Resolve<IPatientApptService> ();
            IGroupPresentationModel model = container.Resolve<IGroupPresentationModel> ();
            bool result = false;
            LaunchNewAppointmentDialog(appointment, (PatientApptService)patientApptService, (GroupPresentationModel)model, ref result);
        }

        public void LaunchNewAppointmentDialog(SchdAppointment appointment, PatientApptService patientApptService, GroupPresentationModel model, ref bool result)
        {
            model.Controller = this;
            string errorMessage = string.Empty;
            bool bOverbookAppointment;
            bool isTelehealth;
            model.AppointmentRequest = appointment.AppointmentRequest;

            if (!this.dataAccessService.CanAddNewAppointmentPreCheck("New Appointment", appointment, out bOverbookAppointment, out isTelehealth))
                return;

            model.GetNewAppointment(appointment, bOverbookAppointment);
            model.Command = "New Appointment";
            model.PaneTitle = "New Appointment";
            model.IsNoteEnabled = true;
            patientApptService.ShowDialog(model.View, model, () => model.OnClose());

            result = (model.View.DialogResult.HasValue ? model.View.DialogResult.Value : false);

            if (result)
                appointment.APPOINTMENTID = model.AppointmentData.AppointmentID;
        }

        private void LaunchEditAppointmentDialog (SchdAppointment appointment)
        {
            IPatientApptService patientApptService = container.Resolve<IPatientApptService> ();
            IGroupPresentationModel model = container.Resolve<IGroupPresentationModel> ();
            model.Controller = this;
            model.AppointmentRequest = appointment.AppointmentRequest;
            model.GetEditAppointment (appointment);
            model.Command = "Edit Appointment";
            model.PaneTitle = "Edit Appointment";
            model.IsNoteEnabled = true;
            patientApptService.ShowDialog (model.View, model, () => model.OnClose ());
        }

        private void LaunchWalkinAppointmentDialog (SchdAppointment appointment)
        {
            IPatientApptService PatientApptService = container.Resolve<IPatientApptService> ();
            IGroupPresentationModel Model = container.Resolve<IGroupPresentationModel> ();

            if (appointment.SelectedResource != null)
                dataAccessService.RefreshAvailabilitySchedule(appointment.SelectedResource, Convert.ToDateTime(appointment.START_TIME), Convert.ToDateTime(appointment.END_TIME));
            else
                dataAccessService.RefreshAvailabilitySchedule(appointment.RESOURCENAME, Convert.ToDateTime(appointment.START_TIME), Convert.ToDateTime(appointment.END_TIME));

            Model.Controller = this;
            Model.AppointmentRequest = appointment.AppointmentRequest;
            if (Convert.ToDateTime(appointment.START_TIME).Date > DateTime.Today.Date)
            {
                Model.View.AlertUser ("You cannot create a walk-in appointment for a date in the future.\n Select today's date and try again.", "Walk-In Appointment");
            } 
            else
            {
                // Check for conflicts, warning only, proceed even if conflicts exist
                bool isTelehealth;
                this.dataAccessService.CheckConflictAppointment("New Appointment", appointment.RESOURCENAME, appointment.PATIENTID, appointment.PATIENTNAME, appointment.START_TIME, appointment.END_TIME, out isTelehealth);

                // We check slots available to determine if the new walkin appointment will be an overbook, No messages shold be displayed
                var errorMessage = this.dataAccessService.CheckSlotsAvailable (appointment.START_TIME, appointment.END_TIME, appointment.RESOURCENAME);

                if (isTelehealth)
                    // Because this is Telehealth there is a conflicting appointment, add 1 minute to the start time to appease VistA
                    appointment.START_TIME = Convert.ToDateTime(appointment.START_TIME).AddMinutes(1).ToString("MMM d,yyyy hh:mm");

                Model.GetWalkInAppointment(appointment, errorMessage.Contains(" overbook "));
                Model.Command = "WalkIn Appointment";
                Model.PaneTitle = "Walk In Appointment";
                Model.IsNoteEnabled = true;
                PatientApptService.ShowDialog(Model.View, Model, () => Model.OnClose());
            }
        }
    }
}
