using System.Collections.Generic;
using System.ComponentModel;

using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Commands;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.PatientSelection.Services;
using Microsoft.Practices.Unity;

namespace ClinSchd.Modules.PatientSelection.PatientSelection
{
    public class PatientSelectionPresentationModel : IPatientSelectionPresentationModel, INotifyPropertyChanged
    {
        private readonly IPatientSelectionService _patientSelectionService;
        private readonly IDataAccessService _dataAccessService;
        private readonly IEventAggregator _eventAggregator;
        private string _searchString;
        private IList<Patient> _patientList;
        private Patient _selectedPatient;

        [Dependency]
        public Factory<Patient> PatientFactory { get; set; }

        public PatientSelectionPresentationModel(
            IPatientSelectionView view,
            IPatientSelectionService patientSelectionService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            View = view;
            View.Model = this;
            _eventAggregator = eventAggregator;
            _patientSelectionService = patientSelectionService;
            _dataAccessService = dataAccessService;

            NewAppointmentCommand = new DelegateCommand<string> (ExecuteNewAppointmentCommand, CanExecuteNewAppointmentCommand);
        }

        public void SearchStringChanged(string newSearchString)
        {
            var newPatientList = _dataAccessService.GetPatients(newSearchString == string.Empty ? null : newSearchString, Keys.MaxPatientsToReturn);
            SearchString = newSearchString;
            PatientList = newPatientList;
        }
        
        public void OnClose()
        {
            View.Close();
        }

        #region IPatientSelectionPresentationModel Members

        public IPatientSelectionView View { get; private set; }
        public SchdAppointment SelectedAppointment { get; set; }
        public string Command { get; set; }
        public DelegateCommand<string> NewAppointmentCommand { get; private set; }

        public void ExecuteNewAppointmentCommand (string patientIEN)
        {
            if (SelectedPatient != null) {
                // Forward the sender's event
                if (Command == "New Appointment") {
                    _eventAggregator.GetEvent<AddNewAppointmentEvent> ().Publish (SelectedAppointment);
                } else if (Command == "WalkIn Appointment") {
                    _eventAggregator.GetEvent<AddWalkInAppointmentEvent> ().Publish (SelectedAppointment);
                }

                // Notify anyone who cares that a patient has been selected
                _eventAggregator.GetEvent<PatientSelectedEvent> ().Publish (SelectedPatient);
            }

            if (Command == "View Patient Appointments") {
                //pop up report dialog to display all appointments for the selected patient
            } else {
                _eventAggregator.GetEvent<ViewAppointmentEvent> ().Publish (SelectedAppointment);
            }
        }

        public bool CanExecuteNewAppointmentCommand (string patientIEN)
        {
            return true;
        }

        public string SearchString
        {
            get { return _searchString; }
            set
            {
                if (_searchString != value)
                {
                    _searchString = value;
                    OnPropertyChanged("SearchString");
                }
            }
        }

        public IList<Patient> PatientList
        {
            get { return _patientList; }
            private set
            {
                if (_patientList != value)
                {
                    _patientList = value;
                    OnPropertyChanged("PatientList");
                }
            }
        }

        public Patient SelectedPatient
        {
            get { return _selectedPatient; }
            set
            {
                if (_selectedPatient != value && value != null && value.IEN != null)
                {
                    _selectedPatient = value;
                    OnPropertyChanged("SelectedPatient");
                }
            }
        }

        #endregion

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
