﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using ClinSchd.Infrastructure.Interfaces;
using Telerik.Windows.Controls;
using MessageBox = System.Windows.Forms.MessageBox;

namespace ClinSchd.Modules.Reports.Reports
{
    /// <summary>
    /// Interaction logic for ClinicReportView.xaml
    /// </summary>
    public partial class ClinicReportView
    {
        public ClinicReportViewModel ViewModel { get; }

        private bool _isClinicSearch;
        private bool _allRecordsDisplayed;

        public ClinicReportView(IDataAccessService dataAccessService)
        {
            ViewModel = new ClinicReportViewModel(dataAccessService, this);
            DataContext = ViewModel;
            InitializeComponent();
            _allRecordsDisplayed = false;
        }

        private void Exit_Click(object sender, RoutedEventArgs e)
        {
            CloseWindow();
        }

        private void CloseWindow()
        {
            var window = Window.GetWindow(this);
            if (window != null)
                window.Close();
        }

        private void ClinicReportView_Unloaded(object sender, RoutedEventArgs e)
        {
            CloseWindow();
        }

        private void ExportClinicReport_Click(object sender, RoutedEventArgs e)
        {
            RadGridView grid = ClinicReportGrid;
            IList<ClinicRecord> reportData = new List<ClinicRecord>();

            foreach (var item in grid.Items)
            {
                var req = item as ClinicRecord;
                reportData.Add(req);
            }

            /* The AdditionalProvs field contains a delimited string of Provider Names, this can be lengthy, so we insure Excel can handle the exported data */
            foreach (var item in reportData)
                item.AdditionalProvs = item.AdditionalProvs.Length > 32767 ? item.AdditionalProvs.Substring(0, 32763) + "..." : item.AdditionalProvs;

            ExportReportToExcel.ExportReport("Clinic Report", reportData);
        }

        private void ClinicReport_DistinctValuesLoading(object sender, Telerik.Windows.Controls.GridView.GridViewDistinctValuesLoadingEventArgs e)
        {
            e.ItemsSource = ((RadGridView) sender).GetDistinctValues(e.Column, true, 700);
        }

        private void SearchClinic_Click(object sender, RoutedEventArgs e)
        {
            if (TxtClinicSearch.Text.Length < Infrastructure.Keys.ClinicResourceSearchCharMin)
            {
                MessageBox.Show(string.Format("Please provide at least {0} characters to search for Clinics.", Infrastructure.Keys.ClinicResourceSearchCharMin));
                return;
            }
            if (ViewModel != null)
            {
                if (string.IsNullOrEmpty(TxtClinicSearch.Text))
                    _allRecordsDisplayed = false;
                else  
                    _isClinicSearch = true;
                ViewModel.SearchStr = TxtClinicSearch.Text;
                ViewModel.LastSub = string.Empty;
                ViewModel.GetClinicData("Searching");
                TxtClinicSearch.Focus();
                Keyboard.Focus(TxtClinicSearch);
            }
        }

        private void ClinicReport_ScrollChanged(object sender, RoutedEventArgs e)
        {
            var scrollViewer = e.OriginalSource as ScrollViewer;
            if (scrollViewer != null && scrollViewer.VerticalOffset == scrollViewer.ScrollableHeight)
            {
                if (ViewModel != null)
                {
                    if (string.IsNullOrEmpty(ViewModel.LastSub) && (_allRecordsDisplayed || ClinicReportGrid.Items.Count <= ViewModel.PageSize))
                    {
                        Mouse.OverrideCursor = Cursors.Arrow;
                        return;
                    }

                    if (string.IsNullOrEmpty(ViewModel.LastSub) && !_isClinicSearch)
                    {
                        MessageBox.Show("All records have been displayed.");
                        _allRecordsDisplayed = true;
                        Mouse.OverrideCursor = Cursors.Arrow;
                        return;
                    }
                    _isClinicSearch = false;
                    ViewModel.GetClinicData("Loading");
                }
            }

            Mouse.OverrideCursor = Cursors.Arrow;
        }

        private void Clear_Click(object sender, RoutedEventArgs e)
        {
            TxtClinicSearch.Text = string.Empty;
            ViewModel.SearchStr = string.Empty;
            ViewModel.LastSub = string.Empty;
            _isClinicSearch = false;
            ViewModel.GetClinicData("Loading");
        }

        private void ClinicReportView_Loaded(object sender, RoutedEventArgs e)
        {
            Infrastructure.Keys.SetRenderingMode(this);
            TxtClinicSearch.Focus();
            Keyboard.Focus(TxtClinicSearch);
        }

        private void UserControl_Initialized(object sender, EventArgs e)
        {
            ViewModel.GetClinicData("Loading");
            Btn_Exit.Focus();
            Keyboard.Focus(Btn_Exit);
        }

        private void UserControl_PreviewKeyUp(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                Key key = e.Key == Key.System ? e.SystemKey : e.Key;

                switch (key)
                {
                    case Key.A:
                        Keyboard.Focus(TxtClinicSearch);
                        break;
                    case Key.S:
                        Keyboard.Focus(BtnClinicSearch);
                        break;
                    case Key.C:
                        Keyboard.Focus(BtnClearReport);
                        break;
                    case Key.E:
                        Keyboard.Focus(BtnExportClinicReport);
                        break;
                    case Key.X:
                        Keyboard.Focus(Btn_Exit);
                        break;
                    case Key.G:
                        if (ClinicReportGrid.Items.Count > 0)
                        {
                            ClinicReportGrid.SelectedItem = ClinicReportGrid.Items[0];
                            ClinicReportGrid.ScrollIntoView(ClinicReportGrid.SelectedItem);
                            ClinicReportGrid.Focus();
                        }
                        break;
                }
            }
        }

        private void ClinicReportGrid_DataLoaded(object sender, EventArgs e)
        {
            if (ClinicReportGrid.Items.Count > 0)
            {
                ClinicReportGrid.SelectedItem = ClinicReportGrid.Items[0];
                ClinicReportGrid.ScrollIntoView(ClinicReportGrid.SelectedItem);
                ClinicReportGrid.Focus();
            }
        }
    }
}
