﻿using System;
using System.Collections.ObjectModel;
using System.Data;
using System.Windows;
using System.Windows.Input;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;

namespace ClinSchd.Modules.Reports.Reports
{
    public class ClinicReportViewModel : AsyncValidatableModel
    {
        private readonly IDataAccessService _dataAccessService;
        private ObservableCollection<ClinicRecord> _clinicRecords = new ObservableCollection<ClinicRecord>();
        private const int INITIAL_PAGE_SIZE = 100;
        public int PageSize = INITIAL_PAGE_SIZE;
        private bool _isLoadingData;

        public ClinicReportViewModel(IDataAccessService dataAccessService, ClinicReportView view)
        {
            _dataAccessService = dataAccessService;
            View = view;
        }

        public ClinicReportViewModel ViewModel { get; set; }

        public ClinicReportView View { get; }

        public ObservableCollection<ClinicRecord> ClinicRecords
        {
            get { return _clinicRecords; }
            set { _clinicRecords = value; }
        }

        public string LastSub { get; set; }
         
        public string SearchStr { get; set; }

        public bool IsExportButtonEnabled
        {
            get { return !_isLoadingData && (ClinicRecords.Count > 0); }
        }

        public void GetClinicData(string command)
        {
            _isLoadingData = true;
            NotifyPropertyChanged("IsExportButtonEnabled");
            var lastSub = LastSub;
            var searchStr = SearchStr;

            if (string.IsNullOrEmpty(LastSub))
                _clinicRecords.Clear();

            View.AsyncWrapper.ShowBusyIndicator(command + " Clinic Data");

            IsAsync = true;
            DoWorkAsync(
                (s, args) =>
                {
                    lock (args)
                    {
                        PageSize = (string.IsNullOrEmpty(searchStr) ? INITIAL_PAGE_SIZE : Keys.ClinicResourceSearchMaxRecords);
                        args.Result = _dataAccessService.GetClinicData(PageSize, ref lastSub, ref searchStr);
                    }
                },
                (s, args) =>
                {
                    lock (args)
                    {
                        try
                        {
                            if (args.Result != null)
                            {
                            }
                        }
                        finally
                        {
                            DataTable dt = (DataTable)args.Result;
                            LastSub = lastSub;

                            try
                            {
                                foreach (DataRow row in dt.Rows)
                                {
                                    var clinicRec = new ClinicRecord
                                    {
                                        Clinic   = row["CLINNAME"].ToString(),
                                        Division = row["DIVNAME"].ToString(),
                                        StopCode = row["STOP_CODE_ID"].ToString(),
                                        Service  = row["SERVICE"].ToString()
                                    };


                                    if (!string.IsNullOrEmpty(row["DATE"].ToString()))
                                    {
                                        string[] dates = row["DATE"].ToString().Split('@');
                                        clinicRec.CreateDT = DateTime.Parse(dates[0]);
                                    }
                                    DateTime inactivateDt;
                                    DateTime reactivateDt;
                                    if (DateTime.TryParse(row["INACTIVATE_DT"].ToString(), out inactivateDt))
                                    {
                                        clinicRec.InactiveDT = inactivateDt;
                                    }
                                    if (DateTime.TryParse(row["REACTIVATE_DT"].ToString(), out reactivateDt))
                                    {
                                        clinicRec.ReactiveDT = reactivateDt;
                                    }
                                    clinicRec.AppType = row["DEF-APPT-TYPE_NAME"].ToString();
                                    clinicRec.DefaultProv = "";
                                    clinicRec.AdditionalProvs = "";
                                    string providers = row["PROVIDERS"].ToString();

                                    if (!string.IsNullOrEmpty(providers))
                                    {
                                        string[] providerArray = providers.Split('|');
                                        string[] stringSeparators = { ";;" };
                                        string providerNames = "";
                                        string defProviderNames = "";

                                        foreach (string provider in providerArray)
                                        {
                                            string[] providerRecord = provider.Split(stringSeparators, StringSplitOptions.None);

                                            if (providerRecord[2] == "YES")
                                                defProviderNames += (string.IsNullOrEmpty(defProviderNames) ? "" : ";") + providerRecord[1];
                                            else
                                                providerNames += (string.IsNullOrEmpty(providerNames) ? "" : ";") + providerRecord[1];
                                        }

                                        clinicRec.DefaultProv = defProviderNames;
                                        clinicRec.AdditionalProvs = providerNames;
                                    }

                                    var duration = row["APPTLEN"].ToString();
                                    if (string.IsNullOrEmpty(duration))
                                        duration = "0";
                                    clinicRec.ApptLength = int.Parse(duration);
                                    clinicRec.Variable = row["VAPPTLEN"].ToString() == "V" ? "YES" : "";
                                    int maxDays;
                                    if (int.TryParse(row["MAXDAYS"].ToString(), out maxDays))
                                    {
                                        clinicRec.MaxDays = maxDays;
                                    }
                                    _clinicRecords.Add(clinicRec);
                                }

                                if ((PageSize != INITIAL_PAGE_SIZE) && (_clinicRecords.Count == PageSize) && !string.IsNullOrEmpty(LastSub))
                                {
                                    Keys.AlertUser(View, $"The current search criteria returns more than {PageSize} records.\nPlease refine your search or scroll to the bottom to review other clinic data.", "Clinic Search Criteria");
                                    View.TxtClinicSearch.Focus();
                                    Keyboard.Focus(View.TxtClinicSearch);
                                }
                            }
                            catch (Exception ex)
                            {
                                Mouse.OverrideCursor = Cursors.Arrow;
                                MessageBox.Show(ex.Message, "Clinic Data Processing Error");
                                throw;
                            }

                            _isLoadingData = false;
                            NotifyPropertyChanged("IsExportButtonEnabled");
                            View.AsyncWrapper.HideBusyIndicator();
                        }
                    }
                },
                false
            );
        }
    }
}
