﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Windows;
using System.Windows.Input;

using System.Xml;
using Microsoft.Practices.Composite.Presentation.Commands;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;

namespace ClinSchd.Modules.Reports.Reports
{
    public class EncounterReportDashboardViewModel : INotifyPropertyChanged
    {
        #region constants

        private const string REPORT_FILE_NAME = "SDEC_VSE.XML";
        private const string JAVA_FILE_NAME = "SDEC_VSE_ReportConsole.jar";

        #endregion constants

        #region fields

        private Cursor _oldCursor;
        private readonly IDataAccessService _dataAccessService;

        private IList<DateAggregateFilter> _dateAgregatesList;
        private IList<DateFilter> _dateFilterList;
        private IList<ReportTypeFilter> _reportTypesList;

        private DateAggregateFilter _selectedDateAgregate;
        private ReportTypeFilter _selectedReportType;
        private DateFilter _selectedDateFilter;

        private readonly EncounterReportDashboardView _view;
        private FilterRoot _filterRoot;

        private Visibility _filtersVisibility = Visibility.Collapsed;
        private Visibility _waitVisibility = Visibility.Visible;
        private string _waitText = string.Empty;

        #endregion fields

        #region constructor

        public EncounterReportDashboardViewModel(IDataAccessService dataAccessService, EncounterReportDashboardView view)
        {
            _view = view;
            _dataAccessService = dataAccessService;
            SubmitCommand = new DelegateCommand<string>(ExecuteSubmitCommand, CanExecuteSubmitCommand);
            CloseCommand = new DelegateCommand<string>(ExecuteCloseCommand, CanExecuteCloseCommand);
            WaitShow("Loading Filter Data, Please Wait");
            LoadFilters();
        }

        #endregion constructor

        #region Command Members

        public DelegateCommand<string> CloseCommand { get; private set; }

        public DelegateCommand<string> SubmitCommand { get; }

        public bool CanExecuteCloseCommand(string title) { return true; }

        public bool CanExecuteSubmitCommand(string title) { return SelectedDateFilter != null; }

        public void ExecuteCloseCommand(string title) { _view.CloseWindow(); }

        public void ExecuteSubmitCommand(string title)
        {
            try
            {
                WaitShow("Opening Report Console");
                LoadReportData();
            }
            catch (Exception ex)
            {
                string errorMessage = ex.Message;

                if (ex.InnerException != null)
                {
                    Exception inner = ex.InnerException;

                    while (inner != null)
                    {
                        if (inner.Message.Contains("ADTDataRepository."))
                            if (inner.InnerException != null)
                            {
                                errorMessage = inner.InnerException.Message;
                                break;
                            }

                        errorMessage = inner.Message;
                        inner = inner.InnerException;
                    }
                }

                MessageBox.Show("A problem was encountered while loading the Management Report data.\r\n" + errorMessage, "Management Report Error");
                WaitHide();
            }
        }

        #endregion Command Members

        #region data bindings

        public Visibility FiltersVisibility
        {
            get { return _filtersVisibility; }
            set
            {
                _filtersVisibility = value;
                NotifyPropertyChanged("FiltersVisibility");
            }
        }

        public Visibility WaitVisibility
        {
            get { return _waitVisibility; }
            set
            {
                _waitVisibility = value;
                NotifyPropertyChanged("WaitVisibility");
            }
        }

        public string WaitText
        {
            get { return _waitText; }
            set
            {
                _waitText = value;
                NotifyPropertyChanged("WaitText");
            }
        }

        public IList<ReportTypeFilter> ReportTypesList
        {
            get { return _reportTypesList; }
            set
            {
                _reportTypesList = value;
                NotifyPropertyChanged("ReportTypesList");

                if ( _reportTypesList != null && _reportTypesList.Count == 1 )
                    SelectedReportType = _reportTypesList[0];
            }
        }

        public ReportTypeFilter SelectedReportType
        {
            get { return _selectedReportType; }
            set
            {
                SelectedDateAgregate = null;
                _selectedReportType = value;
                NotifyPropertyChanged("SelectedReportType");

                DateAgregatesList = _selectedReportType == null ? null : _selectedReportType.SubFilterList;
            }
        }

        public IList<DateAggregateFilter> DateAgregatesList
        {
            get { return _dateAgregatesList; }
            set
            {
                _dateAgregatesList = value;
                NotifyPropertyChanged("DateAgregatesList");

                if ( _dateAgregatesList != null && _dateAgregatesList.Count == 1 )
                    SelectedDateAgregate = _dateAgregatesList[0];
            }
        }

        public DateAggregateFilter SelectedDateAgregate
        {
            get { return _selectedDateAgregate; }
            set
            {
                SelectedDateFilter = null;
                _selectedDateAgregate = value;
                NotifyPropertyChanged("SelectedDateAgregate");

                DateFilterList = _selectedDateAgregate == null ? null : _selectedDateAgregate.SubFilterList;
            }
        }

        public IList<DateFilter> DateFilterList
        {
            get { return _dateFilterList; }
            set
            {
                _dateFilterList = value;
                NotifyPropertyChanged("DateFilterList");

                if ( _dateFilterList != null && _dateFilterList.Count == 1 )
                    SelectedDateFilter = _dateFilterList[0];
            }
        }

        public DateFilter SelectedDateFilter
        {
            get { return _selectedDateFilter; }
            set
            {
                if (value != _selectedDateFilter)
                {
                    _selectedDateFilter = value;
                    NotifyPropertyChanged("SelectedDateFilter");
                    SubmitCommand.RaiseCanExecuteChanged();
                }
            }
        }

        #endregion data bindings

        #region private members

        private void LoadReportData()
        {
            string reportType;
            string dateAggregate;
            string dateFilter;

            switch (SelectedDateFilter.Version)
            {
                case 1:
                    reportType = SelectedReportType.Value;
                    dateAggregate = SelectedDateAgregate.Value;
                    dateFilter = SelectedDateFilter.Value;
                    break;
                case 2:
                    reportType = SelectedReportType.Value;
                    dateAggregate = SelectedDateFilter.BeginDate;
                    dateFilter = SelectedDateFilter.EndDate;
                    break;
                default:
                    throw new Exception(SelectedDateFilter.Version.ToString() + " is an unknown xml version");
            }

            _dataAccessService.GetEncounterDataReportData(
                reportType,
                dateAggregate,
                dateFilter,
                (s, args) =>
                {
                    lock (args)
                    {
                        ProcessReportData(args.Result);
                    }
                }
            );
        }

        private void ProcessReportData(object results)
        {
            string errorMessageType = "reading";

            try
            {
                try
                {
                    if (results.GetType() == typeof(RPCException))
                        throw (RPCException)results;

                    if (results.GetType() == typeof(Exception))
                        throw (Exception)results;

                    errorMessageType = "processing";
                    var targetPath = Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData);
                    targetPath += "\\" + Keys.UsersTempFolder;
                    if (!Directory.Exists(targetPath))
                        throw new Exception("The user's VistA Scheduler temporary folder does not exist");

                    var targetFile = Path.Combine(targetPath, REPORT_FILE_NAME);
                    var jarFile = Path.Combine(targetPath, JAVA_FILE_NAME);
                    File.WriteAllText(targetFile, (string)results);

                    if (!File.Exists(targetFile))
                        throw new Exception("The data file \"" + targetFile + "\" was not created.");

                    if (!File.Exists(jarFile))
                        throw new Exception("The report file  \"" + jarFile + "\" does not exist.");
                    var args = "\"" + targetPath + "\"";
                    System.Diagnostics.Process.Start(jarFile, args);
                }
                finally
                {
                    System.Threading.Thread.Sleep(2000);
                    WaitHide();
                }
            }
            catch (Exception ex)
            {
                string errorMessage = ex.Message;

                if (ex.InnerException != null)
                {
                    Exception inner = ex.InnerException;

                    while (inner != null)
                    {
                        if (inner.Message.Contains("ADTDataRepository."))
                            if (inner.InnerException != null)
                            {
                                errorMessage = inner.InnerException.Message;
                                break;
                            }

                        errorMessage = inner.Message;
                        inner = inner.InnerException;
                    }
                }

                MessageBox.Show(string.Format("A problem was encountered while {0} the Management Report data.\r\n{1}", errorMessageType, errorMessage), "Management Report Error");
                //this.ExecuteCloseCommand(String.Empty);
            }
        }

        private void LoadFilters()
        {
            _dataAccessService.GetEncounterDataFilterData(
            (s, args) =>
            {
                lock (args)
                {
                    ProcessFilters(args.Result);
                }
            });
        }

        private void ProcessFilters(object results)
        {

            try
            {
                try
                {
                    if (results.GetType() == typeof(Exception))
                        throw (Exception)results;

                    string filters = (string)results;
                    XmlDocument xmlFilterDoc = new XmlDocument();
                    xmlFilterDoc.LoadXml(filters);
                    XmlNode vaNode = xmlFilterDoc.DocumentElement;

                    if (vaNode == null)
                        throw new Exception("The VAFacility root node was not found in the filter xml string.");

                    _filterRoot = new FilterRoot(vaNode);
                    ReportTypesList = _filterRoot == null ? null : _filterRoot.SubFilterList;
                }
                finally
                {
                    WaitHide();
                }
            }
            catch (Exception ex)
            {
                string errorMessage = ex.Message;

                if (ex.InnerException != null)
                {
                    Exception inner = ex.InnerException;

                    while (inner != null)
                    {
                        if (inner.Message.Contains("ADTDataRepository."))
                            if (inner.InnerException != null)
                            {
                                errorMessage = inner.InnerException.Message;
                                break;
                            }

                        errorMessage = inner.Message;
                        inner = inner.InnerException;
                    }
                }

                MessageBox.Show("A problem was encountered while loading the Management Report filters.\r\n" + errorMessage, "Management Report Error");
                ExecuteCloseCommand(string.Empty);
            }
        }

        public void WaitHide()
        {
            FiltersVisibility = Visibility.Visible;
            WaitVisibility = Visibility.Collapsed;
            WaitText = string.Empty;
            _view.Refresh();

            if (_oldCursor != null)
            {
                Mouse.OverrideCursor = _oldCursor;
                _oldCursor = null;
            }
            else
                Mouse.OverrideCursor = Cursors.Arrow;
        }

        private void WaitShow(string waitText)
        {
            _oldCursor = Mouse.OverrideCursor;
            Mouse.OverrideCursor = Cursors.Wait;

            if (!string.IsNullOrEmpty(waitText))
            {
                FiltersVisibility = Visibility.Collapsed;
                WaitVisibility = Visibility.Visible;
                WaitText = waitText;
                _view.Refresh();
            }
        }

        #endregion private members

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        public void NotifyPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion INotifyPropertyChanged Members

        #region internal filter classes

        public abstract class BaseFilter
        {
            protected XmlNode _node;

            public BaseFilter(XmlNode node)
            {
                _node = node;
                LoadChildren();
            }

            public string Name { get { return GetName(); } }

            public XmlNode Node { get { return _node; } }

            public string Value { get { return GetValue(); } }

            protected abstract string GetName();

            protected abstract string GetValue();

            protected abstract void LoadChildren();

            protected void ValidateNode(string nodeName, string className)
            {
                if (_node.Name != nodeName)
                    throw new Exception(_node.Name + " is an invalid XML Node for the " + className + " class.");
            }
        }

        public class FilterRoot : BaseFilter
        {
            protected IList<ReportTypeFilter> List;

            public FilterRoot(XmlNode node) : base(node)
            {
                ValidateNode("VAFacility", GetType().ToString());
            }

            public IList<ReportTypeFilter> SubFilterList { get { return List; } }

            protected override string GetName()
            {
                return _node == null ? "" : _node.Attributes["Name"].Value;
            }

            protected override string GetValue()
            {
                return _node == null ? "" : _node.Attributes["ID"].Value;
            }

            protected override void LoadChildren()
            {
                List = new List<ReportTypeFilter>();

                if (_node.HasChildNodes)
                    foreach (XmlNode child in _node.ChildNodes)
                        // Ignore any tags other than ReportType
                        if (child.Name == "ReportType")
                            // Ignore if the node has no children
                            if (child.HasChildNodes)
                                List.Add(new ReportTypeFilter(child));
            }
        }

        public class ReportTypeFilter : BaseFilter
        {
            protected IList<DateAggregateFilter> List;

            public ReportTypeFilter(XmlNode node) : base(node)
            {
                ValidateNode("ReportType", GetType().ToString());
            }

            public IList<DateAggregateFilter> SubFilterList { get { return List; } }

            protected override string GetName()
            {
                return _node == null ? "" : _node.Attributes["ReportType"].Value;
            }

            protected override string GetValue()
            {
                return _node == null ? "" : Name.Substring(0, 1).ToUpper();
            }

            protected override void LoadChildren()
            {
                List = new List<DateAggregateFilter>();

                if (_node.HasChildNodes)
                    foreach (XmlNode child in _node.ChildNodes)
                        if (child.HasChildNodes)
                            List.Add(new DateAggregateFilter(child));
            }
        }

        public class DateAggregateFilter : BaseFilter
        {
            protected IList<DateFilter> List;

            public DateAggregateFilter(XmlNode node) : base(node)
            {
                ValidateNode("DateAggregate", GetType().ToString());
            }

            public IList<DateFilter> SubFilterList { get { return List; } }

            protected override string GetName()
            {
                return _node == null ? "" : _node.Attributes["DateAgg"].Value;
            }

            protected override string GetValue()
            {
                return _node == null ? "" : Name.Substring(0, 1).ToUpper();
            }

            protected override void LoadChildren()
            {
                List = new List<DateFilter>();

                if (_node.HasChildNodes)
                    foreach (XmlNode child in _node.ChildNodes)
                        List.Add(new DateFilter(child));
            }
        }

        public class DateFilter : BaseFilter
        {
            public int Version { get { return Node.Attributes["BeginDate"] == null && Node.Attributes["EndDate"] == null ? 1 : 2; } }

            public string ActivityDate { get; private set; }

            public string DateAggregate { get { return _node == null ? "" : _node.ParentNode.Attributes["DateAgg"].Value.Substring(0, 1); } }

            public DateFilter(XmlNode node) : base(node)
            {
                ValidateNode("Date", GetType().ToString());
            }

            protected override string GetName()
            {
                switch (Version)
                {
                    case 1:
                        if (DateAggregate == "Y")
                            return _node == null ? "" : _node.Attributes["DateName"].Value.Substring(6, 4);
                        return _node == null ? "" : _node.Attributes["DateName"].Value;

                    case 2:
                        return BeginDate + "-" + EndDate;

                    default:
                        throw new Exception(Version + " is an unknown xml version");
                }
            }

            protected override string GetValue()
            {
                if (_node == null)
                    return "";

                switch (DateAggregate)
                {
                    case "Y":
                        return "01/01/" + Name;
                    case "Q":
                        string[] qParts = Name.Split('-');

                        if (qParts.Length == 2)
                            switch (qParts[0])
                            {
                                case "Q1":
                                    return "01/01/" + qParts[1];
                                case "Q2":
                                    return "04/01/" + qParts[1];
                                case "Q3":
                                    return "07/01/" + qParts[1];
                                case "Q4":
                                    return "10/01/" + qParts[1];
                                default:
                                    throw new Exception("\"" + qParts[0] + "\" is an unknown Date Name for a Quarter Date Filter.");
                            }
                        else
                            throw new Exception("\"" + Name + "\" is an invalid Quarter Aggregate Date Name value.");
                    case "M":
                        string[] mParts = Name.Split(' ');

                        if (mParts.Length == 2)
                            switch (mParts[0])
                            {
                                case "Jan":
                                    return "01/01/" + mParts[1];
                                case "Feb":
                                    return "02/01/" + mParts[1];
                                case "Mar":
                                    return "03/01/" + mParts[1];
                                case "Apr":
                                    return "04/01/" + mParts[1];
                                case "May":
                                    return "05/01/" + mParts[1];
                                case "Jun":
                                    return "06/01/" + mParts[1];
                                case "Jul":
                                    return "07/01/" + mParts[1];
                                case "Aug":
                                    return "08/01/" + mParts[1];
                                case "Sep":
                                    return "09/01/" + mParts[1];
                                case "Oct":
                                    return "10/01/" + mParts[1];
                                case "Nov":
                                    return "11/01/" + mParts[1];
                                case "Dec":
                                    return "12/01/" + mParts[1];
                                default:
                                    throw new Exception("\"" + mParts[0] + "\" is an unknown Date Name for a Month Date Filter.");
                            }
                        else
                            throw new Exception("\"" + Name + "\" is an invalid Month Aggregate Date Name value.");
                    case "W":
                    case "D":
                        return Name;
                    default:
                        throw new Exception("\"" + DateAggregate + "\" is an invalid Date Aggregate value.");
                }
            }

            protected override void LoadChildren() { }

            public string BeginDate
            {
                get
                {
                    switch (Version)
                    {
                        case 1:
                            return GetValue();
                        case 2:
                            return Node.Attributes["BeginDate"].Value;
                        default:
                            throw new Exception(Version + " is an unknown xml version");
                    }
                }
            }

            public string EndDate
            {
                get
                {
                    switch (Version)
                    {
                        case 1:
                            return GetValue();
                        case 2:
                            return Node.Attributes["EndDate"].Value;
                        default:
                            throw new Exception(Version + " is an unknown xml version");
                    }
                }
            }
        }

        #endregion internal filter classes
    }
}
