using System.Collections.Generic;
using System.Linq;
using Microsoft.Practices.Composite.Presentation.Commands;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using Microsoft.Practices.Composite.Regions;
using System.Windows.Controls;
using System.Windows;
using ClinSchd.Modules.Reports.Helper_Classes;

namespace ClinSchd.Modules.Reports.Reports
{
    public class ReportsPresentationModel : AsyncValidatableModel, IReportsPresentationModel
    {
        private readonly IRegionManager _regionManager;
        private readonly IDataAccessService _dataAccessService;


        public ReportsPresentationModel(
            IDataAccessService dataAccessService,
            IReportsView view,
            ReportsModel model,
            IRegionManager regionManager
            )
        {
            _dataAccessService = dataAccessService;
            View = view;
            View.DataContext = this;
            Model = model;
            _regionManager = regionManager;

            OKCommand = new DelegateCommand<string> (ExecuteOKCommand, CanExecuteOKCommand);
            PrintCommand = new DelegateCommand<string> (ExecutePrintCommand, CanExecutePrintCommand);
            OKCommand.RaiseCanExecuteChanged();
            PrintCommand.RaiseCanExecuteChanged();
            SetSelectedServerPrinter();
            //regionManager.Regions[RegionNames.TaskGroupRegion].Add (View);
            (View as Window).Show ();
        }

        public ReportsPresentationModel()
        { }

        private void SetSelectedServerPrinter()
        {
            if (Keys.SelectedServerPrinter == null)
                return;

            var addServer = ServerPrinterList.Count == 0;

            if (!addServer)
                addServer = ServerPrinterList.FirstOrDefault(p => p.Ien == Keys.SelectedServerPrinter.Ien) == null;

            if (addServer)
                ServerPrinterList.Add(Keys.SelectedServerPrinter);

            SelectedServerPrinter = Keys.SelectedServerPrinter;
        }

        #region IReportsPresentationModel Members

        public IReportsView View { get; }
        public ReportsModel Model { get; set; }
        public void OnClose ()
        {
            //regionManager.Regions[RegionNames.TaskGroupRegion].Remove (View);
        }

        public DelegateCommand<string> OKCommand { get; }
        public DelegateCommand<string> PrintCommand { get; }

        public void ExecuteOKCommand (string title)
        {
            (View as Window).Close ();
        }

        double _lineHeight;
        double _rowsPerPage;
        double _margin;
        public void ExecutePrintCommand (string printType)
        {
            if (printType == "Local")
            {
                if (Model.IsPatientLetter && Model.PatientHasBadAddress.HasValue && Model.PatientHasBadAddress.Value)
                {
                    Keys.WarnUser(View, "THIS PATIENT HAS BEEN FLAGGED WITH A BAD ADDRESS INDICATOR, NO LETTER WILL BE PRINTED.", "Local Printer Print Warning");
                    return;
                }

                _lineHeight = View.ReportsTextBox.FontFamily.LineSpacing * View.ReportsTextBox.FontSize;
                _margin = 3 * _lineHeight;
                var dialog = new PrintDialog();
                if (dialog.ShowDialog() == true)
                {
                    var pageSize = new Size(dialog.PrintableAreaWidth,
                        dialog.PrintableAreaHeight);
                    _rowsPerPage = ReportPaginator.RowsPerPage(pageSize.Height, _lineHeight, _margin);
                    PrepareTextBoxForPrinting(View.ReportsTextBox, pageSize);
                    var paginator = new ReportPaginator(View.ReportsTextBox,
                      pageSize, _lineHeight, _margin);
                    dialog.PrintDocument(paginator, Model.ReportDisplayName);
                    RestoreTextBoxFromPrinting(View.ReportsTextBox);
                    (View as Window).Close();
                }
            }
            else if (printType == "Server")
            {
                var apptID = Model.ParameterDictionary.FirstOrDefault(p => p.Description == "AppointmentID");
                var letterPrefix = Model.ParameterDictionary.FirstOrDefault(p => p.Description == "LetterPrefix");
                var prefix = (string)letterPrefix.Value;
                prefix = prefix == "A" ? "C" : prefix;
                _dataAccessService.ServerPrinterPrint((string)apptID.Value, prefix, SelectedServerPrinter.Ien);
                (View as Window).Close();
            }
        }

        private double _previousGridHeight;
        private Thickness _previousBorderThickness;
        private Thickness _previousMargin;
        private int _previousMaxLines;
        private void PrepareTextBoxForPrinting (TextBox textBox, Size pageSize)
        {
            _previousGridHeight = ((Grid)textBox.Parent).Height;
            ((Grid)textBox.Parent).Height = _rowsPerPage * _lineHeight + 36 + 2*_margin;
            _previousBorderThickness = textBox.BorderThickness;
            textBox.BorderThickness = new Thickness (0);
            _previousMargin = textBox.Margin;
            textBox.Margin = new Thickness(_margin);
            textBox.VerticalScrollBarVisibility = ScrollBarVisibility.Hidden;
            _previousMaxLines = textBox.MaxLines;
        }

        private void RestoreTextBoxFromPrinting (TextBox textBox)
        {
            ((Grid)textBox.Parent).Height = _previousGridHeight;
            textBox.BorderThickness = _previousBorderThickness;
            textBox.Margin = _previousMargin;
            textBox.VerticalScrollBarVisibility = ScrollBarVisibility.Visible;
            textBox.Height = double.NaN;
            textBox.VerticalAlignment = VerticalAlignment.Stretch;
            textBox.MaxLines = _previousMaxLines;
        }

        public bool CanExecuteOKCommand (string printType)
        {
            return true;
        }

        public bool CanExecutePrintCommand (string printType)
        {
            if (printType == "Local")
                return true;
            if (printType == "Server")
                return SelectedServerPrinter != null;

            return false;
        }

        private IList<ServerPrinter> _serverPrinterList = new List<ServerPrinter>();
        public IList<ServerPrinter> ServerPrinterList
        {
            get { return _serverPrinterList; }
            set
            {
                _serverPrinterList = value;
                NotifyPropertyChanged("ServerPrinterList");
            }
        }

        private ServerPrinter _selectedServerPrinter;
        
        public ServerPrinter SelectedServerPrinter
        {
            get { return _selectedServerPrinter; }
            set
            {
                _selectedServerPrinter = value;
                NotifyPropertyChanged("SelectedServerPrinter");
                PrintCommand.RaiseCanExecuteChanged();
                Keys.SelectedServerPrinter = _selectedServerPrinter;
            }
        }

        public string PrinterFontSize
        {
            get { return Keys.DefaultFacilityFontSize; }
        }

        #endregion IReportsPresentationModel Members
    }
}