﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Input;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.DataAccess.Services;

namespace ClinSchd.Modules.Ribbon.ViewModels
{
    public class ClinicListVM : BaseViewModel
    {
        private readonly IDataAccessService _dataAccessService;
        private readonly AppointmentRequest _appointmentRequest;

        public ClinicListVM(IDataAccessService dataAccessService)
        {
            _dataAccessService = dataAccessService;
            _appointmentRequest = new AppointmentRequest( _dataAccessService );
        }

        public Window View { get; set; }

        private List<ClinicServiceList> _clinicServices = new List<ClinicServiceList>();

        public Action<string> CloseCallback { get; set; }

        public bool ServicesSelected { get; set; }

        public bool ClinicsSelected { get; set; } = true;

        public bool AllSelected { get; set; }

        public string SearchText { get; set; }

        public List<ClinicServiceList> ClinicServices
        {
            get { return _clinicServices ?? ( _clinicServices = new List<ClinicServiceList>() ); }
            set
            {
                _clinicServices = value;
                NotifyPropertyChanged( "ClinicServices" );
                NotifyPropertyChanged( "HasClinics" );
            }
        }
        public IList<Clinic> Clinics( string searchText )
        {
            return _dataAccessService.SearchPtCSchClinics( searchText );
        }

        public IList<ServiceSpecialty> ServiceSpecialties( string searchString )
        {
            return _appointmentRequest.GetApptServiceSpecialties( searchString );
        }

        public bool HasClinics
        {
            get { return ClinicServices.Count > 0; }
        }

        private string SelectedClinicsServices
        {
            get
            {
                var list = "";
                foreach ( var clinicServiceList in ClinicServices.Where( cs=>cs.IsSelected ) )
                {
                    list += clinicServiceList.Name + "=" + clinicServiceList.Ien + ", ";
                }
                if ( list.EndsWith( ", " ) )
                    list = list.Substring( 0, list.Length - 2 );
                return list + ";" + ( ServicesSelected ? "1" : "0" );
            }
        }
        public ICommand FindCommand
        {
            get
            {
                return GetCommand(
                    "FindCommand",
                    param =>
                    {
                        var newSearchString = SearchText.ToUpper();
                        var clinSvcList = ClinicServices.Where( s => s.IsSelected ).ToList();

                        try
                        {
                            if ( ServicesSelected )
                            {
                                if ( newSearchString.Length < Keys.ClinStopSearchCharMin )
                                    return;

                                var list = ServiceSpecialties( newSearchString );

                                foreach ( var item in list )
                                    if ( clinSvcList.FirstOrDefault( i => i.Ien == item.Ien ) == null )
                                    {
                                        var serviceItem = new ClinicServiceList
                                        {
                                            Name = item.Name,
                                            Ien = item.Ien
                                        };
                                        clinSvcList.Add( serviceItem );
                                    }
                            }
                            else
                            {
                                if ( newSearchString.Length < Keys.ClinSetSearchCharMin )
                                    return;

                                var clinics = Clinics( newSearchString );

                                if (clinics != null)
                                    if ((clinics.Count == Keys.ClinicResourceSearchMaxRecords) && clinics[clinics.Count - 1].MoreRecords)
                                    {
                                        Keys.AlertUser(View, $"The current search criteria returns more than {Keys.ClinicResourceSearchMaxRecords} records.\nPlease refine your search and try again.", "Clinic Search Criteria");
                                        ((Views.ClinicListV)View).SetSearchTextFocus();
                                    }

                                foreach ( var item in clinics )
                                    if ( clinSvcList.FirstOrDefault( i => i.Ien == item.ClinicIen ) == null )
                                    {
                                        var clinicItem = new ClinicServiceList
                                        {
                                            Name = item.ClinicName,
                                            Ien = item.ClinicIen
                                        };
                                        clinSvcList.Add( clinicItem );
                                    }
                            }
                        }
                        catch ( Exception ex )
                        {
                            MessageBox.Show( ex.Message, "Find Error" );
                        }

                        ClinicServices = clinSvcList;
                        NotifyPropertyChanged( "ClinicServices" );
                    },
                    _ => true );
            }
        }

        public ICommand SelectAllCommand
        {
            get
            {
                return GetCommand(
                    "SelectAllCommand",
                    param =>
                    {
                        if ( AllSelected )
                            foreach ( var clinList in ClinicServices )
                                clinList.IsSelected = true;
                        else
                            foreach ( var clinList in ClinicServices )
                                clinList.IsSelected = false;
                        NotifyPropertyChanged( "ClinicServices" );
                    },
                    _ => true );
            }
        }

        #region ICloseCommandProvider Members

        public bool IsClosing { get; private set; }

        public ICommand CloseCommand
        {
            get
            {
                return GetCommand(
                    "CloseCommand",
                    _ =>
                    {
                        IsClosing = true;
                        if ( CloseCallback != null )
                        {
                            CloseCallback( null );
                        }
                    },
                    _ => true
                    );
            }
        }

        #endregion ICloseCommandProvider Members

        #region ISaveCommandProvider Members

        public ICommand SaveCommand
        {
            get
            {
                return GetCommand(
                    "SaveCommand",
                    _ =>
                    {
                        if ( CloseCallback != null )
                        {
                            CloseCallback( SelectedClinicsServices );
                        }
                    },
                    _ => {
                        return ClinicServices.Any( iss => iss.IsSelected );
                    }
                    );
            }
        }

        #endregion ISaveCommandProvider Members
    }
}