﻿using System.Collections.Generic;
using System.Windows.Input;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Ribbon.Views;

namespace ClinSchd.Modules.Ribbon.ViewModels
{
    public class ColumnSetting : BaseViewModel
    {
        public delegate void FilterChanged(ColumnSetting sender);
        public event FilterChanged FilterChangedEvent;

        private IDataAccessService _dataAccessService;
        private bool _isFiltered;
        private string _settings;
        private string _sortOrder;
        private bool _isServiceList;

        public void LoadPriorities()
        {
            SortList.Add( "" );
            SortList.Add( "1" );
            SortList.Add( "2" );
            SortList.Add( "3" );
        }

        public ColumnSetting(IDataAccessService dataAccessService)
        {
            _dataAccessService = dataAccessService;
        }

        public ColumnSetting(IDataAccessService dataAccessService, string name, object choices, bool isFiltered, string filterInfo, bool isServiceList, string sortOrder )
            : this(dataAccessService)
        {
            Name       = name.Trim();
            Choices    = choices;
            IsFiltered = isFiltered;
            FilterInfo = filterInfo.Trim();
            IsServiceList = isServiceList;
            SortOrder  = sortOrder.Trim();

            SetColumnChoices( choices );
            LoadPriorities();
        }

        public bool ShowFilterCB
        {
            get { return Choices != null; }
        }

        public List<string> SortList { get; set; } = new List<string>();

        public string SortOrder
        {
            get { return _sortOrder; }
            set
            {
                _sortOrder = value;
                NotifyPropertyChanged( "SortOrder" );
            }
        }

        public bool IsServiceList
        {
            get { return _isServiceList; }
            set
            {
                _isServiceList = value;
                NotifyPropertyChanged( "IsServiceList" );
            }
        }

        public bool IsFiltered
        {
            get { return _isFiltered; }
            set
            {
                _isFiltered = value;
                NotifyPropertyChanged( "IsFiltered" );
            }
        }

        public bool IsEnabled { get; set; } = true;

        private void DisplayDialog()
        {
            switch ( Name.ToUpper() )
            {
                case Keys.RMGridDefaultColumnOrder.REQUEST_TYPE:
                    GetRequestTypesCommand.Execute( Name );
                    break;

                case Keys.RMGridDefaultColumnOrder.WAIT_TIME:
                    GetWaitTimeCommand.Execute( Name );
                    break;

                case Keys.RMGridDefaultColumnOrder.PT_NAME: // No Dialog
                    break;

                case Keys.RMGridDefaultColumnOrder.MRTC:
                    //GetMRTCCommand.Execute( ColumnName );
                    break;

                case Keys.RMGridDefaultColumnOrder.SSN: // No Dialog
                    break;

                case Keys.RMGridDefaultColumnOrder.SCVISIT:
                    GetSCVisitCommand.Execute( Name );
                    break;

                case Keys.RMGridDefaultColumnOrder.TELEPHONE: // No Dialog
                    break;

                case Keys.RMGridDefaultColumnOrder.PRIORITY:
                    GetPriorityGroupsCommand.Execute( Name );
                    break;

                case Keys.RMGridDefaultColumnOrder.CID_PREFERRED_DATE:
                    GetCIDPreferredDatesCommand.Execute( Name );
                    break;

                case Keys.RMGridDefaultColumnOrder.ENTERED_RR_NO_DATE:
                    GetEnteredRRNoDatesCommand.Execute( Name );
                    break;

                case Keys.RMGridDefaultColumnOrder.REQUESTOR: // No Dialog
                    break;

                case Keys.RMGridDefaultColumnOrder.REQUESTED_BY: // No Dialog
                    break;

                case Keys.RMGridDefaultColumnOrder.CLINIC_SERVICE:
                    GetClinicsCommand.Execute( Name );
                    break;

                case Keys.RMGridDefaultColumnOrder.COMMENT: // No Dialog
                    break;
            }
        }

        private void SetColumnChoices( object choices )
        {
            switch ( Name.ToUpper() )
            {
                case Keys.RMGridDefaultColumnOrder.REQUEST_TYPE:
                    RequestTypes = (List<RequestType>)choices;
                    break;
                case Keys.RMGridDefaultColumnOrder.WAIT_TIME:
                    break;
                case Keys.RMGridDefaultColumnOrder.PT_NAME:
                    break;
                case Keys.RMGridDefaultColumnOrder.SSN:
                    break;
                case Keys.RMGridDefaultColumnOrder.MRTC:
                    //MRTCOptions = (List<string>)options;
                    break;
                case Keys.RMGridDefaultColumnOrder.SCVISIT:
                    //SCVisitOptions = (List<string>)choices;
                    break;
                case Keys.RMGridDefaultColumnOrder.TELEPHONE:
                    break;
                case Keys.RMGridDefaultColumnOrder.PRIORITY:
                    PriorityGroups = (List<PriorityGroup>)choices;
                    break;
                case Keys.RMGridDefaultColumnOrder.CID_PREFERRED_DATE:
                    break;
                case Keys.RMGridDefaultColumnOrder.ENTERED_RR_NO_DATE:
                    break;
                case Keys.RMGridDefaultColumnOrder.REQUESTOR:
                    break;
                case Keys.RMGridDefaultColumnOrder.REQUESTED_BY:
                    break;
                case Keys.RMGridDefaultColumnOrder.CLINIC_SERVICE:
                    break;
                case Keys.RMGridDefaultColumnOrder.COMMENT:
                    break;
            }
        }

        public List<string> SCVisitOptions { get; set; } = null;

        public List<string> MRTCOptions { get; set; } = null;

        public string FilterInfo
        {
            get { return _settings; }
            set
            {
                if (value != _settings)
                {
                    _settings = value;
                    NotifyPropertyChanged("FilterInfo");
                    FilterChangedEvent?.Invoke(this);
                }
            }
        }

        public void SetFilterInfo(string filterInfo)
        {
            _settings = filterInfo;
            NotifyPropertyChanged("FilterInfo");
        }

        public string Name { get; set; }

        public object Choices { get; set; }

        public List<RequestType> RequestTypes { get; set; }

        public List<PriorityGroup> PriorityGroups { get; set; }

        #region Commands

        public ICommand FilterCommand
        {
            get
            {
                return GetCommand(
                    "FilterCommand",
                    param =>
                    {
                        if ( !IsFiltered )
                        {
                            FilterInfo = "";
                            return;
                        }
                        DisplayDialog();
                    },
                    _ => true );
            }
        }

        public ICommand GetRequestTypesCommand
        {
            get
            {
                return GetCommand(
                    "GetRequestTypesCommand",
                    param =>
                    {
                        if ( !IsFiltered )
                        {
                            FilterInfo = "";
                            return;
                        }

                        var vm = new RequestTypesVM
                        {
                            Title = "Request Types",
                            RequestTypes = RequestTypes
                        };

                        var dlg = new RequestTypesV
                        {
                            Context = vm
                        };
                        vm.CloseCallback = result =>
                        {
                            dlg.Close();
                            if ( result != null )
                            {
                                FilterInfo = result;
                            }
                            else
                            {
                                IsFiltered = false;
                            }
                        };
                        dlg.ShowDialog();
                    },
                    _ => true );
            }
        }

        public ICommand GetWaitTimeCommand
        {
            get
            {
                return GetCommand(
                    "GetWaitTimeCommand",
                    param =>
                    {
                        if ( !IsFiltered )
                        {
                            FilterInfo = "";
                            return;
                        }

                        var vm = new WaitTimeVM
                        {
                            Title = "Wait Time",
                        };

                        var dlg = new WaitTimeV
                        {
                            Context = vm
                        };
                        vm.CloseCallback = result =>
                        {
                            dlg.Close();
                            if ( result != null )
                            {
                                FilterInfo = result;
                            }
                            else
                            {
                                IsFiltered = false;
                            }
                        };
                        dlg.ShowDialog();
                    },
                    _ => true );
            }
        }

        public ICommand GetPriorityGroupsCommand
        {
            get
            {
                return GetCommand(
                    "GetPriorityGroupsCommand",
                    param =>
                    {
                        if ( !IsFiltered )
                        {
                            FilterInfo = "";
                            return;
                        }

                        var vm = new PriorityGroupsVM
                        {
                            Title = "Priority Groups",
                            PriorityGroups = PriorityGroups
                        };

                        var dlg = new PriorityGroupsV
                        {
                            Context = vm
                        };
                        vm.CloseCallback = result =>
                        {
                            dlg.Close();
                            if ( result != null )
                            {
                                FilterInfo = result;
                            }
                            else
                            {
                                IsFiltered = false;
                            }
                        };
                        dlg.ShowDialog();
                    },
                    _ => true );
            }
        }

        public ICommand GetMRTCCommand
        {
            get
            {
                return GetCommand(
                    "GetMRTCCommand",
                    param =>
                    {
                        if ( !IsFiltered )
                        {
                            FilterInfo = "";
                            return;
                        }

                        var vm = new YesNoBothVM
                        {
                            Title = (string)param,
                        };

                        var dlg = new YesNoBothV
                        {
                            Context = vm
                        };
                        vm.CloseCallback = result =>
                        {
                            dlg.Close();
                            if ( result != null )
                            {
                                FilterInfo = result;
                            }
                            else
                            {
                                IsFiltered = false;
                            }
                        };
                        dlg.ShowDialog();
                    },
                    _ => true );
            }
        }

        public ICommand GetSCVisitCommand
        {
            get
            {
                return GetCommand(
                    "GetSCVisitCommand",
                    param =>
                    {
                        if ( !IsFiltered )
                        {
                            FilterInfo = "";
                            return;
                        }

                        var vm = new YesNoBothVM
                        {
                            Title = (string)param,
                        };

                        var dlg = new YesNoBothV
                        {
                            Context = vm
                        };
                        vm.CloseCallback = result =>
                        {
                            dlg.Close();
                            if ( result != null )
                            {
                                FilterInfo = result;
                            }
                            else
                            {
                                IsFiltered = false;
                            }
                        };
                        dlg.ShowDialog();
                    },
                    _ => true );
            }
        }

        public ICommand GetCIDPreferredDatesCommand
        {
            get
            {
                return GetCommand(
                    "GetCIDPreferredDatesCommand",
                    param =>
                    {
                        if ( !IsFiltered )
                        {
                            FilterInfo = "";
                            return;
                        }

                        var vm = new DatesLogicVM
                        {
                            Title = (string)param,
                        };

                        var dlg = new DatesLogicV
                        {
                            Context = vm
                        };
                        vm.CloseCallback = result =>
                        {
                            dlg.Close();
                            if ( result != null )
                            {
                                FilterInfo = result;
                            }
                            else
                            {
                                IsFiltered = false;
                            }
                        };
                        dlg.ShowDialog();
                    },
                    _ => true );
            }
        }

        public ICommand GetEnteredRRNoDatesCommand
        {
            get
            {
                return GetCommand(
                    "GetEnteredRRNoDatesCommand",
                    param =>
                    {
                        if ( !IsFiltered )
                        {
                            FilterInfo = "";
                            return;
                        }

                        var vm = new DatesLogicVM
                        {
                            Title = (string)param,
                        };

                        var dlg = new DatesLogicV
                        {
                            Context = vm
                        };
                        vm.CloseCallback = result =>
                        {
                            dlg.Close();
                            if ( result != null )
                            {
                                FilterInfo = result;
                            }
                            else
                            {
                                IsFiltered = false;
                            }
                        };
                        dlg.ShowDialog();
                    },
                    _ => true );
            }
        }

        public ICommand GetClinicsCommand
        {
            get
            {
                return GetCommand(
                    "GetClinicsCommand",
                    param =>
                    {
                        if ( !IsFiltered )
                        {
                            FilterInfo = "";
                            return;
                        }

                        var vm = new ClinicListVM(_dataAccessService) { Title = (string)param };
                        var dlg = new ClinicListV(_dataAccessService, vm);

                        vm.CloseCallback = result =>
                        {
                            dlg.Close();
                            if ( result != null )
                            {
                                FilterInfo = result;
                            }
                            else
                            {
                                IsFiltered = false;
                            }
                        };
                        dlg.ShowDialog();
                    },
                    _ => true );
            }
        }

        #endregion Commands
    }
}
