﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Ribbon.ViewModels;

namespace ClinSchd.Modules.Ribbon.Views
{
    /// <summary>
    /// Interaction logic for ClinicListView.xaml
    /// </summary>
    public partial class ClinicListView
    {
        public ClinicListViewModel ViewModel { get; set; }

        public ClinicListView(IDataAccessService dataAccessService)
        {
            ViewModel = new ClinicListViewModel(dataAccessService);
            DataContext = ViewModel;
            InitializeComponent();
        }

        private void ChkSelectUnselectAll_Click(object sender, RoutedEventArgs e)
        {
            if(ChkSelectUnselectAll.IsChecked == true)
                foreach(var clinList in ViewModel.ClinicServiceSelectionList)
                    clinList.IsSelected = true;
            else
                foreach (var clinList in ViewModel.ClinicServiceSelectionList)
                    clinList.IsSelected = false;
            ViewModel.NotifyPropertyChanged("ClinicServiceSelectionList");
        }

        private void OK_Click(object sender, RoutedEventArgs e)
        {
            CloseWindow();
        }

        private void Cancel_Click(object sender, RoutedEventArgs e)
        {
            CloseWindow();
        }

        private void CloseWindow()
        {
            var window = ParentWindow;
            if (window != null)
                window.Close();
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void ClinicListView_Loaded(object sender, RoutedEventArgs e)
        {
            Keys.SetRenderingMode(this);

            if(ViewModel.ClinicServiceSelectionList == null)
                ViewModel.ClinicServiceSelectionList = new List<ClinicServiceList>();

            if (ViewModel.IsServiceList)
            {
                radServices.IsChecked = true;
                radClinics.IsChecked = false;
            }
            else
            {
                radServices.IsChecked = false;
                radClinics.IsChecked = true;
            }

            if (ViewModel.ConsultUrgencyActivated)
            {
                radServices.IsChecked = true;
                radClinics.IsChecked = false;
                radClinics.IsEnabled = false;
            }
            else
                radClinics.IsEnabled = true;

            if (ViewModel.IsPtCSchOnly)
            {
                radServices.IsChecked = false;
                radClinics.IsChecked = true;
                radServices.IsEnabled = false;
            }
            else
                radServices.IsEnabled = true;
            SetTabIndices();
            SearchTextBox.Focus();
            Keyboard.Focus(SearchTextBox);
        }

        private void ClinicServiceRadio_Checked(object sender, RoutedEventArgs e)
        {
            ViewModel.ClinicServiceSelectionList.Clear();
            ViewModel.IsServiceList = radServices.IsChecked.HasValue && radServices.IsChecked.Value;
        }

        private void Find_Click(object sender, RoutedEventArgs e)
        {
            bool radServicesIsChecked = radServices.IsChecked.HasValue && radServices.IsChecked.Value;
            string searchName = radServicesIsChecked ? "Service\\Specialties" : "Clinics";
            string newSearchString = SearchTextBox.Text.ToUpper();
            var clinSvcList = ViewModel.ClinicServiceSelectionList.Where(s => s.IsSelected).ToList();

            try
            {
                if (radServicesIsChecked)
                {
                    if (newSearchString.Length < Keys.ClinStopSearchCharMin)
                        return;
                }
                else
                {
                    if (newSearchString.Length < Keys.ClinSetSearchCharMin)
                        return;
                }

                ViewModel.IsAsync = true;
                ViewModel.DoWorkAsync(
                    (s, args) =>
                    {
                        lock (args)
                        {
                            if (radServicesIsChecked)
                                args.Result = ViewModel.ServiceSpecialties(newSearchString);
                            else
                                args.Result = ViewModel.ClinicSelectionList(newSearchString);
                        }
                    },
                    (s, args) =>
                    {
                        lock (args)
                        {
                            if (args.Result == null)
                                return;

                            if (radServicesIsChecked)
                            {
                                IList<ServiceSpecialty> list = (IList<ServiceSpecialty>)args.Result;

                                foreach (var item in list)
                                    if (clinSvcList.FirstOrDefault(i => i.Ien == item.Ien) == null)
                                    {
                                        var serviceItem = new ClinicServiceList
                                        {
                                            Name = item.Name,
                                            Ien = item.Ien
                                        };
                                        clinSvcList.Add(serviceItem);
                                    }
                            }
                            else
                            {
                                IList<Clinic> clinics = (IList<Clinic>)args.Result;

                                if (clinics != null)
                                    if ((clinics.Count == Keys.ClinicResourceSearchMaxRecords) && clinics[clinics.Count - 1].MoreRecords)
                                    { 
                                        Keys.AlertUser(ParentWindow, $"The current search criteria returns more than {Keys.ClinicResourceSearchMaxRecords} records.\nPlease refine your search and try again.", "Clinic Search Criteria");
                                        SetSearchTextBoxFocus();
                                    }

                                foreach (var item in clinics)
                                    if (clinSvcList.FirstOrDefault(i => i.Ien == item.ClinicIen) == null)
                                    {
                                        var clinicItem = new ClinicServiceList
                                        {
                                            Name = item.ClinicName,
                                            Ien = item.ClinicIen
                                        };
                                        clinSvcList.Add(clinicItem);
                                    }
                            }

                            ViewModel.ClinicServiceSelectionList = clinSvcList;
                            ClinicListGridView.ItemsSource = ViewModel.ClinicServiceSelectionList;
                        }
                    },
                    true,
                    false,
                    string.Empty,
                    string.Empty /*,
                    TimeSpan.FromSeconds(3) //*/
                );
            }
            catch (Exception ex)
            {
                Keys.AlertUserError(this, ex, searchName + " Find Error");
                TraceLog.LogException(GetType().Name, "Find_Click", ex);
            }
        }

        private void SetTabIndices()
        {
            SearchTextBox.TabIndex        = 0;
            BtnFind.TabIndex              = 10;
            ChkSelectUnselectAll.TabIndex = 20;
            radClinics.TabIndex           = 30;
            radServices.TabIndex          = 40;
            ClinicListGridView.TabIndex = 50;
            BtnOk.TabIndex     = 60;
            BtnCancel.TabIndex = 70;
        }

        private void HotkeyHandler(Key k)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                switch (k)
                {
                    case Key.O:
                        Keyboard.Focus(BtnOk);
                        break;
                    case Key.C:
                        Keyboard.Focus(BtnCancel);
                        break;
                    case Key.F:
                        Keyboard.Focus(BtnFind);
                        break;
                }
            }
        }

        private void UserControl_PreviewKeyUp(object sender, KeyEventArgs e)
        {
            Key key = e.Key == Key.System ? e.SystemKey : e.Key;
            HotkeyHandler(key);
        }

        private void ClinicListGridView_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Tab && Keyboard.Modifiers == ModifierKeys.None)
            {
                e.Handled = true;
                Keyboard.Focus(BtnOk);
            }
            if (Keyboard.Modifiers == ModifierKeys.None && e.Key == Key.Space)
            {    
                foreach (var clinList in ViewModel.ClinicServiceSelectionList)
                {
                    if (ClinicListGridView.SelectedItem == clinList)
                    {
                        clinList.IsSelected = !clinList.IsSelected;
                    }
                                        
                }
            }
        }

        private void ClinicListGridView_DataLoaded(object sender, EventArgs e)
        {
            if (ClinicListGridView.HasItems)
            {
                ClinicListGridView.SelectedItem = ClinicListGridView.Items[0];
                ClinicListGridView.ScrollIntoView(ClinicListGridView.SelectedItem);
            }
        }

        private void radServices_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key==Key.Tab && ClinicListGridView.HasItems)
            {
                e.Handled = true;
                ClinicListGridView.SelectedItem = ClinicListGridView.Items[0];
                ClinicListGridView.ScrollIntoView(ClinicListGridView.SelectedItem);
                ClinicListGridView.Focus();
            }
        }

        public void SetSearchTextBoxFocus()
        {
            SearchTextBox.Focus();
            Keyboard.Focus(SearchTextBox);
        }

        public Window ParentWindow { get { return Window.GetWindow(this); } }
    }
}
