﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Ribbon.ViewModels;
using Cursors = System.Windows.Input.Cursors;
using KeyEventArgs = System.Windows.Input.KeyEventArgs;
using Keys = ClinSchd.Infrastructure.Keys;
using MessageBox = System.Windows.MessageBox;
using System.Threading;
using System.Collections.Specialized;
using ClinSchd.Infrastructure.Interfaces;

namespace ClinSchd.Modules.Ribbon.Views
{
    /// <summary>
    /// Interaction logic for WaitListQueryForm.xaml
    /// </summary>
    public partial class WaitListQueryForm
    {
        private readonly IDataAccessService _dataAccessService;

        readonly System.Windows.Threading.DispatcherTimer _dispatcherTimer = new System.Windows.Threading.DispatcherTimer();

        public WaitListQueryFormViewModel ViewModel { get; }

        public string PatientSearchString { get; set; }

        public string WaitTimeSetting { get; set; }

        public bool SubmitClicked { get; set; }

        public WaitListQueryForm( IDataAccessService dataAccessService )
        {
            _dataAccessService = dataAccessService;
            ViewModel = new WaitListQueryFormViewModel( _dataAccessService )
            {
                FilteredClinics        = new ObservableCollection<ClinicServiceList>(),
                FilteredRequestTypes   = new ObservableCollection<RequestType>(),
                FilteredPriorityGroups = new ObservableCollection<PriorityGroup>()
            };
            DataContext = ViewModel;
            InitializeComponent();

            LstSelectedPatients.ItemsSource = ViewModel.SelectedPatients;
            LstPriorityGroups.ItemsSource = ViewModel.FilteredPriorityGroups;
            CmbWaitTime.ItemsSource = ViewModel.WaitTimes;
            CmbWaitTime.Text = Keys.RmGridColumns.WaitTimeGreaterThanEqualTo90;
            WaitTimeSetting = Keys.RmGridColumns.WaitTimeNull;
            CmbServiceRelated.ItemsSource = ViewModel.ServiceRelatedValues;
            CmbServiceVisit.ItemsSource = ViewModel.ScVisitValues;
            CmbServiceRelated.Text = "BOTH";
            CmbServiceVisit.Text = "BOTH";
            CmbQuerySort.ItemsSource = ViewModel.SortFields;
            CmbQuerySort.Text = "Default";

            _dispatcherTimer.Tick += DispatcherTimerTick;
            _dispatcherTimer.Interval = new TimeSpan(0, 0, 0, 0, 250);
            //BtnSubmitClick.IsDefault = true;
            ((INotifyCollectionChanged)PatientSearchResults.Items).CollectionChanged += PatientSearchResults_CollectionChanged;
            ((INotifyCollectionChanged)LstSelectedPatients.Items).CollectionChanged += LstSelectedPatients_CollectionChanged;
            ((INotifyCollectionChanged)LstRequestTypes.Items).CollectionChanged += LstRequestTypes_CollectionChanged;
            ((INotifyCollectionChanged)LstClinics.Items).CollectionChanged += LstClinics_CollectionChanged;
            ((INotifyCollectionChanged)LstPriorityGroups.Items).CollectionChanged += LstPriorityGroups_CollectionChanged;
            Keyboard.Focus(PatientSearchText);
        }

        private void GetMatchingPatients()
        {
            string patientSearchText = PatientSearchText.Text;

            if (ViewModel.PatientList != null)
                ViewModel.PatientList.Clear();

            ViewModel.IsAsync = true;
            ViewModel.DoWorkAsync(
                (s, args) =>
                {
                    lock (args)
                    {
                        args.Result = ViewModel.DataAccessService.GetPatients(patientSearchText, Keys.MaxPatientsToReturn);
                    }
                },
                (s, args) =>
                {
                    lock (args)
                    {
                        if (args.Result == null)
                            return;

                        ViewModel.PatientList = (IList<Patient>)args.Result;
                    }
                },
                true,
                false,
                "Searching Patients",
                String.Empty
            );
        }

        private void PatientSearchKey_Down(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Return)
            {
                if (PatientSearchText.Text.Length >= 3)
                    GetMatchingPatients();
                //e.Handled = true;
            }
        }
        
        public void ExitClick(object sender, RoutedEventArgs e)
        {
            CloseWindow();
        }

        private void CloseWindow()
        {
            var window = Window.GetWindow(this);
            if (window != null)
                window.Close();
        }

        private void LstSelectedPatients_Changed(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0)
                return;
            var patient = e.AddedItems[0] as Patient;
            ViewModel.SelectedPatients.Remove(patient);
            ViewModel.NotifyPropertyChanged("SelectedPatients");
        }

        private void PatientSearchResults_Changed(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0)
                return;
            var patient = e.AddedItems[0] as Patient;
            if (patient == null)
            {
                _dispatcherTimer.Start();
                return;
            }
            if (ViewModel.SelectedPatients.Any(pat => patient.IEN == pat.IEN))
            {
                _dispatcherTimer.Start();
                return;
            }

            var promptRslt = ViewModel.DataAccessService.GetPatientSelectionPrompts(patient.IEN, ref patient, false, false);

            if (promptRslt == "0")
            {
                _dispatcherTimer.Start();
                return;
            }

            //ViewModel.SelectedPatients.Add(patient);
            _dispatcherTimer.Start();
        }

        private void DispatcherTimerTick(object sender, EventArgs e)
        {
            PatientSearchResults.UnselectAll();
            _dispatcherTimer.Stop();
        }

        private void SleepAndUnselect()
        {
            System.Windows.Forms.Application.DoEvents();
            Thread.Sleep(250);
            System.Windows.Forms.Application.DoEvents();
            PatientSearchResults.UnselectAll();
        }

        private void LstRequestTypes_Changed(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0)
                return;
            var item = e.AddedItems[0] as RequestType;
            ViewModel.FilteredRequestTypes.Remove(item);
            ViewModel.NotifyPropertyChanged("RequestTypes");
            if (LstRequestTypes.Items.Count == 0)
                ConsultUrgency.IsEnabled = true;
            if (LstRequestTypes.Items.Count == 1)
            {
                ConsultUrgency.IsEnabled = ViewModel.FilteredRequestTypes[0].RequestName.ToUpper().Contains("CONSULT");
                ViewModel.IsPtCSchOnly = ViewModel.FilteredRequestTypes[0].RequestName.ToUpper().Contains("PtCSch");
            }
            if (LstRequestTypes.Items.Count > 1)
            {
                ConsultUrgency.Text = string.Empty;
                ConsultUrgency.IsEnabled = false;
            }
        }

        private void Submit_Click(object sender, RoutedEventArgs e)
        {
            if (!ValidateQueryRules())
                return;

            Keys.WaitListQueryTotalRecords = 0;

            var waitTime = "";
            if (string.IsNullOrEmpty(CmbWaitTime.Text))
                waitTime = Keys.RmGridColumns.WaitTimeNull;
            else
            {
                if (CmbWaitTime.Text.ToUpper().Contains("ALL DAYS"))
                    waitTime = "ALL DAYS";
                else if (CmbWaitTime.Text.Contains("<30"))
                    waitTime = "<30";
                else if (CmbWaitTime.Text.Contains("30-60"))
                    waitTime = "30-60";
                else if (CmbWaitTime.Text.Contains("60-90"))
                    waitTime = "60-90";
                else if (CmbWaitTime.Text.Contains(">=90"))
                    waitTime = ">=90";
            }

            var filters = "";
            if (!string.IsNullOrEmpty(waitTime))
                filters = "WAITTIME^" + waitTime + "|";

            if (ViewModel.SelectedPatients.Count > 0)
            {
                filters += "PATIENTNAME^";
                foreach (var patient in ViewModel.SelectedPatients)
                    filters += patient.IEN + "~";
                filters = filters.TrimEnd('~');
                filters += "|";
            }
            if (ViewModel.FilteredRequestTypes.Count > 0 && ViewModel.FilteredRequestTypes.Count < 5)
            {
                filters += "REQUESTTYPE^";
                foreach (var reqType in ViewModel.FilteredRequestTypes)
                    if (reqType.RequestName.ToUpper() == "PTCSCH")
                        filters += "R~";
                    else
                        filters += reqType.RequestName.Substring(0, 1) + "~";
                filters = filters.TrimEnd('~');
                filters += "|";
            }
            if (ViewModel.FilteredPriorityGroups.Count > 0)
            {
                filters += "PRIORITYGROUPS^";
                foreach (var pg in ViewModel.FilteredPriorityGroups)
                    filters += pg.PriorityGroupName + "~";
                filters = filters.TrimEnd('~');
                filters += "|";
            }
            //filters += "SERVICERELATED^" + CmbServiceVisit.Text;
            //filters += "|";
            //filters += "SCVISIT^" + CmbServiceRelated.Text;
            //filters += "|";
            if (LstClinics.Items.Count > 0)
            {
                if (ViewModel.IsServiceList)
                    filters += "SERVICES^";
                else
                    filters += "CLINICS^";
                foreach (ClinicServiceList clin in LstClinics.Items)
                    filters += clin.Ien + "~";
                filters = filters.TrimEnd('~');
                filters += "|";
            }
            if (!string.IsNullOrEmpty(TxtOriginationDateBegin.Text))
            {
                filters += "ORIGINATIONDATE^" + TxtOriginationDateBegin.Text;
                filters += "|";
            }
            if (!string.IsNullOrEmpty(TxtOriginationDateEnd.Text))
            {
                filters += "ORIGINATIONDATEEND^" + TxtOriginationDateEnd.Text;
                filters += "|";
            }
            if (!string.IsNullOrEmpty(TxtDesiredDateBegin.Text))
            {
                filters += "DESIREDDATE^" + TxtDesiredDateBegin.Text;
                filters += "|";
            }
            if (!string.IsNullOrEmpty(TxtDesiredDateEnd.Text))
            {
                filters += "DESIREDDATEEND^" + TxtDesiredDateEnd.Text;
                filters += "|";
            }
            if (!string.IsNullOrEmpty(ConsultUrgency.Text))
            {
                filters += "URGENCY^" + ConsultUrgency.SelectedValue;
                filters += "|";
            }
            filters += "SORT^" + CmbQuerySort.Text;
            filters = filters.TrimEnd('|');

            var sort = "";
            if (CmbQuerySort.Text.ToUpper() != "DEFAULT")
                sort = CmbQuerySort.Text.ToUpper();
            if (!string.IsNullOrEmpty(sort))
            {
                switch (sort)
                {
                    case "PATIENT NAME":
                        sort = "PATIENTNAME";
                        break;
                    case "REQUEST TYPE":
                        sort = "REQUESTTYPE";
                        break;
                    case "ORIGINATION DATE":
                        sort = "ORIGINATIONDATE";
                        break;
                    case "SERVICE RELATED":
                        sort = "SCVISIT";
                        break;
                    case "CID/PREFERRED DATE":
                        sort = "DESIREDDATE";
                        break;
                    case "PRIORITY GROUP":
                        sort = "PRIORITYGROUP";
                        break;
                    case "CLINIC":
                        sort = "CLINICS";
                        break;
                    case "WAIT TIME":
                        sort = "WAITTIME";
                        break;
                }
            }

            ViewModel.IsAsync = true;
            ViewModel.DoWorkAsync(
                (s, args) =>
                {
                    lock (args)
                    {
                        List<object> results = new List<object>();

                        try
                        {
                            List<string> messages = new List<string>();
                            bool retVal = _dataAccessService.GetFilteredWaitListRecords(filters, "", true, sort, messages);
                            Keys.QueryFormCurrentFilter = filters; // MES  artf20534 : UAT - Query tool is prepopulating with user preferences  moved this line to here from dataAccess
                            results.Add(retVal);
                            results.Add(messages);
                        }
                        catch (Exception ex)
                        {
                            results.Add(ex);
                        }

                        args.Result = results;
                    }
                },
                (s, args) =>
                {
                    lock (args)
                    {
                        List<object> results = (List<object>)args.Result;

                        if (results[0] is Exception)
                        {
                            Keys.AlertUserError(null, new Exception("Error Processing Query", (Exception)results[0]), "Query Form");
                            return;
                        }

                        if (results.Count > 1)
                        {
                            if (results[1] is List<string>)
                            {
                                List<string> messages = (List<string>)results[1];

                                if (messages.Count > 0)
                                    Keys.AlertUser(null, string.Join("\n", messages.ToArray()), "Query Form");
                            }
                        }

                        if (results[0] is bool)
                        {
                            if (!(bool)results[0])
                                return;
                        }

                        if (Keys.WaitListQueryTotalRecords > 0)
                        {

                            string message;
                            if (Keys.WaitListQueryTotalRecords >= 200)
                            {
                                //Keys.DisplayMaxRecordMessage = false;
                                message = "The maximum number of results has been exceeded.  The first 200 will be displayed.  Do you wish to continue?";
                            }
                            else
                                message = "The current query produced " + Keys.WaitListQueryTotalRecords + " records.\nDo you wish to continue?";
                            var msgBoxRslt = MessageBox.Show(message, "Request Query Confirmation", MessageBoxButton.OKCancel);
                            if (msgBoxRslt == MessageBoxResult.OK)
                            {
                                SubmitClicked = true;
                                CloseWindow();
                            }
                            else
                                SubmitClicked = false;
                        }
                    }
                },
                true,
                true,
                "Requesting Records"
            );
        }

        private bool ValidateQueryRules()
        {
            // Coral Group is in use ONLY when ALL controls in the grou are selected
            var isCoralGroupUsed = ViewModel.SelectedPatients.Count > 0;
            
            // Blue Group is in use ONLY when ALL controls in the grou are selected
            var isBlueGroupUsed = ViewModel.FilteredRequestTypes.Count > 0 && ViewModel.FilteredRequestTypes.Count < 5 && ViewModel.FilteredClinics.Count > 0;
            
            // Green Group is in use if ANY 1 Green Group control has been selected
            var isGreenGroupUsed =
            ViewModel.FilteredPriorityGroups.Count > 0
            ||
            !string.IsNullOrEmpty(CmbWaitTime.Text)
            ||
            !string.IsNullOrEmpty(CmbServiceVisit.Text)
            ||
            !string.IsNullOrEmpty(CmbServiceRelated.Text)
            ||
            !string.IsNullOrEmpty(TxtOriginationDateBegin.Text)
            ||
            !string.IsNullOrEmpty(TxtOriginationDateEnd.Text)
            ||
            !string.IsNullOrEmpty(TxtDesiredDateBegin.Text)
            ||
            !string.IsNullOrEmpty(TxtDesiredDateEnd.Text)
            ||
            !string.IsNullOrEmpty(ConsultUrgency.Text);

            var success =
            (isGreenGroupUsed && (isCoralGroupUsed || isBlueGroupUsed))
            ||
            (isCoralGroupUsed && isBlueGroupUsed)
            ||
            isBlueGroupUsed
            ||
            isCoralGroupUsed;

            if (!success)      
                Keys.AlertUser(this, "Incomplete search criteria selected. Please review the search criteria rules at the top of the window.", "Query Rule Validation");
            
            return success;
        }

        private void LstClinics_Changed(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0)
                return;
            var item = e.AddedItems[0] as ClinicServiceList;
            ViewModel.FilteredClinics.Remove(item);
            ViewModel.NotifyPropertyChanged( "FilteredClinics");
            if (LstClinics.Items.Count == 0)
                ConsultUrgency.IsEnabled = true;
            else
            {
                ConsultUrgency.Text = string.Empty;
                ConsultUrgency.IsEnabled = false;
            }
        }

        private void LstPriorityGroups_Changed(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0)
                return;
            var item = e.AddedItems[0] as PriorityGroup;
            ViewModel.FilteredPriorityGroups.Remove(item);
            ViewModel.NotifyPropertyChanged( "FilteredPriorityGroups");
        }

        private void LaunchClinicListWindow()
        {
            var clinicList = new ClinicListView(_dataAccessService)
            {
                ViewModel =
                {
                    ConsultUrgencyActivated = ViewModel.ConsultUrgencyActivated,
                    IsPtCSchOnly = ViewModel.IsPtCSchOnly
                }
            };
            Point relativePoint = ImgClinicsFilter.TransformToVisual(this).Transform(new Point(0, 0));
            Point absolutePoint = PointToScreen(relativePoint);
            double top = absolutePoint.Y + (ImgClinicsFilter.Height / 2);
            double left = absolutePoint.X + (ImgClinicsFilter.Width / 2);

            CLaunchWindow.LaunchWindow(clinicList, "Clinics", 350, 250, top, left, "ClinicList", false);
            ViewModel.IsServiceList = clinicList.ViewModel.IsServiceList;
            ViewModel.FilteredClinics.Clear();

            foreach (var clv in clinicList.ViewModel.ClinicServiceSelectionList)
            {
                if (clv.IsSelected)
                    ViewModel.FilteredClinics.Add(clv);
                else
                    ViewModel.FilteredClinics.Remove(clv);
            }

            ViewModel.NotifyPropertyChanged( "FilteredClinics");
            Keyboard.Focus(LstClinics);
        }

        private void Clinics_PreviewMouseDown(object sender, MouseButtonEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Arrow;
            LaunchClinicListWindow();
        }

        private void FindPatient_Click(object sender, RoutedEventArgs e)
        {
            GetMatchingPatients();
        }

        private void WaitTime_Changed(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count > 0)
            {
                WaitTimeSetting = e.AddedItems[0].ToString();

                if (!string.IsNullOrEmpty(WaitTimeSetting))
                {
                    TxtDesiredDateBegin.Text = string.Empty;
                    TxtDesiredDateEnd.Text = string.Empty;
                    TxtOriginationDateBegin.Text = string.Empty;
                    TxtOriginationDateEnd.Text = string.Empty;
                }
            }
        }

        private void RequestQueryForm_Loaded(object sender, RoutedEventArgs e)
        {
            Keys.SetRenderingMode(this);

            if (string.IsNullOrEmpty(Keys.QueryFormCurrentFilter))
                return;
            
            //Maintain State... based on last query..
            string[] parms = Keys.QueryFormCurrentFilter.Split('|');
            
            foreach(var parm in parms)
            {
                if(parm.ToUpper().Contains("CLINICS"))
                {
                    var clinics = parm.Split('^')[1];
                    var clinicIens = clinics.Split('~');
                    ViewModel.IsServiceList = false;
                    foreach(var clinIen in clinicIens)
                    {
                        var ptCSchClincs = _dataAccessService.GetPtCSchClinics(clinIen, null);

                        if (ptCSchClincs != null)
                        {
                            var clinic = ptCSchClincs.FirstOrDefault(c => c.ClinicIen == clinIen);

                            if (clinic != null)
                            {
                                var clinServiceList = new ClinicServiceList
                                {
                                    Name = clinic.ClinicName,
                                    Ien = clinic.ClinicIen
                                };
                                ViewModel.FilteredClinics.Add(clinServiceList);
                            }
                        }
                    }
                }
                if (parm.ToUpper().Contains("SERVICES"))
                {
                    ViewModel.IsServiceList = true;
                    var appointmentRequest = new AppointmentRequest(_dataAccessService);
                    var svcs = parm.Split('^')[1];
                    var svcIens = svcs.Split('~');
                    var svcSpecS = appointmentRequest.GetApptServiceSpecialties( "" );
                    foreach (var ien in svcIens)
                    {
                        foreach (var svc in svcSpecS)
                        {
                            if ( ien == svc.Ien)
                            {
                                var clinServiceList = new ClinicServiceList
                                {
                                    Name = svc.Name,
                                    Ien = svc.Ien
                                };
                                ViewModel.FilteredClinics.Add(clinServiceList);
                                break;
                            }
                        }
                    }
                }
                if(parm.ToUpper().Contains("PATIENTNAME"))
                {
                    var patients = parm.Split('^')[1];
                    var patientIens = patients.Split('~');
                    foreach(var ien in patientIens)
                    {
                        var pat = new Patient(_dataAccessService) {IEN = ien};
                        var newPat = _dataAccessService.GetPatient(ref pat);
                        ViewModel.SelectedPatients.Add(newPat);
                    }
                }

                //Flip these for now... as rpc is wrong.. bj5-24-15
                if(parm.ToUpper().Contains("SERVICERELATED"))
                {
                    var svcRelated = parm.Split('^')[1];
                    CmbServiceVisit.Text = svcRelated;
                }

                if(parm.ToUpper().Contains("SCVISIT"))
                {
                    var scVisit = parm.Split('^')[1];
                    CmbServiceRelated.Text = scVisit;
                }

                if(parm.ToUpper().Contains("WAITTIME"))
                {
                    var waitTime = parm.Split('^')[1];
                    if (waitTime.ToUpper().Contains("ALL"))
                        waitTime = "All";
                    waitTime += " Days";
                    CmbWaitTime.Text = "WAIT TIME " + waitTime;
                }

                if(parm.ToUpper().Contains("PRIORITYGROUPS"))
                {
                    var priGroup = parm.Split('^')[1];
                    var priGroups = priGroup.Split('~');
                    foreach (var priGrp in priGroups)
                    {
                        var newPriGroup = new PriorityGroup { IsSelected = true, PriorityGroupName = priGrp };
                        ViewModel.FilteredPriorityGroups.Add(newPriGroup);
                    }
                }

                if (parm.ToUpper().Contains("REQUESTTYPE"))
                {
                    var reqType = parm.Split('^')[1];
                    var reqTypes = reqType.Split('~');
                    foreach (var reqTyp in reqTypes)
                    {
                        var newReqType = new RequestType {IsSelected = true};
                        switch (reqTyp)
                        {
                            case "A": newReqType.RequestName = "APPOINTMENT";
                                break;
                            case "V":
                                newReqType.RequestName = "VETERAN";
                                break;
                            case "R":
                                newReqType.RequestName = "PtCSch";
                                break;
                            case "E":
                                newReqType.RequestName = "EWL";
                                break;
                            case "C":
                                newReqType.RequestName = "CONSULT";
                                break;
                        }

                        ViewModel.FilteredRequestTypes.Add(newReqType);
                    }
                }

                if(parm.ToUpper().Contains("ORIGINATIONDATE"))
                    TxtOriginationDateBegin.Text = parm.Split('^')[1];

                if (parm.ToUpper().Contains("ORIGINATIONDATEEND"))
                    TxtOriginationDateEnd.Text = parm.Split('^')[1];

                if (parm.ToUpper().Contains("DESIREDDATE"))
                    TxtDesiredDateBegin.Text = parm.Split('^')[1];

                if (parm.ToUpper().Contains("DESIREDDATEEND"))
                    TxtDesiredDateEnd.Text = parm.Split('^')[1];

                if (parm.ToUpper().Contains("URGENCY"))
                    ConsultUrgency.Text = GetConsultUrgencyText(parm.Split('^')[1]);

                if (parm.ToUpper().Contains("SORT"))
                    CmbQuerySort.Text = parm.Split('^')[1];
            }
            if (string.IsNullOrEmpty(CmbWaitTime.Text))
                CmbWaitTime.Text = Keys.RmGridColumns.WaitTimeNull;

            Keyboard.Focus(PatientSearchText);
        }
        
        private string GetConsultUrgencyText(string ien)
        {
            foreach(var consult in ViewModel.ConsultUrgencyValues)
                if (ien == consult.Ien)
                    return consult.Name;
            return "";
        }

        private void LaunchRequestTypeListWindow()
        {
            Mouse.OverrideCursor = Cursors.Arrow;
            var requestTypeList = new RequestTypeListView
            {
                FilteredRequestTypes = ViewModel.FilteredRequestTypes,
                ViewModel = { ConsultUrgencyActivated = ViewModel.ConsultUrgencyActivated }
            };
            Point relativePoint = ImgReqTypeFilter.TransformToVisual(this).Transform(new Point(0, 0));
            Point absolutePoint = PointToScreen(relativePoint);
            double top =  absolutePoint.Y + (ImgReqTypeFilter.Height/2);
            double left = absolutePoint.X + (ImgReqTypeFilter.Width/2);

            CLaunchWindow.LaunchWindow(requestTypeList, "Request Types", 250, 200, top, left, "RequestTypes", true);
            ViewModel.FilteredRequestTypes.Clear();
            foreach (var rt in requestTypeList.ViewModel.RequestSelectionList)
            {
                if (rt.IsSelected)
                    ViewModel.FilteredRequestTypes.Add(rt);
                else
                    ViewModel.FilteredRequestTypes.Remove(rt);
            }
            ViewModel.NotifyPropertyChanged( "FilteredRequestTypes");
            if (ViewModel.FilteredRequestTypes.Count == 0)
                ConsultUrgency.IsEnabled = true;

            if (ViewModel.FilteredRequestTypes.Count == 1)
            {
                ConsultUrgency.IsEnabled = ViewModel.FilteredRequestTypes[0].RequestName.Contains("CONSULT");
                ViewModel.IsPtCSchOnly = ViewModel.FilteredRequestTypes[0].RequestName.Contains("PtCSch");
            }
            if (ViewModel.FilteredRequestTypes.Count > 1)
            {
                ConsultUrgency.Text = string.Empty;
                ConsultUrgency.IsEnabled = false;
            }

            Keyboard.Focus(LstRequestTypes);
        }

        private void RequestTypes_PreviewMouseDown(object sender, MouseButtonEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Arrow;
            e.Handled = true;
            LaunchRequestTypeListWindow();
        }

        private void LaunchPriortyGroupListWindow()
        {
            var priGroupList = new PriorityGroupListView
            {
                FilteredPriorityGroups = ViewModel.FilteredPriorityGroups
            };
            Point relativePoint = ImgPrtyGrpFilter.TransformToVisual(this).Transform(new Point(0, 0));
            Point absolutePoint = PointToScreen(relativePoint);
            double top = absolutePoint.Y + (ImgPrtyGrpFilter.Height / 2);
            double left = absolutePoint.X + (ImgPrtyGrpFilter.Width / 2);

            CLaunchWindow.LaunchWindow(priGroupList, "Priority Groups", 380, 225, top, left, "PriorityGroups", true);
            ViewModel.FilteredPriorityGroups.Clear();
            foreach (var rt in priGroupList.ViewModel.PriorityGroupSelectionList)
            {
                if (rt.IsSelected)
                    ViewModel.FilteredPriorityGroups.Add(rt);
                else
                    ViewModel.FilteredPriorityGroups.Remove(rt);
            }
            ViewModel.NotifyPropertyChanged( "FilteredPriorityGroups");
            Keyboard.Focus(LstPriorityGroups);
        }

        private void PriorityGroups_PreviewMouseDown(object sender, MouseButtonEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Arrow;
            LaunchPriortyGroupListWindow();
        }

        private void Clear_Click(object sender, RoutedEventArgs e)
        {
            var rslt = MessageBox.Show("Click OK to clear the current filter.", "Clear Filter", MessageBoxButton.OKCancel);

            if (rslt == MessageBoxResult.OK)
            {
                ViewModel.SelectedPatients.Clear();
                ViewModel.FilteredRequestTypes.Clear();
                ViewModel.FilteredClinics.Clear();
                ViewModel.FilteredPriorityGroups.Clear();
                CmbServiceRelated.Text       = "BOTH";
                CmbServiceVisit.Text         = "BOTH";
                TxtOriginationDateBegin.Text = "";
                TxtOriginationDateEnd.Text   = "";
                TxtDesiredDateBegin.Text     = "";
                TxtDesiredDateEnd.Text       = "";
                ConsultUrgency.Text          = "";
                CmbWaitTime.Text             = "WAIT TIME All Days";
                Keys.WaitListCurrentFilter   = string.Empty;
                Keys.QueryFormCurrentFilter  = string.Empty;
            }

        }

        private void RequestTypeFilter_MouseMove(object sender, MouseEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Hand;
        }

        private void StackPanelRequestTypeFilter_MouseMove(object sender, MouseEventArgs e)
        {
            //Mouse.OverrideCursor = Cursors.Arrow;
        }

        private void QueryText_MouseMove(object sender, MouseEventArgs e)
        {
            //Mouse.OverrideCursor = Cursors.Arrow;
        }

        private void StackPanelClinicFilter_MouseMove(object sender, MouseEventArgs e)
        {
            //Mouse.OverrideCursor = Cursors.Arrow;
        }

        private void StackPanelPriorityGroupFilter_MouseMove(object sender, MouseEventArgs e)
        {
           // Mouse.OverrideCursor = Cursors.Arrow;
        }

        private void UrgencyValue_Changed(object sender, SelectionChangedEventArgs e)
        {
            var consultText = ConsultUrgency.SelectedItem as ConsultUrgency;

            if (consultText != null && !string.IsNullOrEmpty(consultText.Name))
            {
                ViewModel.ConsultUrgencyActivated = true;
                TxtDesiredDateBegin.IsEnabled = false;
                TxtDesiredDateEnd.IsEnabled = false;
                if (!ViewModel.IsServiceList && LstClinics.Items.Count > 0)
                {
                    ViewModel.FilteredClinics.Clear();
                    ViewModel.NotifyPropertyChanged( "FilteredClinics");
                }
            }
            else
            {
                ViewModel.ConsultUrgencyActivated = false;
               
                TxtDesiredDateBegin.IsEnabled = true;
                TxtDesiredDateEnd.IsEnabled = true;
            }
        }

        private void Clinics_Click(object sender, MouseButtonEventArgs e)
        {

        }

        private void PatientSearchText_GotFocus(object sender, RoutedEventArgs e)
        {
            BtnSubmitClick.IsDefault = false;
            BtnFindPatient.IsDefault = true;
        }

        private void PatientSearchText_LostFocus(object sender, RoutedEventArgs e)
        {
            BtnSubmitClick.IsDefault = true;
            BtnFindPatient.IsDefault = false;
        }

        private void HotKeyHandler(Key k)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                switch (k)
                {
                    case Key.L:
                        Keyboard.Focus(BtnClearClick);
                        break;
                    case Key.S:
                        Keyboard.Focus(BtnSubmitClick);
                        break;
                    case Key.C:
                        Keyboard.Focus(BtnExitClick);
                        break;
                    case Key.F:
                        Keyboard.Focus(BtnFindPatient);
                        break;
                    case Key.W:
                        Keyboard.Focus(CmbWaitTime);
                        break;
                    case Key.V:
                        Keyboard.Focus(CmbServiceVisit);
                        break;
                    case Key.N:
                        Keyboard.Focus(CmbServiceRelated);
                        break;
                    case Key.U:
                        Keyboard.Focus(ConsultUrgency);
                        break;
                    case Key.P:
                        Keyboard.Focus(CmbQuerySort);
                        break;
                }
            }
        }

        private void UserControl_PreviewKeyUp(object sender, KeyEventArgs e)
        {
            var key = e.Key == Key.System ? e.SystemKey : e.Key;
            HotKeyHandler(key);
        }

        private void AddToSelectedPatients()
        {
            if (!PatientSearchResults.HasItems)
                return;
            
            var patient = PatientSearchResults.SelectedItem as Patient;
            var selectedIndex = PatientSearchResults.SelectedIndex;
            if (patient == null)
            {
                _dispatcherTimer.Start();
                return;
            }
            if (ViewModel.SelectedPatients.Any(pat => patient.IEN == pat.IEN))
            {
                _dispatcherTimer.Start();
                return;
            }

            var promptRslt = ViewModel.DataAccessService.GetPatientSelectionPrompts(patient.IEN, ref patient, false, false);

            if (promptRslt == "0")
            {
                _dispatcherTimer.Start();
                return;
            }

            ViewModel.SelectedPatients.Add(patient);
            PatientSearchResults.SelectedIndex = selectedIndex;
            _dispatcherTimer.Start();
        }

        private void PatientSearchResults_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            e.Handled = true;
            AddToSelectedPatients();
        }

        private void PatientSearchResults_KeyDown( object sender, KeyEventArgs e )
        {
            if ( ( e.Key == Key.Space || e.Key == Key.Enter ) && Keyboard.Modifiers == ModifierKeys.None )
            {
                AddToSelectedPatients();
                e.Handled = true;
            }
        }

        private void RemoveFromSelectedPatients()
        {
            if (!LstSelectedPatients.HasItems)
                return;
            var patient = LstSelectedPatients.SelectedItem as Patient;
            ViewModel.SelectedPatients.Remove(patient);
            ViewModel.NotifyPropertyChanged( "SelectedPatients");
        }

        private void LstSelectedPatients_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            RemoveFromSelectedPatients();
            e.Handled = true;
        }

        private void LstSelectedPatients_PreviewKeyDown( object sender, KeyEventArgs e )
        {
            if ( ( e.Key == Key.Space || e.Key == Key.Enter ) && Keyboard.Modifiers == ModifierKeys.None )
            {
                RemoveFromSelectedPatients();
                e.Handled = true;
            }
        }

        private void RemoveFromRequestTypes(){
            if (!LstRequestTypes.HasItems)
                return;

            var item = LstRequestTypes.SelectedItem as RequestType;
            ViewModel.FilteredRequestTypes.Remove(item);
            ViewModel.NotifyPropertyChanged( "RequestTypes");

            if (LstRequestTypes.Items.Count == 0)
                ConsultUrgency.IsEnabled = true;

            if (LstRequestTypes.Items.Count == 1)
            {
                ConsultUrgency.IsEnabled = ViewModel.FilteredRequestTypes[0].RequestName.ToUpper().Contains("CONSULT");
                ViewModel.IsPtCSchOnly = ViewModel.FilteredRequestTypes[0].RequestName.ToUpper().Contains("PtCSch");
            }
            if (LstRequestTypes.Items.Count > 1)
            {
                ConsultUrgency.Text = string.Empty;
                ConsultUrgency.IsEnabled = false;
            }
        }

        private void LstRequestTypes_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            RemoveFromRequestTypes();
            e.Handled = true;
        }

        private void LstRequestTypes_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if ((e.Key == Key.Space || e.Key == Key.Enter) && Keyboard.Modifiers == ModifierKeys.None)
            {
                RemoveFromRequestTypes();
                e.Handled = true;
            }
        }

        private void RemoveFromClinics()
        {
            if (!LstClinics.HasItems)
                return;
            var item = LstClinics.SelectedItem as ClinicServiceList;
            ViewModel.FilteredClinics.Remove(item);
            ViewModel.NotifyPropertyChanged( "FilteredClinics");
            if (LstClinics.Items.Count == 0)
                ConsultUrgency.IsEnabled = true;
            else
            {
                ConsultUrgency.Text = string.Empty;
                ConsultUrgency.IsEnabled = false;
            }
        }

        private void LstClinics_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            RemoveFromClinics();
            e.Handled = true;
        }

        private void LstClinics_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if ((e.Key == Key.Space || e.Key == Key.Enter) && Keyboard.Modifiers == ModifierKeys.None)
            {
                RemoveFromClinics();
                e.Handled = true;
            }
        }

        private void RemoveFromPriorityGroups()
        {
            if (!LstPriorityGroups.HasItems)
                return;
            var item = LstPriorityGroups.SelectedItem as PriorityGroup;
            ViewModel.FilteredPriorityGroups.Remove(item);
            ViewModel.NotifyPropertyChanged( "FilteredPriorityGroups");
        }

        private void LstPriorityGroups_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            RemoveFromPriorityGroups();
            e.Handled = true;
        }

        private void LstPriorityGroups_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if ((e.Key == Key.Space || e.Key == Key.Enter) && Keyboard.Modifiers == ModifierKeys.None)
            {
                RemoveFromPriorityGroups();
                e.Handled = true;
            }
        }

        private void BtnExitClick_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.None && e.Key == Key.Tab)
            {
                e.Handled = true;
                Keyboard.Focus(PatientSearchText);
            }
        }

        private void PatientSearchResults_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if (PatientSearchResults.HasItems)
            {   
                var item = PatientSearchResults.ItemContainerGenerator.ContainerFromIndex(0) as ListBoxItem;
                if(item!=null)
                KeyboardNavigation.SetTabIndex(item,20);
            }
        }

        private void LstSelectedPatients_CollectionChanged( object sender, NotifyCollectionChangedEventArgs e )
        {
            if ( LstSelectedPatients.HasItems )
            {
                var item = LstSelectedPatients.ItemContainerGenerator.ContainerFromIndex( 0 ) as ListBoxItem;
                if ( item != null )
                    KeyboardNavigation.SetTabIndex( item, 30 );
            }
        }

        private void LstRequestTypes_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if (LstRequestTypes.HasItems)
            {
                var item = LstRequestTypes.ItemContainerGenerator.ContainerFromIndex(0) as ListBoxItem;
                if(item!=null)
                KeyboardNavigation.SetTabIndex(item,50);
            }
        }

        private void LstClinics_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if (LstClinics.HasItems)
            {
                var item = LstClinics.ItemContainerGenerator.ContainerFromIndex(0) as ListBoxItem;
                if(item!=null)
                KeyboardNavigation.SetTabIndex(item,70);
            }
        }

        private void LstPriorityGroups_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if (LstPriorityGroups.HasItems)
            {
                var item = LstPriorityGroups.ItemContainerGenerator.ContainerFromIndex(0) as ListBoxItem;
                if(item!=null)
                KeyboardNavigation.SetTabIndex(item,90);
            }
        }

        private void ImgReqTypeFilter_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if ((e.Key == Key.Space || e.Key == Key.Enter) && Keyboard.Modifiers == ModifierKeys.None)
            {
                LaunchRequestTypeListWindow();
                e.Handled = true;
            }
        }

        private void Clinics_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if ((e.Key == Key.Space || e.Key == Key.Enter) && Keyboard.Modifiers == ModifierKeys.None)
            {
                e.Handled = true;
                LaunchClinicListWindow();
            }
        }

        private void PriorityGroups_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if ((e.Key == Key.Space || e.Key == Key.Enter) && Keyboard.Modifiers == ModifierKeys.None)
            {
                e.Handled = true;
                LaunchPriortyGroupListWindow();
            }            
        }

        private void TxtDateControl_TextChanged(object sender, TextChangedEventArgs e)
        {
            TextBox textBox = (TextBox)sender;

            if (!string.IsNullOrEmpty(textBox.Text))
            {
                CmbWaitTime.SelectedIndex = CmbWaitTime.Items.IndexOf(Keys.RmGridColumns.WaitTimeNull);

                if (textBox.Name.StartsWith("TxtOriginationDate"))
                {
                    TxtDesiredDateBegin.Text = string.Empty;
                    TxtDesiredDateEnd.Text = string.Empty;
                }
                else if (textBox.Name.StartsWith("TxtDesiredDate"))
                {
                    TxtOriginationDateBegin.Text = string.Empty;
                    TxtOriginationDateEnd.Text = string.Empty;
                }
            }
        }
    }
}
