using System;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Windows;

using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using Microsoft.Practices.Composite.Presentation.Commands;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Composite.Regions;

using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ScheduleView;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Task.Group;
using ClinSchd.Modules.Task.Availability.Controllers;

namespace ClinSchd.Modules.Task.Availability
{
    public class AvailabilityPresentationModel : IAvailabilityPresentationModel, ITaskPresentationModel, INotifyPropertyChanged
    {
        private readonly IRegionManager _regionManager;
        public readonly IDataAccessService DataAccessService;
        private readonly IEventAggregator _eventAggregator;
        public event PropertyChangedEventHandler PropertyChanged;
        private string _paneTitle = "";
        private int _dayRange = 1;
        private TimeSpan _timeScale = TimeSpan.FromMinutes(10);
        private bool _isSelectedTab = true;
        private SchdResource _selectedResource;
        private SchdResourceGroup _selectedResourceGroup;
        private IList<SchdAvailability> _availabilityList;
        private SchdAvailability _schdAvailability;
        private DateTime _visibleStartDate;
        private DateTime _visibleEndDate;
        private string _resourceName;
        private ValidationMessage _validationMessage;
        private int _timeSlotToSkip = 15;
        private bool _isEnableUpdateAppointment = true;
        private string _applyClearOption = "EntireWeek";

        private DateTime _currentDate;
        private Slot _selectedTimeSlot = null;
        private Appointment _selectedAppointment = null;
        private ViewDefinitionBase _activeViewDefinition;
        private readonly Dictionary<string, Category> _appointmentCategories = new Dictionary<string, Category>();
        private readonly ObservableCollection<Category> _categoriesList = new ObservableCollection<Category>();
        private readonly ObservableCollection<Appointment> _appointmentsCollection = new ObservableCollection<Appointment>();
        private readonly ObservableCollection<CustomSlot> _slotsCollection = new ObservableCollection<CustomSlot>();
        private IList<ResourceType> _resourceTypesCollection = null;
        private IList<GroupDescription> _groupsDescriptionCollection = null;

        public AvailabilityPresentationModel(
            IRegionManager regionManager,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            _regionManager = regionManager;
            DataAccessService = dataAccessService;
            _eventAggregator = eventAggregator;
            ValidationMessage = new ValidationMessage();
            var now = DateTime.Today;

            if (now.DayOfWeek != DayOfWeek.Monday)
                now = now.AddDays(DayOfWeek.Monday - now.DayOfWeek);

            SelectedStartDate = now;
            InitialErrorMessage();

            TaskTabSelectedCommand = new DelegateCommand<object>(ExecuteSchedulerTabSelectedCommand, CanExecuteSchedulerTabSelectedCommand);
            AddEditAvailabilityCommand = new DelegateCommand<string>(ExecuteAddEditAvailabilityCommand, CanExecuteAddEditAvailabilityCommand);
            PasteAvailabilityCommand = new DelegateCommand<string>(ExecutePasteAvailabilityCommand, CanExecutePasteAvailabilityCommand);
            CopyAvailabilityCommand = new DelegateCommand<string>(ExecuteCopyAvailabilityCommand, CanExecuteCopyAvailabilityCommand);
            SaveTemplateCommand = new DelegateCommand<string>(ExecuteSaveTemplateCommand, CanExecuteSaveTemplateCommand);
            ApplyTemplateCommand = new DelegateCommand<string>(ExecuteApplyTemplateCommand, CanExecuteApplyTemplateCommand);
            ClearAvailabilityCommand = new DelegateCommand<string>(ExecuteClearAvailabilityCommand, CanExecuteClearAvailabilityCommand);
            ApplyClearRadioButtonCommand = new DelegateCommand<string>(ExecuteApplyClearRadioButtonCommand, CanExecuteApplyClearRadioButtonCommand);
            VisibleRangeChangedCommand = new DelegateCommand<object>(ExecuteVisibleRangeChangedCommand, CanExecuteVisibleRangeChangedCommand);

            _eventAggregator.GetEvent<ViewDayRangeEvent>().Subscribe(DayRangeChanged, ThreadOption.UIThread, true);
            _eventAggregator.GetEvent<ViewTimeScaleEvent>().Subscribe(TimeScaleChanged, ThreadOption.UIThread, true);
            _eventAggregator.GetEvent<ChangeViewModeEvent>().Subscribe(ViewModeChanged, ThreadOption.UIThread, false);
            _eventAggregator.GetEvent<SchedulerViewDateEvent>().Subscribe(SchedulerDateChanged, ThreadOption.UIThread, false);
            _eventAggregator.GetEvent<RefreshAvailability>().Subscribe(RefreshAvailabilityScheduler, ThreadOption.UIThread, true);
            _eventAggregator.GetEvent<ResourceSelectedEvent>().Subscribe(ResourceSelected, ThreadOption.UIThread, false);
            _eventAggregator.GetEvent<ResourceGroupSelectedEvent>().Subscribe(ResourceGroupSelected, ThreadOption.UIThread, false);
            _eventAggregator.GetEvent<PasteAvailabilityEvent>().Subscribe(PasteAppointment, ThreadOption.UIThread, false);
            _eventAggregator.GetEvent<CopyAvailabilityEvent>().Subscribe(CopyAppointment, ThreadOption.UIThread, false);
        }

        public string ScheduleID { get; set; }

        //public Patient Patient { get; set; }

        public DateTime CurrentDate
        {
            get { return _currentDate; }
            set
            {
                if (value.Date.CompareTo(_currentDate.Date) != 0)
                {
                    _currentDate = value.Date;
                    OnPropertyChanged("CurrentDate");
                }
            }
        }

        public ViewDefinitionBase ActiveViewDefinition
        {
            get { return _activeViewDefinition; }
            set
            {
                if (_activeViewDefinition != value)
                {
                    _activeViewDefinition = value;
                    OnPropertyChanged("ActiveViewDefinition");
                }
            }
        }

        public Appointment SelectedAppointment
        {
            get { return _selectedAppointment; }
            set
            {
                _selectedAppointment = value;
                OnPropertyChanged("SelectedAppointment");
            }
        }

        public Slot SelectedTimeSlot
        {
            get { return _selectedTimeSlot; }
            set
            {
                _selectedTimeSlot = value;
                OnPropertyChanged("SelectedTimeSlot");
            }
        }

        public Dictionary<string, Category> AppointmentCategories
        {
            get
            {
                if (!_appointmentCategories.Any())
                {
                    _appointmentCategories.Add(Keys.AccessTypes.Available, new Category("Available", Keys.ColorCodes.AvailableSlotColor));
                    _appointmentCategories.Add(Keys.AccessTypes.Unavailable, new Category("Unavailable", Keys.ColorCodes.UnavailableSlotColor));
                    _appointmentCategories.Add(Keys.AccessTypes.Cancelled, new Category("Cancelled", Keys.ColorCodes.CancelledAvailabilitySlotColor));
                    _appointmentCategories.Add(Keys.Categories.Holiday_Event, new Category("AllDayEvent", Keys.ColorCodes.AllDayEventAreaColor));
                }

                return _appointmentCategories;
            }
        }

        public ObservableCollection<Appointment> AppointmentsCollection { get { return _appointmentsCollection; } }

        public ObservableCollection<Category> CategoriesList
        {
            get
            {
                if (_categoriesList.Count == 0)
                    foreach (var pair in AppointmentCategories)
                        _categoriesList.Add(pair.Value);

                return _categoriesList;
            }
        }

        public ObservableCollection<CustomSlot> SlotsCollection { get; } = new ObservableCollection<CustomSlot>();

        public IList<ResourceType> ResourceTypesCollection
        {
            get
            {
                if (_resourceTypesCollection == null)
                {
                    _resourceTypesCollection = new List<ResourceType>
                    {
                        new ResourceType(Keys.AppointmentGroupNames.Appointment)
                    };
                }

                return _resourceTypesCollection;
            }
        }

        public IList<GroupDescription> GroupsDescriptionCollection
        {
            get
            {
                if (_groupsDescriptionCollection == null)
                {
                    _groupsDescriptionCollection = new List<GroupDescription>();
                    _groupsDescriptionCollection.Add(new DateGroupDescription());
                }

                return _groupsDescriptionCollection;
            }
        }

        public int FirstVisibleHour { get; private set; } = 8;

        /**************************************************************/
        
        public DelegateCommand<object> TaskTabSelectedCommand { get; }
        public DelegateCommand<string> AddEditAvailabilityCommand { get; private set; }
        public DelegateCommand<string> PasteAvailabilityCommand { get; private set; }
        public DelegateCommand<string> CopyAvailabilityCommand { get; private set; }
        public DelegateCommand<string> SaveTemplateCommand { get; private set; }
        public DelegateCommand<string> ApplyTemplateCommand { get; private set; }
        public DelegateCommand<string> ClearAvailabilityCommand { get; private set; }
        public DelegateCommand<string> ApplyClearRadioButtonCommand { get; private set; }
        public DelegateCommand<object> VisibleRangeChangedCommand { get; private set; }

        public void InitialErrorMessage ()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
        }

        public void RefreshAvailabilityScheduler(string command)
        {
            if (command == "INITIAL_LAUNCH")
                CurrentDate = DateTime.Today;

            RefreshAvailabilityScheduler();
        }

        private void RefreshAvailabilityScheduler(DateSpan visibleDateRange)
        {
            // Set the first visible date for this class
            VisibleStartDate = visibleDateRange.Start.Date;
            // Set the end of the last visible date
            VisibleEndDate = visibleDateRange.End.Date.AddDays(1).AddMinutes(-1);
            // Refresh the calendar
            RefreshAvailabilityScheduler();
        }

        private void RefreshAvailabilityScheduler()
        {
            Keys.IsAvailabilitySearch = true;

            // Set the first visible hour
            if (SelectedResource == null)
                return;

            FirstVisibleHour = Convert.ToInt32(SelectedResource.Clinic.HOUR_DISPLAY_BEGIN);

            // Initialize the calendar
            AvailabilityList = SelectedResource.GetAvailability(VisibleStartDate, VisibleEndDate);
            var appointmentsSource = new SchdAppointmentSource(AvailabilityList, AppointmentCategories, IsEnableUpdateAppointment).Appointments;
            var hStart = VisibleStartDate.Date;

            do
            {
                var sHoliday = DataAccessService.IsHoliday(hStart);

                if (!string.IsNullOrEmpty(sHoliday))
                {
                    var holidayEvent = new CustomAppointment
                    {
                        Body = sHoliday,
                        Start = hStart.Date,
                        Subject = sHoliday,
                        Category = _appointmentCategories[Keys.Categories.Holiday_Event],
                        IsAllDayEvent = true
                    };
                    holidayEvent.End = holidayEvent.Start.AddDays(1).AddMinutes(-1);
                    holidayEvent.Resources.Add(new Resource(Keys.AppointmentGroupNames.Appointment, Keys.AppointmentGroupNames.Appointment));
                    appointmentsSource.Add(holidayEvent);
                }

                hStart = hStart.AddDays(1);
            } while (hStart <= VisibleEndDate.Date.AddDays(1));

            AppointmentsCollection.Clear();

            foreach (IAppointment appt in appointmentsSource)
            {
                // Set the Grouping Value
                CustomAppointment customAppt = (CustomAppointment) appt;
                customAppt.Resources.Add(new Resource(SelectedResource.ScheduleTabCaption, Keys.AppointmentGroupNames.Appointment));
                AppointmentsCollection.Add(customAppt);
            }

            View.ScrollSchedulerToOffset();
        }

        public void CopyAppointment (SchdAvailability copyAppt)
        {
            CopySchdAvailability = copyAppt;
        }

        public void PasteAppointment (SchdAppointment pasteAppt)
        {
            if (IsSelectedTab)
            {
                var duration = ( Convert.ToDateTime( CopySchdAvailability.EndTime ) - Convert.ToDateTime( CopySchdAvailability.StartTime ) ).TotalMinutes.ToString();

                var a = new SchdAvailability
                {
                    APPOINTMENTID = null,
                    StartTime     = pasteAppt.START_TIME,
                    EndTime       = Convert.ToDateTime( pasteAppt.START_TIME ).AddMinutes( int.Parse( duration ) ).ToString(),
                    ACCESSTYPEID  = CopySchdAvailability.ACCESSTYPEID,
                    Note          = CopySchdAvailability.Note,
                    RESOURCENAME  = CopySchdAvailability.RESOURCENAME,
                    SLOTS         = CopySchdAvailability.SLOTS
                };

                var errorMessage = DataAccessService.AddNewAvailability (a);

                if (errorMessage != string.Empty)
                {
                    View.AlertUser (errorMessage, "Copy/Paste Availability");
                    a.StartTime = SchdAvailability.StartTime;
                    a.EndTime = SchdAvailability.EndTime;
                    errorMessage = DataAccessService.AddNewAvailability (a);
                }
                RefreshAvailabilityScheduler (string.Empty);
            }
        }

        public void ResourceSelected (SchdResource selectedResource)
        {
            Resource = selectedResource;
            var newGroupList = new List<SchdResource> ();
            newGroupList.Insert (0, Resource);
            SelectedResourceList = newGroupList;
        }

        public void ResourceGroupSelected (SchdResourceGroup selectedResourceGroup)
        {
            ResourceGroup = selectedResourceGroup;
            SelectedResourceList = ResourceGroup.Resources;
        }

        private void SchedulerDateChanged(SchedulerViewDateEventArgs newDate)
        {
            if (IsSelectedTab) {
//				this.View.Scheduler.SelectedSlot .VisibleRange. .Start = newDate.NewDate;
                RefreshAvailabilityScheduler (string.Empty);
            }
        }

        public void DayRangeChanged (string dayRange)
        {
            DayRange = int.Parse (dayRange);

            //if (IsSelectedTab) {
            //	this.View.Scheduler.WeekViewDefinition.VisibleDays = _dayRange;
            //}
        }

        public void ViewModeChanged (string viewMode)
        {
            //if (IsSelectedTab) {
            //	switch (viewMode) {
            //	case "Day":
            //		this.View.Scheduler.ViewMode = SchedulerViewMode.Day;
            //		this.View.Scheduler.SetFirstVisibleDate (this.VisibleStartDate);
            //		break;
            //	case "Month":
            //		this.View.Scheduler.ViewMode = SchedulerViewMode.Month;
            //		this.View.Scheduler.SetFirstVisibleDate (this.VisibleStartDate);
            //		break;
            //	default:
            //		this.View.Scheduler.ViewMode = SchedulerViewMode.Week;
            //		this.View.Scheduler.SetFirstVisibleDate (this.VisibleStartDate);
            //		break;
            //	}
            //}
        }

        public void TimeScaleChanged (string timeScale)
        {
            TimeScale = TimeSpan.FromMinutes(5);
            TimeSlotToSkip = 92;
/*
            switch (timeScale) {
            case "10":
                _timeScale = System.TimeSpan.FromMinutes(10);
                _timeSlotToSkip = 46;
                break;
            case "15":
                _timeScale = System.TimeSpan.FromMinutes(15);
                _timeSlotToSkip = 10;
                break;
            case "20":
                _timeScale = System.TimeSpan.FromMinutes (20);
                _timeSlotToSkip = 15;
                break;
            case "30":
                _timeScale = System.TimeSpan.FromMinutes (30);
                _timeSlotToSkip = 15;
                break;
            case "60":
                _timeScale = TimeSpan.FromMinutes(60);
                _timeSlotToSkip = 30;
                break;
            default:
                _timeScale = System.TimeSpan.FromMinutes (30);
                _timeSlotToSkip = 15;
                break;
            }
//*/
            //if (IsSelectedTab) {
            //	this.View.Scheduler.ViewDefinitions[0].SlotLength = _timeScale;
            //	this.View.Scheduler.DayViewDefinition.TimeSlotLength = _timeScale;
            //}

            //OnPropertyChanged ("TimeScale");
        }

        public void ExecuteCopyAvailabilityCommand (string command)
        {
            _eventAggregator.GetEvent<EnableAppointmentButtonsEvent> ().Publish (command);
            _eventAggregator.GetEvent<CopyAvailabilityEvent> ().Publish (SchdAvailability);
        }

        public bool CanExecuteCopyAvailabilityCommand (string command)
        {
            return true;
        }

        public void ExecutePasteAvailabilityCommand (string command)
        {
            _eventAggregator.GetEvent<EnableAppointmentButtonsEvent> ().Publish (command);
            _eventAggregator.GetEvent<PassAppointmentsEvent> ().Publish (SelectedSchdAppointment);
            _eventAggregator.GetEvent<IsPasteAvailabilityEvent> ().Publish (command);
        }

        public bool CanExecutePasteAvailabilityCommand (string command)
        {
            return true;
        }

        public void ExecuteSchedulerTabSelectedCommand (object resource)
        {
            _selectedResource = resource as SchdResource;
            _eventAggregator.GetEvent<ResourceSelectedEvent> ().Publish (_selectedResource);

            var selectedIndex = 3;
            if (IsSelectedTab)
            {
                //switch (this.View.Scheduler.ViewMode) {
                //case SchedulerViewMode.Week:
                //	selectedMin = this.View.Scheduler.WeekViewDefinition.TimeSlotLength.Minutes;
                //	break;
                //case SchedulerViewMode.Day:
                //	selectedMin = this.View.Scheduler.DayViewDefinition.TimeSlotLength.Minutes;
                //	break;
                //default:
                    var selectedMin = 30;
                //	break;
                //}

                switch (selectedMin) {
                case 10:
                    selectedIndex = 0;
                    break;
                case 15:
                    selectedIndex = 1;
                    break;
                case 20:
                    selectedIndex = 2;
                    break;
                case 30:
                    selectedIndex = 3;
                    break;
                }
                _eventAggregator.GetEvent<SelectTimeScaleEvent> ().Publish (selectedIndex);
            }
        }

        public bool CanExecuteSchedulerTabSelectedCommand (object resource)
        {
            return true;
        }

        public void ExecuteAddEditAvailabilityCommand (string command)
        {
            _eventAggregator.GetEvent<AddEditAvailabilityEvent> ().Publish (SchdAvailability);
        }

        public bool CanExecuteAddEditAvailabilityCommand (object resource)
        {
            return true;
        }

        public void ExecuteSaveTemplateCommand (string command)
        {
            //try
            //{
            //    var saveFileDialog1 = new SaveFileDialog
            //    {
            //        Filter           = "Schedule Template Files (*.sdeca*)|*.sdeca*|All files (*.*)|*.*",
            //        FilterIndex      = 0,
            //        RestoreDirectory = true,
            //        AddExtension     = true,
            //        DefaultExt       = "sdeca4"
            //    };


            //    foreach (var item in AvailabilityList)
            //    {
            //        item.GridColumn = DateTime.Parse( item.StartTime ).Subtract( VisibleStartDate ).Days;
            //    }

            //    if (saveFileDialog1.ShowDialog ().Value)
            //    {
            //        Stream streamFile ;
            //        if ((streamFile = saveFileDialog1.OpenFile ()) != null)
            //        {
            //            var formatter = new BinaryFormatter
            //            {
            //                TypeFormat = System.Runtime.Serialization.Formatters.FormatterTypeStyle.TypesAlways,
            //                AssemblyFormat = System.Runtime.Serialization.Formatters.FormatterAssemblyStyle.Simple
            //            };
            //            formatter.Serialize (streamFile, AvailabilityList);

            //            streamFile.Close ();
            //        }
            //    }
            //}
            //catch (Exception ex)
            //{
            //    MessageBox.Show("Error saving template:  " + ex.Message, "Availablility");
            //}
        }

        private bool CanExecuteSaveTemplateCommand (string command)
        {
            return true;
        }

        private void ExecuteApplyTemplateCommand (string command)
        {
            ///*
            // * Display dialog to collect:
            // * - Number of weeks to apply template
            // * - Starting week Monday
            // * - Template path & filename
            // * 
            // * for each week,
            // *	Delete all availability during that week
            // *  apply the template
            // * 
            // */
            ////DAccessTemplate dlg = new DAccessTemplate ();
            ////dlg.InitializePage ();
            ////if (dlg.ShowDialog (this) == DialogResult.Cancel) {
            ////    return;
            ////}

            //try {
            //    // Get the Template Name
            //    var openFileDialog1 = new OpenFileDialog
            //    {
            //        Filter           = "Schedule Template Files (*.sdeca*)|*.sdeca*|All files (*.*)|*.*",
            //        FilterIndex      = 0,
            //        RestoreDirectory = true,
            //        AddExtension     = true,
            //        DefaultExt       = "sdeca"
            //    };
            //    if ( !openFileDialog1.ShowDialog().Value )
            //        return;

            //    // Open the Template File
            //    Stream streamFile;
            //    if ((streamFile = openFileDialog1.OpenFile ()) == null)
            //    {
            //        MessageBox.Show ("Unable to open template file.");
            //        return;
            //    }

            //    IList<SchdAvailability> cgaTemp = new List<SchdAvailability> ();
            //    var formatter = new BinaryFormatter ();

            //    try
            //    {
            //        // Deserialize the Template Information
            //        cgaTemp = (IList<SchdAvailability>)formatter.Deserialize( streamFile );
            //    }
            //    catch ( InvalidCastException )
            //    {
            //        // Initialial Deserialization failed, try another method
            //        formatter.AssemblyFormat = System.Runtime.Serialization.Formatters.FormatterAssemblyStyle.Simple;
            //        streamFile.Position = 0;
            //        var cgaConvertTemp = (CGAppointments)formatter.Deserialize( streamFile );
            //        foreach ( DictionaryEntry item in cgaConvertTemp )
            //        {
            //            var apptItem = item.Value as CGAppointment;
            //            cgaTemp.Add( new SchdAvailability
            //            {
            //                StartTime      = apptItem.StartTime.ToString(),
            //                EndTime        = apptItem.EndTime.ToString(),
            //                ACCESSTYPEID   = apptItem.AccessTypeID.ToString(),
            //                ACCESSTYPENAME = apptItem.AccessTypeName,
            //                APPOINTMENTID  = apptItem.AppointmentKey.ToString(),
            //                RESOURCENAME   = apptItem.Resource,
            //                SLOTS          = apptItem.Slots
            //            } );
            //        }
            //    }

            //    streamFile.Close ();

            //    // Initialize the ApplyTemplateView variables for Binding
            //    var colCount = ((List<SchdAvailability>)cgaTemp).GroupBy(s => s.GridColumn).Count();
            //    ApplyTemplateViewTitle = "Apply Template";
            //    SelectedDaysListVisibility = colCount == 1 ? Visibility.Visible : Visibility.Collapsed;
            //    SelectOverwriteVisibility = Visibility.Visible;
            //    _applyClearOption = "EntireWeek";

            //    // Open the ApplyTemplateView dialog
            //    var applyTemplateView = new ApplyTemplateView
            //    {
            //        DataContext = this
            //    };
            //    if ( !applyTemplateView.ShowDialog().Value )
            //        return;

            //    var defaultCursor = Mouse.OverrideCursor;
            //    Mouse.OverrideCursor = Cursors.Wait;

            //    try
            //    {
            //        // Initialize processing variables
            //        var dtStart = SelectedStartDate;
            //        if (dtStart.DayOfWeek != DayOfWeek.Monday)
            //            dtStart = dtStart.AddDays(-(dtStart.DayOfWeek - DayOfWeek.Monday));

            //        if (dtStart < MinDate)
            //            dtStart = dtStart.AddDays(7);

            //        var nWeeksToApply = Convert.ToInt32(SelectedNumberOfWeeks);
            //        var dtEnd = dtStart.AddDays(6); // or 7?
            //        var sResourceID = SelectedResource.RESOURCEID;
            //        var sResource = _selectedResource.RESOURCE_NAME;
            //        var clearEntireWeek = _applyClearOption == "EntireWeek";

            //        if (SelectedDaysListVisibility == Visibility.Visible)
            //        {
            //            // Apply a single day template to a specified range of weeks and days

            //            for (var j = 1; j < nWeeksToApply + 1; j++)
            //            {
            //                //Convert start and end to string
            //                var sStart = dtStart.ToString("M/d/yyyy");
            //                var sEnd = dtEnd.ToString("M/d/yyyy");

            //                if (clearEntireWeek)
            //                    //Cancel all existing access blocks in the date range
            //                    DataAccessService.CancelAccessBlocks(sResourceID, sResource, sStart, sEnd);

            //                foreach (var day in SelectedDaysList)
            //                    if (day.IsChecked)
            //                    {
            //                        // Get the column value from the day
            //                        var col = Convert.ToInt32(day.Value);

            //                        if (!clearEntireWeek)
            //                        {
            //                            // Define the Date to Clear
            //                            var clearDate = dtStart.Date.AddDays(col).ToString();
            //                            // Cancel all existing access blocks for the target week
            //                            DataAccessService.CancelAccessBlocks(sResourceID, sResource, clearDate, clearDate);
            //                        }

            //                        // Apply blocks to the current day
            //                        foreach (var a in cgaTemp)
            //                        {
            //                            //Change the resource to the current one
            //                            a.RESOURCENAME = sResource;
            //                            // Define the StartDate\Time
            //                            var dBuildDate = dtStart.Date;
            //                            dBuildDate = dBuildDate.AddDays(col);
            //                            dBuildDate = dBuildDate.AddHours(DateTime.Parse(a.StartTime).Hour);
            //                            dBuildDate = dBuildDate.AddMinutes(DateTime.Parse(a.StartTime).Minute);
            //                            a.StartTime = dBuildDate.ToString();
            //                            // Define the end time
            //                            dBuildDate = dtStart.Date;
            //                            dBuildDate = dBuildDate.AddDays(col);
            //                            dBuildDate = dBuildDate.AddHours(DateTime.Parse(a.EndTime).Hour);
            //                            dBuildDate = dBuildDate.AddMinutes(DateTime.Parse(a.EndTime).Minute);
            //                            a.EndTime = dBuildDate.ToString();

            //                            // Add the new Availability
            //                            DataAccessService.AddNewAvailability(a);
            //                        }
            //                    }

            //                //Increment start and end
            //                dtStart = dtStart.AddDays(7);
            //                dtEnd = dtStart.AddDays(6);
            //            }//end for
            //        }
            //        else
            //        {
            //            // Apply a multi-day template to the specified weeks
            //            for (var j = 1; j < nWeeksToApply + 1; j++)
            //            {
            //                //Convert start and end to string
            //                var sStart = dtStart.ToString("M/d/yyyy");
            //                var sEnd = dtEnd.ToString("M/d/yyyy");

            //                if (clearEntireWeek)
            //                    //Cancel all existing access blocks in the date range
            //                    DataAccessService.CancelAccessBlocks(sResourceID, sResource, sStart, sEnd);

            //                var col = -1;

            //                //for each CGAppointment in AVBlocks, call AddNew
            //                foreach (var a in cgaTemp)
            //                {
            //                    //Change the resource to the current one
            //                    a.RESOURCENAME = sResource;

            //                    //Change the date to correspond to the GridColumn member
            //                    if (a.GridColumn != col)
            //                    {
            //                        col = a.GridColumn;

            //                        if (!clearEntireWeek)
            //                        {
            //                            // Define the Date to Clear
            //                            var clearDate = dtStart.Date.AddDays(col).ToString();
            //                            // Cancel all existing access blocks for the target week
            //                            DataAccessService.CancelAccessBlocks(sResourceID, sResource, clearDate, clearDate);
            //                        }
            //                    }

            //                    var dBuildDate = dtStart.Date;
            //                    dBuildDate = dBuildDate.AddDays(col);
            //                    dBuildDate = dBuildDate.AddHours(DateTime.Parse(a.StartTime).Hour);
            //                    dBuildDate = dBuildDate.AddMinutes(DateTime.Parse(a.StartTime).Minute);
            //                    a.StartTime = dBuildDate.ToString();
            //                    dBuildDate = dtStart.Date;
            //                    dBuildDate = dBuildDate.AddDays(col);
            //                    dBuildDate = dBuildDate.AddHours(DateTime.Parse(a.EndTime).Hour);
            //                    dBuildDate = dBuildDate.AddMinutes(DateTime.Parse(a.EndTime).Minute);
            //                    a.EndTime = dBuildDate.ToString();

            //                    // Add the new Availability
            //                    DataAccessService.AddNewAvailability(a);
            //                }

            //                //Increment start and end
            //                dtStart = dtStart.AddDays(7);
            //                dtEnd = dtStart.AddDays(6);

            //            }//end for
            //        }

            //        RefreshAvailabilityScheduler("APPLY_TEMPLATE"");
            //    }
            //    finally
            //    {
            //        Mouse.OverrideCursor = defaultCursor;
            //    }
            //} catch (Exception ex) {
            //    MessageBox.Show ("Error loading template:  " + ex.Message, "Availablility");
            //}		    
        }

        private bool CanExecuteApplyTemplateCommand (string command)
        {
            return true;
        }

        private void ExecuteClearAvailabilityCommand(string command)
        {
            ///*
            // * Display dialog to collect:
            // * - Number of weeks to apply template
            // * - Starting week Monday
            // * - Template path & filename
            // * 
            // * for each week,
            // *	Delete all availability during that week
            // *  apply the template
            // * 
            // */
            ////DAccessTemplate dlg = new DAccessTemplate ();
            ////dlg.InitializePage ();
            ////if (dlg.ShowDialog (this) == DialogResult.Cancel) {
            ////    return;
            ////}

            //try
            //{
            //    ApplyTemplateViewTitle = "Clear Availability";
            //    SelectedDaysListVisibility = Visibility.Collapsed;
            //    SelectOverwriteVisibility = Visibility.Collapsed;
            //    var applyTemplateView = new ApplyTemplateView();
            //    applyTemplateView.DataContext = this;
            //    if ( !applyTemplateView.ShowDialog().Value )
            //        return;

            //    var defaultCursor = Mouse.OverrideCursor;
            //    Mouse.OverrideCursor = Cursors.Wait;

            //    try
            //    {
            //        var dtStart = SelectedStartDate;
            //        if (dtStart.DayOfWeek != DayOfWeek.Monday)
            //            dtStart = dtStart.AddDays(-(dtStart.DayOfWeek - DayOfWeek.Monday));

            //        if (dtStart < MinDate)
            //            dtStart = dtStart.AddDays(7);

            //        var nWeeksToApply = Convert.ToInt32(SelectedNumberOfWeeks);
            //        var dtEnd = dtStart.AddDays(6); // or 7?
            //        var sResourceID = SelectedResource.RESOURCEID;

            //        for (var j = 1; j < nWeeksToApply + 1; j++)
            //        {
            //            //Convert start and end to string
            //            var sStart = dtStart.ToString("M/d/yyyy");
            //            var sEnd = dtEnd.ToString("M/d/yyyy");

            //            //Cancel all existing access blocks in the date range
            //            DataAccessService.CancelAccessBlocks(sResourceID, SelectedResource.RESOURCE_NAME, sStart, sEnd);

            //            //Increment start and end
            //            dtStart = dtStart.AddDays(7);
            //            dtEnd = dtStart.AddDays(6);

            //        }//end for

            //        RefreshAvailabilityScheduler("CLEAR_AVAILABILITY");
            //    }
            //    finally
            //    {
            //        Mouse.OverrideCursor = defaultCursor;
            //    }
            //}
            //catch (Exception ex)
            //{
            //    MessageBox.Show("Error clearing template:  " + ex.Message, "Availablility");
            //}
        }

        private bool CanExecuteClearAvailabilityCommand(string command)
        {
            return true;
        }

        private void ExecuteApplyClearRadioButtonCommand(string command)
        {
            _applyClearOption = command;
        }

        private bool CanExecuteApplyClearRadioButtonCommand(string command)
        {
            return true;
        }

        public void ExecuteVisibleRangeChangedCommand(object param)
        {
            RefreshAvailabilityScheduler(param as DateSpan);
        }

        public bool CanExecuteVisibleRangeChangedCommand(object param)
        {
            return param != null;
        }

        public void DeleteAvailability(string apptID)
        {
            var errorMessage = DataAccessService.DeleteAvailability (apptID);

            if (errorMessage != string.Empty) {
                View.AlertUser (errorMessage, "Delete Availability");
            } else {
                RefreshAvailabilityScheduler (string.Empty);
                _eventAggregator.GetEvent<RefreshCurrentScheduleEvent>().Publish(string.Empty);
            }
        }

        public void ClearViewFromList ()
        {
            if (Controller != null) {
                Controller.RemoveViewFromList (View);
            }
        }

        #region IAvailabilityPresentationModel Members

        public IAvailabilityView View { get; set; }

        public int TimeSlotToSkip
        {
            get { return _timeSlotToSkip; }
            set
            {
                _timeSlotToSkip = value;
                OnPropertyChanged( "TimeSlotToSkip" );
            }
        }
        public ITaskAvailabilityController Controller { get; set; }
        public SchdResourceGroup ResourceGroup { get; set; }
        public SchdResource Resource { get; set; }
        public IList<SchdResource> SelectedResourceList { get; set; }
        public SchdAppointment SelectedSchdAppointment { get; set; }
        public SchdAvailability CopySchdAvailability { get; set; }

        public bool IsEnableUpdateAppointment
        {
            get { return _isEnableUpdateAppointment; }
            set
            {
                if ( _isEnableUpdateAppointment != value )
                {
                    _isEnableUpdateAppointment = value;
                    OnPropertyChanged( "IsEnableUpdateAppointment" );
                }
            }
        }

        public IList<SchdAvailability> AvailabilityList
        {
            get { return _availabilityList; }
            set
            {
                _availabilityList = value;
                OnPropertyChanged( "AvailabilityList" );
            }
        }

        public SchdAvailability SchdAvailability
        {
            get { return _schdAvailability; }
            set
            {
                _schdAvailability = value;
                OnPropertyChanged( "SchdAvailability" );
            }
        }

        public DateTime VisibleStartDate
        {
            get { return _visibleStartDate; }
            set
            {
                _visibleStartDate = value;
                OnPropertyChanged( "VisibleStartDate" );
            }
        }

        public DateTime VisibleEndDate
        {
            get { return _visibleEndDate; }
            set
            {
                _visibleEndDate = value;
                OnPropertyChanged( "VisibleEndDate" );
            }
        }

        public string PaneTitle
        {
            get { return _paneTitle; }
            set
            {
                _paneTitle = value;
                OnPropertyChanged( "PaneTitle" );
            }
        }

        public string ResourceName
        {
            get { return _resourceName; }
            set
            {
                _resourceName = value;
                OnPropertyChanged( "ResourceName" );
            }
        }

        public int DayRange
        {
            get { return _dayRange; }
            set
            {
                _dayRange = value;
                OnPropertyChanged( "DayRange" );
            }
        }

        public TimeSpan TimeScale
        {
            get { return _timeScale; }
            set
            {
                _timeScale = value;
                OnPropertyChanged( "TimeScale" );
            }
        }

        public bool IsSelectedTab
        {
            get { return _isSelectedTab; }
            set
            {
                _isSelectedTab = value;
                OnPropertyChanged( "IsSelectedTab" );
            }
        }

        public SchdResource SelectedResource
        {
            get { return _selectedResource; }
            set
            {
                if (_selectedResource != value && value != null)
                {
                    _selectedResource = value;
                    OnPropertyChanged( "SelectedResource" );
                }
                //check user's authority for updating the appointments
//				if (!this.dataAccessService.IsEnableUpdateAppointment (this.selectedResource.RESOURCEID)) {
//					this.View.Scheduler. .IsReadOnly = true;
//					this.View.Model.IsEnableUpdateAppointment = false;
//				}
            }
        }

        public ValidationMessage ValidationMessage
        {
            get { return _validationMessage; }
            set
            {
                _validationMessage = value;
                OnPropertyChanged( "ValidationMessage" );
            }
        }

        public SchdResourceGroup SelectedResourceGroup
        {
            get { return _selectedResourceGroup; }
            set
            {
                if ( _selectedResourceGroup != value && value != null )
                {
                    _selectedResourceGroup = value;
                    OnPropertyChanged( "SelectedResourceGroup" );
                }
            }
        }

        #endregion

        public void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if ( handler != null )
                handler( this, new PropertyChangedEventArgs( propertyName ) );
        }

        public void MoveResizeAvailability (IAppointment newAppointment)
        {
            var apptid           = string.Empty;
            var apptnote         = string.Empty;
            var apptresourcename = string.Empty;
            var accesstypeid     = string.Empty;
            var slots            = string.Empty;
            var preStartTime     = string.Empty;
            var preEndTime       = string.Empty;

            foreach (Resource item in newAppointment.Resources)
            {
                switch (item.ResourceType)
                {
                case "APPOINTMENTID":
                    apptid = item.ResourceName;
                    break;
                case "NOTE":
                    apptnote = item.ResourceName;
                    break;
                case "RESOURCENAME":
                    apptresourcename = item.ResourceName;
                    break;
                case "ACCESSTYPEID":
                    accesstypeid = item.ResourceName;
                    break;
                case "SLOTS":
                    slots = item.ResourceName;
                    break;
                case "START_TIME":
                    preStartTime = item.ResourceName;
                    break;
                case "END_TIME":
                    preEndTime = item.ResourceName;
                    break;
                }
            }
            var a = new SchdAvailability
            {
                APPOINTMENTID = null,
                StartTime     = newAppointment.Start.ToString(),
                EndTime       = newAppointment.End.ToString(),
                ACCESSTYPEID  = accesstypeid,
                Note          = apptnote,
                RESOURCENAME  = apptresourcename,
                SLOTS         = int.Parse( slots )
            };

            var errorMessage = DataAccessService.DeleteAvailability (apptid);

            if (errorMessage == string.Empty)
            {
                SelectedResource.IsAsync = false;
                RefreshAvailabilityScheduler (string.Empty);
                SelectedResource.IsAsync = true;
                errorMessage = DataAccessService.AddNewAvailability (a);
            } 
            if (errorMessage != string.Empty)
            {
                View.AlertUser ("Access Block could not be modified:" + Environment.NewLine + Environment.NewLine + errorMessage, "Error");
                a.StartTime = preStartTime;
                a.EndTime = preEndTime;
                errorMessage = DataAccessService.AddNewAvailability (a);
            }
            RefreshAvailabilityScheduler (string.Empty);
        }

        public double SelectedNumberOfWeeks { get; set; }
        public DateTime SelectedStartDate { get; set; }

        public DateTime MinDate
        {
            get { return DateTime.Today; }
        }

        public class SelectedDaysListItem : NameValue
        {
            public bool IsChecked { get; set; }
            public SelectedDaysListItem(string name, string value, bool isChecked) : base(name, value) { IsChecked = isChecked; }
        }

        private IList<SelectedDaysListItem> _selectedDaysList;

        public IList<SelectedDaysListItem> SelectedDaysList
        {
            get
            {
                return _selectedDaysList ?? ( _selectedDaysList = new List<SelectedDaysListItem>
                       {
                           new SelectedDaysListItem( "Mon", "0", true ),
                           new SelectedDaysListItem( "Tue", "1", true ),
                           new SelectedDaysListItem( "Wed", "2", true ),
                           new SelectedDaysListItem( "Thr", "3", true ),
                           new SelectedDaysListItem( "Fri", "4", true ),
                           new SelectedDaysListItem( "Sat", "5", false ),
                           new SelectedDaysListItem( "Sun", "6", false )
                       } );
            }
        }

        public Visibility SelectedDaysListVisibility { get; set; }

        public Visibility SelectOverwriteVisibility { get; set; }

        public string ApplyTemplateViewTitle { get; set; }
    }
}
