using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Input;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using Microsoft.Practices.Composite.Regions;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Task.Scheduler.Services;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Events;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ScheduleView;

namespace ClinSchd.Modules.Task.Scheduler.Controllers
{
    public class TaskSchedulerController : ITaskSchedulerController
    {
        private enum ScheduleTypes
        {
            Clinic,
            Provider,
            ClinicGroup
        };

        private readonly IUnityContainer container;
        private readonly IEventAggregator eventAggregator;
        private readonly IRegionManager regionManager;
        private readonly IDataAccessService dataAccessService;
        private List<ISchedulerView> ChildPanes { get; set; }
        private Patient Patient;
        private AppointmentRequest AppointmentRequest;
        private FuturePatientAppointment PendingAppointment;

        public TaskSchedulerController(
            IUnityContainer container,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator,
            IRegionManager regionManager)
        {
            this.container = container;
            this.eventAggregator = eventAggregator;
            this.dataAccessService = dataAccessService;
            this.regionManager = regionManager;
            ChildPanes = new List<ISchedulerView>();
        }

        public void Run()
        {
            this.container.RegisterType<ISchedulerView, ChildModules.Scheduler.Scheduler.SchedulerView>();
            this.container.RegisterType<ISchedulerPresentationModel, SchedulerPresentationModel>();
            this.container.RegisterType<ITaskSchedulerService, TaskSchedulerService>();

            this.eventAggregator.GetEvent<AppointmentRequestSelectedEvent>().Subscribe(AppointmentRequestSelected, ThreadOption.UIThread, false);
            this.eventAggregator.GetEvent<SchedulerResourcesEvent>().Subscribe(GetResource, ThreadOption.UIThread, true);
            this.eventAggregator.GetEvent<SchedulerDisplayEvent>().Subscribe(SchedulerDisplay, ThreadOption.UIThread, true);
            this.eventAggregator.GetEvent<SchedulerDisplayGroupEvent>().Subscribe(SchedulerDisplayForClinicGroup, ThreadOption.UIThread, true);
            this.eventAggregator.GetEvent<SchedulerDisplayForProvidersEvent>().Subscribe(SchedulerDisplayForProviders, ThreadOption.UIThread, true);
            this.eventAggregator.GetEvent<CloseSchedulesEvent>().Subscribe(CloseSchedules, ThreadOption.UIThread, true);
            this.eventAggregator.GetEvent<PatientSelectedEvent>().Subscribe(PatientSelected, ThreadOption.UIThread, false);
            this.eventAggregator.GetEvent<PatientUnselectEvent>().Subscribe(PatientUnselect, ThreadOption.UIThread, false);
            this.eventAggregator.GetEvent<CloseAllSchedulesEvent>().Subscribe(CloseAllSchedules, ThreadOption.UIThread, true);
            this.eventAggregator.GetEvent<PendingAppointmentSelectedEvent>().Subscribe(PendingAppointmentSelected, ThreadOption.UIThread, true);
            this.eventAggregator.GetEvent<SchedulerDisplayEmptyPaneEvent>().Subscribe(EmptySchedulerDisplay, ThreadOption.UIThread, true);
        }

        private void PatientUnselect(string s)
        {
            this.Patient = null;
        }

        private void SchedulerDisplay(SchdResource selectedResource)
        {
            CloseAllSchedules(String.Empty);

            if (selectedResource == null)
                return;

            Mouse.OverrideCursor = Cursors.Wait;

            try
            {
                Keys.SameClinicSelected = false;
                ISchedulerView view = container.Resolve<ISchedulerView>();
                ISchedulerPresentationModel presenter = view.SchedulerViewModel;
                presenter.ScheduleID = "ClinicSchedule^" + selectedResource.RESOURCEID;
                ChildPanes.Add(view);
                regionManager.Regions[RegionNames.TaskGroupRegion].Add(presenter.View);

                presenter.IsEnableUpdateAppointment = !Keys.LoggedInUserSDECViewOnlyKey;

                presenter.View.Scheduler_EnableEvents(false);
                presenter.View.IsDockedInRegion = true;
                presenter.Controller = this;
                presenter.PaneTitle = selectedResource.RESOURCE_NAME;
                presenter.SelectedResourceGroup = new SchdResourceGroup(dataAccessService) { Name = "Clinic Schedule", IEN = "0" };
                presenter.SelectedResourceGroup.Resources.Add(selectedResource);
                presenter.SelectedResource = selectedResource;
                presenter.CurrentAppointmentRequest = Keys.SelectedAppointmentRequest;
                presenter.CurrentPendingAppointment = this.PendingAppointment;

                if ((Keys.SelectedAppointmentRequest == null) && (this.PendingAppointment == null))
                {
                    presenter.SchedulerDateChanged(new SchedulerViewDateEventArgs("CalendarDateSelection", DateTime.Today));
                    eventAggregator.GetEvent<SchedulerViewDateEvent>().Publish(new SchedulerViewDateEventArgs("Scheduler.SchedulerView", DateTime.Today));
                }

                // we are done with this.PendingAppointment so clear it.
                this.PendingAppointment = null;
                InitializeGroups(presenter, presenter.SelectedResourceGroup, false);
                /*****************************/
                InitializePresenter(presenter, selectedResource.Clinic.DISPLAY_INCREMENTS);
            }
            finally
            {
                Mouse.OverrideCursor = Cursors.Arrow;
            }
        }

        private void SchedulerDisplayForClinicGroup(SchdResourceGroup selectedResourceGroup)
        {
            CloseAllSchedules(String.Empty);

            if (selectedResourceGroup == null)
                return;

            Mouse.OverrideCursor = Cursors.Wait;

            try
            {
                Keys.SameClinicSelected = false;
                ISchedulerView view = this.container.Resolve<ISchedulerView>();
                ISchedulerPresentationModel presenter = view.SchedulerViewModel;
                presenter.ScheduleID = "ClinicGroupSchedule^" + selectedResourceGroup.IEN;
                ChildPanes.Add(view);
                regionManager.Regions[RegionNames.TaskGroupRegion].Add(presenter.View);

                List<SchdResource> newResourceList = new List<SchdResource>();

                foreach (SchdResource res in selectedResourceGroup.Resources)
                {
                    var providerPiece = res.ResourceType.Split('|');

                    if (providerPiece[0].ToUpper() == "P")
                    {
                        IList<SchdResource> providerClinics = dataAccessService.GetClinicsByProvider(res.ResourceTypeIEN);

                        foreach (SchdResource providerClinic in providerClinics)
                        {
                            providerClinic.ParentProviderResourceID = res.RESOURCEID;
                            providerClinic.ParentProviderResourceName = res.RESOURCE_NAME;
                            newResourceList.Add(providerClinic);
                        }
                    }
                    else
                        newResourceList.Add(res);
                }

                selectedResourceGroup.Resources = newResourceList;

                presenter.View.Scheduler_EnableEvents(false);
                presenter.View.IsDockedInRegion = true;
                presenter.Controller = this;
                presenter.IsEnableUpdateAppointment = !Keys.LoggedInUserSDECViewOnlyKey;
                presenter.IsPartofGoupSchedule = true;
                presenter.SelectedResourceGroup = selectedResourceGroup;
                presenter.SelectedResource = selectedResourceGroup.Resources[0];
                presenter.PaneTitle = selectedResourceGroup.Name;
                presenter.SchedulerDateChanged(new SchedulerViewDateEventArgs("CalendarDateSelection", DateTime.Today));
                eventAggregator.GetEvent<SchedulerViewDateEvent>().Publish(new SchedulerViewDateEventArgs("Scheduler.SchedulerView", DateTime.Today));
                int timeScaleIndex = InitializeGroups(presenter, selectedResourceGroup);
                InitializePresenter(presenter, timeScaleIndex);
            }
            finally
            {
                Mouse.OverrideCursor = Cursors.Arrow;
            }
        }

        private void SchedulerDisplayForProviders(SchdResourceGroup selectedResourceGroup)
        {
            CloseAllSchedules(String.Empty);

            if (selectedResourceGroup == null)
                return;

            Mouse.OverrideCursor = Cursors.Wait;

            try
            {
                Keys.SameClinicSelected = false;
                ISchedulerView view = this.container.Resolve<ISchedulerView>();
                ISchedulerPresentationModel presenter = view.SchedulerViewModel;
                presenter.ScheduleID = "ProviderSchedule^" + selectedResourceGroup.IEN;
                ChildPanes.Add(view);
                regionManager.Regions[RegionNames.TaskGroupRegion].Add(presenter.View);

                presenter.View.Scheduler_EnableEvents(false);
                presenter.View.IsDockedInRegion = true;
                presenter.Controller = this;
                presenter.IsEnableUpdateAppointment = !Keys.LoggedInUserSDECViewOnlyKey;
                presenter.IsPartofGoupSchedule = true;
                presenter.SelectedResourceGroup = selectedResourceGroup;
                presenter.SelectedResource = selectedResourceGroup.Resources[0];
                presenter.PaneTitle = selectedResourceGroup.Name;
                presenter.SchedulerDateChanged(new SchedulerViewDateEventArgs("CalendarDateSelection", DateTime.Today));
                eventAggregator.GetEvent<SchedulerViewDateEvent>().Publish(new SchedulerViewDateEventArgs("Scheduler.SchedulerView", DateTime.Today));
                int timeScaleIndex = InitializeGroups(presenter, selectedResourceGroup);
                InitializePresenter(presenter, timeScaleIndex);
            }
            finally
            {
                Mouse.OverrideCursor = Cursors.Arrow;
            }
        }

        private void InitializePresenter(ISchedulerPresentationModel presenter, int timeScaleIndex)
        {
            try
            {
                presenter.PatientSelected(this.Patient);
                eventAggregator.GetEvent<SelectTimeScaleEvent>().Publish(timeScaleIndex);
                TimeSpan _timeScale;

                switch (timeScaleIndex)
                {
                    case 1:
                        _timeScale = TimeSpan.FromMinutes(60);
                        presenter.SelectedResource.TIMESCALE = "60";
                        break;
                    case 3:
                        _timeScale = TimeSpan.FromMinutes(20);
                        presenter.SelectedResource.TIMESCALE = "20";
                        break;
                    case 4:
                        _timeScale = TimeSpan.FromMinutes(15);
                        presenter.SelectedResource.TIMESCALE = "15";
                        break;
                    case 6:
                        _timeScale = TimeSpan.FromMinutes(10);
                        presenter.SelectedResource.TIMESCALE = "10";
                        break;
                    case 12:
                        _timeScale = TimeSpan.FromMinutes(5);
                        presenter.SelectedResource.TIMESCALE = "5";
                        break;
                    default:
                        _timeScale = TimeSpan.FromMinutes(30);
                        presenter.SelectedResource.TIMESCALE = "30";
                        break;

                }

                /*  Day View */
                VSEDayViewDefinition dayView = new VSEDayViewDefinition();
                dayView.TimeScale = (int)_timeScale.TotalMinutes;
                //dayView.GroupHeaderDateStringFormat = "{0:dd dddd}";
                //dayView.ShowAllDayArea = true;
                //dayView.MinorTickLength = new FixedTickProvider(new DateTimeInterval(((timeScaleIndex < 3) ? 15 : (int)_timeScale.TotalMinutes), 0, 0, 0, 0));
                //dayView.TimerulerMinorTickStringFormat = "{0: :mm}";
                //dayView.MinTimeRulerExtent = 17 * 24 * ((timeScaleIndex < 3) ? 4 : timeScaleIndex);
                //dayView.MajorTickLength = new FixedTickProvider(new DateTimeInterval(0, 1, 0, 0, 0));

                /* Week View */
                VSEWeekViewDefinition weekView = new VSEWeekViewDefinition();
                weekView.TimeScale = (int)_timeScale.TotalMinutes;
                //weekView.GroupHeaderDateStringFormat = "{0:dd dddd}";
                //weekView.ShowAllDayArea = true;
                //weekView.MinorTickLength = new FixedTickProvider(new DateTimeInterval(((timeScaleIndex < 3) ? 15 : (int)_timeScale.TotalMinutes), 0, 0, 0, 0));
                //weekView.TimerulerMinorTickStringFormat = "{0: :mm}";
                //weekView.MinTimeRulerExtent = 17 * 24 * ((timeScaleIndex < 3) ? 4 : timeScaleIndex);
                //weekView.MajorTickLength = new FixedTickProvider(new DateTimeInterval(0, 1, 0, 0, 0));

                /* Timeline View */
                //VSETimelineViewDefinition timelineView = new VSETimelineViewDefinition();
                ////timelineView.TimeScale = (int)_timeScale.TotalMinutes;
                //timelineView.GroupHeaderDateStringFormat = "{0:ddd MM/dd/yyyy}";
                //timelineView.TimerulerMajorTickStringFormat = string.Empty;
                //timelineView.TimerulerMinorTickStringFormat = string.Empty;
                //timelineView.MajorTickLength = new FixedTickProvider(new DateTimeInterval(1, 0, 0));
                //timelineView.MinorTickLength = new FixedTickProvider(new DateTimeInterval(1, 0, 0));

                TimelineViewDefinition timelineView = new TimelineViewDefinition();
                timelineView.FirstDayOfWeek = DayOfWeek.Monday;
                timelineView.StretchAppointments = true;
                timelineView.VisibleDays = 7;
                timelineView.TimerulerGroupStringFormat = "{0:MM/dd/yyyy}";
                timelineView.TimerulerMajorTickStringFormat = "{0:ddd MM/dd/yyyy}";
                timelineView.TimerulerMinorTickStringFormat = "";
                timelineView.ApplyStartEndTimeToEachDay = true;
                timelineView.GroupTickLength = new FixedTickProvider(new DateTimeInterval(7, 0, 0));
                timelineView.MajorTickLength = new FixedTickProvider(new DateTimeInterval(1, 0, 0));
                timelineView.MinorTickLength = new FixedTickProvider(new DateTimeInterval(1, 0, 0));
                //timelineView.MinTimeRulerExtent = 17 * 24 * ((timeScaleIndex < 3) ? 4 : timeScaleIndex);
                timelineView.MaxTimeRulerExtent = 45000;
                timelineView.Freeze();

                /* Month View */
                VSEMonthViewDefninition monthView = new VSEMonthViewDefninition();
                // Changes the Top Daily Header
                // monthView.GroupHeaderDateStringFormat = "{0:dddd}";
                // Changes the Lower Daily Date Header
                // monthView.TimerulerMajorTickStringFormat = "{0:d}";
                // Changes the vertical date range header
                // monthView.TimerulerGroupStringFormat = "{0:MM/dd}";


                presenter.View.Scheduler.ViewDefinitions.Clear();
                presenter.View.Scheduler.ViewDefinitions.Add(dayView);
                presenter.View.Scheduler.ViewDefinitions.Add(weekView);
                presenter.View.Scheduler.ViewDefinitions.Add(monthView);
                presenter.View.Scheduler.ViewDefinitions.Add(timelineView);

                if (presenter.IsPartofGoupSchedule)
                    presenter.View.Scheduler.ActiveViewDefinition = dayView;
                else
                    presenter.View.Scheduler.ActiveViewDefinition = weekView;

                presenter.View.Scheduler.MinTimeRulerExtent = 20 * 24 * timeScaleIndex;
                presenter.RefreshAppoinmentScheduler("INITIAL_LAUNCH");

                eventAggregator.GetEvent<ViewTimeScaleEvent>().Publish(presenter.SelectedResource.TIMESCALE);
            }
            finally
            {
                presenter.View.Scheduler_EnableEvents(true);
                Mouse.OverrideCursor = Cursors.Arrow;
            }
        }

        private int InitializeGroups(ISchedulerPresentationModel presenter, SchdResourceGroup selectedResourceGroup, bool createGroups = true)
        {
            int timeScaleIndex = 0;

            // Create Resource Types
            ResourceType resourceType = new ResourceType(Keys.AppointmentGroupNames.Appointment);

            foreach (SchdResource resource in selectedResourceGroup.Resources)
            {
                timeScaleIndex = Math.Max(timeScaleIndex, resource.Clinic.DISPLAY_INCREMENTS);
                resourceType.Resources.Add(new Resource(resource.ScheduleTabCaption));
            }

            presenter.ResourceTypesCollection.Add(resourceType);

            // Create grouping beyond date grouping the groups if needed
            if (createGroups)
            {
                ResourceGroupDescription groupDescription = new ResourceGroupDescription()
                {
                    ResourceType = Keys.AppointmentGroupNames.Appointment
                };

                presenter.GroupsDescriptionCollection.Insert(0, groupDescription);
            }

            return timeScaleIndex;
        }

        public void GetResource(SchdResource selectedResource)
        {
            Keys.AlertUserError(null, new NotImplementedException("TaskSChedulerController.GetResource has been disbled"), "GetResource");
            //ISchedulerPresentationModel presenter = this.container.Resolve<ISchedulerPresentationModel> ();
            //         presenter.RespondToEvents(true);
            //presenter.SelectedResource = selectedResource;
            //presenter.ResourceName = selectedResource.RESOURCE_NAME;
            //presenter.PaneTitle = selectedResource.RESOURCE_NAME;

            //         presenter.AppointmentList = this.dataAccessService.GetAppointments(selectedResource, presenter.VisibleStartDate.ToShortDateString(), presenter.VisibleEndDate.ToShortDateString() + "@23:59");
            //presenter.View.Scheduler.AppointmentsSource = new SchdAppointmentSource (presenter.AppointmentList, presenter.AppointmentCategories, true,dataAccessService).Appointments;
        }

        public void RemoveViewFromList(ISchedulerView View)
        {
            if (ChildPanes.Contains(View))
                ChildPanes.Remove(View);

            if (this.regionManager.Regions[RegionNames.TaskGroupRegion].Views.Contains(View))
                this.regionManager.Regions[RegionNames.TaskGroupRegion].Remove(View);
        }

        private void CloseAllSchedules(string arg)
        {
            List<ISchedulerView> openPanes = new List<ISchedulerView>();
            openPanes.AddRange(ChildPanes);

            foreach (ISchedulerView pane in openPanes)
                CloseSchedule(pane);
        }

        private void CloseSchedules(string ClinicIEN)
        {
            List<ISchedulerView> openPanes = new List<ISchedulerView>();
            foreach (ISchedulerView item in ChildPanes)
            {
                if (item.Model.SelectedResource.Clinic != null &&
                    item.Model.SelectedResource.Clinic.HOSPITAL_LOCATION_ID == ClinicIEN)
                {
                    openPanes.Add(item);
                    break;
                }
            }

            foreach (ISchedulerView pane in openPanes)
                CloseSchedule(pane);
        }

        private void CloseSchedule(ISchedulerView pane)
        {
            ISchedulerPresentationModel m = (ISchedulerPresentationModel)pane.Model;
            pane.Close();
            m.RespondToEvents(false);
            m.Dispose();
            RemoveViewFromList(pane);
        }

        private void PatientSelected(ClinSchd.Infrastructure.Models.Patient selectedPatient)
        {

            if (selectedPatient != null)
            {
                if (string.IsNullOrEmpty(selectedPatient.TypeName))
                {
                    var patients = dataAccessService.GetPatients("#" + selectedPatient.IEN, 1);
                    foreach (var pat in patients)
                    {
                        selectedPatient.TypeIen = pat.TypeIen;
                        selectedPatient.TypeName = pat.TypeName;
                    }

                }
            }

            this.Patient = selectedPatient;
        }

        public void AppointmentRequestSelected(AppointmentRequest appointmentRequest)
        {
            this.AppointmentRequest = appointmentRequest;
        }

        public void PendingAppointmentSelected(FuturePatientAppointment pendingAppointment)
        {
            this.PendingAppointment = pendingAppointment;
        }

        public void EmptySchedulerDisplay(string message)
        {
            CloseAllSchedules(String.Empty);
        }
    }
}