using System;
using System.CodeDom;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using System.ComponentModel;
using System.Threading;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows;
using System.Windows.Controls;
using System.Linq;
using System.Windows.Threading;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using Microsoft.Practices.Composite.Presentation.Commands;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Composite.Regions;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ScheduleView;
using ClinSchd.Modules.Task.Group;
using ClinSchd.Modules.Task.Scheduler.Controllers;

namespace ClinSchd.Modules.Task.Scheduler
{
    public enum ViewModes
    {
        Day,
        Week,
        Month,
        TimeLine,
        None
    }

    public class SchedulerPresentationModel : AsyncValidatableModel, ISchedulerPresentationModel, ITaskPresentationModel, IDisposable
    {
        #region private fields

        private readonly IRegionManager regionManager;
        private readonly IEventAggregator EventAggregator;

        private readonly Dictionary<string, Category> _appointmentCategories = new Dictionary<string, Category>();
        private readonly ObservableCollection<Category> _categoriesList = new ObservableCollection<Category>();
        private readonly ObservableCollection<Appointment> _appointmentsCollection = new ObservableCollection<Appointment>();
        private IList<GroupDescription> _groupsDescriptionCollection = null;

        private Patient _patient;
        private bool _passedSensitiveRecordCheck = false;
        private string _paneTitle = "";
        private bool _patientSelectedByMe = false;
        private int _dayRange = 1;
        private TimeSpan _timeScale = TimeSpan.FromMinutes(10);
        private bool _isSelectedTab;
        private SchdResource _selectedResource;
        private SchdResourceGroup _selectedResourceGroup;
        private IList<SchdAppointment> _appointmentList;
        private DateTime _currentSelectedDate = DateTime.Today;
        private bool _isDisplayWalkInEnabled = true;
        private bool _isEnableUpdateAppointment = true;
        private bool _isInitialized;
        private bool _isTimeSlotViewerVisible;
        private DateTime _lastUpdate = DateTime.Now;
        private bool _respondToEvents;
        private bool _prohibitedClinic;
        private bool _privilegedUser;
        private Slot _selectedTimeSlot;
        private bool _setByTimeSlotViewer = false;
        private Appointment _selectedAppointment;
        private AppointmentRequest _currentAappointmentRequest;
        private FuturePatientAppointment _currentPendingAppointment;
        private ViewDefinitionBase _activeViewDefinition;
        private Brush _timeSlotBrush;
        private string _timeSlotFormattedCaption = string.Empty;
        private IList<Appointment> _timeSlotAppointmentsList = new List<Appointment>();
        private double? _defaultMinAppointmentHeight = null;
        private string _refreshCommand;
        private readonly DispatcherTimer _refreshTimer;
        private bool _scheduleIsInitialized = false;

        #endregion private fields

        #region constructor

        public SchedulerPresentationModel(
            IRegionManager regionManager,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            IsPartofGoupSchedule = false;
            this.regionManager = regionManager;
            DataAccessService = dataAccessService;
            EventAggregator = eventAggregator;
            _refreshTimer = new System.Windows.Threading.DispatcherTimer();
            _refreshTimer.Tick += RefreshAppointmentSchedulerEvent;
            _refreshTimer.Interval = TimeSpan.FromMilliseconds(250);
            _selectedAppointmentTimer = new System.Windows.Threading.DispatcherTimer();
            _selectedAppointmentTimer.Interval = TimeSpan.FromMilliseconds(500);
            _selectedAppointmentTimer.Tick += _selectedAppointmentTimer_Tick;

            ExpandedEntryCommand = new DelegateCommand<string>(ExecuteExpandedEntryCommand);
            CreateAppointmentCommand = new DelegateCommand<string>(ExecuteCreateAppointmentCommand, CanExecuteCreateAppointmentCommand);
            CreateWalkInAppointmentCommand = new DelegateCommand<string>(ExecuteCreateWalkInAppointmentCommand, CanExecuteCreateWalkInAppointmentCommand);
            EditAppointmentCommand = new DelegateCommand<string>(ExecuteEditAppointmentCommand, CanExecuteEditAppointmentCommand);
            ViewAppointmentCommand = new DelegateCommand<string>(ExecuteViewAppointmentCommand, CanExecuteViewAppointmentCommand);
            TaskTabSelectedCommand = new DelegateCommand<object>(ExecuteSchedulerTabSelectedCommand, CanExecuteSchedulerTabSelectedCommand);
            MarkAsNoShowCommand = new DelegateCommand<string>(ExecuteMarkAsNoShowCommand, CanExecuteMarkAsNoShowCommand);
            UndoANoShowCommand = new DelegateCommand<string>(ExecuteUndoANoShowCommand, CanExecuteUndoANoShowCommand);
            CheckInPatientCommand = new DelegateCommand<string>(ExecuteCheckInPatientCommand, CanExecuteCheckInPatientCommand);
            UndoCheckInPatientCommand = new DelegateCommand<string>(ExecuteUndoCheckInPatientCommand, CanExecuteUndoCheckInPatientCommand);
            CheckOutPatientCommand = new DelegateCommand<string>(ExecuteCheckOutPatientCommand, CanExecuteCheckOutPatientCommand);
            UndoCheckOutPatientCommand = new DelegateCommand<string>(ExecuteUndoCheckOutPatientCommand, CanExecuteUndoCheckOutPatientCommand);
            CancelAppointmentCommand = new DelegateCommand<string>(ExecuteCancelAppointmentCommand, CanExecuteCancelAppointmentCommand);
            UndoCancelAppointmentCommand = new DelegateCommand<string>(ExecuteUndoCancelAppointmentCommand, CanExecuteUndoCancelAppointmentCommand);
            CopyAppointmentCommand = new DelegateCommand<string>(ExecuteCopyAppointmentCommand, CanExecuteCopyAppointmentCommand);
            PasteAppointmentCommand = new DelegateCommand<string>(ExecutePasteAppointmentCommand, CanExecutePasteAppointmentCommand);
            VisibleRangeChangedCommand = new DelegateCommand<object>(ExecuteVisibleRangeChangedCommand, CanExecuteVisibleRangeChangedCommand);
            eventAggregator.GetEvent<SchedulerPresentationHotKeysEvent>().Subscribe(HotKeyEvents, ThreadOption.UIThread, false);
            RespondToEvents(true);
        }

        #endregion constructor

        #region ITaskPresentationModel members

        #region RadScheduleViewBindings

        public ObservableCollection<Appointment> AppointmentsCollection { get { return _appointmentsCollection; } }

        public ObservableCollection<Category> CategoriesList
        {
            get
            {
                if (_categoriesList.Count == 0)
                    foreach (var pair in AppointmentCategories)
                        _categoriesList.Add(pair.Value);

                return _categoriesList;
            }
        }

        public ObservableCollection<CustomSlot> SlotsCollection { get; } = new ObservableCollection<CustomSlot>();

        #endregion RadScheduleViewBindings

        public void ClearViewFromList()
        {
            if (Controller != null)
            {
                Controller.RemoveViewFromList(View);
            }

            View = null;
        }

        public bool IsEnableUpdateAppointment
        {
            // 
            get { return _isEnableUpdateAppointment; } // ( Keys.SelectedAppointmentRequest != null ) && SelectedTimeSlotIsToday && DayOfSelectedTimeSlotHasAvailability; }
            set
            {
                if ( _isEnableUpdateAppointment != value )
                {
                    _isEnableUpdateAppointment = value;
                    OnPropertyChanged( "IsEnableUpdateAppointment" );
                }
            }
        }

        public bool IsSelectedTab
        {
            get { return _isSelectedTab; }
            set
            {
                _isSelectedTab = value;
                OnPropertyChanged("IsSelectedTab");
            }
        }

        public SchdResource SelectedResource
        {
            get { return _selectedResource; }
            set
            {
                _selectedResource = value;
                //check user's authority for updating the appointments
                try
                {
                    if ( value != null )
                    {
                        _prohibitedClinic = false;
                        _privilegedUser = false;
                        _selectedResource.StopCodeNumber = _selectedResource.Clinic.CLINIC_STOP;
                        if ( _selectedResource.IsProhibitedClinic )
                        {
                            _prohibitedClinic = true;
                            _privilegedUser = DataAccessService.UserHasAccessToProhibitedClinic( _selectedResource.HOSPITAL_LOCATION_ID );
                        }
                        if ( _prohibitedClinic && !_privilegedUser )
                            View.Model.IsEnableUpdateAppointment = false;
                        else
                            View.Model.IsEnableUpdateAppointment = !Keys.LoggedInUserSDECViewOnlyKey;
                    }
                    OnPropertyChanged( "SelectedResource" );
                }
                catch (Exception ex)
                {
                    TraceLog.LogException(GetType().Name, "SelectedResource", ex);
                    //MessageBox.Show(ex.Source + ": " + ex.Message);
                }
            }
        }

        public SchdResourceGroup SelectedResourceGroup
        {
            get { return _selectedResourceGroup; }
            set
            {
                if (_selectedResourceGroup != value && value != null)
                {
                    _selectedResourceGroup = value;
                    OnPropertyChanged( "SelectedResourceGroup" );
                }
            }
        }

        public IList<ResourceType> ResourceTypesCollection { get; } = new List<ResourceType>();

        public IList<GroupDescription> GroupsDescriptionCollection
        {
            get
            {
                if (_groupsDescriptionCollection == null)
                {
                    _groupsDescriptionCollection = new List<GroupDescription>();
                    _groupsDescriptionCollection.Add(new DateGroupDescription());
                }

                return _groupsDescriptionCollection;
            }
        } 

        public DelegateCommand<object> TaskTabSelectedCommand { get; }

        #endregion ITaskPresentationModel members

        #region ISchedulerPresentationModel Members

        public IDataAccessService DataAccessService { get; private set; }

        public void RespondToEvents(bool respond)
        {
            _respondToEvents = respond;
            if (respond)
            {
                EventAggregator.GetEvent<ViewDayRangeEvent>().Subscribe(DayRangeChanged, ThreadOption.UIThread, true);
                EventAggregator.GetEvent<ChangeViewModeEvent>().Subscribe(ViewModeChanged, ThreadOption.UIThread, false);
                EventAggregator.GetEvent<DisplayWalkInEvent>().Subscribe(DisplayWalkInChanged, ThreadOption.UIThread, true);
                EventAggregator.GetEvent<RefreshScheduler>().Subscribe(RefreshAppoinmentScheduler, ThreadOption.UIThread, true);
                EventAggregator.GetEvent<UndoNoShowAppointmentEvent>().Subscribe(UndoNoShowAppointment, ThreadOption.UIThread, true);
                EventAggregator.GetEvent<UndoCheckInPatientEvent>().Subscribe(UndoCheckInPatient, ThreadOption.UIThread, true);
                EventAggregator.GetEvent<UndoCheckOutPatientEvent>().Subscribe(UndoCheckOutPatient, ThreadOption.UIThread, true);
                EventAggregator.GetEvent<UndoCancelAppointmentEvent>().Subscribe(UndoCancelAppointment, ThreadOption.UIThread, true);
                EventAggregator.GetEvent<PatientSelectedEvent>().Subscribe(PatientSelected, ThreadOption.UIThread, false);
                EventAggregator.GetEvent<PatientUnselectEvent>().Subscribe(PatientUnselect, ThreadOption.UIThread, false);
                EventAggregator.GetEvent<SchedulerViewDateEvent>().Subscribe(SchedulerDateChanged, ThreadOption.UIThread, false);
                EventAggregator.GetEvent<CopyAppointmentEvent>().Subscribe(CopyAppointment, ThreadOption.UIThread, false);
                EventAggregator.GetEvent<PasteAppointmentEvent>().Subscribe(PasteAppointment, ThreadOption.UIThread, false);
                EventAggregator.GetEvent<PrintCurrentSchedulerEvent>().Subscribe(PrintCurrentScheduler, ThreadOption.UIThread, false);
                EventAggregator.GetEvent<CollapseTimeSlotViewerEvent>().Subscribe(CollapseTimeSlotViewer, ThreadOption.UIThread, false);
                EventAggregator.GetEvent<RefreshCurrentScheduleEvent>().Subscribe(RefreshCurrentSchedulerEventHandler, ThreadOption.UIThread, false);
            }
            else
            {
                EventAggregator.GetEvent<ViewDayRangeEvent>().Unsubscribe(DayRangeChanged);
                EventAggregator.GetEvent<ChangeViewModeEvent>().Unsubscribe(ViewModeChanged);
                EventAggregator.GetEvent<DisplayWalkInEvent>().Unsubscribe(DisplayWalkInChanged);
                EventAggregator.GetEvent<RefreshScheduler>().Unsubscribe(RefreshAppoinmentScheduler);
                EventAggregator.GetEvent<UndoNoShowAppointmentEvent>().Unsubscribe(UndoNoShowAppointment);
                EventAggregator.GetEvent<UndoCheckInPatientEvent>().Unsubscribe(UndoCheckInPatient);
                EventAggregator.GetEvent<UndoCheckOutPatientEvent>().Unsubscribe(UndoCheckOutPatient);
                EventAggregator.GetEvent<UndoCancelAppointmentEvent>().Unsubscribe(UndoCancelAppointment);
                EventAggregator.GetEvent<PatientSelectedEvent>().Unsubscribe(PatientSelected);
                EventAggregator.GetEvent<PatientUnselectEvent>().Unsubscribe(PatientUnselect);
                EventAggregator.GetEvent<SchedulerViewDateEvent>().Unsubscribe(SchedulerDateChanged);
                EventAggregator.GetEvent<CopyAppointmentEvent>().Unsubscribe(CopyAppointment);
                EventAggregator.GetEvent<PasteAppointmentEvent>().Unsubscribe(PasteAppointment);
                EventAggregator.GetEvent<PrintCurrentSchedulerEvent>().Unsubscribe(PrintCurrentScheduler);
                EventAggregator.GetEvent<CollapseTimeSlotViewerEvent>().Unsubscribe(CollapseTimeSlotViewer);
                EventAggregator.GetEvent<RefreshCurrentScheduleEvent>().Unsubscribe(RefreshCurrentSchedulerEventHandler);
            }
        }

        public string ScheduleID { get; set; }

        public ISchedulerView View { get; set; }

        public bool IsDisplayWalkInEnabled
        {
            get { return _isDisplayWalkInEnabled; }
            set { _isDisplayWalkInEnabled = value; }
        }

        public Dictionary<string, Category> AppointmentCategories
        {
            get
            {
                if (_appointmentCategories.Count() == 0)
                    foreach (var item in Keys.ColorCodes.CategoriesList)
                        _appointmentCategories.Add(item.Key, new Category(item.Key, item.Value));

                return _appointmentCategories;
            }
        }

        public IList<SchdAppointment> AppointmentList
        {
            get { return _appointmentList; }
            set
            {
                if (_appointmentList != null || _appointmentList != value)
                {
                    _appointmentList = value;
                    //this.OnPropertyChanged ("AppointmentList");
                }
            }
        }

        public SchdAppointment SchdAppointment { get; set; }

        public SchdAppointment CopySchdAppointment { get; set; }

        public DateTime VisibleStartDate { get; set; }

        public DateTime VisibleEndDate { get; set; }

        public string PaneTitle
        {
            get { return _paneTitle; }
            set {
                _paneTitle = value;
                OnPropertyChanged("PaneTitle");
            }
        }

        public string ResourceName
        {
            get
            {
                if (SelectedResource == null)
                    return String.Empty;

                return SelectedResource.RESOURCE_NAME;
            }
        }

        public DateTime CurrentSelectedDate
        {
            get { return _currentSelectedDate; }
            set
            {
                if (_currentSelectedDate.CompareTo(value) != 0)
                {
                    _currentSelectedDate = value;
                    OnPropertyChanged("CurrentSelectedDate");
                    EventAggregator.GetEvent<SchedulerViewDateEvent>().Publish(new SchedulerViewDateEventArgs("Scheduler.SchedulerView", value));
                    RaiseCanExecuteChanged();
                }
            }
        }

        public int DayRange
        {
            get { return _dayRange; }
            set { _dayRange = value; }
        }

        public TimeSpan TimeScale
        {
            get { return _timeScale; }
            set { _timeScale = value; }
        }

        public ITaskSchedulerController Controller { get; set; }

        public bool ProhibitedClinic
        {
            get { return _prohibitedClinic; }
        }

        public bool PrivilegedUser
        {
            get { return _privilegedUser; }
        }

        public Slot SelectedTimeSlot
        {
            get
            {
                try
                {
                    return _selectedTimeSlot;
                }
                catch (Exception ex)
                {
                    TraceLog.LogException(GetType().Name, "SelectedTimeSlot.Getter", ex);
                    return null;
                }
            }
            set
            {
                if (value != null)
                    _selectedAppointmentTimer.Stop();

                try
                {
                    _selectedTimeSlot = value;
                    OnPropertyChanged("SelectedTimeSlot");

                    if (_selectedTimeSlot != null)
                    {
                        CurrentSelectedDate = value.Start.Date;

                        if (_selectedTimeSlot.Resources.Count > 0)
                        {
                            Resource resource = (Resource) _selectedTimeSlot.Resources.FirstOrDefault(r => ((Resource) r).ResourceType == Keys.AppointmentGroupNames.Appointment);
                            SelectedResource = SelectedResourceGroup.Resources.FirstOrDefault(r => (r.ScheduleTabCaption == resource?.ResourceName));
                        }
                        else
                            SelectedResource = SelectedResourceGroup.Resources[0];

                        if (SelectedResource == null)
                        {
                            UpdateTimeSlotViewer(null, null);
                            return;
                        }

                        UpdateTimeSlotViewer(GetCustomSlotFromTimeSlot(_selectedTimeSlot, SelectedResource, false), SelectedResource);
                    }
                }
                catch (Exception ex)
                {
                    TraceLog.LogException(GetType().Name, "SelectedTimeSlot.Setter", ex);
                }
            }
        }

        #region SelectedAppointment timer

        public bool IsContextMenuOpen { get; set; } = false;

        private readonly System.Windows.Threading.DispatcherTimer _selectedAppointmentTimer;
        private bool _selectedAppointmentTimerHasRun = true;

        private void _selectedAppointmentTimer_Tick(object sender, EventArgs e)
        {
            _selectedAppointmentTimer.Stop();

            if (!IsContextMenuOpen)
                if (CheckPatientPrompts())
                    SendPatientSelectedEvent();

            _selectedAppointmentTimerHasRun = true;
        }

        #endregion SelectedAppointment timer

        public Appointment SelectedAppointment
        {
            get { return _selectedAppointment; }
            set
            {
                if (_selectedAppointmentTimerHasRun)
                    _selectedAppointmentTimerHasRun = false;

                _selectedAppointmentTimer.Stop();
                bool isNewValue = (_selectedAppointment != value);

                if (value == null)
                {
                    SchdAppointment = null;
                    SelectedPatient = null;
                }
                else
                {
                    var customAppt = (CustomAppointment)value;
                    SchdAppointment = customAppt.SchdAppointment;
                    SelectedPatient = SchdAppointment?.Patient;
                    CurrentSelectedDate = value.Start.Date;

                    //Handle Special Instructions display
                    if (customAppt.IsAllDayEvent)
                    {
                        UpdateTimeSlotViewer(null, null);

                        if (customAppt.Subject == "Special Instructions")
                            View.AlertUser(customAppt.Body, "Special Instructions");
                    }
                    else if (isNewValue)
                    {
                        if (SelectedResource?.ScheduleID != customAppt.SchdAppointment.ResourceScheduleID)
                            SelectedResource = SelectedResourceGroup.Resources.FirstOrDefault(r => (r.ScheduleID == customAppt.SchdAppointment.ResourceScheduleID));

                        if (SelectedResource == null)
                            UpdateTimeSlotViewer(null, null);
                        else
                        {
                            // Update the Time Slot Viewer
                            if (!_setByTimeSlotViewer)
                                UpdateTimeSlotViewer(GetCustomSlotFromAppointment(value, SelectedResource, false), SelectedResource);
                        }
                    }
                }

                _setByTimeSlotViewer = false;
                _selectedAppointment = value;
                OnPropertyChanged("SelectedAppointment");
                OnPropertyChanged("SelectedTimeSlotViewerAppointment");
                RaiseCanExecuteChanged();

                if (!_selectedAppointmentTimerHasRun)
                    _selectedAppointmentTimer.Start();
            }
        }

        public IAppointment SelectedTimeSlotViewerAppointment
        {
            get { return SelectedAppointment; }
            set
            {
                _setByTimeSlotViewer = true;
                SelectedAppointment = (Appointment)value;
            }
        }

        #endregion ISchedulePresentationModel members

        #region public members

        public ViewDefinitionBase ActiveViewDefinition
        {
            get { return _activeViewDefinition; }
            set
            {
                if (_activeViewDefinition != value)
                {
                    _activeViewDefinition = value;
                    OnPropertyChanged("ActiveViewDefinition");
                    EventAggregator.GetEvent<ChangeViewModeEvent>().Publish("Scheduler.Navigation^" + value.Title);
                    RefreshAppoinmentScheduler("SCHEDULER_NAVIGATION");
                }
            }
        }

        public Brush TimeSlotBrush
        {
            get { return _timeSlotBrush; }
            set
            {
                _timeSlotBrush = value;
                OnPropertyChanged("TimeSlotBrush");
            }
        }

        public string TimeSlotFormattedCaption
        {
            get { return _timeSlotFormattedCaption; }
            set
            {
                _timeSlotFormattedCaption = value;
                OnPropertyChanged("TimeSlotFormattedCaption");
            }
        }

        public IList<Appointment> TimeSlotAppointmentsList
        {
            get { return _timeSlotAppointmentsList; }
            set
            {
                _timeSlotAppointmentsList = value;
                OnPropertyChanged("TimeSlotAppointmentsList");
            }
        } 

        #endregion public members

        #region private members

        private bool CheckProhibitedClinic(SchdResource resource)
        {
            if (resource != null)
                if (resource.IsProhibitedClinic)
                    if(!DataAccessService.UserHasAccessToProhibitedClinic(resource.HOSPITAL_LOCATION_ID))
                    {
                        View.AlertUser("You do not have access to the Prohibited Clinic " + resource.RESOURCE_NAME, "Prohibited Clinic");
                        return false;
                    }

            return true;
        }

        private CustomAppointment GetWorkingAppointment(string menuName)
        {
            if (menuName == "AppointmentContextMenu")
                return (CustomAppointment)SelectedAppointment;

            if (menuName == "TimeSlotViewer_ContextMenu")
                return (CustomAppointment)SelectedTimeSlotViewerAppointment;

            return null;
        }

        private SchdAppointment LoadNewAppointmentInfo(Slot selectedTimeSlot, SchdResource resource, SchdAppointment sourceAppointment = null)
        {
            SchdAppointment newAppointment = null;

            if (selectedTimeSlot != null)
            {
                CustomSlot customSlot = GetCustomSlotFromTimeSlot(selectedTimeSlot, SelectedResource);

                if (customSlot == null)
                    throw new Exception("Unable to determine an Access Block for the Appointment of the the Selected Time Slot.");

                if (sourceAppointment == null)
                    newAppointment = SchdAppointmentFactory.Create();
                else
                    newAppointment = sourceAppointment.Copy(true);

                newAppointment.RESOURCENAME = resource.RESOURCE_NAME;
                newAppointment.ResourceId = resource.RESOURCEID;
                DateTime startTime = selectedTimeSlot.Start;
                newAppointment.START_TIME = startTime.ToString();
                newAppointment.END_TIME = startTime.AddMinutes(resource.Clinic.Duration).ToString();

                if (resource.Clinic.VariableAppointmentLength)
                {
                    // Using SELECTED duration, or Appointment MAX duration, which ever is smaller
                    var duration = Math.Min((int) (selectedTimeSlot.End - selectedTimeSlot.Start).TotalMinutes, Keys.NewAppointmentMaxDuration);

                    if (duration < resource.Clinic.Duration)
                        newAppointment.END_TIME = startTime.AddMinutes(resource.Clinic.Duration).ToString();
                    else if (duration > resource.Clinic.Duration)
                        newAppointment.END_TIME = startTime.AddMinutes(duration).ToString();
                }
            }

            return newAppointment;
        }

        private void LoadExistAppointmentInfo(CustomAppointment selectedAppointment)
        {
            try
            {
                SchdAppointment = selectedAppointment.SchdAppointment;
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "LoadExistAppointmentInfo", ex);
            }
        }

        private bool ProcessAppointmentCommand(string headerText, CustomAppointment workingAppointment)
        {
            try
            {
                if (workingAppointment == null)
                    return false;

                SchdResource resource = workingAppointment.SchdAppointment.SelectedResource;

                if (resource == null)
                {
                    if (String.IsNullOrEmpty(workingAppointment.SchdAppointment.ResourceId))
                    {
                        IList<SchdResource> list = DataAccessService.GetSdecHResources(workingAppointment.SchdAppointment.RESOURCENAME);

                        if (list.Count > 0)
                            resource = list[0];
                    }
                    else
                        resource = DataAccessService.GetSdecResource(workingAppointment.SchdAppointment.ResourceId);
                }

                if (!CheckPatientPrompts(workingAppointment.SchdAppointment.Patient)) 
                    return false;

                if (!CheckProhibitedClinic(resource))
                    return false;

                SelectedPatient = workingAppointment.SchdAppointment.Patient;
                LoadExistAppointmentInfo(workingAppointment);
                IsContextMenuOpen = false;
                SendPatientSelectedEvent();
                return true;
            }
            catch (Exception ex)
            {
                Exception e = new Exception("There was a problem processing the selected Appointment Command.", ex);
                Keys.AlertUserError(View, e, headerText);
                TraceLog.LogException(GetType().Name, "ProcessAppointmentCommand", e);
                return false;
            }
        }

        private bool ProcessSlotCommand(string headerText)
        {
            try
            {
                if (Keys.SelectedPatient == null)
                {
                    View.AlertUser("Please select a patient", headerText);
                    return false;
                }

                if (!CheckPatientPrompts(Keys.SelectedPatient))
                    return false;

                SelectedPatient = Keys.SelectedPatient;

                if (!CheckProhibitedClinic(SelectedResource))
                    return false;

                if (Keys.SelectedAppointmentRequest == null)
                    return false;

                if (Keys.SelectedAppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Consult || Keys.SelectedAppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Procedure)
                {
                    if (Keys.Methods.IsNumeric(SelectedResource.StopCodeNumber))
                        if (!Keys.SelectedAppointmentRequest.StopCodes.Contains(Convert.ToInt32(SelectedResource.StopCodeNumber)))
                            if (!Keys.ConfirmUser((DependencyObject)View, "The clinic stop code that you are attempting to book to for the selected consult request does not match the consult service stop code.  Do you wish to continue booking?", "Clinic Stop Code", true))
                                return false;
                }

                SchdAppointment = LoadNewAppointmentInfo(SelectedTimeSlot, SelectedResource);
                Keys.IsAddAppointmentOrLeftClickOnScheduler = true;
                return true;
            }
            catch (Exception ex)
            {
                Exception e = new Exception("There was a problem processing the selected Slot Command.", ex);
                Keys.AlertUserError(View, e, headerText);
                TraceLog.LogException(GetType().Name, "ProcessSlotCommand", e);
                return false;
            }
        }

        private bool CheckPatientPrompts(Patient patient = null)
        {
            if (patient == null)
                patient = SelectedPatient;

            if (patient != null)
            {
                var prompts = DataAccessService.GetPatientSelectionPrompts(patient.IEN, ref patient, false);
                _passedSensitiveRecordCheck = (prompts != "0");

                foreach (Appointment appointment in AppointmentsCollection)
                    ((CustomAppointment)appointment).RaisePropertyChanged();
            }
            else
                _passedSensitiveRecordCheck = false;

            return _passedSensitiveRecordCheck;
        }

        private void RaiseCanExecuteChanged()
        {
            OnPropertyChanged("IsSlotContextMenuVisible");
            OnPropertyChanged("IsAppointmentContextMenuVisible");
            OnPropertyChanged("IsContextMenuVisible");
            OnPropertyChanged("IsTimeSlotViewerAppointmentContextMenuVisible");
            OnPropertyChanged("IsTimeSlotViewerSlotContextMenuVisible");
            OnPropertyChanged("IsTimeSlotViewerContextMenuVisible");

            CreateAppointmentCommand.RaiseCanExecuteChanged();
            CreateWalkInAppointmentCommand.RaiseCanExecuteChanged();
            EditAppointmentCommand.RaiseCanExecuteChanged();
            ViewAppointmentCommand.RaiseCanExecuteChanged();
            TaskTabSelectedCommand.RaiseCanExecuteChanged();
            MarkAsNoShowCommand.RaiseCanExecuteChanged();
            UndoANoShowCommand.RaiseCanExecuteChanged();
            CheckInPatientCommand.RaiseCanExecuteChanged();
            UndoCheckInPatientCommand.RaiseCanExecuteChanged();
            CheckOutPatientCommand.RaiseCanExecuteChanged();
            UndoCheckOutPatientCommand.RaiseCanExecuteChanged();
            CancelAppointmentCommand.RaiseCanExecuteChanged();
            UndoCancelAppointmentCommand.RaiseCanExecuteChanged();
            CopyAppointmentCommand.RaiseCanExecuteChanged();
            PasteAppointmentCommand.RaiseCanExecuteChanged();
        }

        private IList<Appointment> GetAppointmentsListForResource(SchdResource schdResource)
        {
            IList<Appointment> list = new List<Appointment>();

            foreach (SchdAppointment schdAppointment in schdResource.Appointments)
            {
                Appointment appointment = AppointmentsCollection.FirstOrDefault(a => (a.UniqueId == schdAppointment.APPOINTMENTID));

                if (appointment != null)
                    list.Add(appointment);
            }

            return list;
        }

        private IList<Appointment> GetAppointmentsListForSlot(CustomSlot customSlot, SchdResource schdResource)
        {
            return GetAppointmentsListForResource(schdResource).Where(a => (Keys.Methods.TimesOverlap(a.Start, a.End, customSlot.Start, customSlot.End))).ToList();
        }

        private CustomSlot GetCustomSlotFromAppointment(Appointment appointment, SchdResource schdResource, bool findClosestStartTime = true)
        {
            CustomSlot customSlot = null;

            // Do we have an exact match CustomSlot
            customSlot = schdResource.CustomSlots.FirstOrDefault(s => (s.Start.CompareTo(appointment.Start) == 0));

            // If not find the CustomSlot with the closest StartTime;
            if (customSlot == null)
            {
                if (findClosestStartTime)
                {
                    // Find Closest AccessBlock
                    double smallestInterval = 24 * 60; // Start with 1 day worth of minutes, should always be larger than any difference between start times

                    foreach (CustomSlot slotItem in schdResource.CustomSlots.Where(s => (s.Start.Date.CompareTo(appointment.Start.Date) == 0)))
                    {
                        // get absolute value of difference between block start time and selected slot start time
                        double interval = Math.Abs((slotItem.Start - appointment.Start).TotalMinutes);

                        if (interval < smallestInterval)
                        {
                            customSlot = slotItem;
                            smallestInterval = interval;
                        }
                    }
                }
                else
                    customSlot = schdResource.CustomSlots.FirstOrDefault(s => (Keys.Methods.TimesOverlap(s.Start, s.End, appointment.Start, appointment.End)));
            }

            return customSlot;
        }

        private CustomSlot GetCustomSlotFromTimeSlot(Slot slot, SchdResource schdResource, bool findClosestStartTime = true)
        {
            CustomSlot customSlot = null;

            // Do we have an exact match CustomSlot
            customSlot = schdResource.CustomSlots.FirstOrDefault(s => (s.Start.CompareTo(slot.Start) == 0));

            // If not find the CustomSlot with the closest StartTime;
            if (customSlot == null)
            {
                if (findClosestStartTime)
                {
                    // Find Closest AccessBlock
                    double smallestInterval = 24 * 60; // Start with 1 day worth of minutes, should always be larger than any difference between start times

                    foreach (CustomSlot slotItem in schdResource.CustomSlots.Where(s => (s.Start.Date.CompareTo(slot.Start.Date) == 0)))
                    {
                        // get absolute value of difference between block start time and selected slot start time
                        double interval = Math.Abs((slotItem.Start - slot.Start).TotalMinutes);

                        if (interval < smallestInterval)
                        {
                            customSlot = slotItem;
                            smallestInterval = interval;
                        }
                    }
                }
                else
                    customSlot = schdResource.CustomSlots.FirstOrDefault(s => (Keys.Methods.TimesOverlap(s.Start, s.End, slot.Start, slot.End)));
            }

            return customSlot;
        }

        private void UpdateTimeSlotViewer(CustomSlot customSlot, SchdResource schdResource)
        {
            if (!_isTimeSlotViewerVisible)
                return;

            if (customSlot == null)
            {
                TimeSlotBrush = Keys.ColorCodes.NonWorkingSlotColor;
                TimeSlotFormattedCaption = String.Empty;
                TimeSlotAppointmentsList = new List<Appointment>();
            }
            else
            {
                TimeSlotBrush = customSlot.SlotCategory.CategoryBrush;
                TimeSlotFormattedCaption = $"{customSlot.Start.ToString("MM/dd HH:mm")} - {customSlot.End.ToString("MM/dd HH:mm")} [{customSlot.SlotCount}]";

                if (schdResource != null)
                    TimeSlotAppointmentsList = GetAppointmentsListForSlot(customSlot, schdResource);
                else
                    TimeSlotAppointmentsList = new List<Appointment>();
            }
        }

        private void SendPatientSelectedEvent()
        {
            if (SelectedAppointment == null)
                return;

            if (SchdAppointment?.Patient == null)
                return;

            if (!IsContextMenuOpen && ((Keys.SelectedPatient == null) || (Keys.SelectedPatient.IEN != SelectedPatient.IEN)))
            {
                // Update the Ribbon Bar and RM Grid
                SchdAppointment.Patient.LoadRMGrid = true;
                EventAggregator.GetEvent<PatientSelectedEvent>().Publish(SchdAppointment.Patient);
            }
        }

        #endregion private members

        #region INotifyPropertyChanged members

        public void OnPropertyChanged(string propertyName)
        {
            base.NotifyPropertyChanged(propertyName);
        }

        #endregion INotifyPropertyChanged members

        public bool UndoCheckinSucceeded { get; set; }

        public AppointmentRequest CurrentAppointmentRequest
        {
            get { return _currentAappointmentRequest; }
            set
            {
                if (_currentAappointmentRequest != value)
                {
                    _currentAappointmentRequest = value;

                    if (_currentAappointmentRequest != null)
                    {
                        DateTime newDate = _currentAappointmentRequest.DesiredApptDate.HasValue ? ((_currentAappointmentRequest.DesiredApptDate.Value.CompareTo(DateTime.Today) < 0) ? DateTime.Today : _currentAappointmentRequest.DesiredApptDate.Value) : DateTime.Today;
                        SchedulerDateChanged(new SchedulerViewDateEventArgs("PendingAppointment", newDate));
                        EventAggregator.GetEvent<SchedulerViewDateEvent>().Publish(new SchedulerViewDateEventArgs("Scheduler.SchedulerView", newDate));
                    }
                }
            }
        }

        public FuturePatientAppointment CurrentPendingAppointment
        {
            get { return _currentPendingAppointment; }
            set
            {
                if (_currentPendingAppointment != value)
                {
                    _currentPendingAppointment = value;

                    if (_currentPendingAppointment != null)
                        SchedulerDateChanged(new SchedulerViewDateEventArgs("AppointmentRequest", _currentPendingAppointment.AppointmentDateTime.Date));
                }
            }
        }

        public Patient SelectedPatient
        {
            get { return _patient; }
            set
            {
                _passedSensitiveRecordCheck = (value?.IEN != Keys.PatientIen_SensitiveRecordCheck);
                _patient = value;
                OnPropertyChanged( "SelectedPatient" );
            }
        }

        public bool IsContextMenuVisible
        {
            get
            {
                return (
                    (ViewMode != ViewModes.TimeLine)
                &&
                    (
                        IsAppointmentContextMenuVisible
                    ||
                       IsSlotContextMenuVisible
                    )
                );
            }
        }

        public bool IsAppointmentContextMenuVisible
        {
            get
            {
                // No Appointment Selected
                if (SelectedAppointment == null)
                    return false;

                // Cannot display menu for all day events
                if (((CustomAppointment)SelectedAppointment).IsAllDayEvent)
                    return false;

                return true;
            }
        }

        public bool IsSlotContextMenuVisible
        {
            get
            {
                // If the current View Mode is Month or Time Line disable the context menu
                if ((ViewMode == ViewModes.Month) || (ViewMode == ViewModes.TimeLine))
                    return false;

                //  Is the currently selected request a Master MRTC Record
                if ((Keys.SelectedAppointmentRequest != null) && Keys.SelectedAppointmentRequest.IsMRTCParent)
                    return false;

                // No Timeslot selected ?
                if (SelectedTimeSlot == null)
                    return false;

                // Check to see if the clinic has availability defined for today
                if (!DayOfSelectedTimeSlotHasAvailability)
                    return false;

                return true;
            }
        }

        public bool IsTimeSlotViewerVisible
        {
            get { return _isTimeSlotViewerVisible; }
            set
            {
                _isTimeSlotViewerVisible = value;
                OnPropertyChanged("IsTimeSlotViewerVisible");
            }
        }

        public bool IsTimeSlotViewerContextMenuVisible
        {
            get
            {
                return IsTimeSlotViewerAppointmentContextMenuVisible
                       ||
                       IsTimeSlotViewerSlotContextMenuVisible;
            }
        }

        public bool IsTimeSlotViewerAppointmentContextMenuVisible
        {
            get
            {
                // No Appointment Selected
                if (SelectedTimeSlotViewerAppointment == null)
                    return false;

                // Cannot display menu for all day events
                if (((CustomAppointment)SelectedTimeSlotViewerAppointment).IsAllDayEvent)
                    return false;

                return true;
            }
        }

        public bool IsTimeSlotViewerSlotContextMenuVisible
        {
            get { return !IsTimeSlotViewerAppointmentContextMenuVisible; }
        }

        public bool IsInitialized
        {
            get { return _isInitialized; }
        }

        public DateTime LastUpdate
        {
            get { return _lastUpdate; }
        }

        public DateTime ActiveMonthStartDate { get; set; }

        public DateTime? MultiCalendarDayViewDate { get; set; }

        public bool IsPartofGoupSchedule { get; set; }

        public ViewModes ViewMode
        {
            get
            {
                if (View?.Scheduler == null)
                    return ViewModes.None;
                else
                    return GetViewMode(View.Scheduler.ActiveViewDefinition);
            }
        }

        public ViewModes GetViewMode(ViewDefinitionBase viewDef)
        {
            var deftype = viewDef?.GetType();

            if (deftype != null)
            {
                if (deftype == typeof (VSEDayViewDefinition))
                    return ViewModes.Day;

                if (deftype == typeof (VSEWeekViewDefinition))
                    return ViewModes.Week;

                if (deftype == typeof (VSEMonthViewDefninition))
                    return ViewModes.Month;

                if (deftype == typeof (TimelineViewDefinition))
                    return ViewModes.TimeLine;
            }

            return ViewModes.None;
        }

        public ViewDefinitionBase GetViewDefinition(string viewTitle)
        {
            return View.Scheduler.ViewDefinitions.FirstOrDefault(d => (d.Title == viewTitle));
        }

        public bool SelectedTimeSlotIsToday
        {
            get
            {
                if (SelectedTimeSlot == null)
                    return false;

                return SelectedTimeSlot.Start.Date.CompareTo(DateTime.Today) == 0;
            }
        }

        public bool DayOfSelectedTimeSlotHasAvailability
        {
            get { return SelectedSlotDayHasAvailability(SelectedTimeSlot); }
        }

        public bool SelectedSlotDayHasAvailability(Slot slot)
        {
            if (slot == null)
                return false;

            if (SelectedResource == null)
                return false;

            return (SelectedResource.AccessBlocks.Count(ab => ((ab.START_TIME.Value.Date.CompareTo(slot.Start.Date) == 0) && (ab.ACCESSTYPEID != Keys.AccessTypes.Cancelled))) > 0);
        }

        public bool SelectedTimeSlotIsAfterClinicStart
        {
            get
            {
                if (SelectedTimeSlot == null)
                    return false;

                if (SelectedResource == null)
                    return false;

                if (SelectedResource.Clinic == null)
                    return false;

                var HOUR_DISPLAY_BEGIN = SelectedResource.Clinic.HOUR_DISPLAY_BEGIN;

                if (string.IsNullOrEmpty(HOUR_DISPLAY_BEGIN))
                    return true;

                int hourDisplayBegin;

                if (int.TryParse(HOUR_DISPLAY_BEGIN, out hourDisplayBegin))
                {
                    var clinicStart = SelectedTimeSlot.Start.Date.AddHours(hourDisplayBegin);
                    return SelectedTimeSlot.Start.CompareTo(clinicStart) >= 0;
                }
                return false;
            }
        }

        private void CollapseTimeSlotViewer( bool expand )
        {
            if ( _respondToEvents )
                View.CollapseTimeSlotViewerExpander( expand );
        }

        private void PatientUnselect( string s )
        {
            SelectedPatient = null;
        }

        public Thickness MouseOverHighlightBinding
        {
            get
            {
                if (ViewMode == ViewModes.TimeLine)
                    return new Thickness(0, 0, 0, 0);
                else
                    return new Thickness(0, 0, 15, 0);
            }
        }

        [Dependency]
        public Factory<SchdAppointment> SchdAppointmentFactory { get; set; }

        public DelegateCommand<string> CreateAppointmentCommand { get; private set; }

        public DelegateCommand<string> ExpandedEntryCommand { get; private set; }
        public void ExecuteExpandedEntryCommand(string command)
        {
            var workingAppointment = GetWorkingAppointment(command);

            if (workingAppointment == null)
                return;

            Dictionary<string, string> patientIenAndAppointmentStartTimeDictionary = new Dictionary<string, string>();
            patientIenAndAppointmentStartTimeDictionary.Add("IEN", workingAppointment.SchdAppointment.Patient.IEN);
            patientIenAndAppointmentStartTimeDictionary.Add("AppointmentStartTime",
                ExtensionMethods.ConvertToFilemanDateTime(Convert.ToDateTime(workingAppointment.SchdAppointment.START_TIME)));

            if (ProcessAppointmentCommand("Expanded Entry", workingAppointment))
                EventAggregator.GetEvent<LaunchExpandedEntryDialogEvent>().Publish(patientIenAndAppointmentStartTimeDictionary);
        }

        public DelegateCommand<string> CreateWalkInAppointmentCommand { get; private set; }

        public DelegateCommand<string> EditAppointmentCommand { get; private set; }

        public DelegateCommand<string> ViewAppointmentCommand { get; private set; }

        public DelegateCommand<string> MarkAsNoShowCommand { get; private set; }

        public DelegateCommand<string> UndoANoShowCommand { get; private set; }

        public DelegateCommand<string> CheckInPatientCommand { get; private set; }

        public DelegateCommand<string> UndoCheckInPatientCommand { get; private set; }

        public DelegateCommand<string> CheckOutPatientCommand { get; private set; }

        public DelegateCommand<string> UndoCheckOutPatientCommand { get; private set; }

        public DelegateCommand<string> CancelAppointmentCommand { get; }

        public DelegateCommand<string> UndoCancelAppointmentCommand { get; private set; }

        public DelegateCommand<string> CopyAppointmentCommand { get; private set; }

        public DelegateCommand<string> PasteAppointmentCommand { get; private set; }

        public DelegateCommand<object> VisibleRangeChangedCommand { get; private set; }

        public int FirstVisibleHour { get; private set; }

        private void RefreshAppointmentSchedulerEvent(object sender, EventArgs e)
        {
            _refreshTimer.Stop();
            DoRefreshScheduler(_refreshCommand);
        }

        public virtual void RefreshAppoinmentScheduler(string command)
        {
            _refreshTimer.Stop();

            try
            {
                _isInitialized = _isInitialized || (command == "INITIAL_LAUNCH");
                _refreshCommand = command;
            }
            finally
            {
                _refreshTimer.Start();
            }
        }

        private void DoRefreshScheduler(string command)
        {
            if (View?.Scheduler == null)
                return;

            if (!_defaultMinAppointmentHeight.HasValue)
                _defaultMinAppointmentHeight = View?.Scheduler?.MinAppointmentHeight;

            if ( _isInitialized && IsSelectedTab )
            {
                if (ViewMode == ViewModes.TimeLine)
                    View.Scheduler.MinAppointmentHeight = _defaultMinAppointmentHeight.Value + 8;
                else
                    View.Scheduler.MinAppointmentHeight = _defaultMinAppointmentHeight.Value;

                // Disable View Events
                View.Scheduler_EnableEvents(false);

                try
                {
                    // Get the first visible date
                    VisibleStartDate = View.Scheduler.VisibleRange.Start.Date;
                    // Get the end of the last visible date
                    VisibleEndDate = View.Scheduler.VisibleRange.End.Date;
                    // Initialize the calendar
                    InitializeSchedule(command, VisibleStartDate, VisibleEndDate);
                }
                catch ( Exception ex )
                {
                    TraceLog.LogException( GetType().Name, "RefreshAppoinmentScheduler", ex );
                    //MessageBox.Show(ex.Source + ": " + ex.Message);   
                }
            }
        }

        // This maskes a unique set of locks for each instance
        private readonly object _initializeScheduleWorkerLock = Guid.NewGuid();
        private readonly object _initializeScheduleCompleted = Guid.NewGuid();

        protected void InitializeSchedule(string command, DateTime startDate, DateTime endDate)
        {
            Slot oldSelectedTimeSlot = SelectedTimeSlot;
            Appointment oldSelectedAppointment = SelectedAppointment;

            IsAsync = true;
            DoWorkAsync(
                (s, args) =>
                {
                    lock (_initializeScheduleWorkerLock)
                    {
                        WorkAsyncResults results = new WorkAsyncResults();
                        args.Result = results;
                        string action = "Initializing";
                        List<string> messages = new List<string>();
                        int errorCounter = 0;
                        List<string> nonAvailabilitySchedules = new List<string>();

                        try
                        {
                            action = "Reading";

                            foreach (SchdResource resource in SelectedResourceGroup.Resources)
                                try
                                {
                                    if (!DataAccessService.CheckIfAvailabilityEverDefined(resource, true))
                                        nonAvailabilitySchedules.Add("    " + resource.RESOURCE_NAME);

                                    resource.GetSchedule(startDate, endDate, false);
                                }
                                catch (Exception ex)
                                {
                                    errorCounter++;
                                    TraceLog.LogException(GetType().Name, "InitializeSchedule", new Exception(String.Format("While reading the {0} schedule there was a problem.", resource.RESOURCE_NAME), ex));
                                }

                            if (errorCounter > 0)
                                messages.Add("Schedule Loading Errors^There were problems loading 1 or more schedules.\nPlease review the Trace Log for details.");

                            if (nonAvailabilitySchedules.Count > 0)
                            {
                                string message = "The listed Schedule(s) have never had availability defined.\n\n" + String.Join("\n", nonAvailabilitySchedules.ToArray());
                                messages.Add("VS Availability^" + message);
                            }

                            if (messages.Count > 0)
                                results.Messages = messages;
                            results.Success = true;
                        }
                        catch (Exception ex)
                        {
                            results.Error = new Exception(String.Format("There was a problem {0} the {1} schedule..", action, SelectedResourceGroup.Name), ex);
                        }
                    }
                },
                (s, args) =>
                {
                    lock (_initializeScheduleCompleted)
                    {
                        WorkAsyncResults results = (WorkAsyncResults)args.Result;

                        if (results.Messages != null)
                            foreach(string message in results.Messages)
                            {
                                string[] parts = message.Split('^');

                                if (parts.Count() > 1)
                                    View.AlertUser(parts[1], parts[0]);
                                else
                                    View.AlertUser(parts[0], "Schedule Loading Messages");
                            }

                        if (results.Success)
                        {
                            // Let's count any errors
                            int errorCount = 0;
                            // Clear existing appointments
                            AppointmentsCollection.Clear();
                            // Clear existing slots
                            SlotsCollection.Clear();
                            // Clear Time Slot Viewer
                            TimeSlotAppointmentsList = new List<Appointment>();

                            // Process the schedules
                            foreach (SchdResource resource in SelectedResourceGroup.Resources)
                                try
                                {
                                    // Initialize Appointment display
                                    InitializeAppointments(resource, startDate, endDate);
                                    // Initialize Slots display
                                    InitializeSlots(resource, startDate, endDate);
                                }
                                catch (Exception ex)
                                {
                                    TraceLog.LogException(GetType().Name, "InitializeSchedule", new Exception(string.Format("While reading the {0} schedule there was a problem.", resource.RESOURCE_NAME), ex));
                                    errorCount++;
                                }

                            if (errorCount > 0)
                                View?.AlertUser("There were problems initializing some schedules.\nSee the Trace Log for details.", "Initialize Schedule(s)");

                            if ((ViewMode == ViewModes.Day) || (ViewMode == ViewModes.Week))
                                View?.ScrollSchedulerToOffset();

                            if (oldSelectedTimeSlot != null)
                            {
                                if (Keys.Methods.TimesOverlap(oldSelectedTimeSlot.Start, oldSelectedTimeSlot.End, this.VisibleStartDate.Date, this.VisibleEndDate.Date))
                                {
                                    Resource resource = (Resource) oldSelectedTimeSlot.Resources.FirstOrDefault(r => ((Resource) r).ResourceType == Keys.AppointmentGroupNames.Appointment);
                                    SchdResource schdResource = SelectedResourceGroup.Resources.FirstOrDefault(r => (r.ScheduleTabCaption == resource?.ResourceName));

                                    if (schdResource != null)
                                        SelectedTimeSlot = GetCustomSlotFromTimeSlot(oldSelectedTimeSlot, schdResource);
                                }
                            }
                            else if (oldSelectedAppointment != null)
                            {
                                if (Keys.Methods.TimesOverlap(oldSelectedAppointment.Start, oldSelectedAppointment.End, this.VisibleStartDate.Date, this.VisibleEndDate.Date))
                                {
                                    var customAppt = (CustomAppointment) oldSelectedAppointment;
                                    SchdResource schdResource = SelectedResourceGroup.Resources.FirstOrDefault(r => (r.ScheduleID == customAppt.SchdAppointment.ResourceScheduleID));

                                    if (schdResource != null)
                                        SelectedAppointment = schdResource.CustomAppointments.FirstOrDefault(a => (a.UniqueId == customAppt.UniqueId));
                                }
                            }
                        }
                        else
                        {
                            if (results.Error == null)
                                Keys.AlertUser("The " + PaneTitle + " schedule failed to load.", "Initialize Schedule");
                            else
                                Keys.AlertUserError(View, results.Error, "Initialize Schedule Error");
                        }

                        _lastUpdate = DateTime.Now;
                    }

                    View.Scheduler_EnableEvents(true);
                    _scheduleIsInitialized = true;
                },
                IsPartofGoupSchedule || !_scheduleIsInitialized,
                false,
                "Loading " + SelectedResourceGroup.Name + " Schedule"
            );
        }

        protected void InitializeAppointments(SchdResource resource, DateTime startDate, DateTime endDate)
        {
            // Get the appointments list
            //AppointmentList = resource.Appointments;
            // Initialize the appointments
            var appointmentsSource = new SchdAppointmentSource(resource.Appointments, IsEnableUpdateAppointment).Appointments;
            resource.CustomAppointments.Clear();

            var hStart = startDate.Date;

            // Add any holidays to the all day appointments
            do
            {
                var sHoliday = DataAccessService.IsHoliday(hStart);

                if (!string.IsNullOrEmpty(sHoliday))
                {
                    var holidayEvent = new CustomAppointment
                    {
                        Body = sHoliday,
                        Start = hStart.Date,
                        Subject = sHoliday,
                        Category = _appointmentCategories[Keys.Categories.Holiday_Event],
                        IsAllDayEvent = true
                    };
                    holidayEvent.End = holidayEvent.Start.AddDays(1).AddMinutes(-1);
                    appointmentsSource.Add(holidayEvent);
                }

                hStart = hStart.AddDays(1);
            } while (hStart <= endDate.Date.AddDays(1));

            if (ViewMode != ViewModes.TimeLine)
            {
                // Add appointment type information to the all day events
                var vApptEvent = new CustomAppointment
                {
                    Body = string.Format(
                        "{0}{1} min Appt{2}",
                        resource.Clinic.VariableAppointmentLength ? "VL " : string.Empty,
                        resource.Clinic.Duration,
                        string.IsNullOrEmpty(resource.Clinic.OVERBOOK) ? string.Empty : " | Max OB: " + resource.Clinic.OVERBOOK
                        ),
                    Start = VisibleStartDate.Date,
                    IsAllDayEvent = true,
                };

                switch (ViewMode)
                {
                    case ViewModes.Day:
                        vApptEvent.End = vApptEvent.Start.AddDays(1).AddMinutes(-1);
                        break;
                    case ViewModes.Week:
                    case ViewModes.Month:
                    case ViewModes.TimeLine:
                        vApptEvent.End = vApptEvent.Start.AddDays(7).AddMinutes(-1);
                        break;
                }

                vApptEvent.Subject = vApptEvent.Body;
                vApptEvent.Category = _appointmentCategories[Keys.Categories.VariableLength_Event];

                appointmentsSource.Add(vApptEvent);

                // Add any Special Instructions to the all day events
                if (!string.IsNullOrEmpty(resource.Clinic.SpecialInstructions))
                {
                    var instructionEvent = new CustomAppointment
                    {
                        Subject = "Special Instructions",
                        Body = resource.Clinic.SpecialInstructions,
                        Start = VisibleStartDate.Date,
                        IsAllDayEvent = true,
                        Category = _appointmentCategories[Keys.Categories.SpecialInstructionsEvent]
                    };

                    switch (ViewMode)
                    {
                        case ViewModes.Day:
                            instructionEvent.End = vApptEvent.Start.AddDays(1).AddMinutes(-1);
                            break;
                        case ViewModes.Week:
                        case ViewModes.Month:
                        case ViewModes.TimeLine:
                            instructionEvent.End = vApptEvent.Start.AddDays(7).AddMinutes(-1);
                            break;
                    }

                    instructionEvent.Resources.Add(new Resource(Keys.AppointmentGroupNames.SpecialInstructions));
                    appointmentsSource.Add(instructionEvent);
                }
            }

            foreach (CustomAppointment appt in appointmentsSource)
            {
                // Set the Appointment IsTimeLineView property
                // This modifies the Start\End
                appt.IsTimeLineView = (ViewMode == ViewModes.TimeLine);
                // Set the Grouping Value
                appt.Resources.Add(new Resource(resource.ScheduleTabCaption, Keys.AppointmentGroupNames.Appointment));
                AppointmentsCollection.Add(appt);
                resource.CustomAppointments.Add(appt);
            }
        }

        protected void InitializeSlots(SchdResource resource, DateTime startDate, DateTime endDate)
        {
            resource.CustomSlots.Clear();
            // Clinic Start Time
            int clinicStartingHour;

            if (!int.TryParse(resource.Clinic.HOUR_DISPLAY_BEGIN, out clinicStartingHour))
                clinicStartingHour = 8;

            if (FirstVisibleHour == 0)
                FirstVisibleHour = clinicStartingHour;
            else
                FirstVisibleHour = Math.Min(FirstVisibleHour, clinicStartingHour);

            List<CustomSlot> slotsCollection = null;

            switch (ViewMode)
            {
                case ViewModes.Day:
                case ViewModes.Week:
                    slotsCollection = InitializeDayWeekSlots(clinicStartingHour, resource, startDate, endDate);
                    break;
                case ViewModes.Month:
                    slotsCollection = InitializeMonthSlots(resource, startDate, endDate);
                    break;
                case ViewModes.TimeLine:
                    slotsCollection = InitializeTimeLineSlots(resource, startDate, endDate);
                    break;
                case ViewModes.None:
                    Keys.AlertUserError(View, new Exception($"Unable to initialize Availability for {resource.RESOURCE_NAME}, unknown View Mode."), "Availability Initialization");
                    break;
            }

            // Set the Grouping Value
            if (slotsCollection != null)
                foreach (CustomSlot slot in slotsCollection)
                {
                    slot.Resources.Add(new Resource(resource.ScheduleTabCaption, Keys.AppointmentGroupNames.Appointment));
                    SlotsCollection.Add(slot);
                    resource.CustomSlots.Add(slot);
                }
        }

        private List<CustomSlot> InitializeDayWeekSlots(int clinicStartingHour, SchdResource resource, DateTime startDate, DateTime endDate)
        {
            List<CustomSlot> slotsCollection = new List<CustomSlot>();
            // Do we show the Time Slot Viewer ?
            IsTimeSlotViewerVisible = !IsPartofGoupSchedule;
            // Can Clinic book Holidays
            var canBookHolidays = resource.Clinic.ClinicCanBookOnHolidays;

            for (var workingDay = startDate.Date; workingDay.CompareTo(endDate.Date) < 0; workingDay = workingDay.AddDays(1))
            {
                // Get today's slots for the current resource, sorted by time
                List<SchdAvailability> todaysAccessBlocks = resource.AccessBlocks.Where(ab => (ab.START_TIME.Value.Date.CompareTo(workingDay.Date) == 0)).OrderBy(t => t.START_TIME.Value).ToList();

                // Get today 's appointments for the current resource, sorted by time
                var todaysAppointments = resource.CustomAppointments.Where(a => a.Start.Date.CompareTo(workingDay.Date) == 0).OrderBy(t => t.Start).ToList();
                var clinicStartingTime = workingDay.AddHours(clinicStartingHour);
                var endOfDay = workingDay.AddDays(1);
                DateTime endingTime;
                // Is the current date a holiday
                var isHoliday = !string.IsNullOrEmpty(DataAccessService.IsHoliday(workingDay));

                // Non holiday categories
                var nonWorkingCategory            = Keys.ColorCodes.SlotCategories[Keys.SlotCategoryNames.NonWorkingSlot];
                var unavailableCategory           = Keys.ColorCodes.SlotCategories[Keys.SlotCategoryNames.UnavailableSlot];
                var availableCategory             = Keys.ColorCodes.SlotCategories[Keys.SlotCategoryNames.AvailableSlot];
                var cancelledAvailabilityCategory = Keys.ColorCodes.SlotCategories[Keys.SlotCategoryNames.CancelledAvailabilitySlot];

                if (isHoliday)
                {
                    // Holiday categories
                    nonWorkingCategory            = Keys.ColorCodes.SlotCategories[Keys.SlotCategoryNames.HolidayNotWorkingSlot];
                    unavailableCategory           = Keys.ColorCodes.SlotCategories[Keys.SlotCategoryNames.HolidayUnavailableSlot];
                    availableCategory             = Keys.ColorCodes.SlotCategories[Keys.SlotCategoryNames.HolidayAvailableSlot];
                    cancelledAvailabilityCategory = Keys.ColorCodes.SlotCategories[Keys.SlotCategoryNames.HolidayCancelledAvailabilitySlot];
                }

                if ((todaysAccessBlocks.Count == 0) || (isHoliday && !canBookHolidays))
                {
                    // because there is no availability or it is a holiday and we cannot book holidays we make 1 big slot for the entire day
                    var daySlot = new CustomSlot(workingDay, endOfDay, nonWorkingCategory);
                    daySlot.IsReadOnly = true;
                    slotsCollection.Add(daySlot);
                    // go to the next day
                    continue;
                }
                else
                    // Make a single non-working slot for the time from the beginning of the day up to clinic Start
                    slotsCollection.Add(new CustomSlot(workingDay, clinicStartingTime, nonWorkingCategory));

                // Make non-working slots for any Appointments prior to the Clinic start
                var beforeClinicStart = todaysAppointments.Where(a => !a.IsAllDayEvent && a.Start.CompareTo(clinicStartingTime) < 0).ToList();

                if (beforeClinicStart.Count > 0)
                    foreach (CustomAppointment appointment in beforeClinicStart)
                        if (slotsCollection.FirstOrDefault(sl => (sl.Start.CompareTo(appointment.Start) == 0)) == null)
                        {
                            var s = new CustomSlot(appointment.Start, appointment.End, nonWorkingCategory);
                            s.SlotCount = "*";
                            slotsCollection.Add(s);
                        }

                // If there Clinic Start < Fist Availability then make a single unavailable slot from the Clinic start of day to the first availability
                var startingTime = workingDay.AddHours(clinicStartingHour);
                endingTime = todaysAccessBlocks[0].START_TIME.Value;

                if (startingTime.CompareTo(endingTime) < 0)
                    slotsCollection.Add(new CustomSlot(startingTime, endingTime, unavailableCategory));

                CustomSlot previousSlot = null;

                // Remove any end of day access blocks with no availabiltiy
                for (int idx = todaysAccessBlocks.Count - 1; idx >= 0; idx--)
                {
                    var accessBlock = todaysAccessBlocks[idx];

                    if (string.IsNullOrEmpty(accessBlock.SlotCount))
                        todaysAccessBlocks.Remove(accessBlock);
                    else
                        break;
                }

                // Next make slots for all of the working day's access blocks
                for (int idx = 0; idx < todaysAccessBlocks.Count; idx++)
                {
                    var accessBlock = todaysAccessBlocks[idx];
                    var category = unavailableCategory;

                    // determine the slot availability category    
                    if (accessBlock.ACCESSTYPEID == Keys.AccessTypes.Available)
                        category = availableCategory;
                    else if (accessBlock.ACCESSTYPEID == Keys.AccessTypes.Unavailable)
                        category = unavailableCategory;
                    else if (accessBlock.ACCESSTYPEID == Keys.AccessTypes.Cancelled)
                        category = cancelledAvailabilityCategory;

                    // make the new CustomSlot
                    var slot = new CustomSlot(accessBlock, category);

                    //  if Slot availability = "*"
                    if (slot.SlotCount == "*")
                        //  if the count of appointments overlapping this accessblock = 0
                        if (todaysAppointments.Count(a => !a.IsAllDayEvent && Keys.Methods.TimesOverlap(slot.Start, slot.End, a.Start, a.End)) == 0)
                            // then the "*" availability represents appointments booked prior to the clinic start of day or after the last availability of the day
                            // Set the availability value to an empty string
                            slot.SlotCount = "";

                    // Add the new slot
                    slotsCollection.Add(slot);
                    // Set the Previous Slot
                    previousSlot = slot;
                }

                // Make a single unavailable slot from the end of availability to the end of the day
                // Set the working to start with here is the end of the last access block
                startingTime = todaysAccessBlocks[todaysAccessBlocks.Count - 1].END_TIME.Value;
                // Set the ending time to the end of the day
                slotsCollection.Add(new CustomSlot(startingTime, endOfDay, unavailableCategory));

                // Make unavailable slots for any Appointments after the last availability of the day
                var afterAvailability = todaysAppointments.Where(a => a.Start.CompareTo(todaysAccessBlocks[todaysAccessBlocks.Count - 1].END_TIME) >= 0).ToList();

                if (afterAvailability.Count > 0)
                    foreach (var appointment in afterAvailability)
                        if (SlotsCollection.FirstOrDefault(sl => sl.Start.CompareTo(appointment.Start) == 0) == null)
                        {
                            var s = new CustomSlot(appointment.Start, appointment.End, unavailableCategory);
                            s.SlotCount = "*";
                            slotsCollection.Add(s);
                        }
            }

            return slotsCollection;
        }

        private List<CustomSlot> InitializeMonthSlots(SchdResource resource, DateTime startDate, DateTime endDate)
        {
            List<CustomSlot> slotsCollection = new List<CustomSlot>();
            // Hide the TimeSlotViewer
            IsTimeSlotViewerVisible = false;

            for (var workingDay = startDate.Date; workingDay.CompareTo(endDate.Date) < 0; workingDay = workingDay.AddDays(1))
            {
                // Get today's slots, sorted by time
                List<SchdAvailability> todaysAccessBlocks = resource.AccessBlocks.Where(ab => (ab.START_TIME.Value.Date.CompareTo(workingDay.Date) == 0)).OrderBy(t => t.START_TIME.Value).ToList();
                // Set End of Day
                var endOfDay = workingDay.AddDays(1);
                // Non holiday categories
                var nonWorkingCategory = Keys.ColorCodes.SlotCategories[Keys.SlotCategoryNames.NonWorkingSlot];
                var availableCategory = Keys.ColorCodes.SlotCategories[Keys.SlotCategoryNames.AvailableSlot];

                // Is the current date a holiday
                if (!string.IsNullOrEmpty(DataAccessService.IsHoliday(workingDay)))
                {
                    // Holiday categories
                    nonWorkingCategory = Keys.ColorCodes.SlotCategories[Keys.SlotCategoryNames.HolidayNotWorkingSlot];
                    availableCategory = Keys.ColorCodes.SlotCategories[Keys.SlotCategoryNames.HolidayAvailableSlot];
                }

                if (todaysAccessBlocks.Count == 0)
                {
                    // No availability today
                    var daySlot = new CustomSlot(workingDay, endOfDay, nonWorkingCategory);
                    daySlot.IsReadOnly = true;
                    slotsCollection.Add(daySlot);
                }
                else
                {
                    var daySlot = new CustomSlot(workingDay, endOfDay, availableCategory);
                    var slotCount = todaysAccessBlocks.Sum(a => a.SLOTS);
                    daySlot.SLOTS = slotCount;
                    daySlot.SlotCountVerticalAlignment = VerticalAlignment.Center;

                    if (slotCount > 0)
                        daySlot.SlotCount = "+";

                    slotsCollection.Add(daySlot);
                }
            }

            return slotsCollection;
        }

        private List<CustomSlot> InitializeTimeLineSlots(SchdResource resource, DateTime startDate, DateTime endDate)
        {
            List<CustomSlot> slotsCollection = new List<CustomSlot>();
            // Hide the TimeSlotViewer
            IsTimeSlotViewerVisible = false;

            for (var workingDay = startDate.Date; workingDay.CompareTo(endDate.Date) < 0; workingDay = workingDay.AddDays(1))
            {
                // Get today's slots, sorted by time
                List<SchdAvailability> todaysAccessBlocks = resource.AccessBlocks.Where(ab => (ab.START_TIME.Value.Date.CompareTo(workingDay.Date) == 0)).OrderBy(t => t.START_TIME.Value).ToList();
                // Set End of Day
                var endOfDay = workingDay.AddDays(1).AddMinutes(-1);
                // Non holiday categories
                var nonWorkingCategory = Keys.ColorCodes.SlotCategories[Keys.SlotCategoryNames.NonWorkingSlot];
                var availableCategory = Keys.ColorCodes.SlotCategories[Keys.SlotCategoryNames.AvailableSlot];

                // Is the current date a holiday
                if (!string.IsNullOrEmpty(DataAccessService.IsHoliday(workingDay)))
                {
                    // Holiday categories
                    nonWorkingCategory = Keys.ColorCodes.SlotCategories[Keys.SlotCategoryNames.HolidayNotWorkingSlot];
                    availableCategory = Keys.ColorCodes.SlotCategories[Keys.SlotCategoryNames.HolidayAvailableSlot];
                }

                CustomSlot daySlot =
                    todaysAccessBlocks.Any()
                        ? new CustomSlot(workingDay, endOfDay, availableCategory)
                        : new CustomSlot(workingDay, endOfDay, nonWorkingCategory);

                daySlot.IsReadOnly = true;
                slotsCollection.Add(daySlot);
            }

            return slotsCollection;
        }

        private void PrintCurrentScheduler( object T )
        {
            if ( IsSelectedTab )
            {
                var dialog = new PrintDialog();
                if ( dialog.ShowDialog() == true )
                {
                    dialog.PrintVisual( View.Scheduler, "Schedule" );
                }
            }
        }

        public void DisplayWalkInChanged( string isDisplayWalkInEnabled )
        {
            if ( IsSelectedTab )
            {
                VisibleStartDate = View.Scheduler.VisibleRange.Start;
                VisibleEndDate = View.Scheduler.VisibleRange.End;
                _isDisplayWalkInEnabled = isDisplayWalkInEnabled == "true";
            }
        }

        public void CopyAppointment(SchdAppointment copyAppt)
        {
            throw new NotImplementedException();
            //CopySchdAppointment = copyAppt;
        }

        public void PasteAppointment(SchdAppointment pasteAppt)
        {
            throw new NotImplementedException();
            //try
            //{
            //    if (IsSelectedTab && CopySchdAppointment != null)
            //    {
            //        var a = new AppointmentData
            //        {
            //            AppointmentID = "0",
            //            StartTime = pasteAppt.START_TIME,
            //            Duration = (Convert.ToDateTime(CopySchdAppointment.END_TIME) - Convert.ToDateTime(CopySchdAppointment.START_TIME)).TotalMinutes.ToString(),
            //            Notes = CopySchdAppointment.NOTE,
            //            ResourceName = CopySchdAppointment.RESOURCENAME
            //        };
            //        a.EndTime = Convert.ToDateTime(pasteAppt.START_TIME).AddMinutes(int.Parse(a.Duration)).ToString();

            //        var p = new PatientInformation
            //        {
            //            IEN = CopySchdAppointment.PATIENTID,
            //            Name = CopySchdAppointment.PATIENTNAME
            //        };

            //        var errorMessages = new List<string>();

            //        var errorMessage = DataAccessService.IsHoliday(Convert.ToDateTime(pasteAppt.START_TIME));
            //        var canBookHoliday = false;
            //        if (CurrentAppointmentRequest != null && !string.IsNullOrEmpty(errorMessage))
            //        {
            //            if (!string.IsNullOrEmpty(CurrentAppointmentRequest.ClinicIen))
            //                canBookHoliday = DataAccessService.ClinicCanBookOnHoliday(CurrentAppointmentRequest.ClinicResourceId, null);
            //            if (!canBookHoliday)
            //            {
            //                var msg = Convert.ToDateTime(pasteAppt.START_TIME).ToShortDateString() + " is a holiday (" + errorMessage + ") and is not allowed to be scheduled.";
            //                MessageBox.Show(msg);
            //                return;
            //            }
            //        }

            //        bool isTelehealth;

            //        //check conflict appointment
            //        if (DataAccessService.CheckConflictAppointment("Paste Appointment", a.ResourceName, p.IEN, p.Name, a.StartTime, a.EndTime, out isTelehealth))
            //            return;

            //        //check overbook authority
            //        var bOverbookAppointment = DataAccessService.OverBookAuthority(pasteAppt);

            //        // refresh available slots
            //        if (pasteAppt.SelectedResource == null)
            //            DataAccessService.RefreshAvailabilitySchedule(pasteAppt.RESOURCENAME, Convert.ToDateTime(pasteAppt.START_TIME), Convert.ToDateTime(pasteAppt.END_TIME));
            //        else
            //            DataAccessService.RefreshAvailabilitySchedule(pasteAppt.SelectedResource, Convert.ToDateTime(pasteAppt.START_TIME), Convert.ToDateTime(pasteAppt.END_TIME));

            //        //Check No Shows
            //        errorMessage = string.Empty;
            //        if (Convert.ToDateTime(pasteAppt.START_TIME).Date > DateTime.Now.Date)
            //            errorMessage = DataAccessService.NoShow(p.IEN, pasteAppt.RESOURCENAME, "AddNew");
            //        if (errorMessage != string.Empty)
            //            View.AlertUser(errorMessage, "Patient No-Shows");

            //        //check slots available
            //        Keys.IsAddAppointmentOrLeftClickOnScheduler = true;
            //        errorMessage = DataAccessService.CheckSlotsAvailable(SelectedResource, pasteAppt.START_TIME, pasteAppt.END_TIME);

            //        if (isTelehealth)
            //            // If this is part of a Telehealth pair then a conflict DOES exist, must add i minute to the starttime to appease VistA
            //            a.StartTime = Convert.ToDateTime(a.StartTime).AddMinutes(1).ToString("MMM d,yyyy hh:mm");

            //        if (!string.IsNullOrEmpty(errorMessage) && !Keys.Methods.IsNumeric(errorMessage))
            //        {
            //            if (bOverbookAppointment)
            //            {
            //                if (Keys.ConfirmUser((DependencyObject)View, errorMessage, "Paste Appointment", true))
            //                {
            //                    a.IsOverbook = true;
            //                    errorMessages = DataAccessService.AddNewAppointment(a, p, false);
            //                }
            //            }
            //            else
            //            {
            //                View.AlertUser("You do not have permission to overbook this appointment.", "New Appointment");
            //                return;
            //            }
            //        }
            //        else
            //        {
            //            errorMessages = DataAccessService.AddNewAppointment(a, p, false);
            //        }

            //        if (errorMessages.Count > 0)
            //        {
            //            errorMessage = errorMessages[1];
            //        }
            //        if (errorMessage != string.Empty)
            //        {
            //            View.AlertUser(errorMessage, "Copy/Paste Appointment");
            //        }
            //        RefreshCurrentSchedulerEventHandler(String.Empty);
            //    }
            //}
            //catch (Exception ex)
            //{
            //    TraceLog.LogException(GetType().Name, "PasteAppointment", ex);
            //    //MessageBox.Show(ex.Source + ": " + ex.Message);
            //}
        }

        public void UndoCancelAppointment( string apptID )
        {
            try
            {
                var errorMessage = DataAccessService.UndoCancelAppointment( SchdAppointment.APPOINTMENTID );

                if ( errorMessage != string.Empty )
                {
                    View.AlertUser( errorMessage, "Undo Cancel Appointment" );
                }
                else
                {
                    RefreshCurrentSchedulerEventHandler(String.Empty);
                    EventAggregator.GetEvent<UpdateFutureAppointmentsEvent>().Publish( null );
                }
            }
            catch ( Exception ex )
            {
                TraceLog.LogException( GetType().Name, "UndoCancelAppointment", ex );
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }

        public void UndoNoShowAppointment( string apptID )
        {
            try
            {
                var errorMessage = DataAccessService.AppointmentNoShow( SchdAppointment.APPOINTMENTID, false );

                if ( errorMessage != string.Empty )
                {
                    View.AlertUser( errorMessage, "Undo NoShow Appointment" );
                }
                else
                {
                    var request = SchdAppointment.AppointmentRequest;
                    if ( request == null )
                    {
                        var reqType = SchdAppointment.APPTREQTYPE_REQTYPE;
                        request = DataAccessService.GetAppointmentRequestByAppointmentIen( SchdAppointment.APPOINTMENTID, ref reqType );
                    }

                    var payload = string.Format( "{0}|{1}|{2}|{3}",
                        request.AppointmentRequestType,
                        request.RequestManagmentIen,
                        "Undo No Show",
                        "1"
                    );
                    EventAggregator.GetEvent<RemoveRequestEvent>().Publish( payload );
                    RefreshCurrentSchedulerEventHandler(String.Empty);
                    EventAggregator.GetEvent<UpdateFutureAppointmentsEvent>().Publish( null );
                    Dispatcher.BeginInvoke((Action) (() => { Mouse.OverrideCursor = Cursors.Arrow; }), DispatcherPriority.Normal);
                }
            }
            catch ( Exception ex )
            {
                TraceLog.LogException( GetType().Name, "UndoNoShowAppointment", ex );
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }

        public void UndoCheckInPatient( string apptID )
        {
            try
            {
                UndoCheckinSucceeded = false;
                var errorMessage = DataAccessService.UndoCheckIn( SchdAppointment.APPOINTMENTID );

                if ( errorMessage != string.Empty )
                {
                    View.AlertUser( errorMessage, "Undo CheckIn Patient" );
                }
                else
                {
                    RefreshCurrentSchedulerEventHandler(String.Empty);
                    EventAggregator.GetEvent<UpdateFutureAppointmentsEvent>().Publish( null );
                }
            }
            catch ( Exception ex )
            {
                TraceLog.LogException( GetType().Name, "UndoCheckInPatient", ex );
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }

        public void UndoCheckOutPatient( string apptID )
        {
            try
            {
                var errorMessage = DataAccessService.UndoCheckOut( SchdAppointment.APPOINTMENTID );

                if ( errorMessage != string.Empty )
                {
                    View.AlertUser( errorMessage, "Undo CheckOut Patient" );
                }
                else
                {
                    RefreshCurrentSchedulerEventHandler(String.Empty);
                    EventAggregator.GetEvent<UpdateFutureAppointmentsEvent>().Publish( null );
                }
            }
            catch ( Exception ex )
            {
                TraceLog.LogException( GetType().Name, "UndoCheckOutPatient", ex );
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }

        public void SchedulerDateChanged( SchedulerViewDateEventArgs newDate )
        {
            try
            {
                if ( Keys.IsEwlTransfer )
                    return;

                if ( IsSelectedTab )
                {
                    if ( ( newDate.Sender != "Scheduler.SchedulerView" ) && ( newDate.NewDate.Date != CurrentSelectedDate.Date ) )
                    {
                        CurrentSelectedDate = newDate.NewDate.Date;
                        RefreshAppoinmentScheduler( "DATE_CHANGED" );
                        RaiseCanExecuteChanged();
                    }
                }
            }
            catch ( Exception ex )
            {
                TraceLog.LogException( GetType().Name, "SchedulerDateChanged", ex );
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }

        public void DayRangeChanged( string dayRange )
        {
            _dayRange = int.Parse( dayRange );

            if ( IsSelectedTab )
            {
//                View.Scheduler.WeekViewDefinition.VisibleDays = _dayRange;
            }
        }

        public void ViewModeChanged( string viewMode )
        {
            try
            {
                if ( IsSelectedTab )
                {
                    string[] parts = viewMode.Split('^');

                    if (parts[0] != "Scheduler.Navigation")
                    {
                        ViewDefinitionBase activeViewDefinition = GetViewDefinition(parts[1]);

                        if (activeViewDefinition != null)
                            View.Scheduler.ActiveViewDefinition = activeViewDefinition;
                    }
                }
            }
            catch ( Exception ex )
            {
                TraceLog.LogException( GetType().Name, "ViewModeChanged", ex );
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }

        public void ExecuteCopyAppointmentCommand( string command )
        {
            EventAggregator.GetEvent<CopyAppointmentEvent>().Publish( SchdAppointment );
        }

        public bool CanExecuteCopyAppointmentCommand( string command )
        {
            return true;
        }

        public void ExecutePasteAppointmentCommand( string command )
        {
            EventAggregator.GetEvent<IsPasteAvailabilityEvent>().Publish( command );
        }

        public bool CanExecutePasteAppointmentCommand( string command )
        {
            return true;
        }

        public void ExecuteSchedulerTabSelectedCommand( object resource )
        {
            try
            {
                SelectedResource = resource as SchdResource;
                EventAggregator.GetEvent<ResourceSelectedEvent>().Publish( SelectedResource );

                var selectedIndex = 3;
                if ( IsSelectedTab )
                {
                    int selectedMin;
                    //switch ( ViewMode )
                    //{
                    //    case ViewModes.Week :
                    //        selectedMin = ((WeekViewDefinition)View.Scheduler.ActiveViewDefinition). .MinorTickLength.
                    //        if ( SelectedResource != null && selectedMin == 0 )
                    //            selectedMin = SelectedResource.Clinic.Duration;
                    //        break;
                    //    case SViewModes.Day :
                    //        selectedMin = View.Scheduler.DayViewDefinition.TimeSlotLength.Minutes;
                    //        if ( SelectedResource != null && selectedMin == 0 )
                    //            selectedMin = SelectedResource.Clinic.Duration;
                    //        break;
                    //    default :
                            selectedMin = 30;
                    //        break;
                    //}

                    switch ( selectedMin )
                    {
                        case 10 :
                            selectedIndex = 0;
                            break;
                        case 15 :
                            selectedIndex = 1;
                            break;
                        case 20 :
                            selectedIndex = 2;
                            break;
                        case 30 :
                            selectedIndex = 3;
                            break;
                        case 60 :
                            selectedIndex = 4;
                            break;
                    }
                    EventAggregator.GetEvent<SelectTimeScaleEvent>().Publish( selectedIndex );
                }
            }
            catch ( Exception ex )
            {
                TraceLog.LogException( GetType().Name, "ExecuteSchedulerTabSelectedCommand", ex );
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }

        public void PatientSelected( Patient selectedPatient )
        {
            try
            {
                if ( selectedPatient != null )
                {
                    Keys.SelectedPatient = selectedPatient;
                    SelectedPatient = selectedPatient;

                    if ( !_patientSelectedByMe )
                        SelectedPatient = selectedPatient;
                    else
                        _patientSelectedByMe = false;
                }
            }
            catch ( Exception ex )
            {
                TraceLog.LogException( GetType().Name, "PatientSelected", ex );
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }

        public void ExecuteEditAppointmentCommand( string command )
        {
            var workingAppointment = GetWorkingAppointment(command);

            if (workingAppointment == null)
                return;

            if (ProcessAppointmentCommand("Edit Appointment", workingAppointment))
                EventAggregator.GetEvent<EditAppointmentEvent>().Publish(SchdAppointment);
        }

        public void ExecuteViewAppointmentCommand( string command )
        {
            var workingAppointment = GetWorkingAppointment(command);

            if (workingAppointment == null)
                return;

            if (ProcessAppointmentCommand("View Appointment", workingAppointment))
                EventAggregator.GetEvent<ViewAppointmentEvent>().Publish( SchdAppointment );
        }
        
        public void ExecuteVisibleRangeChangedCommand( object param )
        {
            DateSpan visibleRange = (DateSpan) param;

            if ((visibleRange.Start.Date.CompareTo(VisibleStartDate) != 0) || (visibleRange.End.Date.CompareTo(VisibleEndDate) != 0))
                RefreshAppoinmentScheduler("SCHEDULER_NAVIGATION");
        }

        public void ExecuteCreateAppointmentCommand( string title )
        {
            try
            {
                if (ProcessSlotCommand("Add Appointment"))
                {
                    SchdAppointment.WALKIN = "0";
                    SchdAppointment.PATIENTID = SelectedPatient.IEN;
                    SchdAppointment.PATIENTNAME = SelectedPatient.Name;

                    if (SchdAppointment.AppointmentRequest == null)
                        SchdAppointment.AppointmentRequest = Keys.SelectedAppointmentRequest;

                    if (SchdAppointment.SelectedResource == null)
                        SchdAppointment.SelectedResource = SelectedResource;

                    EventAggregator.GetEvent<AddNewAppointmentEvent>().Publish(SchdAppointment);
                }
            }
            catch ( Exception ex )
            {
                var e = new Exception("There was a problem executing the Add Appointment Command", ex);
                Keys.AlertUserError(View, e, "Add Appointment");
                TraceLog.LogException( GetType().Name, "ExecuteCreateAppointmentCommand", e );
            }
        }

        public void ExecuteCreateWalkInAppointmentCommand( string title )
        {
            try
            {
                if (ProcessSlotCommand("Create Walk-in Appointment"))
                {
                    SchdAppointment.WALKIN = "1";
                    SchdAppointment.PATIENTID = SelectedPatient.IEN;
                    SchdAppointment.PATIENTNAME = SelectedPatient.Name;

                    if (SchdAppointment.AppointmentRequest == null)
                        SchdAppointment.AppointmentRequest = Keys.SelectedAppointmentRequest;

                    if (SchdAppointment.SelectedResource == null)
                        SchdAppointment.SelectedResource = SelectedResource;

                    EventAggregator.GetEvent<AddWalkInAppointmentEvent>().Publish(SchdAppointment);
                }
            }
            catch ( Exception ex )
            {
                var e = new Exception("There was a problem executing the Create Walk-in Appointment Command", ex);
                Keys.AlertUserError(View, e, "Create Walk-in Appointment");
                TraceLog.LogException(GetType().Name, "ExecuteCreateWalkInAppointmentCommand", e);
            }
        }

        public void ExecuteMarkAsNoShowCommand( string command )
        {
            var workingAppointment = GetWorkingAppointment(command);

            if (workingAppointment == null)
                return;

            if (ProcessAppointmentCommand("Mark as No Show", workingAppointment))
                EventAggregator.GetEvent<LaunchMarkAsNoShowDialogEvent>().Publish(SchdAppointment);
        }

        public void ExecuteUndoANoShowCommand( string command )
        {
            var workingAppointment = GetWorkingAppointment(command);

            if (workingAppointment == null)
                return;

            if (ProcessAppointmentCommand("Undo NoShow", workingAppointment))
                UndoNoShowAppointment(SchdAppointment.APPOINTMENTID);
        }

        public void ExecuteCheckInPatientCommand( string command )
        {
            var workingAppointment = GetWorkingAppointment(command);

            if (workingAppointment == null)
                return;

            if (ProcessAppointmentCommand("Check In Patient", workingAppointment))
                EventAggregator.GetEvent<LaunchCheckInDialogEvent>().Publish( SchdAppointment );
        }

        public void ExecuteUndoCheckInPatientCommand( string command )
        {
            var workingAppointment = GetWorkingAppointment(command);

            if (workingAppointment == null)
                return;

            if (ProcessAppointmentCommand("Undo Check In Patient", workingAppointment))
                UndoCheckInPatient( SchdAppointment.APPOINTMENTID );
        }

        public void ExecuteCheckOutPatientCommand( string command )
        {
            var workingAppointment = GetWorkingAppointment(command);

            if (workingAppointment == null)
                return;

            if (ProcessAppointmentCommand("Check Out Patient", workingAppointment))
                EventAggregator.GetEvent<LaunchCheckOutDialogEvent>().Publish( SchdAppointment );
        }

        public void ExecuteUndoCheckOutPatientCommand( string command )
        {
            var workingAppointment = GetWorkingAppointment(command);

            if (workingAppointment == null)
                return;

            if (ProcessAppointmentCommand("Undo Check Out Patient", workingAppointment))
                UndoCheckOutPatient( SchdAppointment.APPOINTMENTID );
        }

        public void ExecuteCancelAppointmentCommand( string command )
        {
            var workingAppointment = GetWorkingAppointment(command);

            if (workingAppointment == null)
                return;

            if (SelectedResource != null && SelectedResource.Clinic != null)
                SchdAppointment.ClinicIen = SelectedResource.Clinic.ClinicIen;
            
            if (ProcessAppointmentCommand("Cancel Appointment", workingAppointment))
                ExecuteCancelAppointment(SchdAppointment, null);
        }

        public void ExecuteCancelAppointment(SchdAppointment oldAppointment, Action<bool> successCallback, bool addToWaitList = true)
        {
            if (string.IsNullOrEmpty(oldAppointment.MRTCParentRequestIEN))
                EventAggregator.GetEvent<LaunchCancelApptDialogEvent>().Publish(new CancelApptDialogEventPayload(oldAppointment, successCallback, addToWaitList));
            else
                EventAggregator.GetEvent<LaunchMRTCCancelDialog>().Publish(new LaunchMRTCCancelDialogArgs(null, oldAppointment, null, successCallback));
        }

        public void ExecuteUndoCancelAppointmentCommand( string command )
        {
            var workingAppointment = GetWorkingAppointment(command);

            if (workingAppointment == null)
                return;

            if (ProcessAppointmentCommand("Undo Cancel Appointment", workingAppointment))
                UndoCancelAppointment( SchdAppointment.APPOINTMENTID );
        }

        public bool CanExecuteSchedulerTabSelectedCommand( object resource )
        {
            return true;
        }

        public bool CanExecuteViewAppointmentCommand( string title )
        {
            var workingAppointment = GetWorkingAppointment(title);

            if (workingAppointment == null)
                return false;

            return true;
        }

        public bool CanExecuteVisibleRangeChangedCommand (object param)
        {
            return (param != null);
        }

        public bool CanExecuteEditAppointmentCommand( string title )
        {
            var workingAppointment = GetWorkingAppointment(title);

            if (workingAppointment == null)
                return false;

            return workingAppointment.IsEditAppointmentEnabled;
        }

        public bool CanExecuteCreateAppointmentCommand( string title )
        {
            return !Keys.LoggedInUserSDECViewOnlyKey
                   &&
                   (Keys.SelectedAppointmentRequest != null)
                   &&
                   DayOfSelectedTimeSlotHasAvailability
                   &&
                   SelectedTimeSlotIsAfterClinicStart;
        }

        public bool CanExecuteCreateWalkInAppointmentCommand( string title )
        {
            return !Keys.LoggedInUserSDECViewOnlyKey
                   &&
                   Keys.SelectedAppointmentRequest != null
                   &&
                   SelectedTimeSlotIsToday
                   &&
                   DayOfSelectedTimeSlotHasAvailability; 
        }

        public bool CanExecuteMarkAsNoShowCommand( string title )
        {
            var workingAppointment = GetWorkingAppointment(title);

            if (workingAppointment == null)
                return false;

            return workingAppointment.IsNotNoShowAppointmentEnabled;
        }

        public bool CanExecuteUndoANoShowCommand( string title )
        {
            var workingAppointment = GetWorkingAppointment(title);

            if (workingAppointment == null)
                return false;

            return workingAppointment.IsNoShowAppointmentEnabled;
        }

        public bool CanExecuteCheckInPatientCommand( string title )
        {
            var workingAppointment = GetWorkingAppointment(title);

            if (workingAppointment == null)
                return false;

            return workingAppointment.IsNotCheckedInAppointmentEnabled;
        }

        public bool CanExecuteUndoCheckInPatientCommand( string title )
        {
            var workingAppointment = GetWorkingAppointment(title);

            if (workingAppointment == null)
                return false;

            return workingAppointment.IsCheckedInAppointmentEnabled;
        }

        public bool CanExecuteCheckOutPatientCommand( string title )
        {
            var workingAppointment = GetWorkingAppointment(title);

            if (workingAppointment == null)
                return false;

            return workingAppointment.IsNotCheckedOutAppointmentEnabled;
        }

        public bool CanExecuteUndoCheckOutPatientCommand( string title )
        {
            var workingAppointment = GetWorkingAppointment(title);

            if (workingAppointment == null)
                return false;

            return workingAppointment.IsCheckedOutAppointmentEnabled;
        }

        public bool CanExecuteCancelAppointmentCommand( string title )
        {
            var workingAppointment = GetWorkingAppointment(title);

            if (workingAppointment == null)
                return false;

            return workingAppointment.IsNotCancelledAppointmentEnabled;
        }

        public bool CanExecuteUndoCancelAppointmentCommand( string title )
        {
            // We are not allowing this at present, but menu stays visible, so ALWAY disable;
            return false;
            //CustomAppointment workingAppointment = GetWorkingAppointment(title);

            //if (workingAppointment == null)
            //    return false;

            //return workingAppointment.IsNotCancelledAppointmentEnabled;
        }

        public void ResizeAppointment( Appointment newAppointment, int duration, bool isVariableLengthClinic)
        {
            if (!isVariableLengthClinic)
            {
                View.AlertUser("Appointment could not be modified:" + Environment.NewLine + Environment.NewLine +
                                "Variable Length Appointments are not enabled for this clinic in VistA Scheduling Management.", "Modify Appointment Error");
                return;
            }

            try
            {
                CustomAppointment customAppointment = (CustomAppointment)newAppointment;

                var a = new AppointmentData()
                {
                    Notes = customAppointment.SchdAppointment.NOTE,
                    Duration = duration.ToString(),
                    AppointmentID = customAppointment.SchdAppointment.APPOINTMENTID
                };

                var errorMessage = DataAccessService.EditAppointment( a );

                if (errorMessage != string.Empty)
                    View.AlertUser("Appointment could not be modified:" + Environment.NewLine + Environment.NewLine + errorMessage, "Modify Appointment Error");

                RefreshCurrentSchedulerEventHandler(String.Empty);
            }
            catch ( Exception ex )
            {
                TraceLog.LogException( GetType().Name, "ResizeAppointment", ex );
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }

        public bool MoveAppointment( CustomAppointment originalAppointment, SchdResource originalResource, Slot targetSlot, SchdResource targetResource )
        {
            try
            {
                // Make sure we have the original Appointment Request
                if (Keys.Methods.IsNumeric(originalAppointment.SchdAppointment.APPOINTMENTID) && SchdAppointment.AppointmentRequest == null)
                {
                    var reqType = string.Empty;
                    SchdAppointment.AppointmentRequest = DataAccessService.GetAppointmentRequestByAppointmentIen(originalAppointment.SchdAppointment.APPOINTMENTID, ref reqType);
                }

                SchdAppointment newAppointment = LoadNewAppointmentInfo(targetSlot, targetResource, originalAppointment.SchdAppointment);
                bool bOverbookAppointment;
                bool isTelehealth;

                if (!DataAccessService.CanAddNewAppointmentPreCheck("Move Appointment", newAppointment, out isTelehealth, out bOverbookAppointment, originalAppointment.SchdAppointment.APPOINTMENTID, false))
                    return false;

                string errorMessage = string.Empty;
                List<string> errorMessages = new List<string>();
                string newAppointmentId = string.Empty;

                originalAppointment.SchdAppointment.MRTCParentRequestIEN = string.Empty;

                ExecuteCancelAppointment( originalAppointment.SchdAppointment, ( success ) =>
                {
                    if ( success )
                    {
                        AppointmentData a = new AppointmentData
                        {
                            AppointmentID = string.Empty,
                            StartTime = newAppointment.START_TIME,
                            EndTime = newAppointment.END_TIME,
                            Duration = (Convert.ToDateTime(newAppointment.END_TIME) - Convert.ToDateTime(newAppointment.START_TIME)).TotalMinutes.ToString(),
                            Notes = newAppointment.NOTE,
                            ResourceName = newAppointment.RESOURCENAME                            
                        };   

                        PatientInformation p = new PatientInformation
                        {
                            IEN = newAppointment.PATIENTID,
                            Name = newAppointment.PATIENTNAME
                        };

                        if ( newAppointment.AppointmentRequest == null )
                            errorMessages = DataAccessService.AddNewAppointment( a, p, false, true );
                        else
                            errorMessages = DataAccessService.AddNewAppointment( a, p, false, newAppointment.AppointmentRequest );

                        if ( errorMessages != null && errorMessages.Count > 0 )
                        {
                            newAppointmentId = errorMessages[0];
                            errorMessage = errorMessages[1].ToString();
                        }

                        if ( !string.IsNullOrEmpty( newAppointmentId ) && Keys.Methods.IsNumeric( newAppointmentId ) )
                        {
                            Thread.Sleep( 200 );
                            newAppointment.APPOINTMENTID = newAppointmentId;

                            if ( originalAppointment?.SchdAppointment.AppointmentRequest != null )
                                EventAggregator.GetEvent<RemoveRequestEvent>().Publish( originalAppointment.SchdAppointment.AppointmentRequest.RequestManagmentIen );

                            CustomAppointment newCustomAppointment = new CustomAppointment(newAppointment);
                            newCustomAppointment.Resources.Add(new Resource(newAppointment.SelectedResource.ScheduleTabCaption, Keys.AppointmentGroupNames.Appointment));
                            ((ObservableCollection<Appointment>)View.Scheduler.AppointmentsSource).Add( newCustomAppointment );
                        }
                        if ( string.IsNullOrEmpty( errorMessage ) )
                        {
                            if ( newAppointment.AppointmentRequest != null )
                            {
                                var errorText = string.Empty;

                                if (newAppointment.AppointmentRequest.IsMRTCChild)
                                    DataAccessService.SetMRTCChildRequest(newAppointment.AppointmentRequest, newAppointment.AppointmentRequest.ApptRequestIen.ToString(), newAppointmentId);

                                if ( newAppointment.AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl )
                                    errorText = DataAccessService.WLClose(
                                        newAppointment.AppointmentRequest.RequestManagmentIen,
                                        Keys.WaitListDispositions.RemovedScheduledAssigned
                                    );
                                else if ( newAppointment.AppointmentRequest.IsAppt )
                                    errorText = DataAccessService.ARClose(
                                        newAppointment.AppointmentRequest.RequestManagmentIen,
                                        Keys.WaitListDispositions.RemovedScheduledAssigned
                                    );

                                if ( errorText != string.Empty )
                                    View.AlertUser(
                                        "Appointment Request could not be reclosed:" + Environment.NewLine +
                                        Environment.NewLine + errorText, "Error" );
                            }
                        }

                        EventAggregator.GetEvent<UpdateFutureAppointmentsEvent>().Publish(null);
                        EventAggregator.GetEvent<RefreshWaitlistEvent>().Publish(string.Empty);
                        EventAggregator.GetEvent<AppointmentAddedEvent>().Publish(a);
                        RefreshCurrentSchedulerEventHandler(String.Empty);
                    }
                }, false );

                return true;
            }
            catch ( Exception ex )
            {
                TraceLog.LogException( GetType().Name, "MoveAppointment", ex );
                //MessageBox.Show(ex.Source + ": " + ex.Message);
                return false;
            }
        }

        public void RefreshCurrentSchedulerEventHandler(string command)
        {
            if (_isInitialized && IsSelectedTab && _respondToEvents)
            {
                // Disable View Events
                View.Scheduler_EnableEvents(false);

                try
                {
                    // All  that is changing is the content of the curent schedule, so all we do is re-read the current schedule
                    // Initialize the calendar
                    InitializeSchedule(command, VisibleStartDate, VisibleEndDate);
                } 
                catch (Exception ex)
                {
                    TraceLog.LogException(GetType().Name, "RefreshCurrentSchedulerEventHandler", ex);
                    //MessageBox.Show(ex.Source + ": " + ex.Message);   
                }
            }
        }

        public void HotKeyEvents( Key k )
        {
            View.HotKeyHandler( k );
        }

        #region IDisposable members

        protected void Dispose( bool itIsSafeToAlsoFreeManagedObjects )
        {
            try
            {
                // Free unmanaged resources
                SelectedResource = null;

                //Free managed resources too, but only if I'm being called from Dispose
                //(If I'm being called from Finalize then the objects might not exist
                //anymore
                if ( itIsSafeToAlsoFreeManagedObjects )
                {
                    if ( View != null )
                    {
                        View.Dispose();
                        View = null;
                    }
                }
            }
            catch ( Exception ex )
            {
                TraceLog.LogException( GetType().Name, "Dispose", ex );
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }

        public void Dispose()
        {
            Dispose( true ); //true: safe to free managed resources
            GC.SuppressFinalize( this ); //Hey, GC: don't bother calling finalize later
        }

        ~SchedulerPresentationModel()
        {
            //we're being finalized (i.e. destroyed), call Dispose in case the user forgot to
            Dispose( false );
        }

        #endregion IDisposable members
    }

    internal class ScheduleDates
    {
        private DateTime _initialDate;
        private DateTime _selectedDate;

        public ScheduleDates( DateTime initialDate )
        {
            _initialDate = initialDate;
            _selectedDate = initialDate;
        }

        public DateTime InitialDate
        {
            get { return _initialDate; }
            set { _initialDate = value; }
        }

        public DateTime SelectedDate
        {
            get { return _selectedDate; }
            set { _selectedDate = value; }
        }

        public DayOfWeek DOW
        {
            get { return InitialDate.DayOfWeek; }
        }

        public DateTime FirstDateOfMonth
        {
            get
            {
                if ( _selectedDate.Day == 1 )
                    return SelectedDate;
                return new DateTime( _selectedDate.Year, _selectedDate.Month, 1 );
            }
        }

        public DateTime FirstDateOfWeek
        {
            get
            {
                try
                {
                    var delta = DOW - SelectedDate.DayOfWeek;
                    if ( delta > 0 )
                        delta += 7;
                    return SelectedDate.AddDays( -1*delta );
                }
                catch ( Exception ex )
                {
                    TraceLog.LogException( GetType().Name, "FirstDateOfWeek", ex );
                    //MessageBox.Show(ex.Source + ": " + ex.Message);
                    return DateTime.MinValue;
                }
            }
        }

        public DateTime LastDateOfMonth
        {
            get { return FirstDateOfMonth.AddMonths( 1 ).AddDays( -1 ); }
        }

        public DateTime LastDateOfWeek
        {
            get { return FirstDateOfWeek.AddDays( 6 ); }
        }
    }
}