﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Input;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Models;

using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ScheduleView;

namespace ClinSchd.Modules.Task.Scheduler
{
    internal class VSEScheduleViewDragDropBehavior : ScheduleViewDragDropBehavior
    {
        /* DragDropState Parameter ******************************************
        DragDropState Parameter

        Each of the methods provided in the ScheduleViewDragDropBehavior class receives an object of type DragDropState.The object contains all the needed information in order to help you achieve the required drag and drop and resize functionality.The object provides the following properties:

            Appointment - Holds a reference to the appointment that is held with the mouse.

            DestinationAppointmentsSource - Holds a reference to the AppointmentsSource collection of the drag destination.

            DestinationSlots - Gets a collection containing the new slots of the dragged Appointments.

            DraggedAppointments - Contains all appointments that are being dragged, including the appointment that is selected while the user clicks and holds the mouse button.When resizing this property is null.

            IsControlPressed - Gets or sets a value indicating the control key is pressed.

            ServiceProvider - Gets the IServiceProvider associated with the current instance.

            SlotDuration - Gets the duration between two minor ticks.

            SourceAppointmentsSource - Holds a reference to the AppointmentsSource collection of the drag source.

            SourceResources (available with Q2 2014 version of UI for WPF) - Gets a collection containing the original resources from which the dragging operation has started.

            TargetedAppointment(available with Q2 2014 version of UI for WPF) - Holds a reference to the targeted appointment on which the drop operation will be performed.

        /*******************************************************************/

        private SchdResource _resizingSchedResource = null;

        public ISchedulerPresentationModel SchedulerViewModel { get; set; }

        /// <summary>
        /// CanDrop(DragDropState state): Using this method you can change the cursor that indicates a resize operation is happening.
        /// That could be done initially or while resizing an Appointment,
        /// </summary>
        /// <param name="state"></param>
        /// <returns></returns>
        public override bool CanDrop(DragDropState state)
        {
            if (!Keys.EnableScheduleDragAndDrop)
                return false;

            if (state.IsControlPressed)
                return false;

            return base.CanDrop(state);
        }

        /// <summary>
        /// CanResize(DragDropState state): Gets the value specifying whether the resize operation can be finished or not.
        /// The method is called multiple times during the Resize operation and you can easily stop the operation by returning false.
        /// </summary>
        /// <param name="state"></param>
        /// <returns></returns>
        public override bool CanResize(DragDropState state)
        {
            if (!Keys.EnableScheduleDragAndDrop)
                return false;

            if (_resizingSchedResource == null)
                _resizingSchedResource = GetSchdResourceFromAppointment((Appointment)state.Appointment);

            if (_resizingSchedResource != null)
            {
                var destinationSlot = state.DestinationSlots.First() as Slot;
                int duration = (int)(destinationSlot.End - destinationSlot.Start).TotalMinutes;

                // Cancel if duration < clinic duration or duration > 240 - VistA Max
                if ((duration < _resizingSchedResource.Clinic.Duration) || (duration > 240))
                    return false;
            }

            return true;
        }

        /// <summary>
        /// CanStartDrag(DragDropState state): Gets the value specifying whether the drag operation can be performed or not.
        /// The method is called at the moment when the operation is starting.
        /// </summary>
        /// <param name="state"></param>
        /// <returns></returns>
        public override bool CanStartDrag(DragDropState state)
        {
            if (!Keys.EnableScheduleDragAndDrop)
                return false;

            if (state.DraggedAppointments.Count() != 1)
                return false;

            CustomAppointment customAppointment = (CustomAppointment) state.Appointment;

            if (customAppointment.Status != AppointmentStatuses.New)
                return false;

            return base.CanStartDrag(state);
        }

        /// <summary>
        /// CanStartResize(DragDropState state): Gets the value specifying whether the resize operation can be performed or not.
        /// The method is called at the moment when the operation is starting.
        /// </summary>
        /// <param name="state"></param>
        /// <returns></returns>
        public override bool CanStartResize(DragDropState state)
        {
            if (!Keys.EnableScheduleDragAndDrop)
                return false;

            return false;

            _resizingSchedResource = null;
            // Appointments can be resized only for Variable Length Clinics
            // Get Custom Appointment
            CustomAppointment appointment = (CustomAppointment)state.Appointment;

            // cannot resize all day event
            if (appointment.IsAllDayEvent)
                return false;

            // Get Resource From Custom Appointment
            SchdResource schdResource = GetSchdResourceFromAppointment(appointment);

            if (schdResource != null)
                return schdResource.VariableLengthAppointments && appointment.IsEditAppointmentEnabled;

            // Schedule Resource NOT Found, returning false
            return false;
        }

        /// <summary>
        /// DragDropCanceled( DragDropState state) - The method is called whenever the execution of the operation has failed for some reason.
        /// </summary>
        /// <param name="state"></param>
        public override void DragDropCanceled(DragDropState state)
        {
            this.ResizeCursor = Cursors.Arrow;
            _resizingSchedResource = null;
            base.DragDropCompleted(state);
        }

        /// <summary>
        /// DragDropCompleted(DragDropState state): This method is called when the drag-drop operation is finished.
        /// </summary>
        /// <param name="state"></param>
        public override void DragDropCompleted(DragDropState state)
        {
            this.ResizeCursor = Cursors.Arrow;
            _resizingSchedResource = null;
            base.DragDropCompleted(state);
        }

        /// <summary>
        /// Drop(DragDropState state): The method is called right after the Appointment is dropped and before the DragDropCompleted method is called.
        /// </summary>
        /// <param name="state"></param>
        public override void Drop(DragDropState state)
        {
            if (!Keys.EnableScheduleDragAndDrop)
                return;

            this.ResizeCursor = Cursors.Arrow;

            // Original Appointment
            Appointment originalAppt = state.Appointment as Appointment;

            if (originalAppt == null)
                return;

            // target time slot
            Slot targetSlot = state.DestinationSlots.First() as Slot;

            if (targetSlot == null)
                return;


            // original CustomAppointment
            CustomAppointment originalCustomAppt = (CustomAppointment)originalAppt;
            // Original Clinic
            SchdResource originalResource = SchedulerViewModel.SelectedResourceGroup.Resources.FirstOrDefault(r => (r.ScheduleID == originalCustomAppt.SchdAppointment.ResourceScheduleID));

            if (originalResource == null)
                originalResource = SchedulerViewModel.SelectedResourceGroup.Resources[0];

            // target clinic
            SchdResource targetResource = null;

            if (targetSlot.Resources.Count > 0)
            {
                Resource resource = (Resource)targetSlot.Resources.FirstOrDefault(r => ((Resource)r).ResourceType == Keys.AppointmentGroupNames.Appointment);
                targetResource = SchedulerViewModel.SelectedResourceGroup.Resources.FirstOrDefault(r => (r.ScheduleTabCaption == resource?.ResourceName));
            }

            if (targetResource  == null)
                targetResource = SchedulerViewModel.SelectedResourceGroup.Resources[0];

            // target duration
            int targetDuration = (int)(targetSlot.End - targetSlot.Start).TotalMinutes;

            // Are we dropping on the original slot
            if ((targetResource.ScheduleID == originalResource.ScheduleID) && (targetSlot.Start.CompareTo(originalAppt.Start) == 0))
                // yes, cancel \ return
                return;

            string newClinicName = string.Empty;

            // If the dragged appointment is MRTC, need to make sure dropped on resource is same as, or in same stop code as original resource
            if ((originalResource.RESOURCEID != targetResource.RESOURCEID) && originalCustomAppt.IsMRTC)
            {
                if (originalCustomAppt.IsMRTC)
                {
                    List<Clinic> ptCSchClinics = SchedulerViewModel.DataAccessService.GetClinicsWithMatchingStopCode(originalResource.HOSPITAL_LOCATION_ID, targetResource.RESOURCE_NAME);

                    // Nothing returned, or nothing returned with the target resource
                    if (!ptCSchClinics.Any(c => (c.HospitalLocationId == targetResource.HOSPITAL_LOCATION_ID)))
                    {
                        SchedulerViewModel.View.AlertUser($"An MRTC appointment can only be dropped on a Clinic in the same Stop Code as {originalResource.RESOURCE_NAME}", "Move Appointment");
                        return;
                    }
                }

                newClinicName = Environment.NewLine + "at " + targetResource.RESOURCE_NAME;
            }

            // Confirn the user intent
            string message = "Are you sure you want to move the selected appointment?" +
                             Environment.NewLine + Environment.NewLine + "New appointment time: " + targetSlot.Start.ToString() +
                             newClinicName;

            if (SchedulerViewModel.View.ConfirmUser(message, "Move Appointment"))
                if (SchedulerViewModel.MoveAppointment(originalCustomAppt, originalResource, targetSlot, targetResource))
                    base.Drop(state);
        }

        /// <summary>
        /// Resize(DragDropState state) - The method is called right after the resize operation is finished.
        /// You can find the initial Appointment in the Appointment property of the DragDropState and the new information in the DestinationSlot property.
        /// </summary>
        /// <param name="state"></param>
        public override void Resize(DragDropState state)
        {
            if (!Keys.EnableScheduleDragAndDrop)
                return;

            // original appointment
            Appointment appointment = state.Appointment as Appointment;
            // target time slot
            Slot destinationSlot = state.DestinationSlots.First() as Slot;
            // target duration
            int duration = (int)(destinationSlot.End - destinationSlot.Start).TotalMinutes;

            if ((appointment.Start.CompareTo(destinationSlot.Start) == 0) && (appointment.End.CompareTo(destinationSlot.End) != 0))
            {
                string message = "Are you sure you want to change the length of this appointment?" +
                                 Environment.NewLine + Environment.NewLine + "New appointment length: " + duration.ToString() + " minutes";

                if (Keys.ConfirmUser(null, message, "Modify Appointment Length?", true))
                    SchedulerViewModel.ResizeAppointment(appointment, duration, _resizingSchedResource.VariableLengthAppointments);
            }
        }

        /// <summary>
        /// ConvertDraggedData(Object data): This method is fired when you drag appointments from a different source (another control or application) 
        /// and is used to convert the data to an appointment.
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        public override IEnumerable<IOccurrence> ConvertDraggedData(object data)
        {
            // Example Code
            //if (Telerik.Windows.DragDrop.Behaviors.DataObjectHelper.GetDataPresent(data, typeof(Meeting), false))
            //{
            //    var customers = Telerik.Windows.DragDrop.Behaviors.DataObjectHelper.GetData(data, typeof(Customer), true) as IEnumerable;
            //    if (customers != null)
            //    {
            //        var newApp = customers.OfType<Customer>().Select(c => new Appointment { Subject = c.Name });

            //        return newApp;
            //    }
            //}

            return base.ConvertDraggedData(data);
        }

        /// <summary>
        /// CoerceDraggedItems(DragDropState state): Initializes the drag operation. 
        /// This method could be used to filter the selected appointments. 
        /// This method allows adding or removing Appointments to the selection depending on a specific condition.
        /// </summary>
        /// <param name="state"></param>
        /// <returns></returns>
        public override IEnumerable<IOccurrence> CoerceDraggedItems(DragDropState state)
        {
            // Example code
            //var resource = (state.Appointment as Appointment).Resources.First();
            //var allAppointments = state.SourceAppointmentsSource.Cast<IOccurrence>();
            //var desiredAppointments = allAppointments.Where(a => (a as Appointment).Resources.Any(r => r == resource));
            //return desiredAppointments;
            return base.CoerceDraggedItems(state);
        }

        private SchdResource GetSchdResourceFromAppointment(Appointment appointment)
        {
            CustomAppointment customAppointment = (CustomAppointment)appointment;
            SchdResource schdResource = null;

            // Get Resource From Custom Appointment
            schdResource = customAppointment.SchdAppointment.SelectedResource;

            if (schdResource == null)
            {
                // Custom Appointment Resource NOT found
                // Get resource from Appointment resources
                Resource resource = (Resource)appointment.Resources.FirstOrDefault(r => (
                    (((Resource)r).ResourceType == Keys.AppointmentGroupNames.Appointment)
                &&
                    (((Resource)r).ResourceName != Keys.AppointmentGroupNames.Appointment)
                ));

                if (resource != null)
                    // Appointment Resource found, get the Schedule Resource
                    schdResource = SchedulerViewModel.SelectedResourceGroup.Resources.FirstOrDefault(r => (r.RESOURCE_NAME == resource.ResourceName));
            }

            return schdResource;
        }
    }
}
