﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Windows;

using Microsoft.Practices.Composite.Presentation.Commands;
using Microsoft.Practices.Composite.Events;

using ClinSchd.Infrastructure.Models;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Modules.VAR.Models;
using ClinSchd.Infrastructure;

using ClinSchd.Modules.VAR.Events;
using ClinSchd.Modules.VAR.Services;

using VSE_VAR_Service;

namespace ClinSchd.Modules.VAR.ProcessRequest
{
    public class ProcessRequestPresentationModel : IProcessRequestPresentationModel, INotifyPropertyChanged
    {
        #region private fields

        private readonly IEventAggregator _eventAggregator;
        private readonly IDataAccessService _vistaDataAccessService;
        private readonly IVARWebServiceAPIService _webServiceAPI;
        private ValidationMessage _validationMessage;

        private IProcessRequestModel _processRequestModel;
        private Visibility _appointmentBarVisibility;
        private Visibility _bookedAppointmentVisibility = Visibility.Collapsed;
        private Visibility _pickAppointmentVisibility = Visibility.Visible;
        private SchdResource _selectedResource;

        private IList<SchdResource> _resourceList = new List<SchdResource>();
        private IList<DETAIL_CODE> _detailCodeList;

        #endregion private fields

        #region constructor

        public ProcessRequestPresentationModel(IEventAggregator eventAggregator, IProcessRequestView view, IVARWebServiceAPIService webServiceAPI, IDataAccessService vistaDataAccessService)
        {
            _eventAggregator = eventAggregator;
            View = view;
            View.Model = this;
            _webServiceAPI = webServiceAPI;
            _vistaDataAccessService = vistaDataAccessService;
            ValidationMessage = new ValidationMessage ();
            InitialErrorMessage();
            CloseCommand = new DelegateCommand<string>(ExecuteCloseCommand, CanExecuteCloseCommand);
            SaveCommand = new DelegateCommand<string>(ExecuteSaveCommand, CanExecuteSaveCommand);
        }

        #endregion constructor

        #region command members

        public bool CanExecuteCloseCommand(string command)
        {
            return true;
        }

        public bool CanExecuteSaveCommand(string command)
        {
            bool results = true;

            if (AppointmentBarVisibility == Visibility.Visible)
            {
                if (PickAppointmentVisibility == Visibility.Visible)
                    results = AppointmentDateTime.HasValue && !string.IsNullOrEmpty(ResourceName);
            }
            return results;
        }

        public void ExecuteCloseCommand(string command)
        {
            View.Close();
        }

        public void ExecuteSaveCommand(string command)
        {
            bool result = false;

            switch (_processRequestModel.ProcessType)
            {
                case ProcessTypes.Booked:
                    result = _webServiceAPI.BookAppointment(_processRequestModel.Request);
                    break;
                case ProcessTypes.NotBooked:
                    result = _webServiceAPI.NotBookAppointment(_processRequestModel.Request);
                    break;
            }

            if (result)
            {
                View.Close();
                _eventAggregator.GetEvent<VAR_CloseRequestDetailsDialog>().Publish(string.Empty);
            }
        }

        #endregion command members

        #region IProcessRequestPresentationModel members

        public IProcessRequestView View { get; }

        public ValidationMessage ValidationMessage
        {
            get { return _validationMessage; }
            set { _validationMessage = value; }
        }

        public DelegateCommand<string> CloseCommand { get; }

        public DelegateCommand<string> SaveCommand { get; }

        public IProcessRequestModel ProcessRequestModel
        {
            get { return _processRequestModel; }
            set
            {
                _processRequestModel = value;
                PickAppointmentVisibility = _processRequestModel.Request.BOOKED_APPT_DATETIME.HasValue ? Visibility.Collapsed : Visibility.Visible;
                BookedAppointmentVisibility = _processRequestModel.Request.BOOKED_APPT_DATETIME.HasValue ? Visibility.Visible : Visibility.Collapsed;
                SetProcessType();
                SetDetailCode();
                SchdResource res = null;

                if (!string.IsNullOrEmpty(_processRequestModel.ResourceID))
                    res = _vistaDataAccessService.GetSdecResource(_processRequestModel.ResourceID);
                else if (!string.IsNullOrEmpty(_processRequestModel.ResourceName))
                    res = _vistaDataAccessService.GetSdecResourceByName(_processRequestModel.ResourceName);

                if (res != null)
                {
                    ResourceList.Add(res);
                    SelectedResource = res;
                }

                SaveCommand.RaiseCanExecuteChanged();
            }
        }

        public Visibility AppointmentBarVisibility
        {
            get { return _appointmentBarVisibility; }
            private set
            {
                _appointmentBarVisibility = value;
                OnPropertyChanged("AppointmentBarVisibility");
            }
        }

        public string ProcessTitle
        {
            get { return _processRequestModel.ProcessTitle; }
        }

        public string StatusText
        {
            get { return _processRequestModel.Request.STATUS; }
        } // +((_processRequestModel.Request.DetailCode == null) ? String.Empty : ": " + _processRequestModel.Request.DetailCode.PROVIDER_MESSAGE); } }

        public Visibility BookedAppointmentVisibility
        {
            get { return _bookedAppointmentVisibility; }
            set
            {
                _bookedAppointmentVisibility = value;
                OnPropertyChanged("BookedAppointmentVisibility");
            }
        }

        public Visibility PickAppointmentVisibility
        {
            get { return _pickAppointmentVisibility; }
            set
            {
                _pickAppointmentVisibility = value;
                OnPropertyChanged("PickAppointmentVisibility");
            }
        }

        public string FormattedAppointmentTime
        {
            get { return AppointmentDateTime.HasValue ? AppointmentDateTime.Value.ToString() : string.Empty; }
        }

        public DateTime? AppointmentDateTime
        {
            get
            {
                if (_processRequestModel.Request.BOOKED_APPT_DATETIME.HasValue)
                    return _processRequestModel.Request.BOOKED_APPT_DATETIME;

                string working =
                    _processRequestModel.Request.OPTION_DATE_1 +
                    (_processRequestModel.Request.OPTION_TIME_1 == "AM" ? " 10:00" : (_processRequestModel.Request.OPTION_TIME_1 == "PM" ? " 14:00" : ""));

                if (PickAppointmentVisibility == Visibility.Collapsed)
                    return Convert.ToDateTime(working);

                AppointmentDateTime = Convert.ToDateTime(working);
                return _processRequestModel.Request.BOOKED_APPT_DATETIME;
            }
            set
            {
                _processRequestModel.Request.BOOKED_APPT_DATETIME = value;
                DateTime? bookedTime = _processRequestModel.Request.BOOKED_APPT_DATETIME;
                _processRequestModel.Request.APPOINTMENT_DATE = bookedTime.HasValue ? bookedTime.Value.ToShortDateString() : "";
                _processRequestModel.Request.APPOINTMENT_TIME = bookedTime.HasValue ? bookedTime.Value.ToShortTimeString() : "";
                SetDetailCode();
                OnPropertyChanged("AppointmentDateTime");
                OnPropertyChanged("FormattedAppointmentTime");
                SaveCommand.RaiseCanExecuteChanged();
            }
        }

        public string ResourceName
        {
            get { return _selectedResource == null ? string.Empty : _selectedResource.RESOURCE_NAME; }
        }

        public IList<SchdResource> ResourceList
        {
            get { return _resourceList; }
            private set
            {
                _resourceList = value;
                OnPropertyChanged("ResourceList");
                if ( _resourceList != null )
                    View.SetResourceComboBoxOpen(_resourceList.Count > 0);
            }
        }

        public SchdResource SelectedResource
        {
            get { return _selectedResource; }
            set
            {
                _selectedResource = value;

                if (_selectedResource != null)
                {
                    _processRequestModel.Request.PROVIDER_ID = _selectedResource.RESOURCEID;
                    _processRequestModel.Request.PROVIDER_NAME = _selectedResource.RESOURCE_NAME;
                }

                OnPropertyChanged("SelectedResource");
                OnPropertyChanged("ResourceName");
                SaveCommand.RaiseCanExecuteChanged();
            }
        }

        public string MessageToSend
        {
            get { return _processRequestModel.Request.NewMessage; }
            set
            {
                _processRequestModel.Request.NewMessage = value;
                OnPropertyChanged("MessageToSend");
            }
        }

        public void OnClose()
        {
            View.Close();
        }

        #endregion IProcessRequestPresentationModel members

        #region private methods

        private IList<DETAIL_CODE> DetailCodeList
        {
            get
            {
                // If the list has not been read or is empty try to read it
                if ((_detailCodeList == null) || (_detailCodeList.Count == 0))
                    _detailCodeList = _webServiceAPI.GetDetailCodeList();

                return _detailCodeList;
            }
        }

        private void SetDetailCode()
        {
            DETAIL_CODE detailCode = null;

            switch (_processRequestModel.ProcessType)
            {
                case ProcessTypes.Booked:
                    if (AppointmentDateTime.HasValue)
                    {
                        if ( Convert.ToDateTime( _processRequestModel.Request.OPTION_DATE_1 ).Date == AppointmentDateTime.Value.Date )
                        {
                            detailCode = DetailCodeList.FirstOrDefault( dc => dc.DETAIL_CODE_ID == "DETCODE1" );

                            if ( detailCode == null )
                                detailCode = new DETAIL_CODE
                                {
                                    DETAIL_CODE_ID = "DETCODE1"
                                };
                        }
                        else
                        {
                            detailCode = DetailCodeList.FirstOrDefault( dc => dc.DETAIL_CODE_ID == "DETCODE2" );

                            if ( detailCode == null )
                                detailCode = new DETAIL_CODE
                                {
                                    DETAIL_CODE_ID = "DETCODE2"
                                };
                        }
                    }

                    break;
                case ProcessTypes.NotBooked:
                    //if (_processRequestModel.Request.SECOND_REQUEST == 1)
                    //{
                    //    detailCode = DetailCodeList.FirstOrDefault(dc => (dc.DETAIL_CODE_ID == "DETCODE4"));

                    //    if (detailCode == null)
                    //        detailCode = new DETAIL_CODE() { DETAIL_CODE_ID = "DETCODE4" };

                    //    _processRequestModel.Request.STATUS = "Completed";
                    //}
                    //else
                    //{
                        detailCode = DetailCodeList.FirstOrDefault(dc => dc.DETAIL_CODE_ID == "DETCODE3");

                        if (detailCode == null)
                            detailCode = new DETAIL_CODE
                            {
                                DETAIL_CODE_ID = "DETCODE3"
                            };

                        _processRequestModel.Request.STATUS = "Not Booked";
                    //}

                    break;
            }

            _processRequestModel.Request.DetailCode = detailCode;
            OnPropertyChanged("StatusText");
        }

        private void SetProcessType()
        {
            switch (_processRequestModel.ProcessType)
            {
                case ProcessTypes.Booked:
                    AppointmentBarVisibility = Visibility.Visible;
                    break;
                case ProcessTypes.NotBooked:
                    AppointmentBarVisibility = Visibility.Collapsed;
                    break;
            }
        }

        #endregion private methods

        #region error handling

        public void InitialErrorMessage()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
        }

        #endregion error handling

        #region INotify members

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion INotify members

        #region clinicComboBox search members

        public void ResourceSearchStringChanged(string newSearchString)
        {
            if (string.IsNullOrEmpty(newSearchString) || (newSearchString == "BOOK_ANY_PROVIDER") || (newSearchString == "PROVIDER_NOT_FOUND"))
                return;

            if (newSearchString.Length < Keys.ClinicHResourceSearchCharMin)
                return;

            _lastSearchString = newSearchString;
            var list = _vistaDataAccessService.SearchSdecHResources(newSearchString);

            if (list == null)
                ResourceList = new List<SchdResource>();
            else
            { 
                ResourceList = list;
                View.SetResourceComboBoxOpen(ResourceList.Count > 0);
            }
        }

        private string _resourceVeryLastSub = string.Empty;
        private bool _resourceScrollJustRan = true;
        private string _lastSearchString = string.Empty;

        public void ResourceScrollChanged(System.Windows.Controls.ScrollViewer scrollViewer)
        {
            double vOffSet = scrollViewer.VerticalOffset;

            if (!_resourceScrollJustRan)
            {
                _resourceScrollJustRan = !_resourceScrollJustRan;
                return;
            }

            if (ResourceList != null && ResourceList.Count > 0)
            {
                string lastSub = ResourceList[ResourceList.Count - 1].LastSub;
                // save previous very last sub for use in testing
                string veryLastSub = _resourceVeryLastSub;
                // save the current very last sub for future testing
                _resourceVeryLastSub = lastSub;

                // test using the saved previous last sub
                if (veryLastSub == lastSub)
                    return;

                IList<SchdResource> moreResources = _vistaDataAccessService.SearchSdecHResources(_lastSearchString, lastSub);

                if (moreResources != null && moreResources.Count > 0)
                {
                    foreach (SchdResource moreResource in moreResources)
                    {
                        if (_resourceList.FirstOrDefault(r => r.RESOURCEID == moreResource.RESOURCEID) == null)
                            _resourceList.Add(moreResource);

                        if (ResourceList.FirstOrDefault(r => r.RESOURCEID == moreResource.RESOURCEID) == null)
                            ResourceList.Add(moreResource);
                    }

                    ((ProcessRequestView)View).ResourceComboBox.Items.Refresh();
                    scrollViewer.ScrollToVerticalOffset(vOffSet);
                }
            }

            _resourceScrollJustRan = !_resourceScrollJustRan;
        }

        #endregion clinicComboBox search members
    }
}
