﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Windows;
using System.Windows.Input;

using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using Microsoft.Practices.Composite.Presentation.Commands;
using Microsoft.Practices.Unity;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Infrastructure.Models;

using ClinSchd.Modules.VAR.Events;
using ClinSchd.Modules.VAR.Models;
using ClinSchd.Modules.VAR.ProcessRequest;
using ClinSchd.Modules.VAR.Services;
using VSE_VAR_Service;

namespace ClinSchd.Modules.VAR.VAR
{
    public class AppointmentRequestItemPresentationModel : INotifyPropertyChanged, IAppointmentRequestItemPresentationModel
    {
        #region constants

        private const string SORTABLE_DATE_FORMAT = "yyyy/MM/dd";

        #endregion constants

        #region private members

        private readonly IUnityContainer _container;
        private readonly IEventAggregator _eventAggregator;
        private readonly APPOINTMENT_REQUEST_LIST_ITEM _model;
        private ValidationMessage _validationMessage;
        private string _messageToVeteran = string.Empty;
        private AppointmentData _apptData;

        #endregion private members

        #region constructor

        public AppointmentRequestItemPresentationModel(IUnityContainer container, IEventAggregator eventAggregator, APPOINTMENT_REQUEST_LIST_ITEM model)
        {
            _model = model;
            _container = container;
            _eventAggregator = eventAggregator;
            IsLocked = false;
            ValidationMessage = new ValidationMessage();
            InitialErrorMessage();
            CloseCommand = new DelegateCommand<string>(ExecuteCloseCommand, CanExecuteCloseCommand);
            RequestApprovedCommand = new DelegateCommand<string>(ExecuteRequestApprovedCommand, CanExecuteRequestApprovedCommand);
            RequestNotAvailableCommand = new DelegateCommand<string> (ExecuteRequestNotAvailableCommand, CanExecuteRequestNotAvailableCommand);
        }

        #endregion constructor

        #region event handlers

        private void AppointmentAddedEventHandler(AppointmentData apptData)
        {
            _apptData = apptData;
        }

        public void HotKeyEventHandler(Key key)
        {
            if (Keyboard.Modifiers == ModifierKeys.Control)
                switch (key)
                {
                    case Key.M:
                        if (View != null)
                            View.SetFocus();

                        break;
                }
        }

        private void OnClose(string command)
        {
            OnClose();
        }

        #endregion event handlers

        #region public members

        public APPOINTMENT_REQUEST Request { get; set; }

        public APPOINTMENT_REQUEST_LIST_ITEM Model { get { return _model; } }

        public string APPOINTMENT_REQUEST_ID { get { return _model.APPOINTMENT_REQUEST_ID; } }

        public string FIRST_NAME { get { return _model.FIRST_NAME; } }

        public string LAST_NAME { get { return _model.LAST_NAME; } }

        public string PatientName { get { return LAST_NAME + "," + FIRST_NAME; } }

        public string SSN { get { return _model.SSN; } }

        public DateTime DOB { get { return _model.DOB; } }

        public string DOBFormatted { get { return _model.DOB.ToShortDateString(); } }

        public string DOBShortStringSortable { get { return _model.DOB.Date.ToString(SORTABLE_DATE_FORMAT); } }

        public string OPTION_DATE_1 { get { return _model.OPTION_DATE_1; } }

        public string OPTION_TIME_1 { get { return _model.OPTION_TIME_1; } }

        public string Option_1_DateTime { get { return ReformatDateString(OPTION_DATE_1) + " " + OPTION_TIME_1; } }

        public string Option_1_DateTimeSortable { get { return MakeSortDateString(OPTION_DATE_1) + " " + OPTION_TIME_1; } }

        public string OPTION_DATE_2 { get { return _model.OPTION_DATE_2; } }

        public string OPTION_TIME_2 { get { return _model.OPTION_TIME_2; } }

        public string Option_2_DateTime { get { return ReformatDateString(OPTION_DATE_2) + " " + OPTION_TIME_2; } }

        public string Option_2_DateTimeSortable { get { return MakeSortDateString(OPTION_DATE_2) + " " + OPTION_TIME_2; } }

        public string OPTION_DATE_3 { get { return _model.OPTION_DATE_3; } }

        public string OPTION_TIME_3 { get { return _model.OPTION_TIME_3; } }

        public string Option_3_DateTime { get { return ReformatDateString(OPTION_DATE_3) + " " + OPTION_TIME_3; } }

        public string Option_3_DateTimeSortable { get { return MakeSortDateString(OPTION_DATE_3) + " " + OPTION_TIME_3; } }

        public DateTime CREATED_DATE { get { return _model.CREATED_DATE; } }

        public string CREATED_DATEFormatted { get { return _model.CREATED_DATE.ToShortDateString(); } }

        public string CREATED_DATEShortStringSortable { get { return _model.CREATED_DATE.Date.ToString(SORTABLE_DATE_FORMAT); } }

        public string APPOINTMENT_TYPE { get { return _model.APPOINTMENT_TYPE; } }

        public string VISIT_TYPE { get { return _model.VISIT_TYPE; } }

        public string PURPOSE_OF_VISIT { get { return _model.PURPOSE_OF_VISIT; } }

        public string OTHER_PURPOSE_OF_VISIT { get { return _model.OTHER_PURPOSE_OF_VISIT; } }

        public string PurposeOfVisitFormatted { get { return PURPOSE_OF_VISIT + (string.IsNullOrEmpty(OTHER_PURPOSE_OF_VISIT) ? string.Empty : ": " + OTHER_PURPOSE_OF_VISIT); } }

        public bool REQUESTED_PHONE_CALL { get { return _model.REQUESTED_PHONE_CALL == 1; } }

        public string PHONE_NUMBER { get { return _model.PHONE_NUMBER; } }

        public string PHONE_NUMBER_FORMATTED { get { return Keys.Methods.StandardPhoneFormat(PHONE_NUMBER); } }

        public IList<string> BEST_TIME { get { return _model.BEST_TIME.ToList(); } }

        public string BestTimesFormatted { get { return string.Join("\r\n", _model.BEST_TIME); } }

        public string FACILITY_CODE { get { return _model.FACILITY_CODE; } }

        public string FACILITY_NAME { get { return _model.FACILITY_NAME; } }

        public string PROVIDER_ID { get { return _model.PROVIDER_ID; } }

        public string PROVIDER_NAME { get { return _model.PROVIDER_NAME; } }

        public string ProviderFormatted { get { return PROVIDER_ID + " " + PROVIDER_NAME; } }

        public string LockedByUser { get { return _model.LockedByUser; } }

        public string LastMessage { get { return _model.LastMessage; } }

        public string Title { get { return "Mobile Appointment Request Details"; } }

        public IVARWebServiceAPIService WebServiceAPI { get; set; }

        public bool IsLocked { get; set; }

        public void OnClose()
        {
            EnableEventHandlers(false);

            if (IsLocked && (WebServiceAPI != null))
            {
                if (WebServiceAPI.UnlockAppointmentRequest(APPOINTMENT_REQUEST_ID))
                    IsLocked = false;
                else
                    View.AlertUser(
                        "Unable to unlock the currently selected Mobile Request.\n" +
                        "The request will remain locked until you reopen the " + Title + " window, " +
                        "reslect the currently selected request and either select another request or close the window.",
                        "Unlock Error"
                    );
            }

            _eventAggregator.GetEvent<VAR_Refresh>().Publish("UNHOLD");
            ((Window)Parent).Show();
            View.Close();
            View = null;
        }

        public IVARDetailView View { get; set; }

        public object Parent { get; set; }

        public int? WaitTime
        {
            get
            {
                TimeSpan diff = DateTime.Today - CREATED_DATE.Date;
                var waitTime = diff.Days;
                return waitTime;
            }
        }

        public string WaitTimeFormatted
        {
            get { return WaitTime.HasValue ? WaitTime + " days" : string.Empty; }
        }

        public string MessageToVeteran
        {
            get { return _messageToVeteran; }
            set
            {
                _messageToVeteran = value;
                OnPropertyChanged("MessageToVeteran");
            }
        }

        public ValidationMessage ValidationMessage
        {
            get
            {
                return _validationMessage;
            }
            set
            {
                _validationMessage = value;
            }
        }

        public void EnableEventHandlers(bool enable)
        {
            if (enable)
            {
                _eventAggregator.GetEvent<AppointmentAddedEvent>().Subscribe(AppointmentAddedEventHandler, ThreadOption.UIThread, false);
                _eventAggregator.GetEvent<VAR_CloseRequestDetailsDialog>().Subscribe(OnClose, ThreadOption.UIThread, false);
                _eventAggregator.GetEvent<VAR_HotKeyEvent>().Subscribe(HotKeyEventHandler, ThreadOption.UIThread, false);
            }
            else
            {
                _eventAggregator.GetEvent<AppointmentAddedEvent>().Unsubscribe(AppointmentAddedEventHandler);
                _eventAggregator.GetEvent<VAR_CloseRequestDetailsDialog>().Unsubscribe(OnClose);
                _eventAggregator.GetEvent<VAR_HotKeyEvent>().Unsubscribe(HotKeyEventHandler);
            }
        }

        #endregion public members

        #region command members

        public DelegateCommand<string> CloseCommand { get; }
        public DelegateCommand<string> RequestApprovedCommand { get; }
        public DelegateCommand<string> RequestNotAvailableCommand { get; }

        public bool CanExecuteCloseCommand(string command)
        {
            return true;
        }

        public bool CanExecuteRequestApprovedCommand(string command)
        {
            return Request != null;
        }

        public bool CanExecuteRequestNotAvailableCommand(string command)
        {
            return Request != null;
        }

        public void ExecuteCloseCommand(string command)
        {
            View.Close();
        }

        public void ExecuteRequestApprovedCommand(string command)
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
            ProcessRequestModel processRequest = (ProcessRequestModel)_container.Resolve<IProcessRequestModel>();
            processRequest.Owner = View;
            processRequest.ProcessTitle = command;
            processRequest.ProcessType = ProcessTypes.Booked;
            Request.STATUS = "Booked";

            if (_apptData != null)
            {
                DateTime apptDate = Convert.ToDateTime(_apptData.Appointment.START_TIME);
                Request.APPOINTMENT_DATE = apptDate.ToShortDateString();
                Request.APPOINTMENT_TIME = apptDate.ToShortTimeString();
                Request.BOOKED_APPT_DATETIME = apptDate;
                Request.APPOINTMENT_TYPE = _apptData.RequestType;
                processRequest.ResourceName =_apptData.Appointment.RESOURCENAME;
                processRequest.ResourceID = _apptData.Appointment.ResourceId;
            }

            processRequest.Request = Request;
            _eventAggregator.GetEvent<VAR_Refresh>().Publish("HOLD");
            _eventAggregator.GetEvent<VAR_LaunchProcessDialog>().Publish(processRequest);
        }

        public void ExecuteRequestNotAvailableCommand(string command)
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
            ProcessRequestModel processRequest = (ProcessRequestModel)_container.Resolve<IProcessRequestModel>();
            processRequest.Owner = View;
            processRequest.ProcessTitle = command;
            processRequest.ProcessType = ProcessTypes.NotBooked;
            Request.APPOINTMENT_DATE = string.Empty;
            Request.APPOINTMENT_TIME = string.Empty;
            Request.BOOKED_APPT_DATETIME = null;
            Request.PROVIDER_NAME = string.Empty;
            Request.STATUS = "Not Booked";
            processRequest.Request = Request;
            _eventAggregator.GetEvent<VAR_Refresh>().Publish("HOLD");
            _eventAggregator.GetEvent<VAR_LaunchProcessDialog>().Publish(processRequest);
        }

        #endregion command members

        #region private members

        private string ReformatDateString(string source)
        {
            DateTime temp;

            if (DateTime.TryParse(source, out temp))
                return temp.ToShortDateString();
            else
                return source;
        }

        private string MakeSortDateString(string source)
        {
            DateTime temp;

            if (DateTime.TryParse(source, out temp))
                return temp.ToString(SORTABLE_DATE_FORMAT);
            else
                return source;
        }

        #endregion private members

        #region error handling

        public void InitialErrorMessage()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
        }

        #endregion error handling

        #region INotify members

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion INotify members
    }
}
