using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Windows;
using System.Windows.Data;
using System.Windows.Input;

using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using Microsoft.Practices.Composite.Presentation.Commands;
using Microsoft.Practices.Unity;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.VAR.Events;
using ClinSchd.Modules.VAR.Services;

using VSE_VAR_Service;

namespace ClinSchd.Modules.VAR.VAR
{
    public class VARPresentationModel : AsyncValidatableModel, IVARPresentationModel
    {
        #region fields

        private readonly IUnityContainer _container;
        private readonly IEventAggregator _eventAggregator;
        private readonly IVARWebServiceAPIService _webServiceAPI;
        private ValidationMessage _validationMessage;
        private readonly List<string> _waitTimeValues = new List<string>(
            new[]
            {
                Keys.RmGridColumnsShort.WaitTimeAll,
                Keys.RmGridColumnsShort.WaitTimeLessThan30,
                Keys.RmGridColumnsShort.WaitTimeBetween30And60,
                Keys.RmGridColumnsShort.WaitTimeBetween60And90,
                Keys.RmGridColumnsShort.WaitTimeGreaterThanEqualTo90
            }
        );
        private string _waitTimeValue = Keys.RmGridColumns.WaitTimeAll;
        private ObservableCollection<AppointmentRequestItemPresentationModel> _mobileRequestList = new ObservableCollection<AppointmentRequestItemPresentationModel>();
        private List<AppointmentRequestItemPresentationModel> _unfilteredMobileRequestList = new List<AppointmentRequestItemPresentationModel>();
        private AppointmentRequestItemPresentationModel _selectedMobileRequest;
        //private APPOINTMENT_REQUEST _request = null;
        private bool _refreshingVAR;
        //private IVARDetailView _detailView = null;

        #endregion fields

        #region constructor

        public VARPresentationModel (IVARView view, IUnityContainer container, IEventAggregator eventAggregator, IVARWebServiceAPIService webServiceAPI)
        {
            View = view;
            View.Model = this;
            _container = container;
            _eventAggregator = eventAggregator;
            _webServiceAPI = webServiceAPI;
            ValidationMessage = new ValidationMessage ();
            InitialErrorMessage ();

            MobileRequestViewSource = new CollectionViewSource
            {
                Source = MobileRequestList
            };
            MobileRequestViewSource.SortDescriptions.Add(new SortDescription("WaitTIme", ListSortDirection.Descending));

            _eventAggregator.GetEvent<VAR_Refresh>().Subscribe(VAR_RefreshEventHandler, ThreadOption.UIThread, false);
            _eventAggregator.GetEvent<VAR_HotKeyEvent>().Subscribe(HotKeyEventHandler, ThreadOption.UIThread, false);

            CloseCommand = new DelegateCommand<string>(ExecuteCloseCommand, CanExecuteCloseCommand);
            RequestSelectedCommand = new DelegateCommand<string>(ExecuteRequestSelectedCommand, CanExecuteRequestSelectedCommand);
            VAR_RefreshEventHandler("INITIALIZING");
        }

        #endregion constructor

        #region error handling

        public void InitialErrorMessage ()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
        }

        #endregion error handling

        #region event handlers

        public void HotKeyEventHandler(Key key)
        {
            if (Keyboard.Modifiers == ModifierKeys.Control)
                switch (key)
                {
                    case Key.M:
                        if (View != null)
                            View.SetFocus();

                        break;
                }
        }

        #endregion event handlers

        #region command members

        public DelegateCommand<string> CloseCommand { get; }

        public DelegateCommand<string> RequestSelectedCommand { get; }

        public bool CanExecuteCloseCommand(string command)
        {
            return true;
        }

        public bool CanExecuteRequestSelectedCommand(string command)
        {
            return SelectedMobileRequest != null;
        }

        public void ExecuteCloseCommand(string command)
        {
            View.Close();
        }

        public void ExecuteRequestSelectedCommand(string command)
        {
            if (SelectedMobileRequest == null)
                return;

            // Lock the selected request
            bool isLocked;
            var request = _webServiceAPI.GetAppointmentRequest(SelectedMobileRequest.APPOINTMENT_REQUEST_ID, out isLocked);

            if (isLocked)
            {
                View.AlertUser("The selected Appointment Request has been locked by another user.", "Appointment Request Locked");
                return;
            }

            SelectedMobileRequest.IsLocked = true;
            SelectedMobileRequest.WebServiceAPI = _webServiceAPI;
            SelectedMobileRequest.Parent = View;
            SelectedMobileRequest.Request = request;
            SelectedMobileRequest.EnableEventHandlers(true);
            VAR_RefreshEventHandler("HOLD");
            _eventAggregator.GetEvent<VAR_PatientLookupEvent>().Publish(SelectedMobileRequest.SSN + "^" + SelectedMobileRequest.Option_1_DateTime);
            _eventAggregator.GetEvent<VAR_LaunchRequestDetailsDialog>().Publish(SelectedMobileRequest);
            ((Window)View).Hide();
        }

        #endregion command members

        #region IVARPresentationModel Members

        public void OnClose()
        {
            _eventAggregator.GetEvent<VAR_Refresh>().Unsubscribe(VAR_RefreshEventHandler);
            _eventAggregator.GetEvent<VAR_HotKeyEvent>().Unsubscribe(HotKeyEventHandler);
            _eventAggregator.GetEvent<VAR_LaunchRequestDialog>().Publish("CLOSING");
        }

        public ObservableCollection<AppointmentRequestItemPresentationModel> MobileRequestList
        {
            get { return _mobileRequestList; }
            set
            {
                _mobileRequestList = value;
                NotifyPropertyChanged("MobileRequestList");
                NotifyPropertyChanged("VARRequestCountDisplay");
            }
        }

        public CollectionViewSource MobileRequestViewSource { get; }

        public AppointmentRequestItemPresentationModel SelectedMobileRequest
        {
            get { return _selectedMobileRequest; }
            set
            {
                _selectedMobileRequest = value;
                NotifyPropertyChanged( "SelectedMobileRequest");
                RequestSelectedCommand.RaiseCanExecuteChanged();
            }
        }

        public string VARRequestCountDisplay
        {
            get { return MobileRequestList.Count == 0 ? " None " : " " + MobileRequestList.Count + " "; }
        }

        public string Title
        {
            get { return "Mobile Appointment Requests"; }
        }

        public IVARView View { get; }

        public ValidationMessage ValidationMessage
        {
            get { return _validationMessage; }
            set { _validationMessage = value; }
        }

        public List<string> WaitTimeValues
        {
            get { return _waitTimeValues; }
        }

        public string WaitTimeValue
        {
            get { return _waitTimeValue; }
            set
            {

                if (value != _waitTimeValue)
                {
                    _waitTimeValue = value;
                    FilterByWaitTime(string.Empty);
                    NotifyPropertyChanged( "WaitTimeValue");
                }
            }
        }

        #endregion IVARPresentationModel Members

        #region private members

        private void FilterByWaitTime(string command)
        {
            MobileRequestList.Clear();

            foreach (var request in _unfilteredMobileRequestList)
                MobileRequestList.Add(request);

            MobileRequestViewSource.View.Refresh();
        }

        private void LoadMobileRequests(WorkCompletedMethod workCompletedMethod)
        {
            DoWorkAsync((s, args) =>
            {
                args.Result = LoadMobileRequests();
            }, workCompletedMethod);
        }

        private List<AppointmentRequestItemPresentationModel> LoadMobileRequests()
        {
            _unfilteredMobileRequestList.Clear();
            IList<APPOINTMENT_REQUEST_LIST_ITEM> list = _webServiceAPI.GetAppointmentRequestList();
            IList<AppointmentRequestItemPresentationModel> newList = new List<AppointmentRequestItemPresentationModel>();
            List<AppointmentRequestItemPresentationModel> unfilteredList = new List<AppointmentRequestItemPresentationModel>();

            foreach (var rec in list)
                newList.Add(new AppointmentRequestItemPresentationModel(_container, _eventAggregator, rec));

            var eligibilityBlank = newList.OrderByDescending(c => c.WaitTime);

            foreach (var rec in eligibilityBlank)
                unfilteredList.Add(rec);

            return unfilteredList;
        }

        private bool _doRefresh = true;
        private void VAR_RefreshEventHandler(string command)
        {
            if (command == "HOLD")
            {
                _doRefresh = false;
                return;
            }

            if (command == "UNHOLD")
            {
                _doRefresh = true;
            }

            if (_doRefresh)
            {
                _refreshingVAR = true;

                LoadMobileRequests((s, args) =>
                {
                    lock (args)
                    {
                        var selectedRequest = SelectedMobileRequest;
                        
                        try
                        {
                            _unfilteredMobileRequestList = (List<AppointmentRequestItemPresentationModel>)args.Result;
                            FilterByWaitTime(command);
                        }
                        finally
                        {
                            if (selectedRequest != null)
                            {
                                SelectedMobileRequest = MobileRequestList.FirstOrDefault(r => r.APPOINTMENT_REQUEST_ID == selectedRequest.APPOINTMENT_REQUEST_ID);

                                if (SelectedMobileRequest != null)
                                    SelectedMobileRequest.IsLocked = selectedRequest.IsLocked;

                                View.ScrollSelectedRequestIntoView();
                            }

                            _refreshingVAR = false;
                        }
                    }
                });
            }
        }

        #endregion private members
    }
}
