﻿using System;
using System.Linq;
using System.Timers;
using System.Windows;

using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using Microsoft.Practices.Composite.Presentation.Commands;
using Microsoft.Practices.Unity;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Modules.VAR.Events;
using ClinSchd.Modules.VAR.Models;
using ClinSchd.Modules.VAR.Services;

namespace ClinSchd.Modules.VAR.VARButton
{
    class VARButtonPresentationModel : AsyncValidatableModel, IVARButtonPresentationModel
    {
        private readonly IUnityContainer _container;
        public readonly IEventAggregator EventAggregator;
        private readonly IVARWebServiceAPIService _webServiceAPI;
        private bool _varRequestEnabled = true;
        private bool _varWindowOpen;

        private readonly Timer _refreshTimer;

        #region constructor

        public VARButtonPresentationModel(IVARButtonView view, IUnityContainer container, IEventAggregator eventAggregator, IVARWebServiceAPIService webServiceAPI)
        {
            View = view;
            _container = container;
            EventAggregator = eventAggregator;
            _webServiceAPI = webServiceAPI;

            EventAggregator.GetEvent<DisplayDefaultDivisionEvent>().Subscribe(SaveDivisionInfo, ThreadOption.UIThread, true);
            EventAggregator.GetEvent<VAR_Refresh>().Subscribe(VAR_RefreshEventHandler, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<VAR_LaunchRequestDialog>().Subscribe(LaunchRequestDialogEvent, ThreadOption.UIThread, true);

            LaunchVARRequestCommand = new DelegateCommand<string>(ExecuteLaunchVARRequestCommand, CanLaunchVARRequestCommand);

            _refreshTimer = new Timer( new TimeSpan( 0, 1, 00 ).TotalMilliseconds )
            {
                Enabled = false
            };
            _refreshTimer.Elapsed += refreshTimer_Elapsed;
        }

        #endregion constructor

        #region IVARButtonPresentationModel members

        private bool _isVARRequestUser;

        public bool IsVARRequestUser
        {
            get { return _isVARRequestUser; }
            set
            {
                _isVARRequestUser = value;

                if (_isVARRequestUser)
                    _refreshTimer.Start();

                NotifyPropertyChanged("VARRequestCountDisplay");
                NotifyPropertyChanged( "VARRequestToolTip");
                NotifyPropertyChanged( "VARCounterVisibility");
                LaunchVARRequestCommand.RaiseCanExecuteChanged();
            }
        }

        public IVARButtonView View { get; }

        #endregion IVARButtonPresentationModel members

        #region VAR Mobile Request Members

        public DelegateCommand<string> LaunchVARRequestCommand { get; }

        public bool CanLaunchVARRequestCommand(string command)
        {
            return _isVARRequestUser && _varRequestCount > -1 && _varRequestEnabled && !_varWindowOpen;
        }

        public void ExecuteLaunchVARRequestCommand(string command)
        {
            if (VARRequestCount == 0)
                View.AlertUser("There are no pending Mobile Requests", "Mobile Requests");
            else
            {
                EventAggregator.GetEvent<VAR_LaunchRequestDialog>().Publish("LAUNCH");
                _varWindowOpen = true;
                LaunchVARRequestCommand.RaiseCanExecuteChanged();
            }
        }

        public void IsVARUserEventHandler(bool isVARRequestUser)
        {
            IsVARRequestUser = isVARRequestUser;
        }

        public Visibility VARCounterVisibility { get { return _isVARRequestUser ? Visibility.Visible : Visibility.Hidden; } }

        private int _varRequestCount;
        public int VARRequestCount
        {
            get { return _varRequestCount; }
            set
            {
                _varRequestCount = value;
                NotifyPropertyChanged( "VARRequestCount");
                NotifyPropertyChanged( "VARRequestCountDisplay");
                NotifyPropertyChanged( "VARRequestToolTip");
                LaunchVARRequestCommand.RaiseCanExecuteChanged();
            }
        }

        public string VARRequestCountDisplay
        {
            get
            {
                return _isVARRequestUser ? (_varRequestCount == -1 ? " UNAV " : (_varRequestCount == 0 ? " None " : " " + _varRequestCount + " ")) : string.Empty;
            }
        }

        public string VARRequestToolTip
        {
            get
            {
                if (_varRequestCount == -1)
                    return "The Mobile Appointment Request Service is unavailable.";
                else
                    return (_varRequestCount == 0 ? "No" : _varRequestCount.ToString()) + " Mobile Appointment Request" + (_varRequestCount == 1 ? "" : "s") + " pending.";
            }
        }

        public string VARUserPreferenceToolTip
        {
            get { return "Set User Preferences and Default User Preferences."; }
        }

        #endregion VAR Mobile Request Members

        #region private members

        private void refreshTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            EventAggregator.GetEvent<VAR_Refresh>().Publish("REFRESH");
        }

        public void SaveDivisionInfo(string divisionInfo)
        {
            var vseInformation = _container.Resolve<IVSE_Information>();

            if (divisionInfo.Contains("^"))
            {
                //DivisionIEN^DivisionName
                string[] parts = divisionInfo.Split('^');
                vseInformation.FACILITY_CODE = parts[0];
                vseInformation.FACILITY_NAME = parts[1];
            }
            else
            {
                vseInformation.FACILITY_CODE = string.Empty;
                vseInformation.FACILITY_NAME = divisionInfo;
            }

            refreshTimer_Elapsed(_refreshTimer, null);
        }

        private void UpdateRequestCount(WorkCompletedMethod workCompletedMethod)
        {
            DoWorkAsync((s, args) =>
            {
                args.Result = UpdateRequestCount();
            }, workCompletedMethod);
        }

        private RequestCountReturn UpdateRequestCount()
        {
            string errorMessages;
            int retVal = _webServiceAPI.GetAppointmentRequestCount(out errorMessages);
            return new RequestCountReturn
            {
                RequestCount = retVal,
                ErrorMessages = errorMessages
            };
        }

        private void VAR_RefreshEventHandler(string command)
        {
            if (_refreshTimer.Enabled)
                _refreshTimer.Stop();

            UpdateRequestCount((s, args) =>
            {
                lock (args)
                {
                    RequestCountReturn requestCountReturn = (RequestCountReturn)args.Result;

                    if (requestCountReturn.ErrorMessages.Any())
                    {
                        TraceLog.LogInfo("VARButton", "VSE_VAR_Service Error", requestCountReturn.ErrorMessages);

                        if (requestCountReturn.RequestCount == 0)
                        {
                            MessageBox.Show(
                                "There was a problem retrieving the Mobile Request List Count." +
                                "\r\nPlease contact the system administrator with the Trace Log information." +
                                "\r\nThe Mobile Request List will be unavailable.",
                                "Mobile Request Error",
                                MessageBoxButton.OK,
                                MessageBoxImage.Warning
                            );
                            _varRequestEnabled = false;
                            LaunchVARRequestCommand.RaiseCanExecuteChanged();
                        }
                        else if ((requestCountReturn.RequestCount == -1) && (_varRequestCount != -1))
                        {
                            MessageBox.Show(
                                "There was a problem communicating with the Mobile Request Service." +
                                "\r\nPlease review the Trace Log for more detailed information." +
                                "\r\nThe Mobile Request List will be unavailable until further notice.",
                                "Mobile Request Error",
                                MessageBoxButton.OK,
                                MessageBoxImage.Warning
                            );

                            _refreshTimer.Start();
                        }
                        else if (requestCountReturn.RequestCount == -1)
                            _refreshTimer.Start();
                    }
                    else
                    {
                        _varRequestEnabled = true;
                        LaunchVARRequestCommand.RaiseCanExecuteChanged();
                        _refreshTimer.Start();
                    }

                    VARRequestCount = requestCountReturn.RequestCount;
                }
            });
        }

        private void LaunchRequestDialogEvent(string command)
        {
            if (command == "CLOSING")
            {
                _varWindowOpen = false;
                LaunchVARRequestCommand.RaiseCanExecuteChanged();
            }
        }

        #endregion private members

        private class RequestCountReturn
        {
            public int RequestCount { get; set; }
            public string ErrorMessages { get; set; }
        }
    }
}
