﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Runtime.InteropServices;
using System.Windows;
using System.Windows.Interop;
using System.Threading;

namespace ClinSchd
{
    internal class GlobalHotKeyHandler : IDisposable
    {
        public GlobalHotKeyHandler(string name, int hotKey, int modifiers, short id, Action<object> action)
        {
            Name = name;
            HotKey = hotKey;
            Modifiers = modifiers;
            ID = id;
            _handler = action;
        }

        public string Name { get; private set; }
        public int HotKey { get; private set; }
        public int Modifiers { get; private set; }
        public short ID { get; private set; }
        private Action<object> _handler { get; set; }

        public void ExecuteAction(Window window)
        {
            if (_handler != null)
                _handler(window);
        }

        public void UnhookHandler()
        {
            _handler = null;
        }

        #region IDisposable members

        protected void Dispose(Boolean itIsSafeToAlsoFreeManagedObjects)
        {
            try
            {
                // Free unmanaged resources

                //Free managed resources too, but only if I'm being called from Dispose
                //(If I'm being called from Finalize then the objects might not exist
                //anymore
                if (itIsSafeToAlsoFreeManagedObjects)
                {
                    if (itIsSafeToAlsoFreeManagedObjects)
                    {
                        UnhookHandler();
                    }
                }
            }
            catch (Exception)
            {
                //_traceLog.LogException(this.GetType().Name, "Dispose", ex);
            }
        }

        public void Dispose()
        {
            Dispose(true); //true: safe to free managed resources
            GC.SuppressFinalize(this); //Hey, GC: don't bother calling finalize later
        }

        ~GlobalHotKeyHandler()
        {
            //we're being finalized (i.e. destroyed), call Dispose in case the user forgot to
            Dispose(false);
        }

        #endregion IDisposable members
    }

    public class GlobalHotKeyManager : IDisposable
    {
        [DllImport("user32", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
#pragma warning disable CS3001 // Argument type is not CLS-compliant
        public static extern bool RegisterHotKey(IntPtr hwnd, int id, uint fsModifiers, uint vk);
#pragma warning restore CS3001 // Argument type is not CLS-compliant
        [DllImport("user32", SetLastError = true)]
        public static extern int UnregisterHotKey(IntPtr hwnd, int id);
        [DllImport("kernel32", SetLastError = true)]
        public static extern short GlobalAddAtom(string lpString);
        [DllImport("kernel32", SetLastError = true)]
        public static extern short GlobalDeleteAtom(short nAtom);


        public const int MOD_ALT = 1;
        public const int MOD_CONTROL = 2;
        public const int MOD_SHIFT = 4;
        public const int MOD_WIN = 8;

        public const int WM_HOTKEY = 0x312;

        private HwndSource _source;
        private Window _window = null;
        private Dictionary<string, GlobalHotKeyHandler> _hotKeysList = new Dictionary<string, GlobalHotKeyHandler>();

        public GlobalHotKeyManager(Window window)
        {
            _window = window;
            var helper = new WindowInteropHelper(_window);
            _source = HwndSource.FromHwnd(helper.Handle);
            _source.AddHook(HwndHook);
            //            RegisterHotKey();
        }

        private IntPtr HwndHook(IntPtr hwnd, int msg, IntPtr wParam, IntPtr lParam, ref bool handled)
        {
            const int WM_HOTKEY = 0x0312;
            switch (msg)
            {
                case WM_HOTKEY:
                    
                    if ((_window.Visibility == Visibility.Visible) && (_window.WindowState != WindowState.Minimized) && ApplicationIsActivated())
                    {
                        GlobalHotKeyHandler gHandler = _hotKeysList.Values.FirstOrDefault(h => (h.ID == wParam.ToInt32()));

                        if (gHandler != null)
                        {
                            gHandler.ExecuteAction(_window);
                            handled = true;
                        }
                    }
                    break;
            }
            return IntPtr.Zero;
        }

        /// <summary>Register the hotkey</summary>
        public void RegisterGlobalHotKey(string name, int hotkey, int modifiers, Action<object> handler)
        {
            UnregisterGlobalHotKey(name);

            try
            {
                // use the GlobalAddAtom API to get a unique ID (as suggested by MSDN)
                string atomName = Thread.CurrentThread.ManagedThreadId.ToString("X8") + this.GetType().FullName;
                short HotkeyID = GlobalAddAtom(atomName);

                if (HotkeyID == 0)
                    throw new Exception("Unable to generate unique hotkey ID. Error: " + Marshal.GetLastWin32Error().ToString());

                // register the hotkey, throw if any error
                var helper = new WindowInteropHelper(_window);
                if (!RegisterHotKey(helper.Handle, HotkeyID, (uint)modifiers, (uint)hotkey))
                    throw new Exception("Unable to register hotkey. Error: " + Marshal.GetLastWin32Error().ToString());

                _hotKeysList.Add(name, new GlobalHotKeyHandler(name, hotkey, modifiers, HotkeyID, handler));
            }
            catch (Exception ex)
            {
                // clean up if hotkey registration failed
                Dispose();
                Console.WriteLine(ex);
            }
        }

        public void UnregisterAllHotKeys()
        {
            foreach (string name in _hotKeysList.Keys)
                UnregisterGlobalHotKey(name);

            _hotKeysList.Clear();
        }

        /// <summary>Unregister the hotkey</summary>
        public void UnregisterGlobalHotKey(string name)
        {
            if (!_hotKeysList.ContainsKey(name))
                return;

            GlobalHotKeyHandler gHandler = _hotKeysList[name];

            if ((gHandler != null) && (gHandler.ID != 0))
            {
                gHandler.UnhookHandler();
                var helper = new WindowInteropHelper(_window);
                UnregisterHotKey(helper.Handle, gHandler.ID);
                // clean up the atom list
                GlobalDeleteAtom(gHandler.ID);
                _hotKeysList.Remove(name);
            }
        }

        public static bool ApplicationIsActivated()
        {
            var activatedHandle = GetForegroundWindow();
            if (activatedHandle == IntPtr.Zero)
            {
                return false;       // No window is currently activated
            }

            var procId = Process.GetCurrentProcess().Id;
            int activeProcId;
            GetWindowThreadProcessId(activatedHandle, out activeProcId);

            return activeProcId == procId;
        }


        [DllImport("user32.dll", CharSet = CharSet.Auto, ExactSpelling = true)]
        private static extern IntPtr GetForegroundWindow();

        [DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        private static extern int GetWindowThreadProcessId(IntPtr handle, out int processId);

        #region IDisposable members

        protected void Dispose(Boolean itIsSafeToAlsoFreeManagedObjects)
        {
            try
            {
                // Free unmanaged resources

                //Free managed resources too, but only if I'm being called from Dispose
                //(If I'm being called from Finalize then the objects might not exist
                //anymore
                if (itIsSafeToAlsoFreeManagedObjects)
                {
                    if (itIsSafeToAlsoFreeManagedObjects)
                    {
                        if (_hotKeysList != null)
                            UnregisterAllHotKeys();

                        _source.RemoveHook(HwndHook);
                        _source = null;
                    }
                }
            }
            catch //(Exception ex)
            {
                //_traceLog.LogException(this.GetType().Name, "Dispose", ex);
            }
        }

        public void Dispose()
        {
            Dispose(true); //true: safe to free managed resources
            GC.SuppressFinalize(this); //Hey, GC: don't bother calling finalize later
        }

        ~GlobalHotKeyManager()
        {
            //we're being finalized (i.e. destroyed), call Dispose in case the user forgot to
            Dispose(false);
        }

        #endregion IDisposable members
    }
}