using System;
using System.Windows.Input;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.InactivityTimeout;
using ClinSchd.Modules.VAR.Events;

namespace ClinSchd
{
    public class ShellPresenter
    {
        private readonly IEventAggregator _eventAggregator;
        private readonly IDataAccessService _dataAccessService;
        private int _timeoutSeconds;
        private int _countdownSeconds;
        private InactivityTimer _inactivityTimer;

        public ShellPresenter(IShellView view, IEventAggregator eventAggregator, IDataAccessService dataAccessService)
        {
            View = view;
            View.Presenter = this;
            _eventAggregator = eventAggregator;
            _dataAccessService = dataAccessService;
            _eventAggregator.GetEvent<InitializeIdleTimeTimeOutEvent>().Subscribe(InitializeIdleTimeTimeOutEventHandler);
            _eventAggregator.GetEvent<RibbonBarSelectedTabChangedEvent>().Subscribe(RibbonBarSelectedTabChangedEventHandler, ThreadOption.UIThread, true);
        }

        public IShellView View { get; }

        public IEventAggregator EventAggregator => _eventAggregator;

        internal void KeyPressed(KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.Control)
            {
                switch (e.Key)
                {
                    case Key.X:
                        if (Keys.ConfirmUser(null, "Are you sure you want to close VSE Scheduler ?", "Close VSE Scheduler", true))
                        {
                            EventAggregator.GetEvent<ExitApplicationEvent>().Publish("Exiting");
                            _dataAccessService.Logout();
                            _dataAccessService.RemoveJasperReportDependenciesFromUsersFolder();
                            Environment.Exit(1);
                        }

                        break;
                    case Key.F:
                        _eventAggregator.GetEvent<FocusPatientSearchEvent>().Publish(null);
                        break;
                    case Key.S:
                        _eventAggregator.GetEvent<FocusSchedulerEvent>().Publish(null);
                        break;
                    case Key.C:
                        _eventAggregator.GetEvent<CopyKeysPressedEvent>().Publish(null);
                        break;
                    case Key.V:
                        _eventAggregator.GetEvent<PasteKeysPressedEvent>().Publish(null);
                        break;
                    case Key.G:
                        _eventAggregator.GetEvent<PatientQueueViewHotKeysEvent>().Publish(e.Key);
                        break;
                    case Key.Down:
                        e.Handled = true;
                        _eventAggregator.GetEvent<PatientQueueViewHotKeysEvent>().Publish(e.Key);
                        break;
                    case Key.Up:
                        e.Handled = true;
                        _eventAggregator.GetEvent<PatientQueueViewHotKeysEvent>().Publish(e.Key);
                        break;
                    case Key.D:
                        _eventAggregator.GetEvent<GroupViewHotKeyEvents>().Publish(e.Key);
                        break;
                    case Key.M:
                        _eventAggregator.GetEvent<VAR_HotKeyEvent>().Publish(e.Key);
                        break;
                    case Key.L:
                        TraceLog.ViewLog(View.Handle);
                        break;
                    default:
                        _eventAggregator.GetEvent<RibbonBarHotKeysEvent>().Publish(e.Key);
                        break;
                }
            }

            else if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                var key = e.Key == Key.System ? e.SystemKey : e.Key;
                _eventAggregator.GetEvent<RibbonBarHotKeysEvent>().Publish(key);
            }
        }

        private void InitializeIdleTimeTimeOutEventHandler(string parms)
        {
            var parts = parms.Split('^');

            if ( parts[0] != "INITIALIZE" )
                return;

            if ( parts.Length != 3 )
                return;

            if (Keys.Methods.IsNumeric(parts[1]))
                _timeoutSeconds = Convert.ToInt32(parts[1]);

            if (Keys.Methods.IsNumeric(parts[2]))
                _countdownSeconds = Convert.ToInt32(parts[2]);

            _inactivityTimer = new InactivityTimer(TimeSpan.FromSeconds(_timeoutSeconds));
            _inactivityTimer.Inactivity += InactivityDetected;
        }

        private void InactivityDetected(object sender, EventArgs e)
        {
            var cv = new CountDownView( _countdownSeconds )
            {
                Owner = View.Instance
            };
            var result = cv.ShowDialog();

            if (!(result.HasValue && result.Value))
                TimedOutEventHandler(cv, EventArgs.Empty);
        }

        public void TimedOutEventHandler(object sender, EventArgs e)
        {
            _eventAggregator.GetEvent<ExitApplicationEvent>().Publish("Exiting");
            _dataAccessService.Logout();
            _dataAccessService.RemoveJasperReportDependenciesFromUsersFolder();
            Environment.Exit(1);
        }

        private void RibbonBarSelectedTabChangedEventHandler(string command)
        {
            View.SetSelectedTab(command);
        }

        public void ExecuteAvailabilityResourceCommand(string title)
        {
            CompositePresentationEvent<SchdResource> newforwardingEvent = _eventAggregator.GetEvent<AvailabilityResourcesEvent>();
            _eventAggregator.GetEvent<LaunchResourceAvailabilityDialogEvent>().Publish(newforwardingEvent);
        }

        private bool _firstActivation = true;

        public void ViewActivated()
        {
            if (_firstActivation)
            {
                _eventAggregator.GetEvent<LaunchUserLoginDialogEvent>().Publish("User Login");
                _firstActivation = false;
            }
        }
    }
}